      subroutine atdath
      implicit real*8(a-h,o-z)
c
c  Atomic data for H I ; ie = 1, ion = 1
c
c  This routine gives a hydrogen atom consisting of the
c  lowest nn n-states divided up into individual l-states.
c  The higher n-states are kept as single levels.
c  The same routine as atdath_nl_n in hiatom_nl_n.f .
c 
c  nn = number of n-states split up into individual l-states ;
c       nn >= 2 or the treatment of the 2-photon transition
c       has to be looked over
c  nlev = total number of levels
c  nm = nmn = total number of n-states included
c
c  A-values : A(nl;n'l') are calculated using expressions in
c             Brocklehurst, 1971, MNRAS, 153, p 471.
c             Eqs. used are originally from Gordon (1929).
c             For n.le.20 the A-values are checked (1) by comparing
c             calculated radial integrals (ri2) with those tabulated by
c             Green, Rush, & Chandler, 1957, ApJS, 3, p 37,
c             (2) by comparing the averaged A(n;n') with the expressions
c             given by Johnson, L.C. 1972, ApJ 174, 227.
c             In both tests the agreement was very good. 
c
c  nlev = number of levels included
c  e    = energy of the levels in eV
c  a    = transition probabilities
c  e00  = ionization potential in eV
c  g    = statistical weights of the levels
c
      include 'INCLUDE/PARAM.IOR'
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
c
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/cmlevpop/xhi(nmax),xhei(nl),xoi(nl),xmg1(nl),xsi1(nl),
     &     xcaii(nl),xfei(nlf1),xfeii(nlf2),xfeiii(nlf3),xfeiv(nlf4)
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/cionpot/e00(mie,mion)
      common/cebin/e1(ne),en(ne),jmax
      common/multijbin/jline(12,nl,nl),jlinef1(nlf1,nlf1),
     &     jlinef2(nlf2,nlf2),jlinef3(nlf3,nlf3),jlinef4(nlf4,nlf4),
     &     jlineh(nm,nm)
      common/chirec/tehr(40),alfth(40),reconl(nmx),recon(nm)
c      common/chirec/tehr(40),alfth(40),alfi(20,40)
      common/cvoigt/avhi(nm),avhei(nl),avoi(nl),avcaii(nl),
     &     avfei(nlf1),avfeii(nlf2),avfeiii(nlf3),avfeiv(nlf4)
      dimension ap(0:nm),am(0:nm)
c
      ie=1
      ion=1
c
      nmn=nm
      nn=5
c
      do n=1,nm
         do l=0,nm
            nlm(n,l)=0
         enddo
      enddo
c
      nlevh=0
      do n=1,nmn
         if (n.le.nn) then
            do l=0,n-1
               nlm(n,l)=nlevh+1+l
               nln(nlm(n,l))=n
            enddo
            nlevh=nlevh+n
            nnm(n)=nlm(n,0)
            i=nlevh
         else
            nlevh=nlevh+1
            nnm(n)=nlevh
            nln(nnm(n))=n
c
            do l=0,n-1
               i=i+1
               nlm(n,l)=i
            enddo
c
         endif
      enddo
      nlev(ie,ion)=nlevh
      nlevm(ie,ion)=nlevh
c
      e00(ie,ion)=13.5972
      ehn(1)=0.
      ehn(2)=10.1981
      ehn(3)=12.0866
      ehn(4)=12.7477
      ehn(5)=13.0536
      ehn(6)=13.2198
c
      do i=7,nmn
        rn=dble(i)
        gi=2.*rn**2.
        ghn(i)=gi
        ei=e00(ie,ion)*(1.-1./rn**2.)
        ehn(i)=ei
      enddo
c
      ghn(1)=2.
      ghn(2)=8.
      ghn(3)=18.
      ghn(4)=32.
      ghn(5)=50.
      ghn(6)=72.
c
      do j=1,nmax
         do i=1,nmax
            ah(j,i)=0.d0
         enddo
      enddo
c
c      do j=1,nm
c         do i=1,nm
c            ab(j,i)=0.d0
c         enddo
c      enddo
c
c   Two-photon transition 2 2S - 1 2S
c      
      ah(nlm(2,0),nlm(1,0))=8.23
c
      const=2.6774d9
      do nj=1,nm
         do ni=1,nj-1
            c1=(1./dble(ni*ni)-1./dble(nj*nj))**3.
            xnj=dble(nj)
            xni=dble(ni)
            xnsum=xnj+xni
            xndiff=xnj-xni
            do lj=0,nj-1
c
               xlj=dble(lj)
               li=lj-1
               xli=dble(li)
               am(lj)=0.
               if (li.ge.0.and.li.le.(ni-1)) then
                  c2=c1*dble(lj)/(2.*dble(lj)+1)
c
                  x=-4.*xnj*xni/(xnj-xni)**2
                  hgfp=hgf(-nj+lj+1,-ni+lj,2*lj,x)
c
                  hgfm=hgf(-nj+lj-1,-ni+lj,2*lj,x)
c
                  hgfsum=hgfp-((xndiff*xndiff)/(xnsum*xnsum))*hgfm
c
c cc1 = (-1)^(ni-lj)/4(2lj-1)!
                  xnf1=1
                  do il=2,2*lj-1
                     xnf1=xnf1*dble(il)
                  enddo
                  xnf1=4*xnf1
                  cc1=1./xnf1
c cc2 = sqrt(((nj+lj)!(ni+lj-1)!)/((nj-lj-1)!(ni-lj)!))
                  xnf1=1
                  xnf2=1
                  xnf3=1
                  xnf4=1
                  do il=2,nj+lj
                     xnf1=xnf1*dble(il)
                  enddo
                  do il=2,ni+lj-1
                     xnf2=xnf2*dble(il)
                  enddo
                  do il=2,nj-lj-1
                     xnf3=xnf3*dble(il)
                  enddo
                  do il=2,ni-lj
                     xnf4=xnf4*dble(il)
                  enddo
c
c                  cc2=sqrt((xnf1*xnf2)/(xnf3*xnf4))
                  cc2=sqrt(xnf1/xnf3)*sqrt(xnf2/xnf4)
c cc3 = (4*nj*ni)^(lj+1)/(nj+ni)^(nj+ni)
                  cc3=(((4.*xnj*xni)**(xlj+1.))/(xnsum**xnsum))
c cc4 = (nj-ni)^(nj+ni-2lj-2)
                  x1=xnsum-2.*xlj-2.
c                  cc4=((xnj-xni)**x1)
                  cc4=xndiff**x1
c
                  ri=cc1*cc2*cc3*cc4*hgfsum
                  ri2=ri*ri
                  nj_lj=nlm(nj,lj)
                  ni_li=nlm(ni,li)
                  am(lj)=const*c2*ri2
               endif
c
               li=lj+1
               xli=dble(li)
               ap(lj)=0.
               if (li.ge.0.and.li.le.(ni-1)) then
                  c2=c1*dble(li)/(2.*dble(lj)+1)
c
                  x=-4.*xnj*xni/(xnj-xni)**2
                  hgfp=hgf(-ni+li+1,-nj+li,2*li,x)
c
                  hgfm=hgf(-ni+li-1,-nj+li,2*li,x)
c
                  hgfsum=hgfp-((xndiff*xndiff)/(xnsum*xnsum))*hgfm
c
c cc1 = (-1)^(nj-li)/4(2li-1)!
                  xnf1=1
                  do il=2,2*li-1
                     xnf1=xnf1*dble(il)
                  enddo
                  xnf1=4*xnf1
                  cc1=1./xnf1
c cc2 = sqrt(((ni+li)!(nj+li-1)!)/((ni-li-1)!(nj-li)!))
                  xnf1=1
                  xnf2=1
                  xnf3=1
                  xnf4=1
                  do il=2,ni+li
                     xnf1=xnf1*dble(il)
                  enddo
                  do il=2,nj+li-1
                     xnf2=xnf2*dble(il)
                  enddo
                  do il=2,ni-li-1
                     xnf3=xnf3*dble(il)
                  enddo
                  do il=2,nj-li
                     xnf4=xnf4*dble(il)
                  enddo
c
c                  cc2=sqrt((xnf1*xnf2)/(xnf3*xnf4))
                  cc2=sqrt(xnf1/xnf3)*sqrt(xnf2/xnf4)
c cc3 = (4*nj*ni)^(li+1)/(nj+ni)^(nj+ni)
                  cc3=(((4.*xnj*xni)**(xli+1.))/(xnsum**xnsum))
c cc4 = (ni-nj)^(nj+ni-2li-2)
                  x1=xnsum-2.*xli-2.
c                  cc4=((xni-xnj)**x1)
                  cc4=xndiff**x1
c
                  ri=cc1*cc2*cc3*cc4*hgfsum
                  ri2=ri*ri
                  nj_lj=nlm(nj,lj)
                  ni_li=nlm(ni,li)
                  ap(lj)=const*c2*ri2
               endif
c
            enddo
c
            if (nj.le.nn) then
               do lj=0,nj-1
                  nj_lj=nlm(nj,lj)
c
                  li=lj-1
                  if (li.ge.0.and.li.le.(ni-1)) then
                     ni_li=nlm(ni,li)
                     ah(nj_lj,ni_li)=am(lj)
                  endif
c
                  li=lj+1
                  if (li.le.(ni-1)) then
                     ni_li=nlm(ni,li)
                     ah(nj_lj,ni_li)=ap(lj)
                  endif
               enddo
c
               asum=0.d0
               gnj=2.*dble(nj)*dble(nj)
               do lj=0,nj-1
                  do li=0,ni-1
                     nj_lj=nlm(nj,lj)
                     ni_li=nlm(ni,li)
                     gnjlj=2.*(2.*dble(lj)+1.)
                     asum=asum+ah(nj_lj,ni_li)*gnjlj/gnj
                  enddo
               enddo
c               ab(nj,ni)=asum
c            
            elseif (nj.gt.nn.and.ni.le.nn) then
               inj=nnm(nj)
               gnj=2.*dble(nj)*dble(nj)
               do li=0,ni-1
                  ni_li=nlm(ni,li)
                  lj=li-1
                  a1=0.
                  if (lj.ge.0) then
                     glj=2.*(2.*dble(lj)+1.)
                     a1=ap(lj)*glj/gnj
                  endif
                  lj=li+1
                  glj=2.*(2.*dble(lj)+1.)
                  a2=am(lj)*glj/gnj
                  ah(inj,ni_li)=a1+a2
               enddo
c
               asum=0.
               inj=nnm(nj)
               do li=0,ni-1
                  ni_li=nlm(ni,li)
                  asum=asum+ah(inj,ni_li)
               enddo
c               ab(nj,ni)=asum
            elseif (ni.gt.nn) then
               inj=nnm(nj)
               ini=nnm(ni)
               asum=0.
               gnj=2.*dble(nj)*dble(nj)
               do lj=0,nj-1
                  nj_lj=nlm(nj,lj)
                  glj=2.*(2.*dble(lj)+1.)
                  li=lj-1
                  if (li.ge.0.and.li.le.(ni-1)) then
                     asum=asum+am(lj)*glj/gnj
                  endif
c
                  li=lj+1
                  if (li.le.(ni-1)) then
                     asum=asum+ap(lj)*glj/gnj
                  endif
               enddo   
               ah(inj,ini)=asum
c               ab(nj,ni)=asum
            endif
c
         enddo
      enddo
c
c      do nj=2,nm
c         avhi(nj)=0.
c         do lj=0,nj-1
c            j=nlm(nj,lj)
c            avhi(j)=0.
c            do ni=1,nj-1
c               do li=0,ni-1
c                  i=nlm(ni,li)
c                  avhi(j)=avhi(j)+ah(j,i)
c               enddo
c            enddo
c         enddo
c      enddo
c
c recombination data
c
      rewind(25)
      open(25,file='ATDAT/hirec.dat',status='old')
      do i=1,10
         read(25,*)
      enddo
c
      do i=1,40
         read(25,701) tehr(i),alfvf,alfth(i)
c         read(25,701) tehr(i),alfvf,alfth(i),alfsum,
c     &        (alfi(il,i),il=1,20)
      enddo
c
      close(25)
 701  format(f12.3,1pe12.4,25e12.4)
c
      return
      end



      subroutine atdath_old
      implicit real*8(a-h,o-z)
c
c  atomic data for H I ; ie = 1, ion = 1
c
c  nlev = number of levels included
c  e    = energy of the levels in eV
c  a    = transition probabilities
c  e00  = ionization potential in eV
c  g    = statistical weights of the levels
c
      include 'INCLUDE/PARAM.IOR'
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/cmlevpop/xhi(nmax),xhei(nl),xoi(nl),xmg1(nl),xsi1(nl),
     &     xcaii(nl),xfei(nlf1),xfeii(nlf2),xfeiii(nlf3),xfeiv(nlf4)
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/cionpot/e00(mie,mion)
      common/cebin/e1(ne),en(ne),jmax
      common/multijbin/jline(12,nl,nl),jlinef1(nlf1,nlf1),
     &     jlinef2(nlf2,nlf2),jlinef3(nlf3,nlf3),jlinef4(nlf4,nlf4),
     &     jlineh(nm,nm)
      common/chirec/tehr(40),alfth(40),reconl(nmx),recon(nm)
c      common/chirec/tehr(40),alfth(40),alfi(20,40)
      dimension alfi(20,40)
c
      ie=1
      ion=1
c
c      nlev(ie,ion)=5
c      nlevm(ie,ion)=5
      nlev(ie,ion)=20
      nlevm(ie,ion)=20
      n=nlev(ie,ion)
      e00(ie,ion)=13.5972
      e(ie,ion,1)=0.
      e(ie,ion,2)=10.1981
      e(ie,ion,3)=12.0866
      e(ie,ion,4)=12.7477
      e(ie,ion,5)=13.0536
      e(ie,ion,6)=13.2198
      do i=7,n
        rn=dble(i)
        g(ie,ion,i)=2.*rn**2.
        e(ie,ion,i)=e00(ie,ion)*(1.-1./rn**2.)
      enddo
c
      g(ie,ion,1)=2.
      g(ie,ion,2)=8.
      g(ie,ion,3)=18.
      g(ie,ion,4)=32.
      g(ie,ion,5)=50.
      g(ie,ion,6)=72.
      g(ie,ion,n+1)=1.
c     
c     A-values (or rather the bound-bound gaunt factor) 
c     from Johnson, L.C. 1972, ApJ 174, 227
c

      do i=1,n
        do j=1,n
          a(ie,ion,i,j)=0.
        enddo
      enddo
c
      do j=1,n
        do i=1,j-1
          rn1=dble(i)
          rn2=dble(j)
          x=1.-(rn1/rn2)**2.
          if (i.eq.1) then
            g0=1.1330
            g1=-.4059
            g2=0.07014
          elseif (i.eq.2) then
            g0=1.0785
            g1=-.2319
            g2=0.02947
          elseif (i.gt.2) then
            g0=.9935+.2328/rn1-.1296/rn1**2.
            g1=-(.6282-.5598/rn1+.5299/rn1**2.)/rn1
            g2=(.3887-1.181/rn1+1.470/rn1**2.)/rn1**2.
          endif
          gaunt=1.
          if(x.gt.0.) gaunt=g0+g1/x+g2/x**2.
          a(ie,ion,j,i)=1.
          if(x.gt.0) a(ie,ion,j,i)=1.57283e10*gaunt/(rn1*rn2**3.*
     &(rn2**2.-rn1**2.))
        enddo
      enddo
c
c recombination data
c
      rewind(25)
      open(25,file='ATDAT/hirec.dat',status='old')
      do i=1,10
         read(25,*)
      enddo
c
      do i=1,40
         read(25,701) tehr(i),alfvf,alfth(i),alfsum,
     &        (alfi(il,i),il=1,20)
      enddo
c
      close(25)
 701  format(f12.3,1pe12.4,25e12.4)
c
      return
      end

      SUBROUTINE ATDATHE
      IMPLICIT REAL*8(A-H,O-Z)
      character*80 dum
      character*6 lab
c
c  atomic data for he i ; ie = 5, ion = 1
c
c  nlev = number of levels included
c  e    = energy of the levels in eV
c  a    = transition probabilities
c  e00  = ionization potential in eV
c  g    = statistical weights of the levels
c
c  Ref :
c  
c
      include 'INCLUDE/PARAM.IOR'
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/cionpot/e00(mie,mion)
      COMMON/Heicex/OM(NL,NL)
      COMMON/HEIRECION/AL0(16),BE(16),B0(16),GA(16)
      common/heirecvf/teheir(12),alfhei(16,12)
      common/cebin/e1(ne),en(ne),jmax
      common/multijbin/jline(12,nl,nl),jlinef1(nlf1,nlf1),
     &     jlinef2(nlf2,nlf2),jlinef3(nlf3,nlf3),jlinef4(nlf4,nlf4),
     &     jlineh(nm,nm)
      common/cvoigt/avhi(nm),avhei(nl),avoi(nl),avcaii(nl),
     &     avfei(nlf1),avfeii(nlf2),avfeiii(nlf3),avfeiv(nlf4)
      dimension xtev(12),av(20)
c
      ie=2
      ion=1
c
      nlev(ie,ion)=16
      nlevm(ie,ion)=16
      n=nlev(ie,ion)
c  ionization potential
      e00(ie,ion)=24.5876
      rewind(25)
      open(25,file='ATDAT/heI.dat2',status='old')
      read(25,987)dum
      read(25,987)dum
987   format(a)
      do i=1,n
        read(25,*)nn,wn,g(ie,ion,i),al0(i),be(i),b0(i),ga(i)
        e(ie,ion,i)=wn/8065.46d0
      enddo
      DO I=1,N
        DO J=1,N
          OM(I,J)=0.
          A(ie,ion,I,J)=0.
        enddo
      enddo
      read(25,987)dum
      read(25,987)dum
      do i=1,n
        do j=i+1,n
          read(25,*)i1,i2,wlq,a(ie,ion,j,i),OM(I,J)
          a(ie,ion,j,i)=a(ie,ion,j,i)*1.e8
        enddo
      enddo
      close(25)
c
c  Radiative recombination coefficients from Verner and Ferland, 
c  1996, ApJS, in press.
c
      do it=1,12
         teheir(it)=0.d0
         do il=1,16
            alfhei(il,it)=0.d0
         enddo
      enddo
c
      rewind(25)
      open(25,file='ATDAT/heirec.dat',status='old')
      do i=1,9
         read(25,987) dum
      enddo
      read(25,701) (xtev(i),i=1,12)
      do it=1,12
         teheir(it)=10**xtev(it)
      enddo
      do i=1,10
         read(25,702) ii,(av(it),it=1,20),lab,il
         do it=1,12
            alfhei(il,it)=av(it)
         enddo
      enddo
      close(25)
 701  format(f8.1,20f9.1)
 702  format(1p,i2,20(1x,e8.2),a6,i3)
c
cqcq
c      nlev(ie,ion)=5
c      nlevm(ie,ion)=5
cqcq
cqcq
c      a(ie,ion,2,1)=a(ie,ion,4,1)
c      e(ie,ion,2)=e(ie,ion,4)
c      g(ie,ion,2)=g(ie,ion,4)
c      al0(2)=al0(4)
c      be(2)=be(4)
c      b0(2)=b0(4)
c      ga(2)=ga(4)
c      om(2,1)=0.
c      om(1,2)=0.
c      a(ie,ion,3,1)=0.
c      a(ie,ion,4,2)=0.
c      nlev(ie,ion)=4
c      nlevm(ie,ion)=4
cqcq
      do i=1,nlev(ie,ion)
         avhei(i)=0.d0
         do j=1,i-1
            avhei(i)=avhei(i)+a(ie,ion,i,j)
         enddo
      enddo
c
      RETURN
      END

      subroutine atdato_old
      implicit real*8(a-h,o-z)
c
c  atomic data for o i ; ie = 5, ion = 1
c
c  nlev = number of levels included
c  e    = energy of the levels in eV
c  a    = transition probabilities
c  e00  = ionization potential in eV
c  g    = statistical weights of the levels
c
      include 'INCLUDE/PARAM.IOR'
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/cionpot/e00(mie,mion)
      common/cebin/e1(ne),en(ne),jmax
      common/multijbin/jline(12,nl,nl),jlinef1(nlf1,nlf1),
     &     jlinef2(nlf2,nlf2),jlinef3(nlf3,nlf3),jlinef4(nlf4,nlf4),
     &     jlineh(nm,nm)
      common/cjbins/j4572mg1,j2852mg1,j2800mg2,j2853na1,
     &     j1334c2,j13ne2,j314s2,j7ar2,
     &     jo21,jo31,jo32,jsi21,jsi31,jsi32,jsi021,jsi031,jsi032
c
      ie=5
      ion=1
c
      nlev(ie,ion)=9
      nlevm(ie,ion)=9
      n=nlev(ie,ion)
c  ionization potential
      e00(ie,ion)=13.618
c  energy levels
c  2p4 3P
      e(ie,ion,1)=0.
c  2p4 1D
      e(ie,ion,2)=1.961
c  2p4 1S
      e(ie,ion,3)=4.172
c  3s 5So
      e(ie,ion,4)=9.137
c  3s 3So
      e(ie,ion,5)=9.508
c  3p 5P
      e(ie,ion,6)=10.732
c  3p 3P
      e(ie,ion,7)=10.976
c  3d 3D
      e(ie,ion,8)=12.073
c  3s' 3Do
      e(ie,ion,9)=12.523
c  statistical weights
      g(ie,ion,1)=9.
      g(ie,ion,2)=5.
      g(ie,ion,3)=1.
      g(ie,ion,4)=5.
      g(ie,ion,5)=3.
      g(ie,ion,6)=15.
      g(ie,ion,7)=9.
      g(ie,ion,8)=15.
      g(ie,ion,9)=15.
      g(ie,ion,n+1)=4.
      do i=1,n
        do j=1,n
          a(ie,ion,i,j)=0.
        enddo 
      enddo
c
c  transition probabilities
c
c     mendoza -83
      a(ie,ion,2,1)=8.45e-3
c     mendoza -83
      a(ie,ion,3,1)=7.35e-2
c     wiese et al skelto & shine list a(4,1)=5.88e3
      a(ie,ion,4,1)=1.70e3
c     wiese and martin, nbs -80
      a(ie,ion,5,1)=5.96e8
c  Bhatia & Kastner, 1995, ApJS, 96, 325. 3s 3S -> 1D
      a(ie,ion,5,2)=1.84e3
c  Bhatia & Kastner, 1995, ApJS, 96, 325. 3s 3S -> 1S
      a(ie,ion,5,3)=4.60
c     christenson and cunningham, j. geophys. res. 83, 4393 -78
      a(ie,ion,8,1)=.770e8
c     christenson and cunningham, j. geophys. res. 83, 4393 -78
      a(ie,ion,9,1)=2.28e8
c     mendoza -83
      a(ie,ion,3,2)=1.22e0
      a(ie,ion,5,4)=0.
c     wiese and martin, nbs -80
      a(ie,ion,6,4)=0.34e8
c     erikson & toft
      a(ie,ion,7,4)=8.52e4
      a(ie,ion,6,5)=0.e0
c     wiese, smith  and glennon, nbs -80
      a(ie,ion,7,5)=2.80e7
c     christenson and cunningham, j. geophys. res. 83, 4393 -78
      a(ie,ion,8,7)=.303e8
c     christenson and cunningham, j. geophys. res. 83, 4393 -78
      a(ie,ion,9,7)=9.59e4
c
c  Energy bins for O I 63.18, 145.5 my (used in forb3nr)
c
      e21=1.965d-2
      do j1=1,jmax
         if(e21.gt.en(j1).and.e21.le.en(j1+1)) then
            jo21=j1
            goto 91
         endif
      enddo
 91   continue
      e32=8.440d-3
      do j1=1,jmax
         if(e32.gt.en(j1).and.e32.le.en(j1+1)) then
            jo32=j1
            goto 93
         endif
      enddo
 93   continue
      e31=e21+e32
      do j1=1,jmax
         if(e31.gt.en(j1).and.e31.le.en(j1+1)) then
            jo31=j1
            goto 92
         endif
      enddo
 92   continue
c
      return
      end


      subroutine atdato
      implicit real*8(a-h,o-z)
c
c  atomic data for o i ; ie = 5, ion = 1
c
c  nlev = number of levels included
c  e    = energy of the levels in eV
c  a    = transition probabilities
c  e00  = ionization potential in eV
c  g    = statistical weights of the levels
c
      include 'INCLUDE/PARAM.IOR'
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/cionpot/e00(mie,mion)
      common/cebin/e1(ne),en(ne),jmax
      common/multijbin/jline(12,nl,nl),jlinef1(nlf1,nlf1),
     &     jlinef2(nlf2,nlf2),jlinef3(nlf3,nlf3),jlinef4(nlf4,nlf4),
     &     jlineh(nm,nm)
      common/cjbins/j4572mg1,j2852mg1,j2800mg2,j2853na1,
     &     j1334c2,j13ne2,j314s2,j7ar2,
     &     jo21,jo31,jo32,jsi21,jsi31,jsi32,jsi021,jsi031,jsi032
      common/comoi/aoi(nl,nl),boi(nl,nl)
      common/cvoigt/avhi(nm),avhei(nl),avoi(nl),avcaii(nl),
     &     avfei(nlf1),avfeii(nlf2),avfeiii(nlf3),avfeiv(nlf4)
c
      ie=5
      ion=1
c
      nlev(ie,ion)=13
      nlevm(ie,ion)=13
      n=nlev(ie,ion)
c  ionization potential
      e00(ie,ion)=13.618
      rewind(25)
      open(25,file='ATDAT/OI.dat',status='old')
      read(25,*) 
      read(25,*) 
      read(25,*) 
      do i=1,n
         read(25,*) nn,ecmm1,g(ie,ion,i)
         e(ie,ion,i)=12398.5d0*ecmm1*1.e-8
      enddo
      g(ie,ion,n+1)=4.
      do i=1,n
        do j=1,n
          a(ie,ion,i,j)=0.
        enddo 
      enddo
c
      read(25,*)
      read(25,*)
      do i=1,n
         do j=i+1,n
            read(25,*) i1,i2,a(ie,ion,j,i),o5000,o10000
            if (i.ne.i1) write(*,*) i,i1
            if (j.ne.i2) write(*,*) j,i2
            aoi(i,j)=o5000
            if (o5000.le.0.) then
               boi(i,j)=1.
            else
               boi(i,j)=log(o10000/o5000)/log(2.)
            endif
         enddo
      enddo
c
      close(25)
c
      do i=1,nlev(ie,ion)
         avoi(i)=0.d0
         do j=1,i-1
            avoi(i)=avoi(i)+a(ie,ion,i,j)
         enddo
      enddo
c
      return
      end

      subroutine atdatmgi_old
      implicit real*8(a-h,o-z)
c
c  atomic data for Mg I 
c
c  e    = energy of the levels in eV
c
      include 'INCLUDE/PARAM.IOR'
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/cebin/e1(ne),en(ne),jmax
      common/cjbins/j4572mg1,j2852mg1,j2800mg2,j2853na1,
     &     j1334c2,j13ne2,j314s2,j7ar2,
     &     jo21,jo31,jo32,jsi21,jsi31,jsi32,jsi021,jsi031,jsi032
c
      ie=8
      ion=1
c
c  3s 1S
      e(ie,ion,1)=0.
c  3p 3P
      e(ie,ion,2)=2.71
c  3p 1P
      e(ie,ion,3)=4.345
c
      e21=e(ie,ion,2)-e(ie,ion,1)
      do j1=1,jmax
         if(e21.gt.en(j1).and.e21.le.en(j1+1)) then
            j4572mg1=j1
            goto 91
         endif
      enddo
 91   continue
c
      e31=e(ie,ion,3)-e(ie,ion,1)
      do j1=1,jmax
         if(e31.gt.en(j1).and.e31.le.en(j1+1)) then
            j2852mg1=j1
            goto 92
         endif
      enddo
 92   continue
c
      return
      end


      subroutine atdatmgi
      implicit real*8(a-h,o-z)
c
c  New Mg I atom 9804  
c
c  10-level atom for Mg I consisting of :
c  3s 1S(1),3p 3Po(2),3p 1Po(3),4s 3S(4),4s 1S(5),
c  3d 1D(6),4p 3Po(7),3d 3D(8),4p 1Po(9),4d 3D(10)
c
c References are given in the data-file /FEDAT/mg1.dat 
c
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/cmg1/om(10,10),alf(10,10),
     &     wl(10,10),jlmg1(10,10),klvmg1(10,10)
      common/cebin/e1(ne),en(ne),jmax
      common/cionpot/e00(mie,mion)
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
c
      ie=iemg
      ion=1
      open(55,file='ATDAT/mg1.dat',status='old')
c      open(55,file='ATDAT/mg1.dat.test',status='old')
c
      do i=1,9
         read(55,701) dum
      enddo
 701  format(a)
c  number of levels
      read(55,*) nmg1
      nlev(ie,ion)=nmg1
      nlevm(ie,ion)=nmg1
c  ionization potential 
      read(55,*) e00mg1
      e00(ie,ion)=e00mg1
c
      read(55,701) dum
      read(55,701) dum
      read(55,701) dum
c  energy levels and statistical weights
      do i=1,nmg1
         read(55,*) il,efx,gfx
c  efx is in cm**-1, and ef is in eV
         e(ie,ion,il)=efx/8067.1
         g(ie,ion,il)=gfx
      enddo
c
      do i=1,nl
         do j=1,nl
            a(ie,ion,i,j)=0.d0
         enddo
      enddo
      do i=1,10
         do j=1,10
            om(i,j)=0.
            alf(i,j)=0.
            wl(i,j)=0.
            jlmg1(i,j)=0
            klvmg1(i,j)=0
         enddo
      enddo
c
      read(55,701) dum
      read(55,701) dum
      do i=1,50000
         read(55,702,end=9) n1,n2,wlx,ax,omx,alfx
         a(ie,ion,n1,n2)=ax
         om(n2,n1)=omx
         om(n1,n2)=omx
         alf(n2,n1)=alfx
         alf(n1,n2)=alfx
         wl(n1,n2)=wlx
         wl(n2,n1)=wlx
      enddo
 9    continue
 702  format(2i4,16x,f14.2,1e13.4,1e12.4,f6.2)
c
      close(55)
c
c  in which energy bin does the line fall?
      do j=2,10
         do i=1,j-1
            enij=e(ie,ion,j)-e(ie,ion,i)
            do j1=1,jmax
               if(enij.gt.en(j1).and.enij.le.en(j1+1)) then
                  jlmg1(j,i)=j1
                  jlmg1(i,j)=j1
                  goto 22
               endif               
            enddo
 22         continue
         enddo
      enddo
c
      return
      end


      subroutine atdatmgii
      implicit real*8(a-h,o-z)
c
c  atomic data for Mg II 
c
c  e    = energy of the levels in eV
c
      include 'INCLUDE/PARAM.IOR'
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/cebin/e1(ne),en(ne),jmax
      common/cjbins/j4572mg1,j2852mg1,j2800mg2,j2853na1,
     &     j1334c2,j13ne2,j314s2,j7ar2,
     &     jo21,jo31,jo32,jsi21,jsi31,jsi32,jsi021,jsi031,jsi032
c
      ie=8
      ion=2
c
c  3s 2S
      e(ie,ion,1)=0.
c  3p 2Po
      e(ie,ion,2)=4.43
c
      e21=e(ie,ion,2)-e(ie,ion,1)
      do j1=1,jmax
         if(e21.gt.en(j1).and.e21.le.en(j1+1)) then
            j2800mg2=j1
            goto 91
         endif
      enddo
 91   continue
c
      return
      end


      subroutine atdatsi1
      implicit real*8(a-h,o-z)
c
c  New Si I atom 9804  
c
c  21-level atom for Si I consisting of :
c  3p 3P0(1),3p 3P1(2),3p 3P2(3),3p 1D(4),3p 1S(5),
c  4s 3Po(6),4s 1Po(7),3p 3Do(8),4p 1P(9),3d 1Do(10),
c  4p 3D(11),4p 3P(12),3d 3Fo(13),4p 1D(14),3d 3Po(15),
c  3d 3Do(16),5p 3D(17),5p 3P(18),4d 3Fo(19),5p 3S(20),
c  5p 1D(21)
c
c References are given in the data-file /FEDAT/si1.dat 
c
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/csi1/om(21,21),alf(21,21),
     &     wl(21,21),jlsi1(21,21),klvsi1(21,21)
      common/cebin/e1(ne),en(ne),jmax
      common/cionpot/e00(mie,mion)
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
c
      ie=iesi
      ion=1
      open(55,file='ATDAT/si1.dat',status='old')
c      open(55,file='ATDAT/si1.dat.test',status='old')
c
      do i=1,19
         read(55,701) dum
      enddo
 701  format(a)
c  number of levels
      read(55,*) nsi1
      nlev(ie,ion)=nsi1
      nlevm(ie,ion)=nsi1
c  ionization potential 
      read(55,*) e00si1
      e00(ie,ion)=e00si1
c
      read(55,701) dum
      read(55,701) dum
      read(55,701) dum
c  energy levels and statistical weights
      do i=1,nsi1
         read(55,*) il,efx,gfx
c  efx is in cm**-1, and ef is in eV
         e(ie,ion,il)=efx/8067.1
         g(ie,ion,il)=gfx
      enddo
c
      do i=1,nl
         do j=1,nl
            a(ie,ion,i,j)=0.
         enddo
      enddo
      do i=1,21
         do j=1,21
            om(i,j)=0.
            alf(i,j)=0.
            wl(i,j)=0.
            jlsi1(i,j)=0
            klvsi1(i,j)=0
         enddo
      enddo
c
      read(55,701) dum
      read(55,701) dum
      do i=1,50000
         read(55,702,end=9) n1,n2,wlx,ax,omx,alfx
         a(ie,ion,n1,n2)=ax
         om(n2,n1)=omx
         om(n1,n2)=omx
         alf(n2,n1)=alfx
         alf(n1,n2)=alfx
         wl(n1,n2)=wlx
         wl(n2,n1)=wlx
      enddo
 9    continue
 702  format(2i4,17x,f13.2,2e12.4,f6.2)
c
      close(55)
c
c  in which energy bin does the line fall?
      do j=2,21
         do i=1,j-1
            enij=e(ie,ion,j)-e(ie,ion,i)
            do j1=1,jmax
               if(enij.gt.en(j1).and.enij.le.en(j1+1)) then
                  jlsi1(j,i)=j1
                  jlsi1(i,j)=j1
                  goto 22
               endif               
            enddo
 22         continue
         enddo
      enddo

cqcq run 99
c      nlev(ie,ion)=5
c      nlevm(ie,ion)=5
c      om(4,5)=0.d0
c      om(5,4)=0.d0
c      om(3,5)=0.d0
c      om(5,3)=0.d0
c      om(2,5)=0.d0
c      om(5,2)=0.d0
c      om(1,5)=0.d0
c      om(5,1)=0.d0      
c      a(ie,ion,5,4)=0.d0
c      a(ie,ion,5,4)=a(ie,ion,5,4)*1000.
c      a(ie,ion,5,3)=0.d0
c      a(ie,ion,5,2)=0.d0
c      a(ie,ion,5,1)=0.d0
c      a(ie,ion,4,5)=0.d0
c
      return
      end


      subroutine atdatca
      implicit real*8(a-h,o-z)
c
c  atomic data for ca ii ; ie = 12, ion = 2
c
c  nlev = number of levels included
c  e    = energy of the levels in eV
c  a    = transition probabilities
c  e00  = ionization potential in eV
c  g    = statistical weights of the levels
c
c  From Peter :
C  A-values of Ca II from Ali & Kim (Phys.Rev.A 38, 3992, '88) and 
C  Zeippen (A & A 229, 248, '90) for 4s-3d. (Smaller than Osterbrock's 
C  value, 1.3, from '51 by 30 %.) 4p-5s and 4s-4p from Wiese et al.
C  ('69) scaled by the multiplet calculations of Burgess et al.
C  (A & A 300, 627, '95). 3d-4p from Wiese et al. ('69). Energies
C  are from the ADS NIST Database ('95). Omegas for most transitions
C  from Burgess et al. ('95), except between the sublevels of 3d which 
C  are from Li & McCray (ApJ 405, 730, '93; based on the results of 
C  Shine ('73)).
c
      include 'INCLUDE/PARAM.IOR'
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/cionpot/e00(mie,mion)
      common/cebin/e1(ne),en(ne),jmax
      common/multijbin/jline(12,nl,nl),jlinef1(nlf1,nlf1),
     &     jlinef2(nlf2,nlf2),jlinef3(nlf3,nlf3),jlinef4(nlf4,nlf4),
     &     jlineh(nm,nm)
      common/cvoigt/avhi(nm),avhei(nl),avoi(nl),avcaii(nl),
     &     avfei(nlf1),avfeii(nlf2),avfeiii(nlf3),avfeiv(nlf4)
c
      ie=12
      ion=2
c
      nlev(ie,ion)=6
      nlevm(ie,ion)=6
      n=nlev(ie,ion)
c  ionization potential
      e00(ie,ion)=11.871
c  energy levels
c  4s 2S
      e(ie,ion,1)=0.
c  3d 2D
      e(ie,ion,2)=1.6926d0
      e(ie,ion,3)=1.7001d0
c  4p 2P
      e(ie,ion,4)=3.1237d0
      e(ie,ion,5)=3.1514d0
c  5s 2S
      e(ie,ion,6)=6.4687d0
c  statistical weights
      g(ie,ion,1)=2.
      g(ie,ion,2)=4.
      g(ie,ion,3)=6.
      g(ie,ion,4)=2.
      g(ie,ion,5)=4.
      g(ie,ion,6)=2.
c     ca iii 1s0
      g(ie,ion,n+1)=1.
      do i=1,n
        do j=1,n
          a(ie,ion,i,j)=0.
        enddo
      enddo
c  transition probabilities
c old values from Wises, Smith & Miles 1969:
c      a(ie,ion,2,1)=1.3
c      a(ie,ion,3,1)=1.3
c      a(ie,ion,4,1)=1.46e8
c      a(ie,ion,5,1)=1.50e8
c      a(ie,ion,3,2)=0.
c      a(ie,ion,4,2)=1.06e7
c      a(ie,ion,5,2)=1.11e6
c      a(ie,ion,4,3)=0.
c      a(ie,ion,5,3)=9.9e6
c      a(ie,ion,5,4)=0.
c
      a(ie,ion,2,1)=0.97d0
      a(ie,ion,3,1)=1.00d0
      a(ie,ion,4,1)=1.54d8
      a(ie,ion,5,1)=1.56d8
      a(ie,ion,3,2)=2.45d-6
      a(ie,ion,4,2)=9.45d6
      a(ie,ion,5,2)=9.90d5
      a(ie,ion,5,3)=8.83d6
      a(ie,ion,6,4)=8.8d7
      a(ie,ion,6,5)=1.7d8
c
      do i=1,nlev(ie,ion)
         avcaii(i)=0.d0
         do j=1,i-1
            avcaii(i)=avcaii(i)+a(ie,ion,i,j)
         enddo
      enddo      
c
      return
      end

      subroutine atdatca3
      implicit real*8(a-h,o-z)
c
c  atomic data for ca ii ; ie = 12, ion = 2
c
c  nlev = number of levels included
c  e    = energy of the levels in eV
c  a    = transition probabilities
c  e00  = ionization potential in eV
c  g    = statistical weights of the levels
c
      include 'INCLUDE/PARAM.IOR'
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/cionpot/e00(mie,mion)
      common/cebin/e1(ne),en(ne),jmax
      common/multijbin/jline(12,nl,nl),jlinef1(nlf1,nlf1),
     &     jlinef2(nlf2,nlf2),jlinef3(nlf3,nlf3),jlinef4(nlf4,nlf4),
     &     jlineh(nm,nm)
c
      ie=12
      ion=2
c
      nlev(ie,ion)=3
      nlevm(ie,ion)=3
      n=nlev(ie,ion)
c  ionization potential
      e00(ie,ion)=11.871
c  energy levels
      e(ie,ion,1)=0.
      e(ie,ion,2)=1.6968
      e(ie,ion,3)=3.1415
c  statistical weights
      g(ie,ion,1)=2.
      g(ie,ion,2)=10.
      g(ie,ion,3)=6.
c     ca iii 1s0
      g(ie,ion,n+1)=1.
      do i=1,n
        do j=1,n
          a(ie,ion,i,j)=0.
        enddo
      enddo
c  transition probabilities
      a(ie,ion,2,1)=1.3
      a(ie,ion,3,1)=2.35e8
      a(ie,ion,3,2)=7.97e6
c
      return
      end



      subroutine atdatfe1
      implicit real*8(a-h,o-z)
c
c  atomic data for Fe I ; ie = 13, ion = 1
c
c  nlev = number of levels included
c  e    = energy of the levels in eV
c  a    = transition probabilities
c  e00  = ionization potential in eV
c  g    = statistical weights of the levels
c
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      character*40 dum
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/cionpot/e00(mie,mion)
      common/cfecoll/omfe1(nlf1,nlf1),omfe2(nlf2,nlf2),
     &     omfe3(nlf3,nlf3),omfe4(nlf4,nlf4)
      common/cfedat/cf1(nlf1,nlf1),cf2(nlf2,nlf2),cf3(nlf3,nlf3),
     &     cf4(nlf4,nlf4),cif1(nlf1),cif2(nlf2),cif3(nlf3),cif4(nlf4),
     &     gf1(nlf1),gf2(nlf2),gf3(nlf3),gf4(nlf4),ef1(nlf1),ef2(nlf2),
     &     ef3(nlf3),ef4(nlf4),af1(nlf1,nlf1),af2(nlf2,nlf2),
     &     af3(nlf3,nlf3),af4(nlf4,nlf4),cpf2(3,3)
      common/cfeem/kupf1(16000),klof1(16000),kupf2(2300),klof2(2300),
     &     kupf3(1000),klof3(1000),kupf4(250),klof4(250),
     &     kfel(mion),kfer(mion)
      common/cebin/e1(ne),en(ne),jmax
      common/multijbin/jline(12,nl,nl),jlinef1(nlf1,nlf1),
     &     jlinef2(nlf2,nlf2),jlinef3(nlf3,nlf3),jlinef4(nlf4,nlf4),
     &     jlineh(nm,nm)
      common/cvoigt/avhi(nm),avhei(nl),avoi(nl),avcaii(nl),
     &     avfei(nlf1),avfeii(nlf2),avfeiii(nlf3),avfeiv(nlf4)
c
cqcq      open(55,file='/cf1/cecilia/DATA/ATDAT/FE/fe1.dat.jh')
c      open(55,file='ATDAT/FEDAT/fe1.dat.jh')
      open(55,file='ATDAT/FEDAT/fe1.dat.jh.kur',status='old')
cqcq run 70.6
c      open(55,file='ATDAT/FEDAT/fe1.dat.jh.kur.sn',status='old')
      ie=iefe
      ion=1
c
      read(55,701) dum
      read(55,701) dum
      read(55,701) dum
cqcq run 70.6
c      do i=1,25
c         read(55,701) dum
c      enddo
cqcq
 701  format(a)
c  number of levels
      read(55,*) n
      nlev(ie,ion)=n 
      nlevm(ie,ion)=n 
c  ionization potential 
      read(55,*) e00fe1
      e00(ie,ion)=e00fe1
      read(55,701) dum
      read(55,701) dum
      read(55,701) dum
c  energy levels and statistical weights
      do i=1,n
         read(55,*) il,efx,gfx
c  efx is in cm**-1, and ef is in eV
         ef1(il)=efx/8067.1
         gf1(il)=gfx
      enddo
c   fe ii 4s a6D9/2
      gf1(n+1)=10.0  
c
      read(55,701) dum
      read(55,701) dum
c
      do i=1,n
         do j=1,n
            af1(i,j)=0.
            omfe1(i,j)=0.
         enddo
      enddo
c
      do i=1,500000
         read(55,*,end=9) n1,n2,wl,ax,omx
         af1(n1,n2)=ax
         omfe1(n2,n1)=omx
      enddo
 9    continue
cqcq run 70.6
c      if (n1.ne.538) write(*,*) n1
c      if (n2.ne.537) write(*,*) n2
c
      close(55)
cqcq
c      nlev(ie,ion)=17
c      nlevm(ie,ion)=17
c      n=17
c      ef1(17)=ef1(38)
c      gf1(17)=gf1(38)
c      do i=1,16
c         af1(17,i)=af1(38,i)
c         omfe1(17,i)=omfe1(38,i)
c      enddo
cqcq
c number of transition which has af > 0.
      k=0
      do j=2,n
         do i=1,j-1
            if (af1(j,i).gt.0.) then
               k=k+1
               kupf1(k)=j
               klof1(k)=i
            endif
         enddo
      enddo
      kfel(ion)=k
c
      if (kfel(ion).gt.16000) then
         write(50,*) 'atdatfe1 : too many Fe I lines !'
      endif
c
      do j=2,n
         do i=1,j-1
            enij=ef1(j)-ef1(i)
            jlinef1(i,j)=0
            jlinef1(j,i)=0
            do j1=1,jmax
               if(enij.gt.en(j1).and.enij.le.en(j1+1)) then
                  jlinef1(j,i)=j1
                  goto 99
               endif
            enddo
 99         continue
         enddo
      enddo
c
cqcq run 99
cqcq (run 19 ; small Fe I atom)
c      nlev(ie,ion)=20
c      nlevm(ie,ion)=20
c      nlev(ie,ion)=16
c      nlevm(ie,ion)=16
cqcq run 99
c      nlev(ie,ion)=40
c      nlevm(ie,ion)=40
c
c      do i=21,22
ccc         do j=1,i-1
c         do j=1,i-1
c            af1(i,j)=0.d0
c            af1(j,i)=0.d0
c         enddo
c      enddo
c
c      do j=1,24
c         af1(25,j)=0.d0
c         af1(j,25)=0.d0
c         af1(24,j)=0.d0
c         af1(j,24)=0.d0
c      enddo
c
cc      do j=2,22
c      do j=3,22
cc      do j=4,22
cc         af1(25,j)=0.d0
cc         af1(j,25)=0.d0
cc         af1(24,j)=0.d0
cc         af1(j,24)=0.d0
c         af1(23,j)=0.d0c
c         af1(j,23)=0.d0
c      enddo
c      af1(23,1)=0.d0
c      af1(1,23)=0.d0
cqcq
      do i=1,nlev(ie,ion)
         avfei(i)=0.d0
         do j=1,i-1
            avfei(i)=avfei(i)+af1(i,j)
         enddo
      enddo      
c
      return
      end



      subroutine atdatfe2
      implicit real*8(a-h,o-z)
c
c  atomic data for Fe II ; ie = 13, ion = 2
c
c  nlev = number of levels included
c  e    = energy of the levels in eV
c  a    = transition probabilities
c  e00  = ionization potential in eV
c  g    = statistical weights of the levels
c
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      character*40 dum
      character*9 uplab,lowlab
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/cionpot/e00(mie,mion)
      common/cfecoll/omfe1(nlf1,nlf1),omfe2(nlf2,nlf2),
     &     omfe3(nlf3,nlf3),omfe4(nlf4,nlf4)
      common/cfeem/kupf1(16000),klof1(16000),kupf2(2300),klof2(2300),
     &     kupf3(1000),klof3(1000),kupf4(250),klof4(250),
     &     kfel(mion),kfer(mion)
      common/cfedat/cf1(nlf1,nlf1),cf2(nlf2,nlf2),cf3(nlf3,nlf3),
     &     cf4(nlf4,nlf4),cif1(nlf1),cif2(nlf2),cif3(nlf3),cif4(nlf4),
     &     gf1(nlf1),gf2(nlf2),gf3(nlf3),gf4(nlf4),ef1(nlf1),ef2(nlf2),
     &     ef3(nlf3),ef4(nlf4),af1(nlf1,nlf1),af2(nlf2,nlf2),
     &     af3(nlf3,nlf3),af4(nlf4,nlf4),cpf2(3,3)
      common/cebin/e1(ne),en(ne),jmax
      common/multijbin/jline(12,nl,nl),jlinef1(nlf1,nlf1),
     &     jlinef2(nlf2,nlf2),jlinef3(nlf3,nlf3),jlinef4(nlf4,nlf4),
     &     jlineh(nm,nm)
      common/cvoigt/avhi(nm),avhei(nl),avoi(nl),avcaii(nl),
     &     avfei(nlf1),avfeii(nlf2),avfeiii(nlf3),avfeiv(nlf4)
c
cqcq      open(55,file='/cf1/cecilia/DATA/ATDAT/FE/fe2.dat.jh')
c      open(55,file='ATDAT/FEDAT/fe2.dat.jh')
cqcq run 97; run 80.6; run 80.7; run 96; run 77.1; run 77.2
cqcq run 77.3; run 77.4; run 77.5; run 77.6; run 77.7
cqcq run 70.8; run 70.9; run 99; run 80.8 > run = 100
      open(55,file='ATDAT/FEDAT/feIInew.dat',status='old')
cqcq run 98
c      open(55,file='ATDAT/FEDAT/feIInew.971126.dat',status='old')
      ie=iefe
      ion=2
c
cqcq run 98
      read(55,701) dum
      read(55,701) dum
      read(55,701) dum
      read(55,701) dum
      read(55,701) dum
c
      read(55,701) dum
 701  format(a)
c  number of levels
      read(55,*) n
c
c  the A-values for the higher levels (91-116) are all zero
c
c      nlev(ie,ion)=90 
c      nlevm(ie,ion)=90 
      nlev(ie,ion)=n
      nlevm(ie,ion)=n
c  ionization potential 
      read(55,*) e00fe2
      e00(ie,ion)=e00fe2
      read(55,701) dum
      read(55,701) dum
      read(55,701) dum
c  energy levels and statistical weights
      do i=1,n
         read(55,*) il,efx,gfx
c  efx is in cm**-1, and ef is in eV
         ef2(il)=efx/8067.1
         gf2(il)=gfx
      enddo
c fe iii 1s0(?)
      gf2(n+1)=1.  
c
      read(55,701) dum
      read(55,701) dum
c
      do i=1,n
         do j=1,n
            af2(i,j)=0.
            omfe2(i,j)=0.
         enddo
      enddo
c
      do i=1,50000
         read(55,702,end=9) n1,n2,uplab,lowlab,wl,ax,omx
c to read fe2.dat.jh:   
c         read(55,*,end=9) n1,n2,wl,ax,omx
         af2(n1,n2)=ax
         omfe2(n2,n1)=omx
      enddo
 9    continue
 702  format(2i4,2a9,f14.2,1pe12.4,e12.4)
cqcq run 80.8 (omfe2_80.8(1,2)=0.5*omfe2(1,2); 26 my-line)
c      omfe2(1,2)=0.5*omfe2(1,2)
c
      close(55)
cqcq
c      nlev(ie,ion)=17
c      nlevm(ie,ion)=17
c      n=17
c      ef2(17)=ef2(36)
c      gf2(17)=gf2(36)
c      do i=1,16
c         af2(17,i)=af2(36,i)
c         omfe2(17,i)=omfe2(36,i)
c      enddo
cqcq
c  number of transition which has af > 0.
c  kup = upper level
c  klo = lower level
      k=0
      do j=2,n
         do i=1,j-1
            if (af2(j,i).gt.0.) then
               k=k+1
               if (k.ge.1000) then
                  xxx=1.
               endif
               kupf2(k)=j
               klof2(k)=i
            endif
         enddo
      enddo
      kfel(ion)=k
c
      if (kfel(ion).gt.2300) then
         write(50,*) 'atdatfe2 : too many Fe II lines !',kfel(ion)
      endif
c
      do j=2,n
         do i=1,j-1
            jlinef2(i,j)=0
            jlinef2(j,i)=0
            enij=ef2(j)-ef2(i)
            do j1=1,jmax
               if(enij.gt.en(j1).and.enij.le.en(j1+1)) then
                  jlinef2(j,i)=j1
                  goto 99
               endif
            enddo
 99         continue
         enddo
      enddo
c
cqcq 
cqcq (run 19 ; small Fe II atom)
c      nlev(ie,ion)=23
c      nlevm(ie,ion)=23
c      nlev(ie,ion)=92
c      nlevm(ie,ion)=92
cqcq run 99
c      nlev(ie,ion)=16
c      nlevm(ie,ion)=16
c      nlev(ie,ion)=40
c      nlevm(ie,ion)=40
cqcq
c
      do i=1,nlev(ie,ion)
         avfeii(i)=0.d0
         do j=1,i-1
            avfeii(i)=avfeii(i)+af2(i,j)
         enddo
      enddo      
      return
      end


      subroutine fe2om(te)
      implicit real*8(a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      common/cfecoll/omfe1(nlf1,nlf1),omfe2(nlf2,nlf2),
     &     omfe3(nlf3,nlf3),omfe4(nlf4,nlf4)
      dimension tev(21),om12(21)
      data  tev/200.0,1000.0,3000.0,5000.0,7000.0,10000.,12000.,
     &15000.,17000.,20000.,25000.,30000.,35000.,40000.,45000.,
     &50000.,60000.,70000.,80000.,90000.,100000./
      data om12/7.000,6.9700,6.2690,5.9740,5.7280,5.5210,5.4710,
     &5.4610,5.4710,5.4830,5.4680,5.4020,5.3010,5.1780,5.0440,
     &4.9060,4.6360,4.3860,4.1590,3.9570,3.7770/
c
c New 990608 :
c Fe II 26 my (1-2)
c Omfe2(2,1) from 'Zhang & Pradhan, 1995, A&A, 293, 953' as
c a function of temperature. Data taken (tev,om12) from upsln.fe2, 
c available on the net (se /milla_1/cecilia/ATDAT/FEDAT/upsln.fe2.ps).
c The value at tev=200 K, om12=7.00 is from Pradhan, mars 1999, 
c private communication. 
c
      om=0.d0
      if (te.le.tev(1)) then
         om=om12(1)
      elseif (te.ge.tev(21)) then
         om=om12(21)
      else
         do i=1,20
            if (te.gt.tev(i).and.te.le.tev(i+1)) then
               om=om12(i)+
     &            (te-tev(i))*(om12(i+1)-om12(i))/(tev(i+1)-tev(i))
               goto 9
            endif
         enddo
      endif
 9    continue
      omfe2(1,2)=om
c
      if (om.le.0.) write(50,*) 'om=0 i fe2om !',om
c
      return
      end


      subroutine atdatfe3
      implicit real*8(a-h,o-z)
c
c  atomic data for Fe III ; ie = 13, ion = 3
c
c  nlev = number of levels included
c  e    = energy of the levels in eV
c  a    = transition probabilities
c  e00  = ionization potential in eV
c  g    = statistical weights of the levels
c
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      character*40 dum
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/cionpot/e00(mie,mion)
      common/cfecoll/omfe1(nlf1,nlf1),omfe2(nlf2,nlf2),
     &     omfe3(nlf3,nlf3),omfe4(nlf4,nlf4)
      common/cfeem/kupf1(16000),klof1(16000),kupf2(2300),klof2(2300),
     &     kupf3(1000),klof3(1000),kupf4(250),klof4(250),
     &     kfel(mion),kfer(mion)
      common/cfedat/cf1(nlf1,nlf1),cf2(nlf2,nlf2),cf3(nlf3,nlf3),
     &     cf4(nlf4,nlf4),cif1(nlf1),cif2(nlf2),cif3(nlf3),cif4(nlf4),
     &     gf1(nlf1),gf2(nlf2),gf3(nlf3),gf4(nlf4),ef1(nlf1),ef2(nlf2),
     &     ef3(nlf3),ef4(nlf4),af1(nlf1,nlf1),af2(nlf2,nlf2),
     &     af3(nlf3,nlf3),af4(nlf4,nlf4),cpf2(3,3)
      common/cebin/e1(ne),en(ne),jmax
      common/multijbin/jline(12,nl,nl),jlinef1(nlf1,nlf1),
     &     jlinef2(nlf2,nlf2),jlinef3(nlf3,nlf3),jlinef4(nlf4,nlf4),
     &     jlineh(nm,nm)
      common/cvoigt/avhi(nm),avhei(nl),avoi(nl),avcaii(nl),
     &     avfei(nlf1),avfeii(nlf2),avfeiii(nlf3),avfeiv(nlf4)
c
cqcq      open(55,file='/cf1/cecilia/DATA/ATDAT/FE/fe3.dat.jh')
      open(55,file='ATDAT/FEDAT/fe3.dat.jh',status='old')
      ie=iefe
      ion=3
c
      read(55,701) dum
      read(55,701) dum
 701  format(a)
c  number of levels
      read(55,*) n
c
c  the levels 111 and 112 are only 'mean values' over a large number 
c  of levels and are not included here
c
      nlev(ie,ion)=110 
      nlevm(ie,ion)=110
c  ionization potential 
      read(55,*) e00fe3
      e00(ie,ion)=e00fe3
      read(55,701) dum
      read(55,701) dum
      read(55,701) dum
c  energy levels and statistical weights
      do i=1,n
         read(55,*) il,efx,gfx
c  efx is in cm**-1, and ef is in eV
         ef3(il)=efx/8067.1
         gf3(il)=gfx
      enddo
c
      read(55,701) dum
      read(55,701) dum
c
      do i=1,n
         do j=1,n
            af3(i,j)=0.
            omfe3(i,j)=0.
         enddo
      enddo
c
      do i=1,50000
         read(55,*,end=9) n1,n2,wl,ax,omx
         af3(n1,n2)=ax
         omfe3(n2,n1)=omx
      enddo
 9    continue
c
      close(55)
c  number of transition which has af > 0.
c  kup = upper level
c  klo = lower level
      k=0
      do j=2,n
         do i=1,j-1
            if (af3(j,i).gt.0.) then
               k=k+1
               kupf3(k)=j
               klof3(k)=i
            endif
         enddo
      enddo
      kfel(ion)=k
c
      if (kfel(ion).gt.1000) then
         write(50,*) 'atdatfe3 : too many Fe III lines !'
      endif
c
      do j=2,n
         do i=1,j-1
            jlinef3(i,j)=0
            jlinef3(j,i)=0
            enij=ef3(j)-ef3(i)
            do j1=1,jmax
               if(enij.gt.en(j1).and.enij.le.en(j1+1)) then
                  jlinef3(j,i)=j1
                  goto 99
               endif
            enddo
 99         continue
         enddo
      enddo
c
cqcq run 99
cqcq (run 19 ; small Fe III atom)
c      nlev(ie,ion)=26
c      nlevm(ie,ion)=26
cqcq run 99
c      nlev(ie,ion)=5
c      nlevm(ie,ion)=5
c      nlev(ie,ion)=40
c      nlevm(ie,ion)=40
cqcq
c
      do i=1,nlev(ie,ion)
         avfeiii(i)=0.d0
         do j=1,i-1
            avfeiii(i)=avfeiii(i)+af3(i,j)
         enddo
      enddo      
c
      return
      end


      subroutine atdatfe4
      implicit real*8(a-h,o-z)
c
c  atomic data for Fe IV ; ie = 13, ion = 4
c
c  nlev = number of levels included
c  e    = energy of the levels in eV
c  a    = transition probabilities
c  e00  = ionization potential in eV
c  g    = statistical weights of the levels
c
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      character*40 dum
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/cionpot/e00(mie,mion)
      common/cfecoll/omfe1(nlf1,nlf1),omfe2(nlf2,nlf2),
     &     omfe3(nlf3,nlf3),omfe4(nlf4,nlf4)
      common/cfeem/kupf1(16000),klof1(16000),kupf2(2300),klof2(2300),
     &     kupf3(1000),klof3(1000),kupf4(250),klof4(250),
     &     kfel(mion),kfer(mion)
      common/cfedat/cf1(nlf1,nlf1),cf2(nlf2,nlf2),cf3(nlf3,nlf3),
     &     cf4(nlf4,nlf4),cif1(nlf1),cif2(nlf2),cif3(nlf3),cif4(nlf4),
     &     gf1(nlf1),gf2(nlf2),gf3(nlf3),gf4(nlf4),ef1(nlf1),ef2(nlf2),
     &     ef3(nlf3),ef4(nlf4),af1(nlf1,nlf1),af2(nlf2,nlf2),
     &     af3(nlf3,nlf3),af4(nlf4,nlf4),cpf2(3,3)
      common/cebin/e1(ne),en(ne),jmax
      common/multijbin/jline(12,nl,nl),jlinef1(nlf1,nlf1),
     &     jlinef2(nlf2,nlf2),jlinef3(nlf3,nlf3),jlinef4(nlf4,nlf4),
     &     jlineh(nm,nm)
      common/cvoigt/avhi(nm),avhei(nl),avoi(nl),avcaii(nl),
     &     avfei(nlf1),avfeii(nlf2),avfeiii(nlf3),avfeiv(nlf4)
c
cqcq      open(55,file='/cf1/cecilia/DATA/ATDAT/FE/fe4.dat.jh')
      open(55,file='ATDAT/FEDAT/fe4.dat.jh',status='old')
      ie=iefe
      ion=4
c
      read(55,701) dum
      read(55,701) dum
 701  format(a)
c  number of levels
      read(55,*) n
c
c  the levels 44 and 45 are only 'mean values' over a large number 
c  of levels and are not included here
c
      nlev(ie,ion)=43
      nlevm(ie,ion)=43
c  ionization potential 
      read(55,*) e00fe4
      e00(ie,ion)=e00fe4
      read(55,701) dum
      read(55,701) dum
      read(55,701) dum
c  energy levels and statistical weights
      do i=1,n
         read(55,*) il,efx,gfx
c  efx is in cm**-1, and ef is in eV
         ef4(il)=efx/8067.1
         gf4(il)=gfx
      enddo
c
      read(55,701) dum
      read(55,701) dum
c
      do i=1,n
         do j=1,n
            af4(i,j)=0.
            omfe4(i,j)=0.
         enddo
      enddo
c
      do i=1,50000
         read(55,*,end=9) n1,n2,wl,ax,omx
         af4(n1,n2)=ax
         omfe4(n2,n1)=omx
      enddo
 9    continue
c
      close(55)
c  number of transition which has af > 0.
c  kup = upper level
c  klo = lower level
      k=0
      do j=2,n
         do i=1,j-1
            if (af4(j,i).gt.0.) then
               k=k+1
               kupf4(k)=j
               klof4(k)=i
            endif
         enddo
      enddo
      kfel(ion)=k
c
      if (kfel(ion).gt.250) then
         write(50,*) 'atdatfe4 : too many Fe IV lines !'
      endif
c
      do j=2,n
         do i=1,j-1
            jlinef4(i,j)=0
            jlinef4(j,i)=0
            enij=ef4(j)-ef4(i)
            do j1=1,jmax
               if(enij.gt.en(j1).and.enij.le.en(j1+1)) then
                  jlinef4(j,i)=j1
                  goto 99
               endif
            enddo
 99         continue
         enddo
      enddo
c
cqcq run 99
cqcq (run 19 ; small Fe IV atom)
c      nlev(ie,ion)=24
c      nlevm(ie,ion)=24
cqcq run 99
c      nlev(ie,ion)=16
c      nlevm(ie,ion)=16
cqcq
c
      do i=1,nlev(ie,ion)
         avfeiv(i)=0.d0
         do j=1,i-1
            avfeiv(i)=avfeiv(i)+af4(i,j)
         enddo
      enddo      
c
      return
      end


      subroutine atdinit
      implicit real*8 (a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/cmlevpop/xhi(nmax),xhei(nl),xoi(nl),xmg1(nl),xsi1(nl),
     &     xcaii(nl),xfei(nlf1),xfeii(nlf2),xfeiii(nlf3),xfeiv(nlf4)
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/ccollh/cnl(nmax,nmax),clep(nm,0:nm),clem(nm,0:nm),
     &     clpp(nm,0:nm),clpm(nm,0:nm)
      common/cfecoll/omfe1(nlf1,nlf1),omfe2(nlf2,nlf2),
     &     omfe3(nlf3,nlf3),omfe4(nlf4,nlf4)
      common/cfeem/kupf1(16000),klof1(16000),kupf2(2300),klof2(2300),
     &     kupf3(1000),klof3(1000),kupf4(250),klof4(250),
     &     kfel(mion),kfer(mion)
      common/cfedat/cf1(nlf1,nlf1),cf2(nlf2,nlf2),cf3(nlf3,nlf3),
     &     cf4(nlf4,nlf4),cif1(nlf1),cif2(nlf2),cif3(nlf3),cif4(nlf4),
     &     gf1(nlf1),gf2(nlf2),gf3(nlf3),gf4(nlf4),ef1(nlf1),ef2(nlf2),
     &     ef3(nlf3),ef4(nlf4),af1(nlf1,nlf1),af2(nlf2,nlf2),
     &     af3(nlf3,nlf3),af4(nlf4,nlf4),cpf2(3,3)
      common/cionpot/e00(mie,mion)
      COMMON/Heicex/OM(NL,NL)
      common/multijbin/jline(12,nl,nl),jlinef1(nlf1,nlf1),
     &     jlinef2(nlf2,nlf2),jlinef3(nlf3,nlf3),jlinef4(nlf4,nlf4),
     &     jlineh(nm,nm)
      common/cabun/amean,ab(mie),xa(mie,mion),xaold(mid,mie,mion),
     &     nion(mie)
      common/cvoigt/avhi(nm),avhei(nl),avoi(nl),avcaii(nl),
     &     avfei(nlf1),avfeii(nlf2),avfeiii(nlf3),avfeiv(nlf4)
      common/cklv/klvh1(nm,nm),klvhe1(nl,nl),klvo1(nl,nl),
     &     klvca2(nl,nl),klvfe1(nlf1,nlf1),klvfe2(nlf2,nlf2),
     &     klvfe3(nlf3,nlf3),klvfe4(nlf4,nlf4)
c
c  xhi(i)  = population of level i , (nlev+1 = continuum)
c  xhei(i)  = population of level i , (nlev+1 = continuum)
c  xoi(i)  = population of level i , (nlev+1 = continuum)
c  xcaii(i)  = population of level i , (nlev+1 = continuum)
c  xfei(i)  = population of level i , (nlev+1 = continuum)
c  xfeii(i)  = population of level i , (nlev+1 = continuum)
c  sum(xhi(i))=1, i=1,nlev+1
c  sum(xhei(i))=1, i=1,nlev+1
c  sum(xoi(i))=1, i=1,nlev+1
c  sum(xmg1(i))=xa(iemg,1), i=1,nlev
c  sum(xsi1(i))=xa(iesi,1), i=1,nlev
c  sum(xcaii(i))=xa(ieca,2), i=1,nlev+1
c  sum(xfei(i))+sum(xfeii(i))+xa(iefe,3)=1, i=1,nlev
c  sum(xfei(i))=xa(iefe,1), i=1,nlev
c  sum(xfeii(i))=xa(iefe,2), i=1,nlev
c
c  h i
      do i=1,nmax
         do j=1,nmax
            cnl(i,j)=0.d0
         enddo
      enddo
c
      do n=1,nm
         avhi(n)=0.d0
         do l=0,nm
            clep(n,l)=0.d0
            clem(n,l)=0.d0
            clpp(n,l)=0.d0
            clpm(n,l)=0.d0
         enddo
      enddo
c
      do ie=1,mie
         nion(ie)=0
         do ion=1,mion
            nlev(ie,ion)=0
            nlevm(ie,ion)=0
            e00(ie,ion)=0.
            if (ie.ne.iefe) then
               do il1=1,nl
                  ci(ie,ion,il1)=0.
                  g(ie,ion,il1)=0.
                  e(ie,ion,il1)=0.
                  do il2=1,nl
                     c(ie,ion,il1,il2)=0.
                     a(ie,ion,il1,il2)=0.
                  enddo
               enddo
            elseif (ie.eq.iefe) then
c Fe I
               do il1=1,nlf1
                  cif1(il1)=0.
                  gf1(il1)=0.
                  ef1(il1)=0.
                  avfei(il1)=0.d0
                  do il2=1,nlf1
                     cf1(il1,il2)=0.
                     af1(il1,il2)=0.
                  enddo
               enddo
c Fe II
               do il1=1,nlf2
                  cif2(il1)=0.
                  gf2(il1)=0.
                  ef2(il1)=0.
                  avfeii(il1)=0.d0
                  do il2=1,nlf2
                     cf2(il1,il2)=0.
                     af2(il1,il2)=0.
                  enddo
               enddo
c Fe III
               do il1=1,nlf3
                  cif3(il1)=0.
                  gf3(il1)=0.
                  ef3(il1)=0.
                  avfeiii(il1)=0.d0
                  do il2=1,nlf3
                     cf3(il1,il2)=0.
                     af3(il1,il2)=0.
                  enddo
               enddo
c Fe IV
               do il1=1,nlf4
                  cif4(il1)=0.
                  gf4(il1)=0.
                  ef4(il1)=0.
                  avfeiv(il1)=0.d0
                  do il2=1,nlf4
                     cf4(il1,il2)=0.
                     af4(il1,il2)=0.
                  enddo
               enddo
           endif
         enddo
      enddo
c  Fe
      do il1=1,nlf1
         xfei(il1)=0.
         do il2=1,nlf1
            omfe1(il1,il2)=0.
         enddo
      enddo
      do il1=1,nlf2
         xfeii(il1)=0.
         do il2=1,nlf2
            omfe2(il1,il2)=0.
         enddo
      enddo
      do il1=1,nlf3
         xfeiii(il1)=0.
         do il2=1,nlf3
            omfe3(il1,il2)=0.
         enddo
      enddo
      do il1=1,nlf4
         xfeiv(il1)=0.
         do il2=1,nlf4
            omfe4(il1,il2)=0.
         enddo
      enddo
      do il=1,nmax
         xhi(il)=0.
      enddo
      do il=1,nl
         xhei(il)=0.
         xoi(il)=0.
         xmg1(il)=0.
         xsi1(il)=0.
         xcaii(il)=0.
         avhei(il)=0.d0
         avoi(il)=0.d0
         avcaii(il)=0.d0
         do il2=1,nl
            om(il,il2)=0.
         enddo
      enddo
c
      do ion=1,mion
         kfel(ion)=0
         kfer(ion)=0
      enddo
      do k=1,16000
         kupf1(k)=0
         klof1(k)=0
      enddo
      do k=1,2300
         kupf2(k)=0
         klof2(k)=0
      enddo
      do k=1,1000
         kupf3(k)=0
         klof3(k)=0
      enddo
      do k=1,250
         kupf4(k)=0
         klof4(k)=0
      enddo
c
      do i=1,nm
         do j=1,nm
            klvh1(i,j)=0
         enddo
      enddo
      do i=1,nl
         do j=1,nl
            klvhe1(i,j)=0
            klvo1(i,j)=0
            klvca2(i,j)=0
         enddo
      enddo
      do i=1,nlf1
         do j=1,nlf1
            klvfe1(i,j)=0
         enddo
      enddo
      do i=1,nlf2
         do j=1,nlf2
            klvfe2(i,j)=0
         enddo
      enddo
      do i=1,nlf3
         do j=1,nlf3
            klvfe3(i,j)=0
         enddo
      enddo
      do i=1,nlf4
         do j=1,nlf4
            klvfe4(i,j)=0
         enddo
      enddo
c
      return
      end



      subroutine atdatni1
      implicit real*8 (a-h,o-z)
c
c New Ni I atom 971204
c
c  14-level atom for Ni I consisting of :
c  a 3F (3 levels;1,4,6), a 3D (3 levels; 2,3,5),
c  a 1D (1 level; 7), b 1D (8), a 1S (9), 3P (10,11,12),
c  1G (13), b 1S (14)
c
c References are given in the data-file /FEDAT/ni1.dat 
c
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      character*40 dum
      common/cni1/e(14),g(14),a(14,14),om(14,14),wl(14,14),nni1,
     &     jlni1(14,14),klvni1(14,14)
      common/cebin/e1(ne),en(ne),jmax
      common/cionpot/e00(mie,mion)
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
c
      open(55,file='ATDAT/ni1.dat',status='old')
      ie=ieni
      ion=1
c
      do i=1,11
         read(55,701) dum
      enddo
 701  format(a)
c  number of levels
      read(55,*) nni1
      nlev(ie,ion)=nni1
      nlevm(ie,ion)=nni1
c  ionization potential 
      read(55,*) e00ni1
      e00(ie,ion)=e00ni1
c
      read(55,701) dum
      read(55,701) dum
      read(55,701) dum
c  energy levels and statistical weights
      do i=1,nni1
         read(55,*) il,efx,gfx
c  efx is in cm**-1, and ef is in eV
         e(il)=efx/8067.1
         g(il)=gfx
      enddo
c
      do i=1,14
         do j=1,14
            a(i,j)=0.
            om(i,j)=0.
            wl(i,j)=0.
            jlni1(i,j)=0
            klvni1(i,j)=0
         enddo
      enddo
c
      read(55,701) dum
      read(55,701) dum
      do i=1,50000
         read(55,702,end=9) n1,n2,wlx,ax,omx
         a(n1,n2)=ax
         om(n2,n1)=omx
         wl(n1,n2)=wlx
         wl(n2,n1)=wlx
      enddo
 9    continue
 702  format(2i4,14x,f14.2,e12.4,f8.4)
c
      close(55)
c
c  in which energy bin does the line fall?
      do j=2,14
         do i=1,j-1
            enij=e(j)-e(i)
            do j1=1,jmax
               if(enij.gt.en(j1).and.enij.le.en(j1+1)) then
                  jlni1(j,i)=j1
                  jlni1(i,j)=j1
                  goto 22
               endif               
            enddo
 22         continue
         enddo
      enddo
c
      return
      end


      subroutine atdatni1_old
c /OLDSR/atdatni1.971204.sr
      implicit real*8 (a-h,o-z)
c
c  7-level atom for Ni I consisting of the
c  three lowest multiplets : a 3F (3 levels;1,4,6), a 3D (3 levels; 2,3,5),
c  and a 1D (1 level; 7).
c
c  References :
c    (A-values, and energies for the a 3F and a 3D multiplets:)
c    Nussbaumer & Storey, 1988, A&A 200, L25
c    (A-values between the a 1D and a 3D multiplets; A(7,2), A(7,3),
c     and A(7,5) from :)
c    Garstang, R.H. 1964, Journal of Research of the National Bureau
c                         of Standards, 68A, 61.
c    (the 7-level atom is discussed by :)
c    Wooden et al. 1993, ApJS, 88, 477
c    (Energy of a 1D :)
c    Kurucs search list via netscape
c
      include 'INCLUDE/PARAM.IOR'
c      common/cni1/e(7),g(7),a(7,7),om(7,7),wl(7,7),nni1,
c     &     jlni1(7,7)
c new 971204
      common/cni1/e(14),g(14),a(14,14),om(14,14),wl(14,14),nni1,
     &     jlni1(14,14),klvni1(14,14)
      common/cebin/e1(ne),en(ne),jmax
c
c  number of levels
      nni1=7
c
      do i=1,7
         e(i)=0.
         g(i)=0.
         do j=1,7
            a(i,j)=0.
            om(i,j)=0.
            wl(i,j)=0.
            jlni1(i,j)=0
            klvni1(i,j)=0
         enddo
      enddo
c
c     e in cm**-1
c
      e(1)=0.00
      e(2)=204.786
      e(3)=879.813
      e(4)=1332.153
      e(5)=1713.080
      e(6)=2216.519
      e(7)=3409.925
c
      do i=1,7
         do j=1,7
            if (i.ne.j) then
               de=abs(e(i)-e(j))
               wl(i,j)=1.e8/de
            endif                  
         enddo
      enddo
c  e in eV
      do i=1,7
         e(i)=e(i)/8067.1
      enddo
c
c  in which energy bin does the line fall?
      do j=2,7
         do i=1,j-1
            enij=e(j)-e(i)
            do j1=1,jmax
               if(enij.gt.en(j1).and.enij.le.en(j1+1)) then
                  jlni1(j,i)=j1
                  jlni1(i,j)=j1
                  goto 22
               endif               
            enddo
 22         continue
         enddo
      enddo
c
c  statistical weights
c
      g(1)=9.
      g(2)=7.
      g(3)=5.
      g(4)=7.
      g(5)=3.
      g(6)=5.
      g(7)=5.
c
c  transition probabilities
c
      a(4,1)=6.2e-2
      a(6,4)=2.5e-2
      a(3,2)=7.0e-3
      a(5,3)=2.1e-2
      a(7,2)=7.8e-2
      a(7,3)=6.2e-3
      a(7,5)=1.1e-2
c
c  collision strengths
c
c  The collision strengths for Ni I are unknown.
c  We assume that the collision strength can be written
c  on the form :
c  om(i,j) = const*g(i)*g(j)
c  This form has been used by Axelrod,1980 in his thesis (his const = 4.e-3),
c  and by Bahcall, J.N. and Wolf, R.A., 1968, ApJ, 152, 701. (refered to by
c  Hollenbach and McKee, 1989)
c  The constant has been determined from the known numbers for Fe II.
c  When determining the constant only the 16 lowest levels in Fe II were 
c  used, that is no allowed transitions.
c
c  The collision strengths are highly uncertain !!!
c
c      const=0.046
c      do i=1,7
c         do j=i+1,7
c            om(i,j)=0.046*g(i)*g(j)
c         enddo
c      enddo
c
      do i=1,7
         do j=i+1,7
            om(i,j)=0.1
         enddo
      enddo
c
      return
      end



      subroutine atdatni2
      implicit real*8 (a-h,o-z)
c
c New Ni II atom 971204
c
c  18-level atom for Ni II consisting of :
c  a 2D (levels;1,2), 4F (3,4,5,6), 2F (7,8),
c  b 2D (9,10), 4P (11,12,13), 2P (14,15),
c  2G (16,17), 2S (18)
c
c References are given in the data-file /FEDAT/ni2.dat 
c
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      character*40 dum
      common/cni2/e(18),g(18),a(18,18),om(18,18),wl(18,18),nni2,
     &     jlni2(18,18),klvni2(18,18)
      common/cebin/e1(ne),en(ne),jmax
      common/cionpot/e00(mie,mion)
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
c
      open(55,file='ATDAT/ni2.dat',status='old')
      ie=ieni
      ion=2
c
      do i=1,18
         read(55,701) dum
      enddo
 701  format(a)
c  number of levels
      read(55,*) nni2
      nlev(ie,ion)=nni2
      nlevm(ie,ion)=nni2
c  ionization potential 
      read(55,*) e00ni2
      e00(ie,ion)=e00ni2
c
      read(55,701) dum
      read(55,701) dum
      read(55,701) dum
c  energy levels and statistical weights
      do i=1,nni2
         read(55,*) il,efx,gfx
c  efx is in cm**-1, and ef is in eV
         e(il)=efx/8067.1
         g(il)=gfx
      enddo
c
      do i=1,18
         do j=1,18
            a(i,j)=0.
            om(i,j)=0.
            wl(i,j)=0.
            jlni2(i,j)=0
            klvni2(i,j)=0
         enddo
      enddo
c
      read(55,701) dum
      read(55,701) dum
      do i=1,50000
         read(55,702,end=9) n1,n2,wlx,ax,omx
         a(n1,n2)=ax
         om(n2,n1)=omx
         wl(n1,n2)=wlx
         wl(n2,n1)=wlx
      enddo
 9    continue
 702  format(2i4,20x,f14.2,e12.4,f10.6)
c
      close(55)
c
c  in which energy bin does the line fall?
      do j=2,18
         do i=1,j-1
            enij=e(j)-e(i)
            do j1=1,jmax
               if(enij.gt.en(j1).and.enij.le.en(j1+1)) then
                  jlni2(j,i)=j1
                  jlni2(i,j)=j1
                  goto 22
               endif               
            enddo
 22         continue
         enddo
      enddo
c
      return
      end


      subroutine atdatni2_old
c /OLDSR/atdatni2.971204.sr
      implicit real*8 (a-h,o-z)
c
c  8-level atom for Ni II consisting of the
c  three lowest multiplets : 2D (2 levels;1-2), 4F (4 levels;3-6),
c  and 2F (2 levels;7-8).
c
c  References :
c    (A-values, energies :)
c    Nussbaumer & Storey, 1982, A&A 110, 295 (NS82)  
c    (A-value and energies for the 6.63 my transition (1->2):)
c    (same as in NS82)
c    Nussbaumer & Storey, 1988, A&A 200, L25
c    Collision strengths from  (at 10000 K):
c       Bautista, M.A. & Pradhan, A.K. 1996 A&AS, 115, 551. (BP96)
c     and (NS82)
c
      include 'INCLUDE/PARAM.IOR'
c      common/cni2/e(8),g(8),a(8,8),om(8,8),wl(8,8),nni2,
c     &     jlni2(8,8),klvni2(18,18)
c new 971204
      common/cni2/e(18),g(18),a(18,18),om(18,18),wl(18,18),nni2,
     &     jlni2(18,18),klvni2(18,18)
      common/cebin/e1(ne),en(ne),jmax
c
c  number of levels
      nni2=8
c
      do i=1,8
         e(i)=0.
         g(i)=0.
         do j=1,8
            a(i,j)=0.
            om(i,j)=0.
            wl(i,j)=0.
            jlni2(i,j)=0
            klvni2(i,j)=0
         enddo
      enddo
c
c     e in cm**-1
c
      e(1)=0.00
      e(2)=1506.94
      e(3)=8393.90
      e(4)=9330.04
      e(5)=10115.66
      e(6)=10663.89
      e(7)=13550.39
      e(8)=14995.57
c
      do i=1,8
         do j=1,8
            if (i.ne.j) then
               de=abs(e(i)-e(j))
               wl(i,j)=1.e8/de
            endif                  
         enddo
      enddo
c  e in eV
      do i=1,8
         e(i)=e(i)/8067.1
      enddo
c
c  in which energy bin does the line fall?
      do j=2,8
         do i=1,j-1
            enij=e(j)-e(i)
            do j1=1,jmax
               if(enij.gt.en(j1).and.enij.le.en(j1+1)) then
                  jlni2(j,i)=j1
                  jlni2(i,j)=j1
                  goto 22
               endif               
            enddo
 22         continue
         enddo
      enddo
c
c  statistical weights
c
      g(1)=6.
      g(2)=4.
      g(3)=10.
      g(4)=8.
      g(5)=6.
      g(6)=4.
      g(7)=8.
      g(8)=6.
c
c  transition probabilities
c
      a(2,1)=5.54e-2
      a(3,1)=7.53e-6
      a(4,1)=1.43e-3
      a(5,1)=1.31e-4
      a(6,1)=8.79e-5
      a(7,1)=2.26e-1
      a(8,1)=9.82e-2
c
      a(4,2)=2.11e-5
      a(5,2)=1.19e-3
      a(6,2)=8.28e-4
      a(7,2)=1.30e-2
      a(8,2)=1.84e-1
c
      a(4,3)=2.71e-2
      a(5,3)=6.85e-10
      a(7,3)=8.67e-2
      a(8,3)=1.32e-8
c
      a(5,4)=2.76e-2
      a(6,4)=4.12e-10
      a(7,4)=5.46e-3
      a(8,4)=2.79e-3
c
      a(6,5)=1.05e-2
      a(7,5)=1.42e-2
      a(8,5)=7.20e-3
c
      a(7,6)=6.30e-11
      a(8,6)=2.87e-2
c
      a(8,7)=4.71e-2
c
c  collision strengths from BP96 (at 10000 K)
c
      om(1,2)=0.113
      om(1,3)=0.165
      om(1,4)=0.0884
      om(1,5)=0.0408
      om(1,6)=0.0151
      om(1,7)=0.749
      om(1,8)=0.135
c
      om(2,3)=0.0191
      om(2,4)=0.0589
      om(2,5)=0.0696
      om(2,6)=0.0585
      om(2,7)=0.0902
      om(2,8)=0.504
c
      om(3,4)=0.410
      om(3,5)=0.0096
      om(3,6)=0.00303
      om(3,7)=0.528
      om(3,8)=0.00183
c
      om(4,5)=0.510
      om(4,6)=0.00875
      om(4,7)=0.242
      om(4,8)=0.182
c
      om(5,6)=0.382
      om(5,7)=0.0765
      om(5,8)=0.241
c
      om(6,7)=0.000976
      om(6,8)=0.211
c
      om(7,8)=0.0344
c
c  collision strengths from NS82
c
c      om(1,2)=0.148
c      om(1,3)=0.230
c      om(1,4)=0.144
c      om(1,5)=0.062
c      om(1,6)=0.024
c      om(1,7)=0.777
c      om(1,8)=0.174
c
c      om(2,3)=0.023
c      om(2,4)=0.071
c      om(2,5)=0.095
c      om(2,6)=0.078
c      om(2,7)=0.116
c      om(2,8)=0.499
c
c      om(3,4)=1.234
c      om(3,5)=0.024
c      om(3,6)=0.010
c      om(3,7)=0.833
c      om(3,8)=0.023
c
c      om(4,5)=1.183
c      om(4,6)=0.025
c      om(4,7)=0.661
c      om(4,8)=0.250
c
c      om(5,6)=0.796
c      om(5,7)=0.317
c      om(5,8)=0.478
c
c      om(6,7)=0.008
c      om(6,8)=0.542
c
c      om(7,8)=0.157
c
c      do i=1,8
c         do j=1,8
c            write(*,701) i,j,a(i,j)
c         enddo
c      enddo
c
c      do i=1,8
c         do j=1,8
c            write(*,702) i,j,om(i,j)
c         enddo
c      enddo
c
c 701  format(2i5,1pe10.2)
c 702  format(2i5,f10.3)
c
      return
      end



      subroutine atdatco2
      implicit real*8 (a-h,o-z)
c
c  3-level atom for Co II consisting of the
c  lowest multiplets : (3d8) a3F
c
c  References :
c    (A-values,  energies :)
c    Nussbaumer & Storey, 1988, A&A 200, L25
c
      include 'INCLUDE/PARAM.IOR'
      common/cco2/e(3),g(3),a(3,3),om(3,3),wl(3,3),nco2,
     &     jlco2(3,3),klvco2(3,3)
      common/cebin/e1(ne),en(ne),jmax
c
c  number of levels
      nco2=3
c
      do i=1,3
         e(i)=0.
         g(i)=0.
         do j=1,3
            a(i,j)=0.
            om(i,j)=0.
            wl(i,j)=0.
            jlco2(i,j)=0
            klvco2(i,j)=0
         enddo
      enddo
c
c     e in cm**-1
c
      e(1)=0.00
      e(2)=950.51
      e(3)=1597.32
c
      do i=1,3
         wl(i,i)=0.d0
         do j=1,3
            if (i.ne.j) then
               de=abs(e(i)-e(j))
               wl(i,j)=1.e8/de
            endif                  
         enddo
      enddo
c  e in eV
      do i=1,3
         e(i)=e(i)/8067.1
      enddo
c
c  in which energy bin does the line fall?
      do j=2,3
         do i=1,j-1
            enij=e(j)-e(i)
            do j1=1,jmax
               if(enij.gt.en(j1).and.enij.le.en(j1+1)) then
                  jlco2(j,i)=j1
                  jlco2(i,j)=j1
                  goto 22
               endif               
            enddo
 22         continue
         enddo
      enddo
c
c  statistical weights
c
      g(1)=9.
      g(2)=7.
      g(3)=5.
c
c  transition probabilities
c
      a(2,1)=2.23e-2
      a(3,2)=9.73e-3
c
c  collision strengths
c
c  The collision strengths for Co II are unknown.
c  We assume that the collision strength can be written
c  on the form :
c  om(i,j) = const*g(i)*g(j)
c  This form has been used by Axelrod,1980 in his thesis (his const = 4.e-3),
c  and by Bahcall, J.N. and Wolf, R.A., 1968, ApJ, 152, 701. (refered to by
c  Hollenbach and McKee, 1989)
c  The constant has been determined from the known numbers for Fe II.
c  When determining the constant only the 16 lowest levels in Fe II were 
c  used, that is no allowed transitions.
c
c  The collision strengths are highly uncertain !!!
c
c      const=0.046
c      do i=1,3
c         do j=i+1,3
c            om(i,j)=0.046*g(i)*g(j)
c         enddo
c      enddo
c
      do i=1,3
         do j=i+1,3
            om(i,j)=0.1
         enddo
      enddo
c
      return
      end

