c*****************************************************************
c
      subroutine matrix(n)
c*****************************************************************
c -- subroutine matrix solves the matrix equation a*x=b
c -- solution is returned in vector b
c
c -- the solution is row-oriented. the solution of loose matrices
c -- are speeded up by skipping zero element operations.
c
c -- in the forward elimination, the routine also stops at the last
c -- non-zero element in each row.
c*****************************************************************
      implicit real*8(a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
c
      common/cmat/a(mmax,mmax),b(mmax)
c
      dimension lastn(mmax),asave(mmax)
c
c -- find the column index of the last non-zero element in each row --
c -- speeds up the solution of loose matrices considerably --
c
      do 100 i=1,n
      do 101 j=n,i,-1
      jsave=j
      if(a(i,j).ne.0.0e0) goto 102
  101 continue
  102 lastn(i)=jsave
  100 continue
c
c -- ************************** --
c -- forward elimination scheme --
c -- ************************** --
c
      do 200 i=1,n-1
c
c -- partial pivoting routine --
c
      ai=0.0e0
      do 201 l=i,n
      asave(l)=0.0e0
      if(dabs(a(l,i)).gt.ai) lsave=l
      if(dabs(a(l,i)).gt.ai) ai=dabs(a(l,i))
  201 continue
      if(lsave.eq.i) goto 202
c
c -- interchange rows a(i,?) and a(lsave,?) if lsave.ne.i --
c
      last=max0(lastn(i),lastn(lsave))
      do 203 l=i,last
      asave(l)=a(i,l)
      a(i,l)=a(lsave,l)
  203 a(lsave,l)=asave(l)
      bsave=b(i)
      b(i)=b(lsave)
      b(lsave)=bsave
      lastni=lastn(i)
      lastn(i)=lastn(lsave)
      lastn(lsave)=lastni
  202 continue
c
c -- elimination routine --
c
      bi=b(i)
      aii=a(i,i)
      do 301 j=i+1,n
      if(a(j,i).eq.0.0e0.or.(i+1).gt.lastn(i)) goto 301
      ajiaii=a(j,i)/a(i,i)
      do 302 k=i+1,lastn(i)
  302 a(j,k)=a(j,k)-ajiaii*a(i,k)
      b(j)=b(j)-ajiaii*bi
      lastn(j)=max0(lastn(j),lastn(i))
  301 continue
c
c -- ************************** --
c -- end of forward elimination --
c -- ************************** --
c
  200 continue
c
c -- *************** --
c -- back-substitute --
c -- *************** --
c
      do 400 k=n,1,-1
      bk=b(k)
      if((k+1).gt.n) goto 401
      do 402 l=k+1,n
      bk=bk-a(k,l)*b(l)
  402 continue
  401 b(k)=bk/a(k,k)
  400 continue
c
      return
      end

      subroutine matrixb(n,irout)
c  samma som cf:s simulb
      implicit real*8(a-h,o-z)
c
      include 'INCLUDE/PARAM.IOR'
c
      character*25 chforb
      common/cforb3nr/chforb
      common/cmat/a(mmax,mmax),b(mmax)
      common/depthind/id
c
      dimension irow(mmax),jcol(mmax),jord(mmax),y(mmax),x(mmax)
      indic=1
      eps=1.d-50
      jmax=n
      if(indic.ge.0) jmax=n+1
      do i=1,n
        a(i,n+1)=b(i)        
      enddo
      if(n.le.50) goto 5
      simulb=0.
      write(0,*) 'stop in matrixb !'
      write(50,*) 'matrixb : stop!'
      stop
      goto 999
c      return
5     deter=1.
      do 18 k=1,n
      km1=k-1
      pivot=0.
      do 11 i=1,n
      do 11 j=1,n
      if(k.eq.1) goto 9
      do 8 iscan=1,km1
      do 8 jscan=1,km1
      if(i.eq.irow(iscan)) goto 11
      if(j.eq.jcol(jscan)) goto 11
8     continue
9     if(abs(a(i,j)).le.abs(pivot)) goto 11
      pivot=a(i,j)
      irow(k)=i
      jcol(k)=j
11    continue
777   format(1x,'matrixb : stopp i pop',3i4,2e12.5)
      if(abs(pivot).gt.eps) goto 13
      simulb=0.
c      write(50,777)k,irow(k),jcol(k),pivot,a(irow(k),jcol(k))
c
      if (irout.eq.0) write(50,*) 'matrixb : error in multilev',id
      if (irout.eq.1) write(50,*) 'matrixb : error in mols',id
      if (irout.eq.2) write(50,*) 'matrixb : error in comul',id
      if (irout.eq.3) then
         write(50,*) 'matrixb : error in forb3nr',id
         write(50,701) chforb
 701     format(a)
      endif
      if (irout.eq.4) write(50,*) 'matrixb : error in ionbal',id
      if (irout.eq.5) write(50,*) 'matrixb : error in ionct',id
      if (irout.eq.6) write(50,*) 'matrixb : error in bforb',id
      if (irout.eq.7) write(50,*) 'matrixb : error in cforb',id
      if (irout.eq.8) write(50,*) 'matrixb : error in nforb',id
      if (irout.eq.9) write(50,*) 'matrixb : error in oforb',id
c
      stop
13    irowk=irow(k)
      jcolk=jcol(k)
      deter=deter*pivot
      do 14 j=1,jmax
14    a(irowk,j)=a(irowk,j)/pivot
      a(irowk,jcolk)=1./pivot
      do 19 i=1,n
      aijck=a(i,jcolk)
      if(i.eq.irowk) goto 19
      a(i,jcolk)=-aijck/pivot
      do 17 j=1,jmax
17    if(j.ne.jcolk) a(i,j)=a(i,j)-aijck*a(irowk,j)
19    continue
18    continue
      do 20 i=1,n
      irowi=irow(i)
      jcoli=jcol(i)
      jord(irowi)=jcoli
20    if(indic.ge.0) x(jcoli)=a(irowi,jmax)
      intch=0
      nm1=n-1
      do 22 i=1,nm1
      ip1=i+1
      do 22 j=ip1,n
      if(jord(j).ge.jord(i)) goto 22
      jtemp=jord(j)
      jord(j)=jord(i)
      jord(i)=jtemp
      intch=intch+1
22    continue
      if(intch/2*2.ne.intch) deter=-deter
      if(indic.le.0) goto 26
      simulb=deter
      goto 999
c      return
26    continue
      do 28j=1,n
      do 27 i=1,n
      irowi=irow(i)
      jcoli=jcol(i)
27    y(jcoli)=a(irowi,j)
      do 28 i=1,n
28    a(i,j)=y(i)
      do 30 i=1,n
      do 29 j=1,n
      irowj=irow(j)
      jcolj=jcol(j)
29    y(irowj)=a(i,jcolj)
      do 30 j=1,n
30    a(i,j)=y(j)
      simulb=deter
c
 999  continue
      do i=1,n
        b(i)=x(i)
      enddo
c
      return
  200  format(1x,'too big!')
      end


      subroutine matrixbt(n,irout,ifail)
c  samma som cf:s simulb
      implicit real*8(a-h,o-z)
c
      include 'INCLUDE/PARAM.IOR'
c
      character*25 chforb
      common/cforb3nr/chforb
      common/cmat/a(mmax,mmax),b(mmax)
      common/depthind/id
c
      dimension irow(mmax),jcol(mmax),jord(mmax),y(mmax),x(mmax)
      ifail=0
      indic=1
      eps=1.d-50
      jmax=n
      if(indic.ge.0) jmax=n+1
      do i=1,n
        a(i,n+1)=b(i)        
      enddo
      if(n.le.50) goto 5
      simulb=0.
      write(0,*) 'stop in matrixb !'
      write(50,*) 'matrixb : stop!'
      stop
      goto 999
c      return
5     deter=1.
      do 18 k=1,n
      km1=k-1
      pivot=0.
      do 11 i=1,n
      do 11 j=1,n
      if(k.eq.1) goto 9
      do 8 iscan=1,km1
      do 8 jscan=1,km1
      if(i.eq.irow(iscan)) goto 11
      if(j.eq.jcol(jscan)) goto 11
8     continue
9     if(abs(a(i,j)).le.abs(pivot)) goto 11
      pivot=a(i,j)
      irow(k)=i
      jcol(k)=j
11    continue
777   format(1x,'matrixb : stopp i pop',3i4,2e12.5)
      if(abs(pivot).gt.eps) goto 13
      simulb=0.
c      write(50,777)k,irow(k),jcol(k),pivot,a(irow(k),jcol(k))
c
      if (irout.eq.0) write(50,*) 'matrixb : error in multilev',id
      if (irout.eq.1) write(50,*) 'matrixb : error in mols',id
      if (irout.eq.2) write(50,*) 'matrixb : error in comul',id
      if (irout.eq.3) then
         write(50,*) 'matrixb : error in forb3nr',id
         write(50,701) chforb
 701     format(a)
      endif
      if (irout.eq.4) write(50,*) 'matrixb : error in ionbal',id
      if (irout.eq.5) write(50,*) 'matrixb : error in ionct',id
      if (irout.eq.6) write(50,*) 'matrixb : error in ionbal5',id
c
      ifail=1
      return
c      stop
13    irowk=irow(k)
      jcolk=jcol(k)
      deter=deter*pivot
      do 14 j=1,jmax
14    a(irowk,j)=a(irowk,j)/pivot
      a(irowk,jcolk)=1./pivot
      do 19 i=1,n
      aijck=a(i,jcolk)
      if(i.eq.irowk) goto 19
      a(i,jcolk)=-aijck/pivot
      do 17 j=1,jmax
17    if(j.ne.jcolk) a(i,j)=a(i,j)-aijck*a(irowk,j)
19    continue
18    continue
      do 20 i=1,n
      irowi=irow(i)
      jcoli=jcol(i)
      jord(irowi)=jcoli
20    if(indic.ge.0) x(jcoli)=a(irowi,jmax)
      intch=0
      nm1=n-1
      do 22 i=1,nm1
      ip1=i+1
      do 22 j=ip1,n
      if(jord(j).ge.jord(i)) goto 22
      jtemp=jord(j)
      jord(j)=jord(i)
      jord(i)=jtemp
      intch=intch+1
22    continue
      if(intch/2*2.ne.intch) deter=-deter
      if(indic.le.0) goto 26
      simulb=deter
      goto 999
c      return
26    continue
      do 28j=1,n
      do 27 i=1,n
      irowi=irow(i)
      jcoli=jcol(i)
27    y(jcoli)=a(irowi,j)
      do 28 i=1,n
28    a(i,j)=y(i)
      do 30 i=1,n
      do 29 j=1,n
      irowj=irow(j)
      jcolj=jcol(j)
29    y(irowj)=a(i,jcolj)
      do 30 j=1,n
30    a(i,j)=y(j)
      simulb=deter
c
 999  continue
      do i=1,n
        b(i)=x(i)
      enddo
c
      return
  200  format(1x,'too big!')
      end


      subroutine romb(aa,bb,err,fi,fail,k)
      implicit real*8(a-h,o-z)
      dimension tr(100),w(100)
      a=aa
      b=bb
      dx=(b-a)/2.
      fi1=0.5*(fx(a)+fx(b))
      fi2=fx(a+dx)
      tr(1)=dx*(fi1+fi2)
      n=1
      k=1
      w(1)=4.
1     fi=tr(1)
      w(k+1)=4.*w(k)
      n=2*n
      k=k+1
      tdx=dx
      dx=dx/2.
      x=a+dx
      do 2 i=1,n
      fi2=fi2+fx(x)
2     x=x+tdx
      tr(k)=dx*(fi1+fi2)
      kk=k-1
      do 3 j=1,kk
      m=k-j
3     tr(m)=(w(j)*tr(m+1)-tr(m))/(w(j)-1.)
      fail=abs(tr(1)-fi)/abs(tr(1))
      if(fail-err)6,6,4
4     if(k-100)1,6,6
6     fi=tr(1)
      return
      end



      double precision function simul(n,a,x,eps,ii,nrc)
      implicit real*8(a-h,o-z)
      parameter (nl=130,nlp1=nl+1)
c     solve a*x = b
c     b contains as input the r.h. side of the system, and as output the
c     solution
c     eps, ii, nrc not used
      dimension a(nlp1,nlp1),x(n),b(nlp1)
      dimension lastn(nlp1),asave(nlp1)                                     
c                                                                       
c -- find the column index of the last non-zero element in each row --  
c -- speeds up the solution of loose matrices considerably --           
c                                                                       
      do i=1,n
      b(i)=a(i,n+1)
      enddo
      do 100 i=1,n                                                      
      do 101 j=n,i,-1                                                   
      jsave=j                                                           
      if(a(i,j).ne.0.0d0) goto 102                                      
  101 continue                                                          
  102 lastn(i)=jsave                                                    
  100 continue                                                          
c                                                                       
c -- ************************** --                                      
c -- forward elimination scheme --                                      
c -- ************************** --                                      
c                                                                       
      do 200 i=1,n-1                                                    
c                                                                       
c -- partial pivoting routine --                                        
c                                                                       
      ai=0.0d0                                                          
      do 201 l=i,n                                                      
      asave(l)=0.0d0                                                    
      if(dabs(a(l,i)).gt.ai) lsave=l                                    
      if(dabs(a(l,i)).gt.ai) ai=dabs(a(l,i))                            
  201 continue                                                          
      if(lsave.eq.i) goto 202                                           
c                                                                       
c -- interchange rows a(i,?) and a(lsave,?) if lsave.ne.i --            
c                                                                       
      last=max0(lastn(i),lastn(lsave))                                  
      do 203 l=i,last                                                   
      asave(l)=a(i,l)                                                   
      a(i,l)=a(lsave,l)                                                 
  203 a(lsave,l)=asave(l)                                               
      bsave=b(i)                                                        
      b(i)=b(lsave)                                                     
      b(lsave)=bsave                                                    
      lastni=lastn(i)                                                   
      lastn(i)=lastn(lsave)                                             
      lastn(lsave)=lastni                                               
  202 continue                                                          
c                                                                       
c -- elimination routine --                                             
c                                                                       
      bi=b(i)                                                           
      aii=a(i,i)                                                        
      do 301 j=i+1,n                                                    
      if(a(j,i).eq.0.0d0.or.(i+1).gt.lastn(i)) goto 301                 
      ajiaii=a(j,i)/a(i,i)                                              
      do 302 k=i+1,lastn(i)                                             
  302 a(j,k)=a(j,k)-ajiaii*a(i,k)                                       
      b(j)=b(j)-ajiaii*bi                                               
      lastn(j)=max0(lastn(j),lastn(i))                                  
  301 continue                                                          
c                                                                       
c -- ************************** --                                      
c -- end of forward elimination --                                      
c -- ************************** --                                      
c                                                                       
  200 continue                                                          
c                                                                       
c -- *************** --                                                 
c -- back-substitute --                                                 
c -- *************** --                                                 
c                                                                       
      do 400 k=n,1,-1                                                   
      bk=b(k)                                                           
      if((k+1).gt.n) goto 401                                           
      do 402 l=k+1,n                                                    
      bk=bk-a(k,l)*b(l)                                                 
  402 continue                                                          
  401 b(k)=bk/a(k,k)                                                    
  400 continue                                                          
c                                                                       
      do i=1,n
      x(i)=b(i)
      enddo
      simul=0.
      return                                                            
      end




      double precision function hgf(a,b,c,x)
      implicit real*8 (a-h,o-z)
      integer a,b,c
      dimension fact(100)
c
c  hgf = 2F1 = The hypergeometric function
c  See Arfken, 'Mathematical Methods for Physicists', p. 748.
c
c  a.le.0 and b.le.0 which makes the sum finite !
c
      nmax=100
      if (a.gt.0) then
         write(50,*) 'a out of range in hgf !' 
         stop
      endif
      if (b.gt.0) then
         write(50,*) 'b out of range in hgf !' 
         stop
      endif
      if (c.le.0) then
         write(50,*) 'c out of range in hgf !' 
         stop
      endif
c
      xa=dble(a)
      xb=dble(b)
      xc=dble(c)
c
      do n=1,nmax
         nm=n
         xn=dble(n)
         fact(n)=((xa+xn-1.)*(xb+xn-1.)/(xc+xn-1.))*x/xn
         if (fact(n).eq.0.) goto 9
      enddo
      if (fact(nmax).ne.0) then
         write(50,*) 'Insufficient summation in hgf'
         stop
      endif
 9    continue
c
      f=1.
      do n=nm,1,-1
         f=(1.+fact(n)*f)
      enddo
      hgf=f
c
      return
      end
