      subroutine rloss(k,wl,om,g1,g2,a21,r,xel,z,te,rl,weq
     &,cint,rfl,l,wli)
      implicit real*8(a-h,o-z)
      parameter (nfel=1000)
c     **************************************************************
c     *****
c     this routine calculates the heat loss and observed flux from
c     a line with the escape probability method.
c     wl = wavelegth (a)
c     om = collision strength
c     g1,g2 = stat.weights of the lower and upper levels
c     a21 = transition probability
c     r = radius (in terms of shock radius)
c     xel = electron fraction
c     z = total abundance of the ion
c     te = temperature
c     rl = loss rate in erg cm**3 /s
c     weq = flux emitted to infinity (erg/s)
c     cint = observed flux
c     rfl = radiation force (not used here)
c     sred = (1-exp(-tau))*source fcn for line profile
c     revised 87-05-15
c
c     *****
c     ****************************************************************
      common/cden/dens
      data pi/3.1415926e0/,elch/1.60219e-12/,amu/1.660531e-24/
      wli=wl
      en=1.4413e8/wl
      encgs=1.989e-8/wl
      eev=encgs/elch
      den=xel*dens
      c21=8.63e-6*om/(g2*dsqrt(te))
      e=c21*den/a21
      ente=en/te
      if(en/te.gt.100.) ente=100.
c
c     n1*g2/n2*g1
c
      rn1g2n2g1=(1.+e)/(e*fexp(-ente))
      rn1n2=g1*rn1g2n2g1/g2
c
c
c     cooling rate of electrons taking thermalization into account
c

      rl=encgs*g2*c21*z*fexp(-ente)*(1.-fexp(ente)/rn1g2n2g1)
     &/(g1*(1.+1./rn1n2))
      xx=g1*(a21+c21*den)/(g2*c21*den*fexp(-ente))
      weq=z*a21*encgs*den/(1.+xx)
      return
      end

      subroutine diel(ar,t0r,br,t1r,te,ald)
      implicit real*8(a-h,o-z)
      real*4 ar,t0r,br,t1r
c     ***************************************************************
c     ******
c     dielectric recombination rate according to formula in ald. and
c     peq.
c     ******
c     ***************************************************************
        a=dble(ar)
        t0=dble(t0r)
        b=dble(br)
        t1=dble(t1r)
        t4=te/1.e4
      ex=t0/te
      ex2=t1/te
      ald=0.
      if(ex2.gt.100.) ex2=100.
      if(ex.lt.100)ald=a*dexp(-ex)*(1.+b*dexp(-ex2))/(te*dsqrt(te))
      if(t4.lt..4) ald=0.
      return
      end

      subroutine dielb(ar,br,cr,dr,fr,t4,aldb)
      implicit real*8(a-h,o-z)
      real*4 ar,br,cr,dr,fr
c     ***************************************************************
c     ******
c     dielectric recombination rate according to formula in nussbaumer
c     and storey (a&a 126,75 (1983))
c     ******
c     ***************************************************************
c     dielectric recombination rate
      a=dble(ar)
      b=dble(br)
      c=dble(cr)
      d=dble(dr)
      f=dble(fr)
      ex=f/t4
      aldb=0.
      if(t4.gt.6.) goto 100
      if(ex.gt.100.) goto 100
      aldb=1.e-12*(a/t4+b+c*t4+d*t4*t4)*dexp(-f/t4)/t4**1.5
 100  continue
      if(t4.lt..2) aldb=0.
      return
      end



      subroutine bis(te,xe,ifail)
      implicit real*8(a-h,o-z)
c
      include 'INCLUDE/PARAM.IOR'
c
      parameter (itmax=30)
      character*20 chc,chh
      common/ctpar/tday,dts,iss,iss1,itd
      common/ther/a1,b1,tin,tol,e20
      common/checo/heat,cool,co(mie,mion),emstot,
     &     pheats,pheat(mie,mion),phex,cov(115),hev(35),
     &     comg1(nl,nl),cosi1(nl,nl),chc(115),chh(35),klcm,klhm
c      common/checo/heat,cool,co(mie,mion),emstot,
c     &     pheats,pheat(mie,mion),phex,
c     &     hphi,hphei,hpoi,hpcaii,hpfei,hpfeii,hpfeiii,hpfeiv,
c     &     chi,crechi,chei,crechei,coci1,coci2,ccii1,ccii2,c221,
c     &     conii1,conii2,coniii1,coniii2,coi,coifs,crecoi,
c     &     coiii1,coiii2,coneiii1,coneiii2,
c     &     c411,c412,cosiii1,cosiii2,cosuli1,cosuli2,csul2,c1222,
c     &     cosuliii1,cosuliii2,cocaii,creccaii,
c     &     cfei1,cfei,crecfei,cfeii1,cfeii,crecfeii,
c     &     cfeiii1,cfeiii,crecfeiii,cfeiv,crecfeiv
      common/depthind/id
c
      ifail=0
      ima=0
      imb=0
      a=a1
      b=b1
      fb=rad(b1,xe)
      fa=rad(a1,xe)
c
      if (fa*fb.gt.0.) then
         write(50,*)'bis : same sign',fa,fb
         ifail=1
         goto 99
      else
cqcq
         if (b1.gt.a1) then
 83         continue
            if (fb.gt.0.) then
               a1=b1
               fa=fb
               b1=1.05*b1
               fb=rad(b1,xe)
               goto 83
            endif
         elseif(a1.gt.b1) then
 84         continue
            if (fa.gt.0.) then
               b1=a1
               fb=fa
               a1=1.05*a1
               fa=rad(a1,xe)
               goto 84
            endif
         endif
cqcq
         do it=1,itmax
            tme=(a+b)/2.
            fm=rad(tme,xe)
            if (fa*fm.gt.0.) then
               a=tme
               fa=fm
c
               imb=0
               ima=ima+1
cqcq               if (ima.gt.8) then
               if (ima.gt.8.and.it.lt.20) then
                  ima=0
 81               continue
                  b=1.05*b
                  fb=rad(b,xe)
                  fm=fb
                  if (fa*fb.gt.0.) goto 81
               endif
            else
               b=tme
               fb=fm
c
               ima=0
               imb=imb+1
cqcq               if (imb.gt.8) then
               if (imb.gt.8.and.it.lt.20) then
                  imb=0
 82               continue
                  a=0.95*a
                  fa=rad(a,xe)
                  fm=fa
                  if (fa*fb.gt.0.) goto 82
               endif
            endif
c
cqcq            if(abs(fm).le.e20*abs(heat)) goto 99
            if(abs(fm).le.e20*abs(heat).and.it.ge.5) goto 99
         enddo
         if (it.ge.itmax) write(50,77)tday,id,tme,fm,e20*abs(heat)
      endif
 99   continue
      te=tme
 77   format('bis : no temp. conv. ',f10.3,i4,1pe12.4,5e12.4)
c
      return
      end


      subroutine rlossb(k,wl,om,g1,g2,a21,r,xel,z,te,rl,weq
     &,cint,rfl,l,wli,t00)
      implicit real*8(a-h,o-z)
c     **************************************************************
c     *****
c     this routine calculates the heat loss and observed flux from
c     a line with the escape probability method.
c     wl = wavelegth (a)
c     om = collision strength
c     g1,g2 = stat.weights of the lower and upper levels
c     a21 = transition probability
c     r = radius (in terms of shock radius)
c     xel = electron fraction
c     z = total abundance of the ion
c     te = temperature
c     rl = loss rate in erg cm**3 /s
c     weq = flux emitted to infinity (erg/s)
c     cint = observed flux
c     rfl = radiation force (not used here)
c     sred = (1-exp(-tau))*source fcn for line profile
c     revised 87-05-15
c
c     *****
c     ****************************************************************
      common/ctpar/tday,dts,iss,iss1,itd
      common/cden/dens
      data pi/3.1415926e0/,elch/1.60219e-12/,amu/1.660531e-24/
      wli=wl
      en=1.4413e8/wl
c  encgs = transition energy in erg
      encgs=1.989e-8/wl
      eev=encgs/elch
      den=xel*dens
      c21=8.63e-6*om/(g2*dsqrt(te))
      e=c21*den/a21
c
c     this part applies for a velocity gradient and a background
c     continuum, fc.
c
      time=8.64e4*tday
c     opt. depth. coeff = 1e-24/8 pi
      t00=3.979e-26*time*wl**3.*g2*a21*z*dens/g1
 200  continue
c
c     calculate the escape probability.
c
      mm=0
      t0=t00
232   be=1.
      top=t0
      ente=en/te
      be=1.-0.5*t0+t0**2/6.
      if(t0.gt.0.005) be=(1.-dexp(-t0))/t0
      if(en/te.gt.250.) ente=250.
      fexpente=fexp(-ente)
c
c     n1*g2/n2*g1
c
c      rn1g2n2g1=(be+e)/(e*dexp(-ente))
c      rn1n2=g1*rn1g2n2g1/g2
c
c     n2*g1/n1*g2
c
      rn2g1n1g2=(e*fexpente)/(be+e)
      rn2n1=g2*rn2g1n1g2/g1
c
c     correct optical depth for stimulated emission
c
c      if(rn1n2.lt.1000.) t0new=t00*(1.-1./rn1g2n2g1)/(1.+1./rn1n2)
c      if(rn1n2.lt.1000.) t0=t0new
c      if(rn1n2.lt.1000.) mm=mm+1
c      if(rn1n2.lt.1000..and.mm.lt.6) goto 232
      if(rn2n1.gt.0.001) t0new=t00*(1.-rn2g1n1g2)/(1.+rn2n1)
      if(rn2n1.gt.0.001) t0=t0new
      if(rn2n1.gt.0.001) mm=mm+1
      if(rn2n1.gt.0.001.and.mm.lt.6) goto 232
c
c     cooling rate of electrons taking thermalization into account
c
c      rl=encgs*g2*c21*z*fexp(-ente)*(1.-dexp(ente)/rn1g2n2g1)
c     &/(g1*(1.+1./rn1n2))
c      xx=g1*(a21*be+c21*den)/(g2*c21*den*fexp(-ente))
c      weq=z*a21*encgs*be*dens/(1.+xx)
      rl=encgs*g2*c21*z*fexpente*(1.-dexp(ente)*rn2g1n1g2)
     &/(g1*(1.+rn2n1))
      xx=g1*(a21*be+c21*den)/(g2*c21*den*fexpente)
      weq=z*a21*encgs*be*dens/(1.+xx)
cqcq
      if(en/te.lt.650.) then
         bpl=2.031e-04*eev**3/(dexp(en/te)-1.)
      else
         bpl=0.
      endif
c     source function incl. stim. emission
      ep=e
      if(en/te.lt.650.) ep=e*(1.-fexpente)
      if(en/te.lt..001) ep=e*en/te
      source=(ep*bpl)/(ep+be)
c     observed intensity
      expm1=1.-fexp(-t0)
      if(t0.lt.1.e-3) expm1=t0-0.5*t0**2
      cint=4.*pi*1.e8*expm1*source/(time*wl*den*dens)
cqcq
      c12=(g2/g1)*c21*fexpente
c      x2=1./(1.+rn1n2)
      x2=rn2n1/(1.+rn2n1)
      x1=1.-x2
      rl=encgs*z*(x1*c12-x2*c21)
      cint=z*x2*a21*be*encgs/(dens*xel)
cqcq
c     convert to intensity/ angstrom
      source=3.e18*source/wl**2
      sred=expm1*source
      return
      end


      subroutine fivel(n,e1,e2,e3,e4,e5,g1,g2,
     &g3,g4,g5,a21,a31,a41,a51,a32,a42,a52,a43,a53,
     &a54,c21,c31,c41,c51,c32,c42,c52,c43,c53,c54,te,den,xel,z,rlt)
      implicit real*8 (a-h,o-z)
      real*4 a21,a31,a41,a51,a32,a42,a52,a43,a53,a54
     &,c21,c31,c41,c51,c32,c42,c52,c43,c53,c54
     &,e1,e2,e3,e4,e5,g1,g2,g3,g4,g5
      integer n
      parameter (nl=130,nlp1=nl+1)
      common/ctpar/tday,dts,iss,iss1,itd
      common/w/wl(5,5)
      dimension  c(5,5),g(5),e(5),a(5,5),be(5,5)
      dimension aa(nlp1,nlp1),x(nl),rl(5,5)
      data pi/3.14159/
 902   format(5e11.4)
      nm1=n-1
      np1=n+1
      do 836 i=1,np1
      do 836 j=1,np1
836   aa(i,j)=0.
       e(1)=dble(e1)
       e(2)=dble(e2)
       e(3)=dble(e3)
       e(4)=dble(e4)
       e(5)=dble(e5)
       g(1)=dble(g1)
       g(2)=dble(g2)
       g(3)=dble(g3)
       g(4)=dble(g4)
       g(5)=dble(g5)
      do 220 i=1,n
 220  e(i)=e(i)/8065.46
      do 5395 i=1,n
      do 5394 j=1,n
      wl(i,j)=0.0
      if(i.eq.j) goto 5394
      if(e(i).eq.e(j)) goto 5394
      wl(i,j)=dabs(12398.54/(e(i)-e(j)))
5394  continue
5395  continue
      do 7352 i=1,n
      do 7352 j=1,n
      c(i,j)=0.
 7352 a(i,j)=0.
      a(2,1)=dble(a21)
      a(3,1)=dble(a31)
      a(4,1)=dble(a41)
      a(5,1)=dble(a51)
      a(3,2)=dble(a32)
      a(4,2)=dble(a42)
      a(5,2)=dble(a52)
      a(4,3)=dble(a43)
      a(5,3)=dble(a53)
      a(5,4)=dble(a54)
      c(2,1)=dble(c21)
      c(3,1)=dble(c31)
      c(4,1)=dble(c41)
      c(5,1)=dble(c51)
      c(3,2)=dble(c32)
      c(4,2)=dble(c42)
      c(5,2)=dble(c52)
      c(4,3)=dble(c43)
      c(5,3)=dble(c53)
      c(5,4)=dble(c54)
      tev=te/1.1609d4
      time=8.64e4*tday
      do 5 i=1,nm1
      ip1=i+1
      do 5 j=ip1,n
      c(j,i)=8.63d-6*den*xel*c(j,i)/(dsqrt(te)*g(j))
5     c(i,j)=c(j,i)*fexp(-(e(j)-e(i))/tev)*g(j)/g(i)
      do l=1,20
cfqq
	do i=1,nlp1
	  do j=1,nlp1
	    aa(i,j)=0.
	  enddo
	enddo
      do 100 i=1,n
      do 100 j=1,n
      if(i.eq.n) aa(i,j)=1.d0
      if(i.eq.n) goto 100
      if(x(i).gt.0.) then
      finduced=1.-g(i)*x(j)/(g(j)*x(i))
      else
      finduced=1.
      endif
      dei=x(i)*z*den*finduced
c
c     calculate the escape probability.
c
      t0=1.e-24*wl(j,i)**3*a(j,i)*g(j)*dei*time/(8.*pi*g(i))
      if(l.eq.1) t0=.0
      if(t0.lt.0.01) be(j,i)=1.-0.5*t0+t0**2/6.
      if(t0.ge.0.01) be(j,i)=(1.-dexp(-t0))/t0
      if(i.ne.j) goto 2
      s=0.
      do 3 k=1,n
3     s=s+c(i,k)+be(i,k)*a(i,k)
      aa(i,i)=-s
      goto 100
2     aa(i,j)=be(j,i)*a(j,i)+c(j,i)
 100  continue
      nrc=n+1
      aa(n,nrc)=1.
      eps=1.d-30
      na=n
      di=simul(na,aa,x,eps,1,nrc)
      enddo
      rlt=0.
      do 10 i=1,nm1
      ip1=i+1
      do 10 j=ip1,n
      rl(j,i)=1.602e-12*(e(j)-e(i))*x(j)*a(j,i)/den
      rlt=rlt+rl(j,i)
c10    write(8,920)i,j,wl(j,i),rl(j,i)
10    continue
920   format(2i4,f14.2,e12.4)
25    continue
      return
      end


      double precision function fir(n,z,te)
c     calculate goulds function psi in gould eq (31)
      implicit real*8(a-h,o-z)
c
      y=z**2*15.79e4/te
      y4=z**2*15.79
      fir=fin(n,y)/fin(n,y4)
c
      return
      end

      double precision function fin(n,y)
      implicit real*8(a-h,o-z)
c
      if(y.lt.0.10) then
        fin=y*(-1.202*dlog(y)-.298)
      else
c            fin=0.4288+0.5*dlog(y)+0.469/(y**.33333)
        fin=0.5*(dlog(y)+1.735+1./(6.*y))
      endif
      do m=1,n-1
        x=y/m**2
      if(x.lt.1.) then
        e1=-dlog(x)-.577216+x-.24991*x*x+.0552*x*x*x-.00976
     &            *x*x*x*x
        e1=e1*dexp(x)
      else
        e1=(x*x+2.334733*x+.250621)/((x*x+3.330657*x
     &            +1.681534)*x)
      endif
        fim=x*e1/m
        fin=fin-fim
      enddo
      return
      end



      subroutine forb3nr(e21,e32,a21,a31,a32,o21,o31,o32,s1,s2,s3,
     &f21,f31,f32,fp21,fp31,fp32,t21,t31,t32,s21,s31,s32,x1,x2,x3,
     &cs21,cs31,cs32,te,z,f,xel,j21,j31,j32,ge2,ge3,fjl21,fjl31,
     &fjl32,phxl21,phxl31,phxl32)
c     cooling/vol= f*x(el)*ab(i)*x(i)*den**2
c  
      implicit real*8(a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
c      parameter(itmax=100)
      parameter(itmax=30)
      common/cden/den
      common/ctpar/tday,dts,iss,iss1,itd
      common/cmat/a(mmax,mmax),b(mmax)
      common/depthind/id
      common/cfj/fj(ne),fjidm1(ne),fjc(0:ne),fjcidm1(ne)
      common/cphed/phed(0:ne),phextot,phextc,phextl,phexlm(mid,lmax),
     &     eq1(1200),klq1(1200),kl1m
      data pi/3.1415926e0/,elch/1.60219e-12/,amu/1.660531e-24/
      data tol/0.00001/
c hc=h*c=6.626e-27 erg s *2.9979e10 cm/s = 1.9864e-16 erg cm
      data hc/1.9864e-16/
c
      if (z.gt.0.) then
         e31=e21+e32
         c21=8.63e-6*den*xel*o21/(s2*dsqrt(te))
         c31=8.63e-6*den*xel*o31/(s3*dsqrt(te))
         c32=8.63e-6*den*xel*o32/(s3*dsqrt(te))
         tev=te/1.1609e4
         c12=0.
         c13=0.
         c23=0.
         if(e21/tev.lt.100.) c12=s2*c21*dexp(-e21*1.1609d4/te)/s1
         if(e31/tev.lt.100.) c13=s3*c31*dexp(-e31*1.1609d4/te)/s1
         if(e32/tev.lt.100.) c23=s3*c32*dexp(-e32*1.1609d4/te)/s2
         wl21=12398.*1.e-8/e21
         wl21u3=wl21**3
         wl31=12398.*1.e-8/e31
         wl31u3=wl31**3
         wl32=12398.*1.e-8/e32
         wl32u3=wl32**3
         deni=z*den
c
c  Photoexcitation/deexcitation (stimulated)
c
c  change the unit from ../eV to ../Hz
         q1=1./(2.41838e14*1.602e-12)
c
         flc21=fjc(j21)/2.41838e14
         flc21=flc21+fjl21
         flc31=fjc(j31)/2.41838e14
         flc31=flc31+fjl31
         flc32=fjc(j32)/2.41838e14
         flc32=flc32+fjl32
c
         pe21=0.5*(wl21*wl21)*a21*flc21/(e21*1.602e-12)
         pe12=(s2/s1)*pe21
         if (pe12.gt.0.) then
            frl=fjl21/flc21
            frc=1.-frl
            pe12c=pe12*frc
            pe21c=pe21*frc
            pe12l=pe12*frl
            pe21l=pe21*frl
         else
            pe12c=0.d0
            pe21c=0.d0
            pe12l=0.d0
            pe21l=0.d0
         endif
         pe31=0.5*(wl31*wl31)*a31*flc31/(e31*1.602e-12)
         pe13=(s3/s1)*pe31
         if (pe13.gt.0.) then
            frl=fjl31/flc31
            frc=1.-frl
            pe13c=pe13*frc
            pe31c=pe31*frc
            pe13l=pe13*frl
            pe31l=pe31*frl
         else
            pe13c=0.d0
            pe31c=0.d0
            pe13l=0.d0
            pe31l=0.d0
         endif
         pe32=0.5*(wl32*wl32)*a32*flc32/(e32*1.602e-12)
         pe23=(s3/s2)*pe32
         if (pe23.gt.0.) then
            frl=fjl32/flc32
            frc=1.-frl
            pe23c=pe23*frc
            pe32c=pe32*frc
            pe23l=pe23*frl
            pe32l=pe32*frl
         else
            pe23c=0.d0
            pe32c=0.d0
            pe23l=0.d0
            pe32l=0.d0
         endif
c
         x1=1.
         x2=1.e-10
         x3=1.e-11
         time=8.64e4*tday
         do it=1,itmax
            de1=x1*deni
            de2=x2*deni
            de3=x3*deni
            x1o=x1
            x2o=x2
c
c     calculate the escape probability.
c
            t0=wl21u3*a21*s2*de1*(1.-s1*x2/(s2*x1))*time/(8.*pi*s1)
            t21=t0
            dt21dx1=wl21u3*a21*s2*z*den*time/(8.*pi*s1)
            dt21dx2=-wl21u3*a21*z*den*time/(8.*pi)
            if (t0.lt.0.01) then
               be21=1.-0.5*t0+(t0**2)/6.
               dbedtau=-0.5+(2./6.)*t0
            elseif(t0.ge.0.01) then
               be21=(1.-fexp(-t0))/t0
               dbedtau=((1.+t0)*fexp(-t0)-1.)/(t0*t0)
            endif
            dbe21dx1=dbedtau*dt21dx1
            dbe21dx2=dbedtau*dt21dx2
c
            t0=wl31u3*a31*s3*de1*(1.-s1*x3/(s3*x1))*time/(8.*pi*s1)
            t31=t0
            dt31dx1=wl31u3*a31*s3*z*den*time/(8.*pi*s1)
            dt31dx3=-wl31u3*a31*z*den*time/(8.*pi)
            if(t0.lt.0.01) then
               be31=1.-0.5*t0+t0**2/6.
               dbedtau=-0.5+(2./6.)*t0
            elseif(t0.ge.0.01) then
               be31=(1.-fexp(-t0))/t0
               dbedtau=((1.+t0)*fexp(-t0)-1.)/(t0*t0)
            endif
            dbe31dx1=dbedtau*dt31dx1
            dbe31dx3=dbedtau*dt31dx3
c
            t0=wl32u3*a32*s3*de2*(1.-s2*x3/(s3*x2))*time/(8.*pi*s2)
            t32=t0
            if (t0.le.0.) then
               t0=0.
               dt32dx2=0.
               dt32dx3=0.
            else
               dt32dx2=wl32u3*a32*s3*z*den*time/(8.*pi*s2)
               dt32dx3=-wl32u3*a32*z*den*time/(8.*pi)
            endif
            if(t0.lt.0.01) then
               be32=1.-0.5*t0+t0**2/6.
               dbedtau=-0.5+(2./6.)*t0
            elseif (t0.ge.0.01) then
               be32=(1.-fexp(-t0))/t0
               dbedtau=((1.+t0)*fexp(-t0)-1.)/(t0*t0)
            endif
            dbe32dx2=dbedtau*dt32dx2
            dbe32dx3=dbedtau*dt32dx3
c
c  calculate the matrix elements
c
c  new by ck 930211 :
c  -  level 1 is replaced by number conservation instead of level 3
c  -  matrixb is used instead of matrix
            do i=1,mmax
               b(i)=0.
               do j=1,mmax
                  a(i,j)=0.
               enddo
            enddo
            b(1)=-(1.-x1-x2-x3)
c            b(1)=-(x1*c12+x1*c13-x2*a21*be21-x2*c21-x3*a31*be31-x3*c31)
c            b(2)=-(x2*c23+x2*c21+x2*a21*be21-x1*c12-x3*c32-x3*a32*be32
c     &             x2*pe21+x2*pe23-x1*pe12-x3*pe32)
            b(2)=-(x2*c23+x2*c21-x1*c12-x3*c32
     &           +be21*(x2*a21+x2*pe21-x1*pe12)
     &           -be32*(x3*a32+x3*pe32-x2*pe23)
     &           -x1*ge2)
c            b(3)=-(x3*c31+x3*c32+x3*a31*be31+x3*a32*be32-x1*c13-x2*c23
c     &             x3*pe31+x3*pe32-x1*pe13-x2*pe23)
            b(3)=-(x3*c31+x3*c32-x1*c13-x2*c23
     &           +be31*(x3*a31+x3*pe31-x1*pe13)
     &           +be32*(x3*a32+x3*pe32-x2*pe23)
     &           -x1*ge3)
c            b(3)=-(1.-x1-x2-x3)
c
c            a(1,1)=c12+c13-x2*a21*dbe21dx1-x3*a31*dbe31dx1
c            a(1,2)=-a21*be21-x2*a21*dbe21dx2-c21
c            a(1,3)=-a31*be31-x3*a31*dbe31dx3-c31
            a(1,1)=-1.
            a(1,2)=-1.
            a(1,3)=-1.
            a(2,1)=dbe21dx1*(x2*a21+x2*pe21-x1*pe12)
     &           -c12-pe12*be21-ge2
c            a(2,2)=c23+c21+a21*be21+x2*a21*dbe21dx2-x3*a32*dbe32dx2+
c     &             pe21+pe23
            a(2,2)=c23+c21+be21*(a21+pe21)+be32*pe23
     &           +dbe21dx2*(x2*a21+x2*pe21-x1*pe12)
     &           -dbe32dx2*(x3*a32+x3*pe32-x2*pe23)
c            a(2,3)=-c32-a32*be32-x3*a32*dbe32dx3-pe32
            a(2,3)=-c32-be32*(a32+pe32)
     &           -dbe32dx3*(x3*a32+x3*pe32-x2*pe23)
c            a(3,1)=x3*a31*dbe31dx1-c13-pe13
            a(3,1)=dbe31dx1*(x3*a31+x3*pe31-x1*pe13)
     &           -c13-pe13*be31-ge3
c            a(3,2)=x3*a32*dbe32dx2-c23-pe23
            a(3,2)=dbe32dx2*(x3*a32+x3*pe32-x2*pe23)
     &           -c23-pe23*be32
c            a(3,3)=c31+c32+a31*be31+x3*a31*dbe31dx3+a32*be32+
c     &           x3*a32*dbe32dx3+pe31+pe32
            a(3,3)=c31+c32+be31*(a31+pe31)+be32*(a32+pe32)
     &           +dbe31dx3*(x3*a31+x3*pe31-x1*pe13)
     &           +dbe32dx3*(x3*a32+x3*pe32-x2*pe23)
c            a(3,1)=-1.
c            a(3,2)=-1.
c            a(3,3)=-1.
c
c            call matrix(3)
            call matrixb(3,3)
c
            if ((x1+b(1)).le.0.) b(1)=b(1)/2.
            if ((x2+b(2)).le.0.) b(2)=b(2)/2.
            if ((x3+b(3)).le.0.) b(3)=b(3)/2.
            x1=x1+b(1)
            x2=x2+b(2)
            x3=x3+b(3)
            dxmax=max(abs(b(1)/x1),abs(b(2)/x2),abs(b(3)/x3))
            if (dxmax.lt.tol) goto 9
c new by ck 921217
            if (abs(x2+x3).lt.1.d-25) then
               x2=0.
               x3=0.
               goto 9
            endif
         enddo
         if (it.ge.itmax) write(50,701) 'forb3nr : no conv!',wl21*1.e8,
     &        tday,z,id
 9       continue
         f21=elch*e21*be21*x2*a21/(xel*den)
         f31=elch*e31*be31*x3*a31/(xel*den)
         f32=elch*e32*be32*x3*a32/(xel*den)
         fp21=elch*e21*be21*x2*pe21/(xel*den)
         phextot=phextot+z*elch*e21*be21*(x1*pe12-x2*pe21)/(xel*den)
         phextc=phextc+z*elch*e21*be21*(x1*pe12c-x2*pe21c)/(xel*den)
         phxl21=z*elch*e21*be21*(x1*pe12l-x2*pe21l)/(xel*den)
         phextl=phextl+phxl21
c
         fp31=elch*e31*be31*x3*pe31/(xel*den)
         phextot=phextot+z*elch*e31*be31*(x1*pe13-x3*pe31)/(xel*den)
         phextc=phextc+z*elch*e31*be31*(x1*pe13c-x3*pe31c)/(xel*den)
         phxl31=z*elch*e31*be31*(x1*pe13l-x3*pe31l)/(xel*den)
         phextl=phextl+phxl31
c
         fp32=elch*e32*be32*x3*pe32/(xel*den)
         phextot=phextot+z*elch*e32*be32*(x2*pe23-x3*pe32)/(xel*den)
         phextc=phextc+z*elch*e32*be32*(x2*pe23c-x3*pe32c)/(xel*den)
         phxl32=z*elch*e32*be32*(x2*pe23l-x3*pe32l)/(xel*den)
         phextl=phextl+phxl32
c
         f=f21+f31+f32+fp21+fp31+fp32
         phed(j21)=phed(j21)+
     &        z*elch*e21*be21*(x1*pe12-x2*pe21)/(xel*den)
         phed(j31)=phed(j31)+
     &        z*elch*e31*be31*(x1*pe13-x3*pe31)/(xel*den)
         phed(j32)=phed(j32)+
     &        z*elch*e32*be32*(x2*pe23-x3*pe32)/(xel*den)
c
         cs21=(2.*hc/wl21u3)
         if (x2.le.1.e-40) then
            s21=0.d0
         else
            s21=cs21/((x1*s2)/(x2*s1)-1)
         endif
         cs31=(2.*hc/wl31u3)
         cs32=(2.*hc/wl32u3)
         if (x3.le.1.e-40) then
            s31=0.d0
            s32=0.d0
         else
            s31=cs31/((x1*s3)/(x3*s1)-1)
            s32=cs32/((x2*s3)/(x3*s2)-1)
         endif
      else
         f21=0.d0
         f31=0.d0
         f32=0.d0
         fp21=0.d0
         fp31=0.d0
         fp32=0.d0
         f=0.d0
         t21=0.d0
         t31=0.d0
         t32=0.d0
         s21=0.d0
         s31=0.d0
         s32=0.d0
         cs21=0.d0
         cs31=0.d0
         cs32=0.d0
         x1=0.d0
         x2=0.d0
         x3=0.d0
      endif
 701  format(a,2f10.2,1pe12.4,i5)
c
      return
      end



      subroutine wlforb(wlf)
      implicit real*8(a-h,o-z)
      dimension wlf(100),wlf1(100)
c     wavelengths of all forbidden lines
      data wlf1/5007.,2321.,4363.,3726.,2470.,7320.,6548.,3063.,
     &         5755.,6300.,2964.,5581.,3468.,5201.,10406.,0.,630000.,
     &         0.,1455600.,9818.,4619.,8729.,6718.,4071.,10320.,
     &         252460.,563060.,1297016.,684932.,16360.,10995.,
     &         3704000.,6091000.,3722.,6312.,9069.,1815.,3342.,
     &         3869.,1602.,2423.,4714.,1575.,2975.,3346.,     
     &         5*0.,11287.,7002.,9264.,6157.,46*0./
      do k=1,100
            wlf(k)=wlf1(k)
      enddo
      return
      end


      subroutine crphiml
      implicit real*8 (a-h,o-z)
c
c  This routine calculates the photoionization cross sections
c  for the different levels in h i (sigh),he i (sighe), o i (sigo), 
c  and ca ii (sigca), fe i (sigfe1), fe ii (sigfe2)
c
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      common/ctpar/tday,dts,iss,iss1,itd
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
      common/cfedat/cf1(nlf1,nlf1),cf2(nlf2,nlf2),cf3(nlf3,nlf3),
     &     cf4(nlf4,nlf4),cif1(nlf1),cif2(nlf2),cif3(nlf3),cif4(nlf4),
     &     gf1(nlf1),gf2(nlf2),gf3(nlf3),gf4(nlf4),ef1(nlf1),ef2(nlf2),
     &     ef3(nlf3),ef4(nlf4),af1(nlf1,nlf1),af2(nlf2,nlf2),
     &     af3(nlf3,nlf3),af4(nlf4,nlf4),cpf2(3,3)
      common/cionpot/e00(mie,mion)
      common/cphsig/sigo(nl,ne),sigca(nl,ne),sigh(nmax,ne),
     &     sigfe1(nlf1,ne),sigfe2(nlf2,ne),sighe(nl,ne)
      common/cebin/e1(ne),en(ne),jmax
      common/cfemul/ife
      common/cgf/G0p(nmx),G0m(nmx),initgf
      common/cphionfl/iflphi
      dimension sig(nl),ga(nl)
c
c     He I cross sections from D. Koester et al. A&A 149, 423 (1985)
c
      dimension a0(10), a1(10), a2(10), wl0(12), gi(12)
      save a0,a1,a2,wl0,gi
      data a0/ -58.229,  -67.310,    -68.438,   -68.936,    -92.020,
     &         -63.778,  -63.408,    -61.027,   -83.287,    -76.903/
      data a1/  4.3965,   6.1831,     5.7453,    5.2666,     10.313,
     &          4.5102,   3.8797,     3.1833,	 7.1751,     6.3639/
      data a2/-0.22134, -0.32244,   -0.26277,  -0.15812,   -0.45090,
     &        -0.18213, -0.12479,  -0.043675,  -0.20821,   -0.21565/
      data wl0/0.00, 159850.318, 166271.70, 169081.189, 171129.148,
     &                183231.08, 185559.085, 186095.90, 190292.46,
     &                191211.42, 191438.83,  191446.61/
      data gi/1, 3, 1, 9, 3, 3, 9, 15, 3, 9, 15, 21/
c
cqcq run 97; run 80.6; run 80.7; run 96; run 77; run 70.8; run 70.9
cqcq run 99; run 80.8 >= run 100
      if (iflphi.eq.0) then
         do j1=1,ne
            do il=1,nmax
               sigh(il,j1)=0.d0
            enddo
            do il=1,nl
               sighe(il,j1)=0.d0
               sigo(il,j1)=0.d0
               sigca(il,j1)=0.d0
            enddo
            do il=1,nlf1
               sigfe1(il,j1)=0.d0
            enddo
            do il=1,nlf2
               sigfe2(il,j1)=0.d0
            enddo
         enddo
         return
      endif
c
c  h i , ie=1, ion=1
chc The expression for the photoionization cross section for i>3
chc can be found in Seaton, 1959, MNRAS, Vol. 119, p. 81.
      ie=1
      ion=1
ch      n=nlev(ie,ion)
c
      e00l=e00(ie,ion)
      sig(1)=7.91
      sig(2)=15.
      ga(1)=2.99
      ga(2)=2.5
      do i=3,nm
        sig(i)=7.91*real(i)
        ga(i)=3.
      enddo
c
ch      do i=1,nm
ch        eth=e00l-ehn(i)
ch        do j1=1,jmax-1
ch          sigh(i,j1)=0.0e0
ch          if(e1(j1).ge.eth) then
ch            if(i.eq.2) then
ch              sigh(2,j1)=15.82e-18*(1.+.3456*(6.80/e1(j1))**(2./3.)*
ch     &                  (e1(j1)/6.80-1.))*(3.40/e1(j1))**3.
ch            elseif(i.eq.1) then
ch              if(e1(j1).gt.13.6) then
ch                ps=dsqrt((e1(j1)-13.6)/13.6)
ch                fe=fexp(-4.d0*datan(ps)/ps)/(1.-fexp(-6.283d0/ps))
ch                sigh(1,j1)=3.402e-16*(13.6/e1(j1))**4.*fe
ch              endif
ch            else
ch              gaunt=gbf(i,e1(j1))
ch              sigh(i,j1)=1.e-18*sig(i)*gaunt*(eth/e1(j1))**ga(i)
ch            endif
ch          endif
ch        enddo
ch      enddo
c
      do j1=1,jmax-1
         ej=e1(j1)
         initgf=1
c  n = 1
         if(e1(j1).gt.13.6) then
            ps=dsqrt((e1(j1)-13.6)/13.6)
            fe=fexp(-4.d0*datan(ps)/ps)/(1.-fexp(-6.283d0/ps))
            sigh(1,j1)=3.402e-16*(13.6/e1(j1))**4.*fe
         endif
c  n = 2
         eth=e00l-ehn(2)
         if (e1(j1).ge.eth) then
            sigh2=15.82e-18*(1.+.3456*(6.80/e1(j1))**(2./3.)*
     &           (e1(j1)/6.80-1.))*(3.40/e1(j1))**3.
         endif
c
         do n=2,nn
c
c  Photoionization cross sections for individual nl-states,
c  for n>2, are calculated using the expressions in 
c  Brockelhurst, 1971, MNRAS, 152, 471.
c
            xn=dble(n)
            xk2=(ej/e00l)-(1./(xn*xn))
c
c photon energy  > ionization potential
c
            if (xk2.ge.0.) then
               xk=sqrt(xk2)
               do l=0,n-1
                  xl=dble(l)
                  il=nlm(n,l)
                  thsum=0.d0
                  lp=l-1
                  xlp=dble(lp)
                  if (lp.ge.0.and.lp.le.(n-1)) then
                     gfk=gf(n,l,xk,lp)
                     thetak=(1.+xn*xn*xk2)*gfk*gfk
                     thsum=thsum+xl*thetak
                  endif
c
                  lp=l+1
                  xlp=dble(lp)
                  if (lp.ge.0.and.lp.le.(n-1)) then
                     gfk=gf(n,l,xk,lp)
                     thetak=(1.+xn*xn*xk2)*gfk*gfk
                     thsum=thsum+xlp*thetak
                  endif
c
                  sigh(il,j1)=8.56e-19*xn*xn*thsum/(2.*xl+1.)
               enddo
            else
               do l=0,n-1
                  il=nlm(n,l)
                  sigh(il,j1)=0.d0                  
               enddo
            endif
         enddo
c
c  for the n=2 level assume that the total cross section equals sigh2
c  but that they are diveded between 2s and 2p according to Brockelhurst.
c
         eth=e00l-ehn(2)
         if (e1(j1).ge.eth) then
            sum2=sigh(2,j1)+sigh(3,j1)
            sigh(2,j1)=sigh(2,j1)*sigh2/sum2
            sigh(3,j1)=sigh(3,j1)*sigh2/sum2
         endif
c
c  nn > 2 ! otherwise changes are needed
         do n=nn+1,nm
c
c  The expression for the photoionization cross section for i>3,
c  n-states, can be found in Seaton, 1959, MNRAS, Vol. 119, p. 81.
c
            il=nnm(n)
            sigh(il,j1)=0.0e0
            eth=e00l-ehn(n)
            if(e1(j1).ge.eth) then
               gaunt=gbf(n,e1(j1))
               sigh(il,j1)=1.e-18*sig(n)*gaunt*(eth/e1(j1))**ga(n)
            endif
         enddo
      enddo
cqcq
c          sigh(i,j1)=0.0e0
cqcq
c
c  he i , ie=2, ion=1
      ie=2
      ion=1
      n=nlev(ie,ion)
      do i=1,n
         do j1=1,jmax
            sighe(i,j1)=0.d0
         enddo
      enddo
cqcq
c      goto 992
cqcq
      do j1=1,jmax-1
        el=e1(j1)
c  He ground state cross section in terms of H cross section
c  960215 new data from Verner et al 1996 for the ground state
c        if (el.gt.24.6 .and. el.lt.65.4) then
c           sighe(1,j1) = sigh(1,j1)*(6.53*(el/24.6d0)-0.22d0)
c        elseif (el.gt.65.4d0) then
c           sighe(1,j1) = sigh(1,j1)*(37.d0-19.1d0*(65.4d0/el)**0.76d0)
c        endif
        sighe(1,j1)=0.d0
        if (el.gt.e00(iehe,1)) then
           call vernercr(el,1.361d1,9.492d2,1.469d0,3.188d0,
     &          2.039d0,4.434d-1,2.136d0,sigf)
           sighe(1,j1)=sigf*1.e-18               
        endif
c
        do j=2,8
           wl=12398.54/e1(j1)
           sighe(j,j1) = 0.
           wlion = 198310. - wl0(j)
           if(wl.lt.1.e8/wlion) then
              if(j.le.6) then
c                                        6 is really 3 1S
                 jk=j
              elseif(j.eq.7.or.j.eq.8) then
                 jk=j+1
              endif
              wllog=log(wl)
              sigl = a0(jk)+(a1(jk)+a2(jk)*wllog)*wllog
              sighe(j,j1) = exp(sigl)/gi(j)
           endif
        enddo
c
c ad hoc !
c        do j=9,n
c           sighe(j,j1)=sighe(8,j1)
c        enddo
      enddo
cqcq
c 992  continue
cqcq
c
c  o i , ie=5, ion=1
      ie=5
      ion=1
      n=nlev(ie,ion)
      do j1=1,jmax-1
         el=e1(j1)
c
c   o i 
c  960215 new data from Verner et al 1996 for the ground state
c
         sigo(1,j1)=0.0e0
         if (el.gt.e00(ieo,1)) then
            call vernercr(el,1.240d0,1.745d3,3.784d0,1.764d1, 
     &           7.589d-2,8.698d0,1.271d-1,sigf)
            sigo(1,j1)=sigf*1.e-18                           
            sigo(2,j1)=sigf*1.e-18                           
            sigo(3,j1)=sigf*1.e-18                           
         endif
         sigo(4,j1)=0.
         sigo(5,j1)=0.                           
         do j=6,13
            s=0.
c
c     crossections of o i from dawies and lewis in 1.e-18 cm
c
c            if(j.eq.1) ery=(el-13.618)/13.618
            if(j.eq.6) ery=(el-4.481)/13.618
            if(j.eq.7) ery=(el-4.110)/13.618
            if(j.eq.8) ery=(el-2.886)/13.618
            if(j.eq.9) ery=(el-2.642)/13.618
            if (j.eq.10) ery=-1 
            if (j.eq.11) ery=-1 
            if (j.eq.12) ery=-1 
            if(j.eq.13) ery=(el-1.545)/13.618
c            if(j.eq.9) ery=(el-1.095)/13.618
c
            if(ery.gt.0.) then
               if(j.eq.6.and.ery.lt..3488) sl=(-1.034*ery+1.365-
     &              .0491/ery)-2.
               if(j.eq.6.and.ery.gt..3488) sl=-0.5481-0.6746*(ery/.4)
               if(j.eq.7.and.ery.lt..5) 
     &              sl=(-.4038*ery+1.783-.0449/ery)-2.
               if(j.eq.7.and.ery.gt..5) sl=-.483*ery-.2672
               s=10.**sl
               if(j.eq.8) s=10.**(-7.204*ery+1.657-1.)+
     &              10.**(-.3748*ery-.5877)
               if(j.eq.9.and.ery.gt.0.25) s=10.**(-.54*ery-.1715)
               if(j.eq.9.and.ery.lt.0.25) s=10.**(-.54*0.25-.1715)
               if(j.eq.9.and.ery.lt.0.12) s=10.**(-6.*ery+.28)
               if(j.eq.13) s=20.5*(1.545/el)**3.56
c               if(j.eq.9) s=20.5*(1.545/el)**3.56
c               if(j.eq.1) then
c                  s=9.05*(4.378*(13.618/el)**1.5-3.378*(13.618/el)**2.5)
c                  s=2.94*(2.66*(13.618/el)**1.-1.66*(13.618/el)**2.)
c                  if(el.gt.100.) 
c     &                 s=1.778*(3.65*(100./el)**3-2.65*(100./el)**4)
c               endif
            endif
c     
            sigo(j,j1)=s*1.e-18
c960616
c
c   o i (mcalpine corrected to agree with reilman and manson at
c        high energies).
c  960215 new data from Verner et al 1996 for the ground state
c
c960616         sigo(1,j1)=0.0e0
c        if (el.gt.13.6.and.el.lt.100.) then
c          sigo(1,j1)=9.05e-18*(4.378*(13.6/el)**1.5-3.378*
c     &(13.6/el)**2.5)
c        elseif (el.gt.100.) then
c          si0=1.778
c          al=3.65
c          s=3.
c          e0=100.
c          sigo(1,j1)=si0*1.e-18*(al*(e0/el)**s+(1.-al)*(e0/el)**(s+1.))
c        endif
c960616         if (el.gt.e00(ieo,1)) then
c960616            call vernercr(el,1.240d0,1.745d3,3.784d0,1.764d1, 
c960616     &           7.589d-2,8.698d0,1.271d-1,sigf)
c960616            sigo(1,j1)=sigf*1.e-18                           
c960616         endif
cqcq
c        sigo(1,j1)=0.0e0
cqcq
cxx? j = 2,3 ??
c960616        do j=4,9
c960616          s=0.
c
c     crossections of o i from dawies and lewis in 1.e-18 cm
c
c960616          if(j.eq.1) ery=(el-13.618)/13.618
c960616          if(j.eq.4) ery=(el-4.481)/13.618
c960616          if(j.eq.5) ery=(el-4.110)/13.618
c960616          if(j.eq.6) ery=(el-2.886)/13.618
c960616          if(j.eq.7) ery=(el-2.642)/13.618
c960616          if(j.eq.8) ery=(el-1.545)/13.618
c960616          if(j.eq.9) ery=(el-1.095)/13.618
c
c960616          if(ery.gt.0.) then
c960616            if(j.eq.4.and.ery.lt..3488) sl=(-1.034*ery+1.365-
c960616     &.0491/ery)-2.
c960616            if(j.eq.4.and.ery.gt..3488) sl=-0.5481-0.6746*(ery/.4)
c960616            if(j.eq.5.and.ery.lt..5) sl=(-.4038*ery+1.783-.0449/ery)-2.
c960616            if(j.eq.5.and.ery.gt..5) sl=-.483*ery-.2672
c960616            s=10.**sl
c960616            if(j.eq.6) s=10.**(-7.204*ery+1.657-1.)+
c960616     &10.**(-.3748*ery-.5877)
c960616            if(j.eq.7.and.ery.gt.0.25) s=10.**(-.54*ery-.1715)
c960616            if(j.eq.7.and.ery.lt.0.25) s=10.**(-.54*0.25-.1715)
c960616            if(j.eq.7.and.ery.lt.0.12) s=10.**(-6.*ery+.28)
c960616            if(j.eq.8) s=20.5*(1.545/el)**3.56
c960616            if(j.eq.9) s=20.5*(1.545/el)**3.56
c960616            if(j.eq.1) then
c960616              s=9.05*(4.378*(13.618/el)**1.5-3.378*(13.618/el)**2.5)
c960616              s=2.94*(2.66*(13.618/el)**1.-1.66*(13.618/el)**2.)
c960616              if(el.gt.100.) 
c960616     &             s=1.778*(3.65*(100./el)**3-2.65*(100./el)**4)
c960616            endif
c960616          endif
c
c960616          sigo(j,j1)=s*1.e-18
cqcq
c          sigo(j,j1)=0.
cqcq
        enddo
c        do j=1,13
c           sigo(j,j1)=0.
c        enddo
      enddo
c  ca ii , ie=12, ion=2
c  960215 new data from Verner et al 1996 for the ground state
      ie=12
      ion=2
      n=nlev(ie,ion)
c
      e01=e00(ie,ion)-e(ie,ion,1)
      e02=e00(ie,ion)-e(ie,ion,2)
      e03=e00(ie,ion)-e(ie,ion,3)
      e04=e00(ie,ion)-e(ie,ion,4)
      e05=e00(ie,ion)-e(ie,ion,5)
      e06=e00(ie,ion)-e(ie,ion,6)
      do j1=1,jmax-1
         el=e1(j1)
c level 1
c        if (el.ge.e01) then
c     approx. fit to reilman and manson
c          if(el.lt.35.) s=0.20*(e01/el)**0.0001
c          if(el.ge.35..and.el.lt.45.) s=0.10*(35./el)**1.
c          if(el.ge.45..and.el.lt.160.) s=1.5
c          if(el.ge.160..and.el.lt.390.) s=1.15*(160./el)**1.66
c          if(el.ge.390..and.el.lt.4000.) s=2.5*(390./el)**2.18
c          if(el.gt.4000.) s=6.85e-2*(4000./el)**2.43
c        else
c          s=0.
c        endif
c        sigca(1,j1)=1.e-18*s
         sigca(1,j1)=0.d0
         if (el.gt.e00(ieca,2)) then
            call vernercr(el,1.553d1,1.064d7,7.790d-1,2.130d1,
     &           6.453d-1,2.161d-3,6.706d-2,sigf)
            sigca(1,j1)=sigf*1.e-18               
         endif

cqcq
c        sigca(1,j1)=0.
cqcq
c level 2
        if (el.ge.e02) then
          s=6.15*(e02/el)**1.
        else
          s=0.
        endif
        sigca(2,j1)=1.e-18*s
cqcq
c        sigca(2,j1)=0.
cqcq
c level 3
        if (el.ge.e03) then
          s=6.15*(e03/el)**1.
        else
          s=0.
        endif
        sigca(3,j1)=1.e-18*s
cqcq
c        sigca(3,j1)=0.
cqcq
c level 4
        if (el.ge.e04) then
          s=2.38*(e04/el)**3.65
        else
          s=0.
        endif
        sigca(4,j1)=1.e-18*s
cqcq
c        sigca(4,j1)=0.
cqcq
c level 5
        if (el.ge.e05) then
          s=2.38*(e05/el)**3.65
        else
          s=0.
        endif
        sigca(5,j1)=1.e-18*s
cqcq
c        sigca(5,j1)=0.
cqcq
c level 6
c        if (el.ge.e06) then
c          s=2.38*(e06/el)**3.65
c        else
c          s=0.
c        endif
c        sigca(6,j1)=1.e-18*s
c
        sigca(6,j1)=0.
c
      enddo
c
c  For Fe I and Fe II :
c
c  Total photoionization cross sections for iron from 
c  Reilman and Manson, 1979, ApJS, 40, 815. OLD.
c
c  NEW data 960211 from Verner, Ferland, Korista, Yakolev
c  1996, in press 'Atomic Data for Astrohysics. II New Analytic Fits 
c  for Photoionization Cross Sections of Atoms and Ions.'
c  The fits and the treatment of excited levels taken
c  from J.H. 930126.
c
cfex crphiml. Better photo ion. cross sections for different 
cfex         levels needed for fe i and fe ii. Just guessed now.
c
c  fe i , ie=iefe, ion=1
c
c  ife=0 : multilev atom for Fe I and Fe II, ion.balance + multinc
c  ife=1 : multilev atom for Fe I and Fe II, multilev
      if (ife.le.1) then
         ie=iefe
         ion=1
         n=nlev(ie,ion)
         nlgs=5
c         
         do j1=1,jmax-1
            el=e1(j1)
            sigf = 0.d0
crm            if (el.gt.750.d0) then
crm               sigf = 1.46d-18*(750.d0/el)**2.09
crm            else if (el.gt.70.d0) then
crm               sigf = 8.11d-18*(70.d0/el)**1.6
crm            else if (el.gt.35.d0) then
crm               sigf = 8.d-18
crm            else if (el.gt.15.d0) then
crm               sigf = 3.288d-18*(el/15.d0)**1.1
crm            else if (el.gt.7.87d0) then
crm               sigf = 0.836d-18*(7.87d0/el)**9.945
crm            else if (el.le.e00(iefe,ion)) then
crm               sigf = 0.d0
crm            endif
            if (el.ge.e00(iefe,ion)) then
               call vernercr(el,5.461d-2,3.062d-1,2.671d7,7.923d0,
     &              2.069d1,1.382d2,2.481d-1,sigf)
               sigf=sigf*1.e-18
            endif
cqcq run 16.4 ; run 40.1; run 41.6; run 60.6
c            sigf=0.
cqcq
cqcq
c            if (j1.lt.59.or.j1.gt.60) then
c            if (j1.lt.59) then
c            if (j1.ge.59) then
c            if (j1.ne.36) then
c               sigf = 0.d0
c            endif
c            if (j1.eq.32.or.j1.eq.36.or.j1.eq.43.or.j1.eq.50) then
c            if (j1.eq.32.or.j1.eq.43.or.j1.eq.50) then
c               sigf=0.d0
c            endif
cqcq
            iphfex=1
c            iphfex=0
            if (iphfex.eq.0.or.sigf.le.0.) then
c  no photoion from exc. levels
               sigfe1(1,j1)=sigf
               do il=2,n
                  sigfe1(il,j1)=0.
               enddo
            elseif (iphfex.eq.1.and.sigf.gt.0.) then
cqcq
c               sume=0.
c               do il=2,n
c                  sume=sume+ef1(il)
c               enddo
c               sume=sume/0.9
c
c               sigfe1(1,j1)=0.1*sigf
c               sigsum=sigfe1(1,j1)
c               do il=2,n
c                  sigfe1(il,j1)=(ef1(il)/sume)*sigf
c                  sigsum=sigsum+sigfe1(il,j1)
c               enddo
               do il=1,nlgs
cqcq                  sigfe1(il,j1)=sigf/(dble(nlgs))
                  sigfe1(il,j1)=sigf
               enddo
               do il=nlgs+1,n
                  sigfe1(il,j1)=0.
               enddo
            endif
         enddo
      endif
c
c  fe ii , ie=iefe, ion=2
c
c  ife=0 : multilev atom for Fe I and Fe II, ion.balance + multinc
c  ife=1 : multilev atom for Fe I and Fe II, multilev
      if (ife.le.1) then
         ie=iefe
         ion=2
         n=nlev(ie,ion)
         nlgs=5
c
         do j1=1,jmax-1
            el=e1(j1)
            sigf = 0.d0
crm            if (el.gt.750.d0) then
crm               sigf = 1.462d-18*(750.d0/el)**1.911
crm            else if (el.gt.45.d0) then
crm               sigf = 8.896d-18*(45.d0/el)**1.378
crm            else if (el.gt.25.d0) then
crm               sigf = 6.095d-18*(el/25.d0)**.643
crm            else if (el.gt.16.16d0) then
crm              sigf = 0.833d-18*(16.16d0/el)**9.945
crm            else if (el.le.e00(iefe,ion)) then
crm               sigf = 0.d0
crm            endif
            if (el.ge.e00(iefe,ion)) then
               call vernercr(el,1.761d-1,4.365d3,6.298d3,5.204d0,
     &              1.141d1,9.272d1,1.075d2,sigf)
               sigf=sigf*1.e-18
            endif
cqcq run 16.4
c            sigf = 0.d0
cqcq
c            iphfex=0
            iphfex=1
            if (iphfex.eq.0.or.sigf.le.0.) then
c  no photoion from exc. levels
               sigfe2(1,j1)=sigf
               do il=2,n
                  sigfe2(il,j1)=0.
               enddo
            elseif (iphfex.eq.1.and.sigf.gt.0.) then
cqcq
c               sume=0.
c               do il=2,n
c                  sume=sume+ef2(il)
c               enddo
c               sume=sume/0.9
c
c               sigfe2(1,j1)=0.1*sigf
c               sigsum=sigfe2(1,j1)
c               do il=2,n
c                  sigfe2(il,j1)=(ef2(il)/sume)*sigf
c                  sigsum=sigsum+sigfe2(il,j1)
c               enddo
               do il=1,nlgs
cqcq                  sigfe2(il,j1)=sigf/(dble(nlgs))
                  sigfe2(il,j1)=sigf
               enddo
               do il=nlgs+1,n
                  sigfe2(il,j1)=0.
               enddo
            endif
         enddo
      endif
c
      return
      end


      subroutine vernercr(e,e0,sig0,ya,p,yw,y0,y1,sig)
      implicit real*8 (a-h,o-z)
c
c  960211
c  Fitting formula for photoionization cross section according to
c  Verner, Ferland, Korista, Yakolev
c  1996, ApJ, 465, 487'Atomic Data for Astrohysics. II New Analytic Fits 
c  for Photoionization Cross Sections of Atoms and Ions.'
c  Resulting cross section (sig) in units of 1.e-18 cm^2
c  e0 in eV.
c
c  The photoionization cross sections have been tested in
c  /milla_1/cecilia/ATDAT/PHOTOION/phcross.f
c
      x=(e/e0)-y0
      y=sqrt(x*x+y1*y1)
      q=0.5*p-5.5
c  missprint in preprint should be x instead for y; then
c  the data agrees with their routine phfit1.f
      fy=((x-1)*(x-1)+yw*yw)*(y**q)*(1.+sqrt(y/ya))**(-p)
      sig=sig0*fy
c
      return
      end



      subroutine ratepiml
      implicit real*8 (a-h,o-z) 
c
c calculate photoionization from excited states of multilevel atoms ; 
c ph(ie,ion,il) 
c
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
      common/cionpot/e00(mie,mion)
      common/cfedat/cf1(nlf1,nlf1),cf2(nlf2,nlf2),cf3(nlf3,nlf3),
     &     cf4(nlf4,nlf4),cif1(nlf1),cif2(nlf2),cif3(nlf3),cif4(nlf4),
     &     gf1(nlf1),gf2(nlf2),gf3(nlf3),gf4(nlf4),ef1(nlf1),ef2(nlf2),
     &     ef3(nlf3),ef4(nlf4),af1(nlf1,nlf1),af2(nlf2,nlf2),
     &     af3(nlf3,nlf3),af4(nlf4,nlf4),cpf2(3,3)
      common/crecph/reco(mie,mion,nl),ph(mie,mion,nl),gh(mie,mion,nl),
     &     recof1(nlf1),recof2(nlf2),recof3(nlf3),recof4(nlf4),
     &     phh1(nmax),ghh1(nmax),
     &     phf1(nlf1),phf2(nlf2),phf3(nlf3),phf4(nlf4),
     &     ghf1(nlf1),ghf2(nlf2),ghf3(nlf3),ghf4(nlf4),
     &     recofr(mie,mion),phto(mie,mion),alsi,pennc
      common/cphsig/sigo(nl,ne),sigca(nl,ne),sigh(nmax,ne),
     &     sigfe1(nlf1,ne),sigfe2(nlf2,ne),sighe(nl,ne)
      common/cebin/e1(ne),en(ne),jmax
      common/opt/tau(ne),opac(ne),opi(mie,mion,ne),
     &     opikh(nmax,ne),opikhe(nl,ne),opiko(nl,ne),opikca(nl,ne),
     &     opikfe1(nlf1,ne),opikfe2(nlf2,ne)
      common/cfj/fj(ne),fjidm1(ne),fjc(0:ne),fjcidm1(ne)
c      common/cfjx/fj(ne),fjex(7,ne),tauex(7,ne)
      common/linephot/phj(ne)
      common/test/phottest,exphtest
c  only for testing
      common/cmlevpop/xhi(nmax),xhei(nl),xoi(nl),xmg1(nl),xsi1(nl),
     &     xcaii(nl),xfei(nlf1),xfeii(nlf2),xfeiii(nlf3),xfeiv(nlf4)
      common/cden/den
      common/cabun/amean,ab(mie),xa(mie,mion),xaold(mid,mie,mion),
     &     nion(mie)
      common/cfemul/ife
      common/cmod/abmod(10,mie),vrmod(mid),rmod(mid),rlab(mid),
     &     dmmod(mid),ameanmod(10),denmod(mid),tdmod,
     &     nrtab(mid),izmod(mid),nrmod,mixm
      common/depthind/id
      data pi/3.1415926e0/,elch/1.60219e-12/
c
c
c  h i ; ie=1, ion=1
      ie=1
      ion=1
      n=nlev(ie,ion)
      do i=1,n
         e0=e00(ie,ion)-ehn(nln(i))
         za=0.
         ga=0.
         do j1=1,jmax-1
            if (e1(j1).ge.e0) then
               sigm=sigh(i,j1)
               dz=exphtest*4.*pi*fj(j1)*sigm*(en(j1+1)-en(j1))
               if(opac(j1).gt.0.) then
                  dz=dz + (opikh(i,j1)/opac(j1))*
     &                 phj(j1)*elch*e1(j1)
               endif
               zc=dz/(elch*e1(j1))
               gc=dz*(1. - e0/e1(j1))
               za=za+zc
               ga=ga+gc
               if(i.eq.2) then
 9466             format(' bal ',2i4,1pe12.4,5e12.4)
c            write(8,9466)i,j1,fj(j1),dz,
c     &           dz*xhi(2)*ab(1)/den,za,ga
               endif
            endif          
         enddo
         phh1(i)=za
         ghh1(i)=ga
ch         ph(ie,ion,i)=za
ch         gh(ie,ion,i)=ga
      enddo
c  he i ; ie=2, ion=1
      ie=2
      ion=1
      n=nlev(ie,ion)
      do i=1,n
        e0=e00(ie,ion)-e(ie,ion,i)
        za=0.
        ga=0.
        do j1=1,jmax-1
           if (e1(j1).ge.e0) then
              sigm=sighe(i,j1)
              dz=exphtest*4.*pi*fj(j1)*sigm*(en(j1+1)-en(j1))
              if(opac(j1).gt.0.) then
                 dz=dz+(opikhe(i,j1)/opac(j1))*
     &                phj(j1)*elch*e1(j1)
              endif
              zc=dz/(elch*e1(j1))
              gc=dz*(1. - e0/e1(j1))
              za=za+zc
              ga=ga+gc
           endif          
        enddo
        ph(ie,ion,i)=za
        gh(ie,ion,i)=ga
      enddo
c  o i ; ie=5, ion=1
      ie=5
      ion=1
      n=nlev(ie,ion)
      do i=1,n
        e0=e00(ie,ion)-e(ie,ion,i)
        za=0.
        ga=0.
        do j1=1,jmax-1
         if (e1(j1).gt.e0) then
            sigm=sigo(i,j1)
            dz=exphtest*4.*pi*fj(j1)*sigm*(en(j1+1)-en(j1))
            if(opac(j1).gt.0.) then
              dz=dz + (opiko(i,j1)/opac(j1))*phj(j1)*elch*e1(j1)
            endif
            zc=dz/(elch*e1(j1))
            gc=dz*(1. - e0/e1(j1))
            za=za+zc
            ga=ga+gc
          endif          
        enddo
        ph(ie,ion,i)=za
        gh(ie,ion,i)=ga
      enddo
cqcq
c  ots for o i except in the hydrogen zone
c      iz=izmod(id)
c  for 87a
c      iz0=6
c      if (iz.ne.iz0) then
c         gh(ie,ion,1)=0.
c         ph(ie,ion,1)=0.
c      endif
cqcq
c  ca ii ; ie=12, ion=2
      ie=12
      ion=2
      n=nlev(ie,ion)
      do i=1,n
        e0=e00(ie,ion)-e(ie,ion,i)
        za=0.
        ga=0.
        do j1=1,jmax-1
          if (e1(j1).ge.e0) then
            sigm=sigca(i,j1)
            dz=exphtest*4.*pi*fj(j1)*sigm*(en(j1+1)-en(j1))
            if(opac(j1).gt.0.) then
              dz=dz + (opikca(i,j1)/opac(j1))*phj(j1)*elch*e1(j1)
            endif
            zc=dz/(elch*e1(j1))
            gc=dz*(1. - e0/e1(j1))
            za=za+zc
            ga=ga+gc
          endif          
        enddo
        ph(ie,ion,i)=za
        gh(ie,ion,i)=ga
      enddo
c  fe i ; ie=iefe, ion=1
      if (ife.le.1) then
         ie=iefe
         ion=1
         n=nlev(ie,ion)
         do i=1,n
            e0=e00(ie,ion)-ef1(i)
            za=0.
            ga=0.
            do j1=1,jmax-1
               if (e1(j1).ge.e0) then
                  sigm=sigfe1(i,j1)
                  dz=exphtest*4.*pi*fj(j1)*sigm*(en(j1+1)-en(j1))
                  if(opac(j1).gt.0.) then
                     dz=dz + (opikfe1(i,j1)/opac(j1))*phj(j1)*
     &                    elch*e1(j1)
                  endif
                  zc=dz/(elch*e1(j1))
                  gc=dz*(1. - e0/e1(j1))
                  za=za+zc
                  ga=ga+gc
               endif          
            enddo
            phf1(i)=za
            ghf1(i)=ga
         enddo
      endif
cqcq
      phfsum1=0.
      do i=1,n
         phfsum1=phfsum1+phf1(i)
      enddo
cqcq
c
c  fe ii ; ie=iefe, ion=2
c
      if (ife.le.1) then
         ie=iefe
         ion=2
         n=nlev(ie,ion)
         do i=1,n
            e0=e00(ie,ion)-ef2(i)
            za=0.
            ga=0.
            do j1=1,jmax-1
               if (e1(j1).ge.e0) then
                  sigm=sigfe2(i,j1)
                  dz=exphtest*4.*pi*fj(j1)*sigm*(en(j1+1)-en(j1))
                  if(opac(j1).gt.0.) then
                     dz=dz + (opikfe2(i,j1)/opac(j1))*phj(j1)*
     &                    elch*e1(j1)
                  endif
                  zc=dz/(elch*e1(j1))
                  gc=dz*(1. - e0/e1(j1))
                  za=za+zc
                  ga=ga+gc
               endif          
            enddo
            phf2(i)=za
            ghf2(i)=ga
         enddo
      endif
cqcq
      phfsum2=0.
      do i=1,n
         phfsum2=phfsum2+phf2(i)
      enddo
cqcq
c
      return
      end


      subroutine colldat(te,xe)
      implicit real*8 (a-h,o-z)
c
c  this routine calculates the collisional excitation (c) and
c  ionization rates (ci) for h i, o i and ca ii
c
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
      common/ccollh/cnl(nmax,nmax),clep(nm,0:nm),clem(nm,0:nm),
     &     clpp(nm,0:nm),clpm(nm,0:nm)
      common/cfedat/cf1(nlf1,nlf1),cf2(nlf2,nlf2),cf3(nlf3,nlf3),
     &     cf4(nlf4,nlf4),cif1(nlf1),cif2(nlf2),cif3(nlf3),cif4(nlf4),
     &     gf1(nlf1),gf2(nlf2),gf3(nlf3),gf4(nlf4),ef1(nlf1),ef2(nlf2),
     &     ef3(nlf3),ef4(nlf4),af1(nlf1,nlf1),af2(nlf2,nlf2),
     &     af3(nlf3,nlf3),af4(nlf4,nlf4),cpf2(3,3)
      common/cionpot/e00(mie,mion)
      common/heirecion/al0(16),be(16),b0(16),ga(16)
      common/heicex/oms(nl,nl)
      common/cfecoll/omfe1(nlf1,nlf1),omfe2(nlf2,nlf2),
     &     omfe3(nlf3,nlf3),omfe4(nlf4,nlf4)
      common/cfemul/ife
      common/cabun/amean,ab(mie),xa(mie,mion),xaold(mid,mie,mion),
     &     nion(mie)
      common/comoi/aoi(nl,nl),boi(nl,nl)
      common/cmg1/ommg1(10,10),alfmg1(10,10),
     &     wlmg1(10,10),jlmg1(10,10),klvmg1(10,10)
      common/csi1/omsi1(21,21),alfsi1(21,21),
     &     wlsi1(21,21),jlsi1(21,21),klvsi1(21,21)
      dimension cil(nl),cl(nl,nl)
      dimension cle(0:nm,0:nm),clp(0:nm,0:nm)
c data for ca ii
      data om1t23/0./,om1t45/0./,om16/0./,om23/21.1d0/
      data om23t45/0./,om23t6/0./,om45/9.85d0/,om45t6/0./
      data al1t23/0./,al1t45/0./,al16/0./,al23/0.01d0/
      data al23t45/0./,al23t6/0./,al45t6/0./,al45/-0.01d0/
c
      tev=te/1.1609d4
      te5000=te/5000
      t4=te/1.e4
      tesqrt=dsqrt(te)
c
c  h i ; ie=1, ion=1
c
      ie=1
      ion=1
      n=nlev(ie,ion)
c
      do i=1,nmax
         do j=1,nmax
            cnl(i,j)=0.d0
         enddo
      enddo
c
c
c ***  collisions between l-states for given n ***
c
c collisions with electrons : ic=1
c collisions with protons : ic=2
c
      do i=1,nm
         do j=0,nm
            clep(i,j)=0.d0
            clem(i,j)=0.d0
            clpp(i,j)=0.d0
            clpm(i,j)=0.d0
         enddo
      enddo
      if (ab(ieh).gt.0.) then
c
         do n=2,nn
            ic=1
            call coll_l(ic,n,te,xe,cle)
            ic=2
            call coll_l(ic,n,te,xe,clp)
            do l=0,n-1
c            anl=0.d0
c            nl=nlm(n,l)
c            do np=1,n-1
c               do lp=0,np-1
c                  nplp=nlm(np,lp)
c                  anl=anl+a(nl,nplp)
c               enddo
c            enddo
               if (l.eq.0) then
                  clep(n,l)=cle(l,l+1)
                  clpp(n,l)=clp(l,l+1)
               elseif (l.eq.(n-1)) then
                  clem(n,l)=cle(l,l-1)
                  clpm(n,l)=clp(l,l-1)
               else
                  clem(n,l)=cle(l,l-1)
                  clpm(n,l)=clp(l,l-1)
                  clep(n,l)=cle(l,l+1)
                  clpp(n,l)=clp(l,l+1)
               endif
            enddo
         enddo
c
c ***  collisions between states of different n ***
c
         call coll_nl_n(te)
      endif
c
ch      do i=1,nl
ch        cil(i)=0.
ch        do j=1,nl
ch          cl(i,j)=0.
ch        enddo
ch      enddo
chc
chc  coll. exc. rates
chc
chc  the 1-2 transition
chc  collision rates for 1-2s, 1-2p, and 1-3 from
chc  aggarwall mn 202, 15p '83. for 2s
ch      tsq = te  * te
ch      t3 = tsq * te
ch      if( te.le.25000. ) then
ch        cs12s = 0.195 + 1.28e-5*te - 4.69e-10*tsq + 6.19e-15*t3
ch        cs12p = 0.335 + 1.45e-5*te + 1.39e-10*tsq - 5.66e-15*t3
ch      else if( te.gt.25000. .and. te.le.5e5 ) then
ch        cs12s = 0.308 + 3.58e-7*te + 6.15e-13*tsq - 1.08e-18*t3
ch        cs12p = 0.328 + 1.43e-5*te - 6.55e-12*tsq - 2.69e-18*t3
ch      else if( te.gt.5e5 ) then
ch        cs12s = 0.506
ch        cs12p = 5.50
ch      endif
chc  scale factor to agree with callaway unnikrishnan and oza
ch      cs12p = cs12p * 0.88
ch      cl(1,2)= cs12p + cs12s
ch      et=10.1981*1.1609e4/te
ch      cl(1,2)=8.63e-6*cl(1,2)*fexp(-et)/(2.*dsqrt(te))
chc
ch      e1=e(ie,ion,1)
ch      e2=e(ie,ion,2)
ch      et=dabs(e1-e2)/tev
ch      if (et.gt.650.) et=650.
ch      g1=g(ie,ion,1)
ch      g2=g(ie,ion,2)      
ch      cl(2,1)=g1*dexp(et)*cl(1,2)/g2
chc
ch      do i=1,n
ch        do j=i+1,n
chc      do i=1,5
chc        do j=i+1,5
chc
ch          if (i.eq.1.and.j.eq.2) goto 9
ch          call cexh(i,j,te,cl(i,j))
chc
ch          ei=e(ie,ion,i)
ch          ej=e(ie,ion,j)
ch          et=dabs(ei-ej)/tev
ch          if (et.gt.650.) et=650.
ch          gi=g(ie,ion,i)
ch          gj=g(ie,ion,j)      
ch          cl(j,i)=gi*dexp(et)*cl(i,j)/gj
ch 9        continue
ch        enddo
ch      enddo
chc
ch      do i=1,n
ch        cl(i,i)=0.
ch      enddo
chc  coll. ion. rates
ch      do i=1,n
ch        call cionh(i,n+1,te,cil(i))
ch      enddo
chc
ch      do i=1,n
ch        ci(ie,ion,i)=cil(i)
ch        do j=1,n
ch          c(ie,ion,i,j)=cl(i,j)
ch        enddo
ch      enddo
c
c  he i ; ie=2, ion=1
c
      do i=1,nl
        cil(i)=0.
        do j=1,nl
          cl(i,j)=0.
        enddo
      enddo
c
      ie=2
      ion=1
c
      t4=te/1.d4
      t3=te/1.d3
c
      n=nlev(ie,ion)
c
c  coll. exc. rates
c
c
      do i=1,n
        do j=i+1,n
          ei=e(ie,ion,i)
          ej=e(ie,ion,j)
          gi=g(ie,ion,i)
          gj=g(ie,ion,j)
          eij=abs(ej-ei)
          et=eij/tev
          if(et.gt.650.) et=650.
          cl(i,j)=8.63e-6*oms(i,j)*dexp(-et)/(gi*tesqrt)
          cl(j,i)=gi*dexp(et)*cl(i,j)/gj
        enddo
c
c     collisional ionization rates from almog & netzer 
c
        eion=24.5876-ei
        cil(i)=b0(i)*t4**ga(i)*fexp(-eion/tev)
      enddo
c
      do i=1,n
        ci(ie,ion,i)=cil(i)
        do j=1,n
          c(ie,ion,i,j)=cl(i,j)
        enddo
      enddo
c
c  o i ; ie=5, ion=1
c
      do i=1,nl
        cil(i)=0.
        do j=1,nl
          cl(i,j)=0.
        enddo
      enddo
c
      ie=5
      ion=1
c
      t4=te/1.d4
      t3=te/1.d3
c
      n=nlev(ie,ion)
c
c  coll. exc. rates
c
      do i=1,n
         do j=i+1,n
            cl(i,j)=aoi(i,j)*(te/5.d3)**boi(i,j)
         enddo
      enddo
c
c     o i 63.18, 145.5 my from Berrington, J.Phys.B 21, 1083 (1987) --
c
      if(te.le.1.d3) then
         cl(1,2)=9.72d-3*(te/1.d3)**.773d0
         cl(1,3)=4.11d-3*(te/1.d3)**.676d0 
         cl(2,3)=6.36d-4*(te/1.d3)**1.301d0
      else 
         cl(1,2)=9.72d-3*(te/1.d3)**1.038d0
         cl(1,3)=4.11d-3*(te/1.d3)**.893d0
         cl(2,3)=6.36d-4*(te/1.d3)**1.648d0
      endif
c960616      if(t4.le.0.5) then
c960616        cl(1,2)=.0151*t3**1.31
c960616        cl(1,3)=.00184*t3**1.32
c960616        cl(2,3)=.031*t3**.534
c960616      elseif (t4.le.1.) then
c960616        cl(1,2)=.266*t4**1.10
c960616        cl(1,3)=.034*t4**1.08
c960616        cl(2,3)=.105*t4**.52
c960616      else
c960616        cl(1,2)=.266*t4**.91
c960616        cl(1,3)=.0324*t4**.91
c960616        cl(2,3)=.105*t4**.50
c960616      endif
c960616      cl(1,4)=.173*t4**1.25
c960616      cl(1,5)=.995*t4**.80
c960616      cl(1,6)=.046*t4**.82
c960616      cl(1,7)=.041*t4**.50
c960616      cl(1,8)=.0017*t4**2.16
c960616      cl(1,9)=.0088*t4**2.21
c960616      cl(4,6)=18.47*tev**.64
c960616      cl(4,7)=2.5e-2*tev**.64
c960616      cl(5,7)=9.63*tev**.64
c960616      cl(7,8)=20.7*tev**.64
c960616      cl(7,9)=5.3e-2*tev**.64
c     mewe formula
c960616      cl(1,4)=1.28
c960616      cl(1,5)=1.59
c960616      cl(1,6)=2.69
c960616      cl(1,7)=2.76
c960616      cl(1,8)=.51
c960616      cl(1,9)=1.34
c960616      cl(4,6)=148.*tev**.64
c960616      cl(5,7)=94.*tev**.64
c960616      cl(7,8)=447.*tev**.64
c960616      cl(7,9)=93.*tev**.64
cqcq not important at all !!  
c      cl(5,2)=1.e-6
cqcq
c
      do i=1,n
        do j=i+1,n
          ei=e(ie,ion,i)
          ej=e(ie,ion,j)
          gi=g(ie,ion,i)
          gj=g(ie,ion,j)
          eij=abs(ej-ei)
          et=eij/tev
          if(eij/tev.gt.650.) et=650.
          cl(i,j)=8.63e-6*cl(i,j)*dexp(-et)/(gi*tesqrt)
          cl(j,i)=gi*dexp(et)*cl(i,j)/gj
        enddo
      enddo
c
      do i=1,n
        cl(i,i)=0.
      enddo
c
c  coll. ion. rates
c
c  collisional ionization rates for o i from summers
c
      if (te.ge.2.e3) then
        cil(1)=1.09e-10*tesqrt*fexp(-1.58d5/te)/(1.+0.1*te/1.58e5)
        cil(2)=cil(1)
        cil(3)=cil(1)
      else
        cil(1)=0.
      endif
      do i=4,n
        ei=e(ie,ion,i)
        e00l=e00(ie,ion)
        gi=g(ie,ion,i)
        if(i.eq.4) om=1.68*t4**.956
        if(i.eq.5) om=.496*t4**.952
        if(i.eq.6) om=2.54*t4**.920
        if(i.eq.7) om=1.80*t4**.915
        if(i.eq.8) om=17.4*t4**.892
        if(i.eq.9) om=12.3*t4**.885
        if(i.eq.10) om=0.
        if(i.eq.11) om=0.
        if(i.eq.12) om=0.
        if(i.eq.13) om=54.6*t4**.840
c        if(i.eq.9) om=101.*t4**.806
        cil(i)=8.63e-6*om*fexp((ei-e00l)*11609.d0/te)/(tesqrt*gi)
      enddo
c960616      if (te.ge.2.e3) then
c960616        cil(1)=1.09e-10*dsqrt(te)*fexp(-1.58d5/te)/(1.+0.1*te/1.58e5)
c960616      else
c960616        cil(1)=0.
c960616      endif
c960616      do i=2,n
c960616        ei=e(ie,ion,i)
c960616        e00l=e00(ie,ion)
c960616        gi=g(ie,ion,i)
c960616        if(i.eq.2) om=1.68*t4**.956
c960616        if(i.eq.3) om=.496*t4**.952
c960616        if(i.eq.4) om=2.54*t4**.920
c960616        if(i.eq.5) om=1.80*t4**.915
c960616        if(i.eq.6) om=17.4*t4**.892
c960616        if(i.eq.7) om=12.3*t4**.885
c960616        if(i.eq.8) om=54.6*t4**.840
c960616        if(i.eq.9) om=101.*t4**.806
c960616        cil(i)=8.63e-6*om*fexp((ei-e00l)*11609.d0/te)/(dsqrt(te)*gi)
c960616      enddo
c
      do i=1,n
        ci(ie,ion,i)=cil(i)
c        ci(ie,ion,i)=0.
        do j=1,n
          c(ie,ion,i,j)=cl(i,j)
c          c(ie,ion,i,j)=0.
        enddo
      enddo
c
cnew9804
c *** Mg I ***
      ie=iemg
      ion=1
      do i=1,nl
        cil(i)=0.
        do j=1,nl
          cl(i,j)=0.
        enddo
      enddo
c
      n=nlev(ie,ion)
      if (ab(iemg).gt.0.) then
         do i=1,n
            do j=i+1,n
               ei=e(iemg,1,i)
               ej=e(iemg,1,j)
               gi=g(iemg,1,i)
               gj=g(iemg,1,j)
               eij=abs(ej-ei)
               et=eij/tev
               if(eij/tev.gt.650.) et=650.
               etexp=dexp(-et)
               omij=ommg1(i,j)*(te5000)**alfmg1(i,j)
               cl(i,j)=8.63e-6*omij*etexp/(gi*tesqrt)
               cl(j,i)=gi*cl(i,j)/(gj*etexp)
            enddo
         enddo
      endif
      do i=1,n
        ci(ie,ion,i)=cil(i)
        do j=1,n
          c(ie,ion,i,j)=cl(i,j)
        enddo
      enddo
cnew9804
c *** Si I ***
      ie=iesi
      ion=1
      do i=1,nl
        cil(i)=0.
        do j=1,nl
          cl(i,j)=0.
        enddo
      enddo
c
      n=nlev(ie,ion)
      if (ab(iesi).gt.0.) then
c     Si I 129.68 68.474 my omegas guessed!! from o i
         o32=9.76e-6*(te-228)+3.46e-11*(te-228.)**2
         o31=3.39e-6*(te-326.)-2.90e-11*(te-326.)**2
         o21=1.89e-6*(te-326.)+8.00e-11*(te-326.)**2
         if(o21.lt.0.) o21=1.e-10
         if(o31.lt.0.) o31=1.e-10
         if(o32.lt.0.) o32=1.e-10
c
         et=(e(iesi,1,3)-e(iesi,1,2))/tev
         if (et.gt.650.) et=650.
         etexp=dexp(-et)
         cl(2,3)=8.63e-6*o32*etexp/(g(iesi,1,2)*tesqrt)
         cl(3,2)=g(iesi,1,2)*cl(2,3)/(g(iesi,1,3)*etexp)
c
         et=(e(iesi,1,3)-e(iesi,1,1))/tev
         if (et.gt.650.) et=650.
         etexp=dexp(-et)
         cl(1,3)=8.63e-6*o31*etexp/(g(iesi,1,1)*tesqrt)
         cl(3,1)=g(iesi,1,1)*cl(1,3)/(g(iesi,1,3)*etexp)
c
         et=(e(iesi,1,2)-e(iesi,1,1))/tev
         if (et.gt.650.) et=650.
         etexp=dexp(-et)
         cl(1,2)=8.63e-6*o21*etexp/(g(iesi,1,1)*tesqrt)
         cl(2,1)=g(iesi,1,1)*cl(1,2)/(g(iesi,1,2)*etexp)
c
c     (Si I) 6591, 10995, 16360  omega for 3p - 1d from
c     m.s. pindzola, a.k. bhatia, a.temkin, phys.rev. 15, 35 (1977)
c     rest scaled from c i with scaling = 3
c      o21=3.97*t4**0.9
         o4_123=3.97*t4**0.9
         o41=o4_123/9.d0
         o42=o4_123/3.d0
         o43=o4_123*5.d0/9.d0
c
         et=(e(iesi,1,4)-e(iesi,1,1))/tev
         if (et.gt.650.) et=650.
         etexp=dexp(-et)
         cl(1,4)=8.63e-6*o41*etexp/(g(iesi,1,1)*tesqrt)
         cl(4,1)=g(iesi,1,1)*cl(1,4)/(g(iesi,1,4)*etexp)
c
         et=(e(iesi,1,4)-e(iesi,1,2))/tev
         if (et.gt.650.) et=650.
         etexp=dexp(-et)
         cl(2,4)=8.63e-6*o42*etexp/(g(iesi,1,2)*tesqrt)
         cl(4,2)=g(iesi,1,2)*cl(2,4)/(g(iesi,1,4)*etexp)
c
         et=(e(iesi,1,4)-e(iesi,1,3))/tev
         if (et.gt.650.) et=650.
         etexp=dexp(-et)
         cl(3,4)=8.63e-6*o43*etexp/(g(iesi,1,3)*tesqrt)
         cl(4,3)=g(iesi,1,3)*cl(3,4)/(g(iesi,1,4)*etexp)
c
c      o31=3.*0.149*(te/5000.)**0.871
         o5_123=3.*0.149*(te5000)**0.871
         o51=o5_123/9.d0
         o52=o5_123/3.d0
         o53=o5_123*5.d0/9.d0
c
         et=(e(iesi,1,5)-e(iesi,1,1))/tev
         if (et.gt.650.) et=650.
         etexp=dexp(-et)
         cl(1,5)=8.63e-6*o51*etexp/(g(iesi,1,1)*tesqrt)
         cl(5,1)=g(iesi,1,1)*cl(1,5)/(g(iesi,1,5)*etexp)
c
         et=(e(iesi,1,5)-e(iesi,1,2))/tev
         if (et.gt.650.) et=650.
         etexp=dexp(-et)
         cl(2,5)=8.63e-6*o52*etexp/(g(iesi,1,2)*tesqrt)
         cl(5,2)=g(iesi,1,2)*cl(2,5)/(g(iesi,1,5)*etexp)
c
         et=(e(iesi,1,5)-e(iesi,1,3))/tev
         if (et.gt.650.) et=650.
         etexp=dexp(-et)
         cl(3,5)=8.63e-6*o53*etexp/(g(iesi,1,3)*tesqrt)
         cl(5,3)=g(iesi,1,3)*cl(3,5)/(g(iesi,1,5)*etexp)
c      o32=3.*0.196*(te/5000.)**0.499
         o54=3.*0.196*(te5000)**0.499
c
         et=(e(iesi,1,5)-e(iesi,1,4))/tev
         if (et.gt.650.) et=650.
         etexp=dexp(-et)
         cl(4,5)=8.63e-6*o54*etexp/(g(iesi,1,4)*tesqrt)
         cl(5,4)=g(iesi,1,4)*cl(4,5)/(g(iesi,1,5)*etexp)
c         write(*,*) o21,o31,o32,o4_123,o5_123
c For the remaining transitions omega=0.1. Just a wild guess!
         do j=6,n
            do i=1,j-1
               gi=g(iesi,1,i)
               gj=g(iesi,1,j)
               eij=abs(e(iesi,1,j)-e(iesi,1,i))
               et=eij/tev
               if(et.gt.650.) et=650.
               etexp=dexp(-et)
               omij=omsi1(i,j)*(te5000**alfsi1(i,j))
               cl(i,j)=8.63e-6*omij*etexp/(gi*tesqrt)
               cl(j,i)=gi*cl(i,j)/(gj*etexp)
            enddo
         enddo

c Collisional ionization rate (cm**3 s**-1) from 
c Vernazza, Avrett, & Loeser 1976, ApJS, 30, 1
         et=(e00(iesi,1)-e(iesi,1,1))/tev
         if(et.gt.650.) et=650.
         cil(1)=2.41d-8*(te5000**0.649)*dexp(-et)
         cil(2)=cil(1)
         cil(3)=cil(1)
c
         et=(e00(iesi,1)-e(iesi,1,4))/tev
         if(et.gt.650.) et=650.
         cil(4)=0.27d-8*(te5000**0.688)*dexp(-et)
c
         et=(e00(iesi,1)-e(iesi,1,5))/tev
         if(et.gt.650.) et=650.
         cil(5)=5.39d-8*(te5000**0.353)*dexp(-et)
c
         et=(e00(iesi,1)-e(iesi,1,6))/tev
         if(et.gt.650.) et=650.
         cil(6)=0.35d-8*(te5000**0.582)*dexp(-et)
c
         et=(e00(iesi,1)-e(iesi,1,7))/tev
         if(et.gt.650.) et=650.
         cil(7)=1.23d-8*(te5000**0.547)*dexp(-et)
c
         et=(e00(iesi,1)-e(iesi,1,8))/tev
         if(et.gt.650.) et=650.
         cil(8)=6.63d-8*(te5000**0.525)*dexp(-et)
      endif
      do i=1,n
        ci(ie,ion,i)=cil(i)
        do j=1,n
          c(ie,ion,i,j)=cl(i,j)
        enddo
      enddo
c
c  ca ii ; ie=12, ion=2
c
      do i=1,nl
        cil(i)=0.
        do j=1,nl
          cl(i,j)=0.
        enddo
      enddo
c
      ie=12
      ion=2
      n=nlev(ie,ion)
c
c  coll. exc. rates
c
c     old data :
c     collisional excitation rates for ca ii from shine as given
c     by Li & McCray, 1993, ApJ, 405, 730 
c
c     collision strengths :
c
c      cl(1,2)=5.67*t3**(-0.01)
c      cl(1,3)=8.51*t3**(-0.01)
c      cl(2,3)=20.60*t3**0.01
c      cl(1,4)=3.99*t3**0.12
c      cl(2,4)=1.68*t3**0.23
c      cl(3,4)=3.93*t3**0.01
c      cl(1,5)=7.99*t3**0.12
c      cl(2,5)=6.27*t3**0.05
c      cl(3,5)=6.49*t3**0.29
c      cl(4,5)=10.08*t3**(-0.01)
c     collisional excitation rates for ca ii from shine
c        cl(1,2)=14.0
c        cl(1,3)=14.3
c        cl(2,3)=19.8
C
C  Omegas for most transitions
C  from Burgess et al. ('95), except between the sublevels of 3d which 
C  are from Li & McCray (ApJ 405, 730, '93; based on the results of 
C  Shine ('73)).
C
      o23=om23*t4**al23
      o45=om45*t4**al45
      o1t23=upsil(2,0.1248d0,1.4d0,2.56d0,2.954d0,3.762d0,
     &     4.339d0,5.023d0,te)
      o16=upsil(2,0.4757d0,1.5d0,2.601d0,1.778d0,1.981d0,
     &     2.261d0,3.382d0,te)
      o23t6=upsil(2,0.3509d0,4.4d0,0.9722d0,0.7578d0,0.6744d0,
     &     0.5929d0,0.4160d0,te)
      o1t45=upsil(1,0.2311d0,2.9d0,2.196d1,2.953d1,3.235d1,
     &     3.743d1,3.756d1,te)
      o23t45=upsil(1,0.1063d0,5.6d0,4.342d1,3.318d1,2.932d1,
     &     2.786d1,2.418d1,te)
      o45t6=upsil(1,0.2446d0,2.6d0,7.667d0,5.636d0,7.14d0,
     &     1.149d1,1.708d1,te)
c
      gsum1=g(ie,ion,2)+g(ie,ion,3)
      gsum2=g(ie,ion,4)+g(ie,ion,5)
      gmult=gsum1*gsum2
      o12=(g(ie,ion,2)*o1t23)/gsum1
      o13=(g(ie,ion,3)*o1t23)/gsum1
      o14=(g(ie,ion,4)*o1t45)/gsum2
      o15=(g(ie,ion,5)*o1t45)/gsum2
      o24=(g(ie,ion,2)*g(ie,ion,4)*o23t45)/gmult
      o25=(o24*g(ie,ion,5))/g(ie,ion,4)
      o26=(g(ie,ion,2)*o23t6)/gsum1
      o34=(g(ie,ion,3)*g(ie,ion,4)*o23t45)/gmult
      o35=(o34*g(ie,ion,5))/g(ie,ion,4)
      o36=(g(ie,ion,3)*o23t6)/gsum1
      o46=(g(ie,ion,4)*o45t6)/gsum2
      o56=(g(ie,ion,5)*o45t6)/gsum2
c
      cl(1,2)=o12
      cl(1,3)=o13
      cl(1,4)=o14
      cl(1,5)=o15
      cl(1,6)=o16
      cl(2,3)=o23
      cl(2,4)=o24
      cl(2,5)=o25
      cl(2,6)=o26
      cl(3,4)=o34
      cl(3,5)=o35
      cl(3,6)=o36
      cl(4,5)=o45
      cl(4,6)=o46
      cl(5,6)=o56
c
      do i=1,n
         do j=i+1,n
            ej=e(ie,ion,j)
            ei=e(ie,ion,i)
            gi=g(ie,ion,i)
            gj=g(ie,ion,j)
            eij=abs(ej-ei)
            et=eij/tev
            if(eij/tev.gt.650.) et=650.
            cl(i,j)=8.63e-6*cl(i,j)*dexp(-et)/(gi*tesqrt)
            cl(j,i)=gi*dexp(et)*cl(i,j)/gj
         enddo
      enddo
c
      do i=1,n
        cl(i,i)=0.
      enddo
c
c  coll. ion. rates
c
c     collisional ionization rates for ca ii from shine
c
c  check these for the excited levels
      do i=1,n
        ei=e00(ie,ion)-e(ie,ion,i)
        cil(i)=1.14e-10*tesqrt*fexp(-ei/tev)/ei**2
      enddo
c
      do i=1,n
        ci(ie,ion,i)=cil(i)
        do j=1,n
          c(ie,ion,i,j)=cl(i,j)
        enddo
      enddo
c
c  fe i ; ie=iefe, ion=1
c
cfex   colldat ; coll.ion rates only from g.s. fe i
      if (ife.le.1) then
         ie=iefe
         ion=1
         n=nlev(ie,ion)
c
c  coll. exc. rates
c
         do i=1,n
            do j=i+1,n
               ej=ef1(j)
               ei=ef1(i)
               gi=gf1(i)
               gj=gf1(j)
               eij=abs(ej-ei)
               et=eij/tev
c               cf1(i,j)=8.629e-6*omfe1(i,j)*dexp(-et)/
c     &              (gi*dsqrt(te))
c               cf1(j,i)=gi*dexp(et)*cf1(i,j)/gj
               cf1(j,i)=8.629e-6*omfe1(i,j)/
     &              (gj*tesqrt)
               cf1(i,j)=0.
               if (et.lt.690.) then
                  cf1(i,j)=gj*dexp(-et)*cf1(j,i)/gi
               endif
cqcq
c               cf1(i,j)=0.
c               cf1(j,i)=0.
cqcq
            enddo
         enddo
c
         do i=1,n
            cf1(i,i)=0.
         enddo
c
c  coll. ion. coefficients from Shull and Van Steenberg, 1982,
c  ApJS, 48, 95.
c
         ai=0.1
         cif1(1)=0.
         if ((9.13/t4).lt.690.) then
            cif1(1)=1.26e-10*tesqrt*dexp(-9.13d0/t4)/
     &           (1.+ai*t4/9.13)
         endif
cqcq
c         cif1(1)=0.
cqcq
         do i=2,n
            cif1(i)=0.
         enddo
      endif
c
c
c  fe ii ; ie=iefe, ion=2
c
cfex   colldat ; coll.ion rates only from g.s. fe ii
      if (ife.le.1) then
         ie=iefe
         ion=2
         n=nlev(ie,ion)
c
c  coll. exc. rates
c
cqcq run 74, run 75, run 311; run 301(rerun); run 310_ss
cqcq run 312; run 401; run 402
         call fe2om(te)
c
         do i=1,n
            do j=i+1,n
               ej=ef2(j)
               ei=ef2(i)
               gi=gf2(i)
               gj=gf2(j)
               eij=abs(ej-ei)
               et=eij/tev
c               cf2(i,j)=8.629e-6*omfe2(i,j)*dexp(-et)/
c     &              (gi*dsqrt(te))
c               cf2(j,i)=gi*dexp(et)*cf2(i,j)/gj
               cf2(j,i)=8.629e-6*omfe2(i,j)/
     &              (gj*tesqrt)
               cf2(i,j)=0.
               if (et.lt.690.) then
                  cf2(i,j)=gj*dexp(-et)*cf2(j,i)/gi
cqcq
c               cf2(i,j)=0.
c               cf2(j,i)=0.
cqcq
               endif
            enddo
         enddo
c
         do i=1,n
            cf2(i,i)=0.
         enddo
c
c  coll. exc. by H-atoms
c  Data from Hollenbach & McKee, 1989, ApJ, 342, 306.
c
         do i=1,3
            cpf2(i,i)=0.d0
         enddo
c
         cpf2(2,1)=9.5e-10
         cpf2(3,1)=5.7e-10
         cpf2(3,2)=4.7e-10
c
         et=(ef2(2)-ef2(1))/tev
         cpf2(1,2)=(gf2(2)/gf2(1))*cpf2(2,1)*fexp(-et)
         et=(ef2(3)-ef2(1))/tev
         cpf2(1,3)=(gf2(3)/gf2(1))*cpf2(3,1)*fexp(-et)
         et=(ef2(3)-ef2(2))/tev
         cpf2(2,3)=(gf2(3)/gf2(2))*cpf2(3,2)*fexp(-et)
c
c  coll. ion. coefficients from Shull and Van Steenberg, 1982,
c  ApJS, 48, 95.
c
         ai=0.1
         cif2(1)=0.
         if((18.8/t4).lt.690.) then
            cif2(1)=4.97e-11*tesqrt*dexp(-18.8d0/t4)/
     &           (1.+ai*t4/18.8)
         endif
         do i=2,n
            cif2(i)=0.
         enddo
c
c  fe iii ; ie=iefe, ion=3
c
         ie=iefe
         ion=3
         n=nlev(ie,ion)
c
c  coll. exc. rates
c
         do i=1,n
            do j=i+1,n
               ej=ef3(j)
               ei=ef3(i)
               gi=gf3(i)
               gj=gf3(j)
               eij=abs(ej-ei)
               et=eij/tev
c               cf3(i,j)=8.629e-6*omfe3(i,j)*dexp(-et)/
c     &              (gi*dsqrt(te))
c               cf3(j,i)=gi*dexp(et)*cf3(i,j)/gj
               cf3(j,i)=8.629e-6*omfe3(i,j)/
     &              (gj*tesqrt)
               cf3(i,j)=0.
               if (et.lt.690.) then
                  cf3(i,j)=gj*dexp(-et)*cf3(j,i)/gi
               endif
            enddo
         enddo
c
         do i=1,n
            cf3(i,i)=0.
         enddo
c
c  coll. ion. coefficients from Shull and Van Steenberg, 1982,
c  ApJS, 48, 95.
cfex   colldat ; coll.ion rates only from g.s. fe iii
c
         ai=0.1
         cif3(1)=0.
         if((35.6/t4).lt.690.) then
            cif3(1)=1.49e-10*tesqrt*dexp(-35.6d0/t4)/
     &           (1.+ai*t4/35.6)
         endif
         do i=2,n
            cif3(i)=0.
         enddo
c
c  fe iv ; ie=iefe, ion=4
c
         ie=iefe
         ion=4
         n=nlev(ie,ion)
c
c  coll. exc. rates
c
         do i=1,n
            do j=i+1,n
               ej=ef4(j)
               ei=ef4(i)
               gi=gf4(i)
               gj=gf4(j)
               eij=abs(ej-ei)
               et=eij/tev
c               cf4(i,j)=8.629e-6*omfe4(i,j)*dexp(-et)/
c     &              (gi*dsqrt(te))
c               cf4(j,i)=gi*dexp(et)*cf4(i,j)/gj
               cf4(j,i)=8.629e-6*omfe4(i,j)/
     &              (gj*tesqrt)
               cf4(i,j)=0.
               if (et.lt.690.) then
                  cf4(i,j)=gj*dexp(-et)*cf4(j,i)/gi
               endif
            enddo
         enddo
c
         do i=1,n
            cf4(i,i)=0.
         enddo
c
c  coll.ion rates are put to zero for now
         do i=1,n
            cif4(i)=0.
         enddo
      endif
c
      return
      end



      subroutine coll_l(ic,n,te,xe,cl)
      implicit real*8 (a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
      common/cden/den
      dimension cl(0:nm,0:nm),anl(0:nmax)
      dimension c1p(0:nm),c1m(0:nm)
      dimension c2p(0:nm),c2m(0:nm)
      data emass/9.10956e-28/,pmass/1.672614e-24/
c
c  References :
c     Brocklehurst, M., 1971, MNRAS, 153, P. 471-490. (B71)
c     Pengelly, R.M.,& Seaton, M.J., 1964, MNRAS, 127, p. 165. (PS64)
c
c  The data is checked by reproducing figure 2. in PS64, in detail.
c  That is to plot xltc (in main code) as a function of l, for 
c  collsions with protons, T=1.e4K, and Ne=1.e4.
c 
c  The collision rates may be negative if the density is very high.
c  In this case the Debye radius gives the smallest r; (lt.1) =>
c  log10(debye radius) is less than 0 and the collision rate may become
c  less than 0. I haven't checked out which is the limiting density.
c
c  Points of confusion :
c
c  - Both PS64 (eq. 33) and B71 (eq. 3.36) uses an expression for
c    the Debye radius containg a 'pi' too much. B71 then uses this
c    'wrong' Debye radius in the following expressions (eq. 3.40).   
c    PS64, on the other hand uses the correct Debye radius (without
c    the extra pi) in the corresponding expression (eq. 46). In PS64 
c    it mostly looks like a misprint in eq. 33, when calculating the 
c    expression they get the correct number.
c    In my calculation I use the correct Debye radius in all expressions. 
c
c  - I can't derive eq. 3.41 in B71 ( and the corresponding eq. 45 in
c    PS64). They get 2log10(0.72*v*tau)=10.95+log10(m*T*tau*tau/my).
c    By using the following expression for the velocity,
c    v = sqrt(2kT/my) (this is the same expression as given by PS64
c    in eq. 31, however when calculating it explicitly they get the
c    wrong answer), I get the expression
c    2log10(0.72*v*tau)=11.196+log10(m*T*tau*tau/my).
c    I'm using this last expression.
c
c zZ = z/Z ; where z is the charge of the particle causing the excitation,
c            (z=1 for electrons), Z is the charge of the nuclear charge 
c            (Z=1 for hydrogen). 
c
c
c collisions with electrons : ic=1
c collisions with protons : ic=2
c
c  xmym = my/m = reduced mass / electron mass
c
      do l1=0,nm
         c1p(l1)=0.d0
         c1m(l1)=0.d0
         c2p(l1)=0.d0
         c2m(l1)=0.d0
         do l2=0,nm
            cl(l1,l2)=0.d0
         enddo
      enddo
c
      xn=dble(n)
      if (ic.eq.1) then
c
c e- + H I - collisions
c
c  xmy = mp*me/(mp+me) = me 
c  xmym = xmy / me = 1.
         zZ=1.
         xmy=emass
         xmym=1.d0
      elseif (ic.eq.2) then
c H+ + H I - collisions
c
c  xmy = mp*mp/(mp+mp) = 0.5 mp 
c  xmym = xmy / me = 0.5 mp/me
         zZ=1.
         xmy=0.5*pmass
         xmym=xmy/emass
      endif
c
      c_dnl=zZ*zZ*6.*xn*xn
c      cnl1=9.933d-6*sqrt(xmym)*xe*den/sqrt(te)
      cnl1=9.933d-6*sqrt(xmym)/sqrt(te)
      vel=sqrt(2.*1.3806d-16*te/xmy)
c
c  rho_c = 0.72*tau*v ; tau = radiative lifetime; v = velocity of 
c                       the incident particle
c  (rho_D = sqrt(kT/4*pi^2*Ne*e^2) = Debye radius/sqrt(pi) (?); not used !)
c  rho_D = sqrt(kT/4*pi*Ne*e^2) = Debye radius  ; used !
c  tau = radiative lifetime
c
c      rho_D=sqrt(15.1581*te/(xe*den))
c      two_log_rho_D=1.181+log10(te/(xe*den))         
      rho_D=sqrt(47.6248*te/(xe*den))
      two_log_rho_D=1.678+log10(te/(xe*den))         
      two_log_rho_D=2.*log10(rho_D)                  
c
      do l=0,n-1
         inl=nlm(n,l)
         anl(l)=0.d0
         do np=1,n-1
            do lp=0,np-1
               nplp=nlm(np,lp)
               anl(l)=anl(l)+ah(inl,nplp)
            enddo
         enddo
      enddo
c
c start from l=0 and climb up (c1p & c1m)
      l=0
      xl=dble(l)
      tau=1./anl(l)
      rho_c=0.72*tau*vel
      if (rho_c.lt.rho_D) then
c         two_log_rho=10.95+log10(te*tau*tau/xmym)
         two_log_rho=11.196+log10(te*tau*tau/xmym)
         two_log_rho=2.*log10(rho_c)
      elseif (rho_D.le.rho_c) then
         two_log_rho=two_log_rho_D
      endif
c
c      dnl=zZ*zZ*6.*xn*xn*(xn*xn-xl*xl-xl-1.)
c      cnl=9.933d-6*sqrt(xmym)*(dnl/sqrt(te))*xe*den*
c     &     (11.538+log10(te/(dnl*xmym))+two_log_rho)
      dnl=c_dnl*(xn*xn-xl*xl-xl-1.)
      cnl=cnl1*dnl*(11.538+log10(te/(dnl*xmym))+
     &     two_log_rho)
c
c      cl(0,1)=cnl
      c1p(0)=cnl
      gl=(2.*dble(l)+1.)
      glp1=(2.*dble(l+1)+1.)
c      cl(1,0)=(gl/glp1)*cl(0,1)
      c1m(1)=(gl/glp1)*c1p(0)
c
      do l=1,n-2
         xl=dble(l)
         tau=1./anl(l)
         rho_c=0.72*tau*vel
         if (rho_c.lt.rho_D) then
c            two_log_rho=10.95+log10(te*tau*tau/xmym)
            two_log_rho=11.196+log10(te*tau*tau/xmym)
            two_log_rho=2.*log10(rho_c)
         elseif (rho_D.le.rho_c) then
            two_log_rho=two_log_rho_D
         endif
c
c         dnl=zZ*zZ*6.*xn*xn*(xn*xn-xl*xl-xl-1.)
c         cnl=9.933d-6*sqrt(xmym)*(dnl/sqrt(te))*xe*den*
c     &        (11.538+log10(te/(dnl*xmym))+two_log_rho)
         dnl=c_dnl*(xn*xn-xl*xl-xl-1.)
         cnl=cnl1*dnl*(11.538+log10(te/(dnl*xmym))+
     &        two_log_rho)
c         cl(l,l+1)=cnl-cl(l,l-1)
         c1p(l)=cnl-c1m(l)
         gl=(2.*xl+1.)
         glp1=(2.*(xl+1)+1.)
c         cl(l+1,l)=(gl/glp1)*cl(l,l+1)
         c1m(l+1)=(gl/glp1)*c1p(l)
      enddo
c
c start from l=n-1 and climb down (c2p & c2m)
c
      l=n-1
      xl=dble(l)
      tau=1./anl(l)
      rho_c=0.72*tau*vel
      if (rho_c.lt.rho_D) then
c         two_log_rho=10.95+log10(te*tau*tau/xmym)
         two_log_rho=11.196+log10(te*tau*tau/xmym)
         two_log_rho=2.*log10(rho_c)
      elseif (rho_D.le.rho_c) then
         two_log_rho=two_log_rho_D
      endif
c
c      dnl=zZ*zZ*6.*xn*xn*(xn*xn-xl*xl-xl-1.)
c      cnl=9.933d-6*sqrt(xmym)*(dnl/sqrt(te))*xe*den*
c     &     (11.538+log10(te/(dnl*xmym))+two_log_rho)
      dnl=c_dnl*(xn*xn-xl*xl-xl-1.)
      cnl=cnl1*dnl*(11.538+log10(te/(dnl*xmym))+
     &     two_log_rho)
c
      c2m(n-1)=cnl
      gl=(2.*dble(l)+1.)
      glm1=(2.*dble(l-1)+1.)
      c2p(n-2)=(gl/glm1)*c2m(n-1)
c
      do l=n-2,1,-1
         xl=dble(l)
         tau=1./anl(l)
         rho_c=0.72*tau*vel
         if (rho_c.lt.rho_D) then
            two_log_rho=11.196+log10(te*tau*tau/xmym)
            two_log_rho=2.*log10(rho_c)
         elseif (rho_D.le.rho_c) then
            two_log_rho=two_log_rho_D
         endif
c
         dnl=c_dnl*(xn*xn-xl*xl-xl-1.)
         cnl=cnl1*dnl*(11.538+log10(te/(dnl*xmym))+
     &        two_log_rho)

         c2m(l)=cnl-c2p(l)
         gl=(2.*xl+1.)
         glm1=(2.*(xl-1)+1.)
         c2p(l-1)=(gl/glm1)*c2m(l)
      enddo
c
      l=0
      cl(l,l+1)=(c1p(l)+c2p(l))*0.5
      do l=1,n-2
         cl(l,l-1)=(c1m(l)+c2m(l))*0.5
         cl(l,l+1)=(c1p(l)+c2p(l))*0.5
      enddo
      l=n-1
      cl(l,l-1)=(c1m(l)+c2m(l))*0.5
c
c      write(*,701) n
c      write(*,702) (c1p(l),l=0,n-2)
c      write(*,702) (c2p(l),l=0,n-2)
c      write(*,702) (c1m(l),l=1,n-1)
c      write(*,702) (c2m(l),l=1,n-1)
c
 701  format(i4)
 702  format(1pe12.4,20e12.4)
c
      return
      end



      subroutine coll_nl_n(te)
      implicit real*8(a-h,o-z)
c
c  this routine calculates the collisional excitation rate (cnl) 
c  between different n-states for h i
c
c  Hummer & Storey, 1987 (MNRAS, 224, p. 801) uses the Bethe-approx.,
c  Q_nl,n'l' = (f_nl,n'l'/f_nn')*Q_nn',
c  for transitions with delta(n)=1. For delta(n) > 1 they use a simpler
c  approximation, Q_nl,n'l' = Q_nn'. Only transitions with delta(l)=1
c  are considered.
c   
c  For all transitions (delta(n) > 0, delta(l)=1), we use the 
c  Bethe-approximation. 
c
c  We use : C_nl,n'l' = (A_nl,n'l'/A_nn')*C_nn' 
c   and     C_n,n'l'  = (A_n,n'l'/A_nn')*C_nn'
c  This assumption  gives :
c     x_n * C_nn' = sum(ll'; x_nl * C_nl,n'l') if the different l-states
c  are assumed to be populated according to statistical weights.
c  The statistical weights necessary to cancel the populations are included
c  in the definitions of the A-values !!!
c
c  Therefore the reasoning below is not used any more :
c  (Our treatment differs from that by Hummer & Storey
c  in that we aslo include the statistical weights,
c  C_nl,n'l' = (g_n/g_nl)*(f_nl,n'l'/f_nn')*C_nn' (g_n/g_nl = 2n^2/2(2l+1)).
c  This assumption  gives :
c     x_n * C_nn' = sum(ll'; x_nl * C_nl,n'l') if the different l-states
c  are assumed to be populated according to statistical weights.)
c
c  The collisional excitation rates (C_nn') are from johnson (ap j 174:227).
c  
      include 'INCLUDE/PARAM.IOR'
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
      common/ccollh/cnl(nmax,nmax),clep(nm,0:nm),clem(nm,0:nm),
     &     clpp(nm,0:nm),clpm(nm,0:nm)
c
      do i=1,nmax
         do j=1,nmax
            cnl(i,j)=0.d0
         enddo
      enddo
c
      tev=te/1.1609d4
c
c  h i 
c
c  coll. exc. rates
c
c  the 1-2 transition
c  collision rates for 1-2s, 1-2p, and 1-3 from
c  aggarwall mn 202, 15p '83. for 2s
      tsq = te  * te
      t3 = tsq * te
      if( te.le.25000. ) then
        cs12s = 0.195 + 1.28e-5*te - 4.69e-10*tsq + 6.19e-15*t3
        cs12p = 0.335 + 1.45e-5*te + 1.39e-10*tsq - 5.66e-15*t3
      else if( te.gt.25000. .and. te.le.5e5 ) then
        cs12s = 0.308 + 3.58e-7*te + 6.15e-13*tsq - 1.08e-18*t3
        cs12p = 0.328 + 1.43e-5*te - 6.55e-12*tsq - 2.69e-18*t3
      else if( te.gt.5e5 ) then
        cs12s = 0.506
        cs12p = 5.50
      endif
c  scale factor to agree with callaway unnikrishnan and oza
      cs12p = cs12p * 0.88
c      cl(1,2)= cs12p + cs12s
      et=10.1981*1.1609e4/te
c      cl(1,2)=8.63e-6*cl(1,2)*fexp(-et)/(2.*dsqrt(te))
c
      nl10=nlm(1,0)
      nl20=nlm(2,0)
      nl21=nlm(2,1)
      cnl(nlm(1,0),nlm(2,0))=8.63e-6*cs12s*fexp(-et)/(2.*dsqrt(te))
      cnl(nlm(1,0),nlm(2,1))=8.63e-6*cs12p*fexp(-et)/(2.*dsqrt(te))
c
      e1=ehn(1)
      e2=ehn(2)
      et=dabs(e1-e2)/tev
      if (et.gt.650.) et=650.
      g1=ghn(1)
      g2=2.      
c      cl(2,1)=g1*dexp(et)*cl(1,2)/g2
      cnl(nl20,nl10)=g1*dexp(et)*cnl(nl10,nl20)/g2
      g2=6.
      cnl(nl21,nl10)=g1*dexp(et)*cnl(nl10,nl21)/g2
c
      do i=1,nn
        do j=i+1,nn
c
          if (i.eq.1.and.j.eq.2) goto 9
          call cexh(i,j,te,clij)
c
          ei=ehn(i)
          ej=ehn(j)
          et=dabs(ei-ej)/tev
          if (et.gt.650.) et=650.
          expet=fexp(-et)
          gi=ghn(i)
          gj=ghn(j)
          clji=gi*dexp(et)*clij/gj
c
          aji=0.d0
          do lj=0,j-1
             do li=0,i-1
                nj_lj=nlm(j,lj)
                ni_li=nlm(i,li)
                gnjlj=2.*(2.*dble(lj)+1.)
                aji=aji+ah(nj_lj,ni_li)*gnjlj/gj
             enddo
          enddo
c     
          li=0
          xli=dble(li)
          gi=2.*(2.*xli+1)
          lj=li+1
          xlj=dble(lj)
          glj=2.*(2.*xlj+1)
          fact=ah(nlm(j,lj),nlm(i,li))/aji
          cnl(nlm(j,lj),nlm(i,li))=clji*fact
c          cnl(nlm(i,li),nlm(j,lj))=clij*fact
          cnl(nlm(i,li),nlm(j,lj))=cnl(nlm(j,lj),nlm(i,li))*
     &         (glj/gi)*expet
c     
          do li=1,i-1
             xli=dble(li)
             gi=2.*(2.*xli+1)
c     
             lj=li+1
             xlj=dble(lj)
             xlj=dble(lj)
             glj=2.*(2.*xlj+1)
             fact=ah(nlm(j,lj),nlm(i,li))/aji
             cnl(nlm(j,lj),nlm(i,li))=clji*fact
c             cnl(nlm(i,li),nlm(j,lj))=clij*fact
             cnl(nlm(i,li),nlm(j,lj))=cnl(nlm(j,lj),nlm(i,li))*
     &            (glj/gi)*expet
c                
             lj=li-1
             xli=dble(lj)
             xlj=dble(lj)
             glj=2.*(2.*xlj+1)
             fact=ah(nlm(j,lj),nlm(i,li))/aji
             cnl(nlm(j,lj),nlm(i,li))=clji*fact
c             cnl(nlm(i,li),nlm(j,lj))=clij*fact
             cnl(nlm(i,li),nlm(j,lj))=cnl(nlm(j,lj),nlm(i,li))*
     &            (glj/gi)*expet
          enddo
c
          sumji=0.d0
          sumij=0.d0
          do li=0,i-1
             do lj=0,j-1
                gnjlj=2.*(2.*dble(lj)+1.)
                sumji=sumji+cnl(nlm(j,lj),nlm(i,li))*gnjlj/gj
                sumij=sumij+cnl(nlm(i,li),nlm(j,lj))
             enddo
          enddo
c
 9        continue
        enddo
      enddo
c
      do i=nn+1,nm
         do j=i+1,nm
            call cexh(i,j,te,clij)
c
            ei=ehn(i)
            ej=ehn(j)
            et=dabs(ei-ej)/tev
            if (et.gt.650.) et=650.
            gi=ghn(i)
            gj=ghn(j)
            clji=gi*dexp(et)*clij/gj
c
            cnl(nnm(j),nnm(i))=clji
            cnl(nnm(i),nnm(j))=clij
c     
         enddo
      enddo
c
      do j=nn+1,nm
         do i=1,nn
            call cexh(i,j,te,clij)
c
            ei=ehn(i)
            ej=ehn(j)
            et=dabs(ei-ej)/tev
            if (et.gt.650.) et=650.
            expet=fexp(-et)
            gi=ghn(i)
            gj=ghn(j)
            clji=gi*dexp(et)*clij/gj
c
            aji=0.d0
            do li=0,i-1
               aji=aji+ah(nnm(j),nlm(i,li))
            enddo
c
            do li=0,i-1
               gli=2.*(2.*dble(li)+1.)
               cnl(nnm(j),nlm(i,li))=clji*ah(nnm(j),nlm(i,li))/aji
c               cnl(nlm(i,li),nnm(j))=clij*ah(nnm(j),nlm(i,li))/aji

               cnl(nlm(i,li),nnm(j))=cnl(nnm(j),nlm(i,li))*
     &              (gj/gli)*expet
            enddo
         enddo
      enddo
c
c  coll. ion. rates
c      do i=1,n
c        call cionh(i,n+1,te,cil(i))
c      enddo
c
c      do i=1,n
c        ci(i)=cil(i)
c        do j=1,n
c          c(i,j)=cl(i,j)
c        enddo
c      enddo
c
      return
      end



      subroutine cionh(n,n0,t,ci)
      implicit real*8(a-h,o-z)
c
c     collisional ionization rates from johnson (ap j 174:227)
c
      pi=3.14159
      con=1.092e-10
      q=dble(n)
      t4=t/1.e4
      x0=1.-(q/real(n0))**2
      yn=x0*15.72/(q**2*t4)
      if(n.ne.1) goto 1
      g0=1.133
      g1=-.4059
      g2=.07014
      bn=-.603
      rn=.45
1     if(n.ne.2) goto 2
      g0=1.0785
      g1=-.2319
      g2=.02947
      bn=(4.-18.63/q+36.24/q**2-28.09/q**3)/q
      rn=1.94/q**1.57
2     if(n.lt.3) goto 3
      g0=.9935+.2328/q-.1296/q**2
      g1=-(.6282-.5598/q+.5299/q**2)/q
      g2=(.3887-1.181/q+1.470/q**2)/q**2
      bn=(4.-18.63/q+36.24/q**2-28.09/q**3)/q
      rn=1.94/q**1.57
3     an=32.*q*(g0/(x0**3*3.)+g1/(x0**4*4.)+g2/(x0**5*5.))
     &/(3.*dsqrt(3.d0)*pi)
      bn=2.*q**2*(3.+2./x0+bn/x0**2)/(3.*x0)
      zn=x0*(rn+15.72/(q**2*t4))
      a1=exp1(yn)/yn-exp1(zn)/zn
      a2=(1.+2./yn)*e2(yn)-dexp(-yn)/yn-(1.+2./zn)*e2(zn)+dexp(
     &-zn)/zn
      ci=con*dsqrt(t)*q**2*yn**2*(an*a1+(bn-an*dlog(2.*q**2
     &/x0))*a2)/x0
      return
      end

      subroutine cexh(n,np,t,ce)
      implicit real*8(a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
c
c     collisional excitation rates from johnson (ap j 174:227)
c
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),aq(mie,mion,nl,nl)
      common/cionpot/e00(mie,mion)
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
c  h i
      ie=1
      ion=1
      pi=3.14159
      con=1.092e-10
      qp=dble(np)
      q=dble(n)
      t4=t/1.e4
      in=n
      inp=np
      eij=abs(ehn(in)-ehn(inp))
ch      eij=abs(e(ie,ion,in)-e(ie,ion,inp))
      tev=t/1.1609e4
      x=1.-(q/qp)**2
      yn=eij/tev
      if(n.ne.1) goto 1
      g0=1.133
      g1=-.4059
      g2=.07014
      bn=-.603
      rn=.45
1     if(n.ne.2) goto 2
      g0=1.0785
      g1=-.2319
      g2=.02947
      bn=(4.-18.63/q+36.24/q**2-28.09/q**3)/q
      rn=1.94/q**1.57
2     if(n.lt.3) goto 3
      g0=.9935+.2328/q-.1296/q**2
      g1=-(.6282-.5598/q+.5299/q**2)/q
      g2=(.3887-1.181/q+1.470/q**2)/q**2
      bn=(4.-18.63/q+36.24/q**2-28.09/q**3)/q
      rn=1.94/q**1.57
 
3     gn=g0+g1/x+g2/x**2
      ann=64.*q**3*gn/(3.*dsqrt(3.d0)*pi*x**4*qp**3)
      fnn=ann*x/(2.*q**2)
      bnn=4.*q**4*(1.+4./(3.*x)+bn/x**2)/(qp**3*x**2)
      rnn=x*rn
      zn=rnn+yn
      a1=(1./yn+.5)*exp1(yn)-(1./zn+.5)*exp1(zn)
      a2=e2(yn)/yn-e2(zn)/zn
      ce=con*dsqrt(t)*q**2*yn**2*(ann*a1+(bnn-ann*dlog(2.*q**2
     &/x))*a2)/x
      return
      end

      double precision function gbf(n,e)
c
c     calculation of bound-free gaunt factor from mihalas, d., 
c     ap.j. 149:169 (1967)
c     e in ev
c     wl0 for n=7-10 not from mihalas, n=1-6 from mihalas ?
c   
      implicit real*8 (a-h,o-z)
      parameter(nl=10)
      dimension a0(nl),a1(nl),a2(nl),a3(nl),am1(nl),am2(nl),am3(nl)
      dimension wl0(nl)
      data wl0/915.3291d0,3704.9034d0,8504.7783d0,15560.594d0,
     &         25260.706d0,38194.187d0,
     &         44676.397d0,58352.845d0,73852.820d0,91176.320d0/
      data a0/1.2302628d0,1.1595421d0,1.1450949d0,1.1306695d0,
     &        1.1190904d0,1.1168376d0,1.1128632d0,1.1093137d0,
     &        1.1078717d0,1.1052734d0/
      data a1/-2.9094219d-3,-2.0735860d-3,-1.9366592d-3,-1.3482273d-3,
     &        -1.0401085d-3,-8.9466573d-4,-7.4833260d-4,-6.2619148d-4,
     &        -5.4837392d-4,-4.4341570d-4/
      data a2/7.3993579d-6,2.7033384d-6,2.3572356d-6,-4.6949424d-6,
     &        -6.9943488d-6,-8.8393113e-6,-1.0244504d-5,-1.1342068d-5,
     &        -1.2157943d-5,-1.3235905d-5/
      data a3/-8.7356966d-9,0.d0,0.d0,2.3548636d-8,2.8496742d-8,
     &        3.4696768d-8,3.8595771d-8,4.1477731d-8,
     &        4.3796716d-8,4.7003140d-8/
      data am1/-5.5759888d0,-1.2709045d0,-0.55936432d0,-0.31190730d0,
     &         -0.16051018d0,-0.13075417d0,-9.5441161d-2,
     &         -7.1010560d-2,-5.6046560d-2,-4.7326370d-2/
      data am2/12.803223d0,2.1325684d0,0.52471924d0,0.19683564d0,
     &         5.5545091d-2,4.1921183d-2,2.3350812d-2,1.3298411d-2,
     &         8.5139736d-3,6.1516856d-3/
      data am3/0.d0,-2.0244141d0,-0.23387146d0,-5.4418565d-2,
     &         -8.9182854d-3,-5.5303574d-3,-2.2752881d-3,
     &         -9.7200274d-4,4.9576163d-4,-2.9467046d-4/
      wlmy=1.239854/e
      gbf=0.
      if (n.le.10) then
         if(wlmy.lt.wl0(n)/1.d4) then
            x=1./wlmy
            gbf=a0(n)+a1(n)*x+a2(n)*x*x+a3(n)*x*x*x+am1(n)/x+
     &           am2(n)/(x*x)+am3(n)/(x*x*x)
         endif
      else
         wlth=dble(n)*dble(n)*1.d8/109677.6
         if(wlmy.lt.wlth/1.d4) then
            gbf=1.
         endif
      endif
c
      return
      end

      subroutine recomb(te)
      implicit real*8 (a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/cfedat/cf1(nlf1,nlf1),cf2(nlf2,nlf2),cf3(nlf3,nlf3),
     &     cf4(nlf4,nlf4),cif1(nlf1),cif2(nlf2),cif3(nlf3),cif4(nlf4),
     &     gf1(nlf1),gf2(nlf2),gf3(nlf3),gf4(nlf4),ef1(nlf1),ef2(nlf2),
     &     ef3(nlf3),ef4(nlf4),af1(nlf1,nlf1),af2(nlf2,nlf2),
     &     af3(nlf3,nlf3),af4(nlf4,nlf4),cpf2(3,3)
      common/crecph/reco(mie,mion,nl),ph(mie,mion,nl),gh(mie,mion,nl),
     &     recof1(nlf1),recof2(nlf2),recof3(nlf3),recof4(nlf4),
     &     phh1(nmax),ghh1(nmax),
     &     phf1(nlf1),phf2(nlf2),phf3(nlf3),phf4(nlf4),
     &     ghf1(nlf1),ghf2(nlf2),ghf3(nlf3),ghf4(nlf4),
     &     recofr(mie,mion),phto(mie,mion),alsi,pennc
      common/chirec/tehr(40),alfth(40),reconl(nmx),recon(nm)
      COMMON/HEIRECION/AL0(16),BE(16),B0(16),GA(16)
      common/heirecvf/teheir(12),alfhei(16,12)
      common/cfemul/ife
      common/ctpar/tday,dts,iss,iss1,itd
      common/cabun/amean,ab(mie),xa(mie,mion),xaold(mid,mie,mion),
     &     nion(mie)
      common/cmod/abmod(10,mie),vrmod(mid),rmod(mid),rlab(mid),
     &     dmmod(mid),ameanmod(10),denmod(mid),tdmod,
     &     nrtab(mid),izmod(mid),nrmod,mixm
      common/depthind/id
      common/csn/i87acl,i87a,i93j,isn
      common/csnmod/absn(mid,mie),ameansn(mid),densn(mid),rsn(midp1),
     &     vrsn(midp1),rsnlab(mid),fillsn(mid),dmsn(mid),iesnm(mid,2),
     &     tdsn,nrsn
      common/ots/iots
      common/cmg1/ommg1(10,10),alfmg1(10,10),
     &     wlmg1(10,10),jlmg1(10,10),klvmg1(10,10)
      common/catdat/cs(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),as(mie,mion,nl,nl)
      dimension ee(nlf)
      dimension xltsi(41),alsiv(41)
c
c  this routine calculates the recombination coefficients (reco)
c  for h i, o i and ca ii, fe i, fe ii
c
      t4=te/1.e4
      t15=te**1.5
      tev=te/1.1609e4
      tev0=tev*10.
      xkt=tev
c
c  h i ; ie=1, ion=1
c
      if (ab(ieh).gt.0.) then
         call recomb_h(te)
c
         ie=1
         ion=1
         n=nlev(ie,ion)      
c
         if (te.lt.tehr(1)) then
            ite1=1
            ite2=1
            dte=0.d0
         elseif (te.gt.tehr(40)) then
            ite1=40
            ite2=40
            dte=0.d0
         else
            do i=1,39
               if (te.ge.tehr(i).and.te.le.tehr(i+1)) then
                  ite1=i
                  ite2=i+1
                  dte=(te-tehr(ite1))/(tehr(ite2)-tehr(ite1))
                  goto 9
               endif
            enddo
 9          continue
         endif
         alftot=alfth(ite1)+(alfth(ite2)-alfth(ite1))*dte
c
c  put the remaining recombination in the highest level to
c  make the total recombination coefficient correct
         alfsum=0.d0
         do nj=1,nm-1
            alfsum=alfsum+recon(nj)
         enddo
         recon(nm)=alftot-alfsum
      else
         do n=1,nm
            recon(n)=0.
         enddo
         do i=1,nmx
            reconl(i)=0.
         enddo
      endif
c
c      do j=1,n
cc     recombination coefficients adjusted for n ge 3 to give approximate
cc     recomb. emission spectrum for case b.see program atdath.f 
c         if(j.eq.1) ali=1.58e-13/t4**0.549
c         if(j.eq.2) ali=7.69e-14/t4**0.668
c         if(j.eq.3) ali=7.10e-14/t4**0.769
c         if(j.eq.4) ali=4.63e-14/t4**0.843
c         if(j.eq.5) ali=4.53e-14/t4**1.05
cc  fits to martin and so that ha/hb and hg/hb and a(tot) are reproduced 
cc  at 10**6 cm-3
c         ali=0.
c         if(te.ge.1250.) then
c            if(j.eq.1) ali=1.582e-13/t4**0.517
c            if(j.eq.2) ali=0.770e-13/t4**0.564
c            if(j.eq.3) ali=0.774e-13/t4**0.933
c            if(j.eq.4) ali=0.514e-13/t4**0.761
c            if(j.eq.5) ali=0.529e-13/t4**0.707
c         elseif(te.lt.1250.) then
c            t1250=te/1250.
c            if(j.eq.1) ali=4.640e-13/t1250**0.502
c            if(j.eq.2) ali=2.490e-13/t1250**0.515
c            if(j.eq.3) ali=5.382e-13/t1250**0.828
c            if(j.eq.4) ali=2.505e-13/t1250**0.619
c            if(j.eq.5) ali=2.302e-13/t1250**0.570
c         endif
c         reco(ie,ion,j)=ali
c      enddo
c
c ***  He i ; ie=2, ion=1 ***
c 
      ie=2
      ion=1
      n=nlev(ie,ion)      
c
c  Data from Almog, Y., and Netzer, H. 1989, MNRAS, 238, 57
c  to the different exc. levels.
c  Their data is given in the temp range : 10000 - 20000 K.
c
c  The sum over the recomb. coeff. is equal to total rec. coeff
c  given by Aldrovandi o Pequignot, -73, with very high accuracy, within the 
c  right temp. interval.
c
c  In order to get the right total recombination coeff also at low 
c  temp. the level reco(j) are normalized to add up to alftot(AoP73) 
c
c      sumrec=0.
c      do j=1,n
c        reco(ie,ion,j)=AL0(j)*T4**BE(j)
c        sumrec=sumrec+reco(ie,ion,j)
c      enddo
c
c  Total radiative recombination coeff. from 
c  Aldrovandi and Pequignot, 1973, A&A, 25, 137
c  The total recmb. coeff. is valid in the 
c  temp range : 100 - 100 000 K
c  (the dielectronic rec. coeff. important only 
c   for T > 50 000 K )
c
c      alftot=4.3e-13*t4**(-0.672)
c
c      do j=1,n
c        reco(ie,ion,j)=reco(ie,ion,j)*alftot/sumrec
c      enddo
c
c -----------------------------------------------------------
c Radiative recomb. coeff. from Verner and Ferland, 1996, ApJS,
c in press. The fit is valid in the temperature range 3K to 1e6 K.
c -----------------------------------------------------------
c
      if (te.lt.teheir(1)) then
         write(50,*) 'out of temp range for recomb. he i',te
         dtefr=0.d0
         it0=1
         it1=1
      elseif (te.ge.teheir(12)) then
         write(50,*) 'out of temp range for recomb. he i',te
         dtefr=0.d0
         it0=12
         it1=12
      else
         do it=2,12
            if (te.ge.teheir(it-1).and.te.lt.teheir(it)) then
               it0=it-1
               it1=it
               te0=teheir(it0)
               te1=teheir(it1)
               dtefr=(te-te0)/(te1-te0)
               goto 3
            endif
         enddo
 3       continue
      endif
      do il=1,16
         reco(ie,ion,il)=alfhei(il,it0)+dtefr*
     &        (alfhei(il,it1)-alfhei(il,it0))
      enddo
c
c
c  2s 1S ; data from Almog and Netzer , effective rate
      reco(ie,ion,3)=AL0(3)*T4**BE(3)
c
c  2p 1P ; data from Almog and Netzer , effective rate
      reco(ie,ion,5)=AL0(5)*T4**BE(5)
c
c  4p 3P ; no topbase data, extrapolated from 2p 3P,
c          3p 3P at low T. Data from Almog and Netzer
c          for 10000 < T < 20000 K.
      if (te.ge.10000) then
         reco(ie,ion,10)=AL0(10)*T4**BE(10)
      elseif (te.ge.5000.and.te.lt.10000.) then
         reco(ie,ion,10)=8.86e-15*t4**(-0.60)
      elseif (te.ge.1000.and.te.lt.5000.) then
         reco(ie,ion,10)=9.13e-15*t4**(-0.557)
      elseif (te.lt.1000.) then
         reco(ie,ion,10)=9.97e-15*t4**(-0.52)
      endif
c
c  4F 3F ;  no topbase data, extrapolated from 4s 3S,
c           4p 3P, 4d 3D at low T.
c           Data from Almog and Netzer for
c           10000 < T < 20000 K. 
c
      if (te.ge.10000) then
         reco(ie,ion,12)=AL0(12)*T4**BE(12)
      elseif (te.ge.5000.and.te.lt.10000.) then
         reco(ie,ion,12)=4.16e-15*t4**(-0.80)
      elseif (te.ge.1000.and.te.lt.5000.) then
         reco(ie,ion,12)=4.6e-15*t4**(-0.64)
      elseif (te.lt.1000.) then
         reco(ie,ion,12)=6.1e-15*t4**(-0.52)
      endif
c
c  il = 13 ;  5 3L -   7 3L
c  il = 14 ;  8 3L -  12 3L
c  il = 15 ; 13 3L -  25 3L
c  il = 16 ; 26 3L - 100 3L
c
c  the exponents guessed for 1000 < T < 10000 K
c  it is set equal to -0.5 for T < 1000 K
c
      if (te.ge.10000.) then
         reco(ie,ion,13)=AL0(13)*T4**BE(13)
         reco(ie,ion,14)=AL0(14)*T4**BE(14)
         reco(ie,ion,15)=AL0(15)*T4**BE(15)
         reco(ie,ion,16)=AL0(16)*T4**BE(16)
      elseif (te.ge.5000.and.te.lt.10000) then
         reco(ie,ion,13)=3.49e-14*T4**(-0.75)
         reco(ie,ion,14)=2.16e-14*T4**(-0.65)
         reco(ie,ion,15)=1.52e-14*T4**(-1.20)
         reco(ie,ion,16)=7.91e-15*T4**(-0.90)
      elseif (te.ge.1000.and.te.lt.5000) then
         reco(ie,ion,13)=3.87e-14*T4**(-0.60)
         reco(ie,ion,14)=2.32e-14*T4**(-0.55)
         reco(ie,ion,15)=2.08e-14*T4**(-0.75)
         reco(ie,ion,16)=9.41e-15*T4**(-0.65)
      elseif (te.lt.1000) then
         reco(ie,ion,13)=4.87e-14*T4**(-0.50)
         reco(ie,ion,14)=2.60e-14*T4**(-0.50)
         reco(ie,ion,15)=3.69e-14*T4**(-0.50)
         reco(ie,ion,16)=1.33e-14*T4**(-0.50)
      endif
c----------------
c
      a=3.294e-11
      b=0.6910
      T0=1.554e1
      T1=3.676e7
      sqrt_tet0=sqrt(te/T0)
c
      alftot=a/(sqrt_tet0*(1.+sqrt_tet0)**(1-b)*
     &     (1.+sqrt(te/T1))**(1.+b))
c
      alfgs=reco(ie,ion,1)
c      alfexc=alftot-alfgs
c
c Adjust the fictitious levels 13 - 16 the get the right 
c total recombination coefficient.
c 
      alfsum=0.d0
      do il=1,12
         alfsum=alfsum+reco(ie,ion,il)
      enddo
c
      alftop=alftot-alfsum
c
      alf4sum=0.d0
      do il=13,16
         alf4sum=alf4sum+reco(ie,ion,il)
      enddo
c
      if (alftop.gt.0.) then
c alftop > 0 for T > 25 K
         do il=13,16
            reco(ie,ion,il)=reco(ie,ion,il)*alftop/alf4sum
         enddo
      endif
c
c      alfsum=0.d0
c      do il=1,16
c         alfsum=alfsum+reco(ie,ion,il)
c      enddo
c
c      write(,777) te,(reco(ie,ion,il),il=1,16),alftot,alfsum      
c
c  o i ; ie = 5, ion = 1
c     temp in 0.1 eV
      t01ev=te/1164.
c
      ie=5
      ion=1
      n=nlev(ie,ion)
c
      do i=1,n
         a=0.
c rec. rates from Julienne, P.S., Davies, J. & Oran, E. 1974, 
c J. Geophys. Res. 79, 2540  
c 3P divided according to statistical weights. Total 4.4e-13
         IF(I.EQ.1) A=2.44
         IF(I.EQ.2) A=1.466
         IF(I.EQ.3) A=0.4888
         IF(I.EQ.4) A=0.
         IF(I.EQ.5) A=0.
         IF(I.EQ.6) A=0.10
         IF(I.EQ.7) A=0.04
         IF(I.EQ.8) A=2.1
         IF(I.EQ.9) A=0.11
         IF(I.EQ.10) A=0.
         IF(I.EQ.11) A=0.
         IF(I.EQ.12) A=2.7
         IF(I.EQ.13) A=1.2
c correction to give same total rate as Chung, Lin & Lee 
c + same temp. dep.      
         a=(1.47/1.065)*a*1.e-13/t01ev**0.8433
         reco(ie,ion,i)=a
      enddo
c
c960616      suma=0.
c960616      do i=1,n
c960616        a=0.
c
c960616        if(i.eq.1) a=1.1e-13/tev**.7
c960616        if(i.eq.4) a=1.6e-14/tev0**.7
c960616        if(i.eq.5) a=.14e-13/tev0**.7
c960616        if(i.eq.6) a=4.77e-13/tev0**.7
c960616        if(i.eq.7) a=0.47e-13/tev0**.7
c960616        if(i.eq.8) a=1.68e-13/tev0**.7
c960616        if(i.eq.9) a=0.52e-13/tev0**.7
c960616        reco(ie,ion,i)=a
c960616        suma=suma+a
c960616      enddo
c
c  The total radiative recomb. coeff. are taken
c  from Chung, Lin, and Lee, 1991, Phys. Rev. A, 43, 3433.
c  Their values are valid in the temp. range 400 - 30000 K. 
c
c960616      if (te.le.10000.) then
c960616         a=5.657d-10
c960616         eta=0.8433
c960616      elseif (te.gt.10000.) then
c960616         a=1.659d-10
c960616         eta=0.7099
c960616      endif
c960616      alocll=a/(te**eta)
c
c  The recomb. coeff. to the various levels are just
c  rescaled to give the total recomb. coeff. as given
c  by Chung, Lin, and Lee, 1991.
c
c960616      do i=1,n
c960616         reco(ie,ion,i)=reco(ie,ion,i)*alocll/suma
c960616      enddo
c  ots for o i except in the hydrogen zone
      if (i87a.eq.1.or.i93j.eq.1) then
         iz=izmod(id)
c  for 87a
         iz0=6
         if (iz.ne.iz0) then
            reco(ie,ion,1)=0.
c960616
            reco(ie,ion,2)=0.
            reco(ie,ion,3)=0.
         endif
      elseif (isn.eq.1) then
         if (absn(id,ieo).lt.1.e-4) then
            reco(ie,ion,1)=0.
            reco(ie,ion,2)=0.
            reco(ie,ion,3)=0.
         endif
      endif
c
c  Dielectronic recombination from 
c  Nussbaumer and Storey, 1983, A&A, 126, 75.
c
c  The total diel. recomb. coeff. is divided up
c  between the exc. levels (ad hoc)
c  in the same proportions as the radiative recomb.
c
c  dielectronic recomb is important only at very high
c  temp. (> 27000 K), exclude for the moment
c
c      if (te.le.20000) then
c         a=-0.0001
c         b= 0.0001
c         c= 0.0956
c         d= 0.0193
c         f= 0.4106
c      elseif (te.gt.20000) then
c         a= 0.3715
c         b=-0.0239
c         c=-0.0597
c         d= 0.0678
c         f= 0.7993
c      endif
c      alfd=1.e-12*((a/t4)+b+c*t4+d*t4*t4)*
c     &     t4**(-1.5)*fexp(-f/t4)
c      if (alfd.le.0.) alfd=0.
c
c      do i=1,n
c         reco(ie,ion,i)=reco(ie,ion,i)+reco(ie,ion,i)*alfd/alocll
c      enddo
c
cnew9804
c *** Mg I ***
      ie=iemg
      ion=1
      n=nlev(iemg,1)
      do il=1,nl
         reco(ie,ion,il)=0.d0
      enddo
      if (ab(iemg).gt.0.) then
c
c Total recomb. coeff. ; Mg II -> Mg I (almg1)
c radiative recomb. coeff. from : Shull & vanSteenberg, 1982 (alrad)
c high-temp. dielectronic recomb. coeff. from : Shull & vanSteenberg, 1982 (aldht)
c low-temp. dielectronic recomb. coeff. from : Nussbaumer & Storey, 1986 (aldlt)
c low-temp. dielectronic recomb. coeff. to g.s. from : Nussbaumer & Storey, 1986 (aldlt1)
c
         alrad=1.40e-13*t4**(-0.855)
c
         aldht=4.49e-4*(te**(-1.5))*dexp(-5.01e4/te)*
     &        (1.+2.10e-2*dexp(-2.81e4/te))
c
         aldlt=1.e-12*(1.2044/t4-4.6836+7.6620*t4-0.5930*t4*t4)*
     &        t4**(-1.5)*dexp(-1.6260/t4)
         almg=alrad+aldht+aldlt
c
         aldlt1=1.e-12*(0.1028/t4-0.3270+1.3742*t4-0.0742*t4*t4)*
     &        t4**(-1.5)*dexp(-1.2735/t4)
         frgsmg=aldlt1/aldlt
c aldlt1,aldlt not good for T<2000 K.
c At T=2000 frgsmg=0.94
c Uncertain !! Use frgsmg=0.30 for all T, for now !!
         frgsmg=0.30
         if (frgsmg.lt.0.or.frgsmg.gt.1) write(50,*) 'Error in recomb for
     &Mg I',te,frgsmg,aldlt1,aldlt
         gsum2=0.d0
         do il=2,n
            gsum2=gsum2+g(iemg,1,il)
         enddo
c
c Place the recomb. in each level prop. to the statistical weight.
c Exception for the g.s..The same fraction of the rad. and high-temp 
c diel. rec. as for the low-temp diel rec to the g.s. is assumed. 
c
         c1=almg*(1-frgsmg)/gsum2
         reco(ie,ion,1)=almg*frgsmg
         if (iots.eq.1) reco(ie,ion,1)=0.d0
         reco(ie,ion,2)=c1*g(iemg,1,2)
         reco(ie,ion,3)=c1*g(iemg,1,3)
         reco(ie,ion,4)=c1*g(iemg,1,4)
         reco(ie,ion,5)=c1*g(iemg,1,5)
         reco(ie,ion,6)=c1*g(iemg,1,6)
         reco(ie,ion,7)=c1*g(iemg,1,7)
         reco(ie,ion,8)=c1*g(iemg,1,8)
         reco(ie,ion,9)=c1*g(iemg,1,9)
         reco(ie,ion,10)=c1*g(iemg,1,10)
      endif
cnew9804
c *** Si I ***
c Data from Nahar, S.N., 1995, ApJS, 101, 423
c Direct recombination to the individual levels
      ie=iesi
      ion=1
      do il=1,nl
         reco(ie,ion,il)=0.d0
      enddo
      if (ab(iesi).gt.0) then
         t3q=1.d0/dsqrt(te/1000.)
c 3p 3P0
         reco(ie,ion,1)=1.68d-13*t3q
c 3p 3P1
         reco(ie,ion,2)=5.03d-13*t3q
c 3p 3P2
         reco(ie,ion,3)=8.39d-13*t3q
c 3p 1D
         reco(ie,ion,4)=3.66d-13*t3q
c 3p 1S
         reco(ie,ion,5)=3.23d-14*t3q
         reco(ie,ion,6)=0.
         reco(ie,ion,7)=0.
c 3p 3Do
         reco(ie,ion,8)=5.21d-14*t3q
c 4p 1P
         reco(ie,ion,9)=1.63d-14*t3q
c 3d 1Do
         reco(ie,ion,10)=3.34d-14*t3q
c 4p 3D
         reco(ie,ion,11)=1.21d-13*t3q
c 4p 3P
         reco(ie,ion,12)=7.16d-14*t3q
c 3d 3Fo
         reco(ie,ion,13)=1.15d-13*t3q
c 4p 1D
         reco(ie,ion,14)=4.27d-14*t3q
c 3d 3Po
         reco(ie,ion,15)=3.33d-14*t3q
c 3d 3Do
         reco(ie,ion,16)=2.07d-14*t3q
c 5p 3D
         reco(ie,ion,17)=4.11d-14*t3q
c 5p 3P
         reco(ie,ion,18)=5.18d-14*t3q
c 4d 3Fo
         reco(ie,ion,19)=3.01d-14*t3q
c 5p 3S
         reco(ie,ion,20)=1.57d-14*t3q
c 5p 1D
         reco(ie,ion,21)=2.98d-14*t3q
         recsum=0.d0
         do il=1,21
            recsum=recsum+reco(ie,ion,il)
         enddo
c
c  Total recombination
c  New recombination for Si II -> Si I 970311
c  from Nahar, 1995, ApJS, 101, 423.
c  Erratum in 1996, ApJS, 106,213 (cnew9804)
c  Both radiative and dielectronic processes included in this rate
c  The rate to the g.s. is valid at least in the interval 100 - 10000 K
         data xltsi/1.0,1.1,1.2,1.3,1.4,1.5,1.6,1.7,1.8,1.9,2.0,
     &        2.1,2.2,2.3,2.4,2.5,2.6,2.7,2.8,2.9,3.0,3.1,3.2,
     &        3.3,3.4,3.5,3.6,3.7,3.8,3.9,4.0,4.1,4.2,4.3,4.4,
     &        4.5,4.6,4.7,4.8,4.9,5.0/
         data alsiv/5.02e-11,4.41e-11,3.87e-11,3.40e-11,2.99e-11,
     &        2.62e-11,2.30e-11,2.02e-11,1.77e-11,1.55e-11,
     &        1.36e-11,1.19e-11,1.05e-11,9.19e-12,8.06e-12,
     &        7.07e-12,6.20e-12,5.45e-12,4.79e-12,4.21e-12,
     &        3.72e-12,3.30e-12,2.94e-12,2.65e-12,2.40e-12,
     &        2.19e-12,2.01e-12,1.87e-12,1.76e-12,1.67e-12,
     &        1.60e-12,1.62e-12,1.83e-12,2.44e-12,3.54e-12,
     &        5.06e-12,6.67e-12,7.98e-12,8.66e-12,8.72e-12,
     &        8.24e-12/
         xlogt=log10(te)
         if (xlogt.ge.xltsi(1).and.xlogt.lt.xltsi(41)) then
            do ic=2,41
               if (xlogt.ge.xltsi(ic-1).and.xlogt.lt.xltsi(ic)) then
                  ic1=ic-1
                  ic2=ic
                  alsi=alsiv(ic1)+
     &                 ((xlogt-xltsi(ic1))/(xltsi(ic2)-xltsi(ic1)))*
     &                 (alsiv(ic2)-alsiv(ic1))
                  goto 41
               endif
            enddo
         elseif (xlogt.lt.xltsi(1)) then
            alsi=alsiv(1)
         elseif (xlogt.ge.xltsi(41)) then
            alsi=alsiv(41)
         endif
 41      continue
         if (recsum.gt.alsi) write(50,*) 'Check Si I recomb !',te,recsum,alsi
         c1=alsi/recsum
         do il=1,21
            reco(ie,ion,il)=c1*reco(ie,ion,il)
         enddo
c         write(*,*) tday,te,alsi,recsum,c1
         if (iots.eq.1) then
            reco(ie,ion,1)=0.d0
            reco(ie,ion,2)=0.d0
            reco(ie,ion,3)=0.d0
         endif
      endif
c
c  ca ii ; ie=12, ion=2
      ie=ieca
      ion=2
      n=nlev(ie,ion)
c
cxx ?? preliminary!!!, include ct with o i !!!
c
c     recombination coeff.  (shull and van steenberg)
c
c      ald=3.28e-4*fexp(-3.64d0/t4)*(1.+.091*fexp(-24.5d0/t4))/t15
c      reco(ie,ion,1)=1.12e-13/t4**.900+ald
      ald=5.84e-2*fexp(-38.5d0/t4)*(1.+.110*fexp(-24.5d0/t4))/te**1.5
      reco(ie,ion,1)=6.78e-13/t4**.800+ald
      do i=2,n
        reco(ie,ion,i)=0.
      enddo
c
c  For Fe I and Fe II :
c
c  Radiative and dielectronic recombination coefficients
c  (recombinations sec**-1 ion**-1 electron**-1 cm**3)
c  from Shull and Van Steenberg, 1982, ApJS, 48, 95.
c  The fits are for the temperature interval 
c  1250 <(T/Zeff**)<20000 K.
c
c  The fraction of the radiative recombinations going to the 
c  ground state,frgs, at a temp. 1.e4 * Zeff**2 is taken from 
c  Sarazin, Shull and Woods, 1981, ApJ, 249, 399.
c
c  Here I just assume that the same fraction holds for the
c  dielectronic recombination. Ad. hoc.
c
c  The fractions going into excited levels are just guessed !
c
c  960215 NEW dielectronic recombination rates for the Fe-ions
c  from Arnaud & Raymond, 1992, ApJ, 398, 394
c
cfex recomb. Better recomb coeff to different levels needed
cfex         for fe i and fe ii. Just guessed now.
c
c  970602 New recombination rates to different levels in Fe I
c  and Fe II as given by Nahar, see /FEDAT/FEREC
c  The recombination coeffients include both 
c  radiative and dielectronic contributions.
c  run 70.1; run 70.2; run 70.3; run 70.4; run 70.5; run 70.7
c  run 80.1; run 80.2 ; runs >= 70.1
c
      call ferecomb(te,recof1,recof2,recof3)
c Big Fe I atom. run 70.6
c      call ferecomb_big(te,recof1,recof2)
      goto 11
c
c  fe i ; ie=iefe, ion=1
c
      if (ife.le.1) then
         ie=iefe
         ion=1
         n=nlev(ie,ion)
css         bdcorr=0.59d0
css         supr=1.d0
css         call diel(1.58e-3,6.e4,0.456e0,8.97e4,te,ald)
css         ald=bdcorr*supr*ald
c      ald=1.58e-3*fexp(-6.d0/t4)*(1.+0.456*fexp(-8.97d0/t4))/t15
c
         ald=2.20e-4*fexp(-5.12/xkt)+1.00e-4*fexp(-12.9/xkt)
         ald=ald/te**1.5
         alr=1.42e-13*(t4**(-0.891))
         alf=ald+alr
c
c  no recombination to excited levels in fe I 
c  right now
c  -------------------------------------------
c         recof1(1)=alf
c         do i=2,n
c            recof1(i)=0.
c         enddo
c  -------------------------------------------
         frgs=0.019
         recof1(1)=frgs*alf
c
c  Recombinations to excited levels. A pure guess !
c
         alfex=(1.-frgs)*alf
c
         sume=0.
cr         do i=2,5
         do i=2,n
            ee(i)=ef1(i)*ef1(i)
            sume=sume+ee(i)
         enddo
c
         sumrec=recof1(1)
cr         do i=2,5
         do i=2,n
            recof1(i)=(ee(i)/sume)*alfex
            sumrec=sumrec+recof1(i)
         enddo
cr>
c         do i=6,n
c            recof1(i)=0.
c         enddo
cr<
      endif
c
c  fe ii ; ie=iefe, ion=2
c
      if (ife.le.1) then
         ie=iefe
         ion=2
         n=nlev(ie,ion)
c
css         ald=8.38e-3*fexp(-19.4d0/t4)*(1.+0.323*fexp(-17.1d0/t4))/t15
         ald=2.30d-3*fexp(-16.7/xkt)+2.70d-3*fexp(-31.4/xkt)
         ald=ald/te**1.5
         alr=1.02e-12*(t4**(-0.843))
         alf=ald+alr
c
c  no recombination to excited levels in fe II 
c  right now
c  -------------------------------------------
c         recof2(1)=alf
c         do i=2,n
c            recof2(i)=0.
c         enddo
c  -------------------------------------------
         frgs=0.004
         recof2(1)=frgs*alf
c
c  Recombinations to excited levels. A pure guess !
c
         alfex=(1.-frgs)*alf
c
         sume=0.
cr         do i=2,5
         do i=2,n
            ee(i)=ef2(i)*ef2(i)
            sume=sume+ee(i)
         enddo
c
         sumrec=recof2(1)
cr         do i=2,5
         do i=2,n
            recof2(i)=(ee(i)/sume)*alfex
            sumrec=sumrec+recof2(i)
         enddo
c
cr>
c         do i=6,n
c            recof2(i)=0.
c         enddo
cr<
      endif
c 11   continue
c
c  fe iii ; ie=iefe, ion=3
c
      if (ife.le.1) then
         ie=iefe
         ion=3
         n=nlev(ie,ion)
c
css         call diel(1.54e-2,3.31e5,0.31e0,2.73e5,te,ald)
css         ald=supr*ald
         ald=1.50d-2*fexp(-28.6/xkt)+4.70d-3*fexp(-52.1/xkt)
         ald=ald/te**1.5
         alr=3.32d-12/t4**.746
         alf=ald+alr
c
c  no recombination to excited levels in fe III 
c  right now
c  -------------------------------------------
c         recof3(1)=alf
c         do i=2,n
c            recof3(i)=0.
c         enddo
c  -------------------------------------------
         frgs=0.37
         recof3(1)=frgs*alf
c
c  Recombinations to excited levels. A pure guess !
c
         alfex=(1.-frgs)*alf

         sume=0.
cr         do i=2,5
         do i=2,n
            ee(i)=ef3(i)*ef3(i)
            sume=sume+ee(i)
         enddo

         sumrec=recof3(1)
cr         do i=2,5
         do i=2,n
            recof3(i)=(ee(i)/sume)*alfex
            sumrec=sumrec+recof3(i)
         enddo
cr>
c         do i=6,n
c            recof3(i)=0.
c         enddo
cr<
      endif
 11   continue
c
c  fe iv ; ie=iefe, ion=4
c
      if (ife.le.1) then
         ie=iefe
         ion=4
         n=nlev(ie,ion)
c
css         supr=1.d0
css         call diel(3.75e-2,4.32e5,0.411e0,3.49e5,te,ald)
css         ald=supr*ald
         ald=3.80e-2*fexp(-37.3/xkt)+1.60e-2*fexp(-67.4/xkt)
         ald=ald/te**1.5
         alr=7.80d-12/t4**0.682
         alf=ald+alr
c
c  no recombination to excited levels in fe IV 
c  right now
c  -------------------------------------------
c         recof4(1)=alf
c         do i=2,n
c            recof4(i)=0.
c         enddo
c  -------------------------------------------
         frgs=0.54
         recof4(1)=frgs*alf
c
c  Recombinations to excited levels. A pure guess !
c
         alfex=(1.-frgs)*alf
c
         sume=0.
cr         do i=2,5
         do i=2,n
            ee(i)=ef4(i)*ef4(i)
            sume=sume+ee(i)
         enddo

         sumrec=recof4(1)
cr         do i=2,5
         do i=2,n
            recof4(i)=(ee(i)/sume)*alfex
            sumrec=sumrec+recof4(i)
         enddo
cr>
c         do i=6,n
c            recof4(i)=0.
c         enddo
cr<
      endif
c
      return
      end


      subroutine recomb_h(te)
      implicit real*8 (a-h,o-z)
      real*8 Imin,Iplus
      include 'INCLUDE/PARAM.IOR'
      common/chirec/tehr(40),alfth(40),reconl(nmx),recon(nm)
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
      common/cgf/G0p(nmx),G0m(nmx),initgf
      common/cint/y,n,l,lp
      data eps/0.000001/
      data err/0.01/
      dimension rnl(0:nm)
c
c  The recombination coefficients are calculated according to 
c  Brocklehurst, M., 1971, MNRAS, 153, 471.
c
      initgf=1
      t4=te/1.e4
      y=15.778/t4
c
      const=2.8128d-15
      a1=0.d0
      do n=1,nmn
         xn=dble(n)
         c1=const*2.*sqrt(y)/(xn*xn)
         do l=0,n-1
            xl=dble(l)
            if (l.eq.0) then
               Imin=0.d0
            else
c l' = l-1
               lp=l-1
c
c  find upper integration limit
               call findup(n,l,lp,y,eps,xkm,fxk)
               b=xkm*xkm
c
               call rombh(a1,b,err,fim,fail,k)
               Imin=xl*y*fim
            endif
c l' = l+1
            lp=l+1
c
c  find upper integration limit
            call findup(n,l,lp,y,eps,xkm,fxk)
            b=xkm*xkm
c
            call rombh(a1,b,err,fip,fail,k)
            Iplus=dble(l+1)*y*fip
c            nl=nlm(n,l)
            rnl(l)=c1*(Imin+Iplus)
c            reconl(nl)=c1*(Imin+Iplus)
         enddo
c
         recon(n)=0.d0
         if (n.le.nn) then
            do il=0,n-1
               nil=nlm(n,il)
               reconl(nil)=rnl(il)
               recon(n)=recon(n)+reconl(nil)
            enddo
         elseif (n.gt.nn) then
            do il=0,n-1
               recon(n)=recon(n)+rnl(il)
            enddo
         endif
      enddo
c
      return
      end



      subroutine findup(n,l,lp,y,eps,xkm,fxk)
      implicit real*8(a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
      common/cionpot/e00q(mie,mion)
c
      e00=e00q(ieh,1)
      xkmax=5.d0*(e00-ehn(n))/e00
c
      gf0=gf(n,l,0.d0,lp)
      theta0=gf0*gf0
      fx0=theta0
c
      xn=dble(n)
      xk2=xkmax*xkmax
      xk=xkmax
      sn2k2=1.+xn*xn*xk2
      gfk=gf(n,l,xk,lp)
      theta=sn2k2*gfk*gfk     
      fxk=sn2k2*sn2k2*theta*exp(-xk2*y)/theta0
      dk=xk*0.11
c     
      if (fxk.lt.eps*0.1) then
         xk=sqrt(-log(eps)/y)
         xk2=xk*xk
         sn2k2=1.+xn*xn*xk2
         gfk=gf(n,l,xk,lp)
         theta=sn2k2*gfk*gfk     
         fxk=sn2k2*sn2k2*theta*exp(-xk2*y)/theta0
         dk=xk*0.11
c     
         if (fxk.ge.eps) then
            do while (fxk.ge.eps)
               xk=xk+dk
               xk2=xk*xk
               sn2k2=1.+xn*xn*xk2
               gfk=gf(n,l,xk,lp)
               theta=sn2k2*gfk*gfk     
               fxk=sn2k2*sn2k2*theta*exp(-xk2*y)/theta0
            enddo
         elseif (fxk.lt.eps) then
            do while (fxk.lt.eps)
               fxko=fxk
               if (xk.lt.dk) dk=xk*0.11
               xk=xk-dk
               xk2=xk*xk
               sn2k2=1.+xn*xn*xk2
               gfk=gf(n,l,xk,lp)
               theta=sn2k2*gfk*gfk     
               fxk=sn2k2*sn2k2*theta*exp(-xk2*y)/theta0
            enddo
            fxk=fxko
            xk=xk+dk
         endif
      elseif (fxk.ge.eps) then
         do while (fxk.ge.eps)
            xk=xk+dk
            xk2=xk*xk
            sn2k2=1.+xn*xn*xk2
            gfk=gf(n,l,xk,lp)
            theta=sn2k2*gfk*gfk     
            fxk=sn2k2*sn2k2*theta*exp(-xk2*y)/theta0
         enddo
      endif
c
      xkm=xk
c
      return
      end




      double precision function gf(n,l,xk,lp)
      implicit real*8 (a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
      common/cgf/G0p(nmx),G0m(nmx),initgf
      dimension GKnp(0:nm),GKnm(0:nm)
      data pi/3.141593/
c
      if (initgf.eq.1) then
         initgf=0
         call fG0
      endif
      xn=dble(n)
      xl=dble(l)
      xlp=dble(lp)
c
      xnf1=1.d0
      do i=2,n+l
         xnf1=xnf1*dble(i)
      enddo
c
      xnf2=1.d0
      do i=2,n-l-1
         xnf2=xnf2*dble(i)
      enddo
c
      xprod=1.d0
      do is=0,lp
         s=dble(is)
         xprod=xprod*(1.+s*s*xk*xk)
      enddo
      cgf=sqrt(xnf1*xprod/xnf2)*((2.*xn)**(l-n))
c
c  Gm = G(n,l,K,l')
c  G0(n,l,lp) = G(n,l,0,l')
c  G0p(n,l) = G(n,l,0,l+1)
c  G0m(n,l) = G(n,l,0,l-1)
c  GKn(l,lp) = G(n,l,K,l')
c  GKnp(l) = G(n,l,K,l+1)
c  GKnm(l) = G(n,l,K,l-1)
c
      if (xk.eq.0.) then
c  K = 0
         inl=nlm(n,l)
         if (lp.eq.l+1) then
            Gm=G0p(inl)
         elseif (lp.eq.l-1) then   
            Gm=G0m(inl)
         else
            Gm=0.d0
         endif
      elseif (xk.gt.0.) then
c  K > 0
         sn2k2=1.+xn*xn*xk*xk
         do i=0,nm
            GKnp(i)=0.d0
            GKnm(i)=0.d0
         enddo
c
         c1=1./sqrt(1.-exp(-2.*pi/xk))
         c2=exp(2.*xn-(2./xk)*atan(xn*xk))
         c3=sn2k2**(n+2)
         n_nm1=nlm(n,n-1)
         GKnp(n-1)=(c1*c2/c3)*G0p(n_nm1)
c
         if (n.eq.2) then
            GKnm(n-1)=(sn2k2/(2.*xn))*GKnp(n-1)
            GKnp(0)=(2.*xn-1)*sn2k2*xn*GKnp(n-1)
         elseif (n.gt.2) then
            GKnm(n-1)=(sn2k2/(2.*xn))*GKnp(n-1)
            GKnp(n-2)=(2.*xn-1)*sn2k2*xn*GKnp(n-1)
            GKnm(n-2)=(2.*xn-1)*(4.+(xn-1.)*sn2k2)*GKnm(n-1)
            do il=n-3,1,-1
               lr=il+2
               xlr=dble(lr)
               GKnp(il)=(4.*xn*xn-4.*xlr*xlr+
     &              xlr*(2.*xlr-1.)*sn2k2)*GKnp(lr-1)-
     &              4.*xn*xn*(xn*xn-xlr*xlr)*
     &              (1.+(xlr+1.)*(xlr+1.)*xk*xk)*GKnp(lr)
c
               lr=il+1
               xlr=dble(lr)
               GKnm(il)=(4.*xn*xn-4.*xlr*xlr+
     &              xlr*(2.*xlr+1.)*sn2k2)*GKnm(lr)-
     &              4.*xn*xn*(xn*xn-(xlr+1.)*(xlr+1.))*
     &              (1.+xlr*xlr*xk*xk)*GKnm(lr+1)
               if (il.le.l) goto 9
            enddo
c l = 0
            lr=2
            xlr=dble(lr)
            GKnp(0)=(4.*xn*xn-4.*xlr*xlr+xlr*(2.*xlr-1.)*sn2k2)*
     &              GKnp(lr-1)-
     &              4.*xn*xn*(xn*xn-xlr*xlr)*
     &              (1.+(xlr+1.)*(xlr+1.)*xk*xk)*GKnp(lr)
c
         endif
 9       continue
c
         if (lp.eq.l+1) then
            Gm=GKnp(l)
         elseif (lp.eq.l-1) then   
            Gm=GKnm(l)
         else
            Gm=0.d0
         endif
      endif
c
      gf=cgf*Gm
c
      return
      end



      subroutine fG0
      implicit real*8(a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
      common/cgf/G0p(nmx),G0m(nmx),initgf
      data pi/3.141593/
c
c  G0(n,l,lp) = G(n,l,0,lp) <=> K=0.
c  G0p(nl) = G(n,l,0,l+1)
c  G0m(nl) = G(n,l,0,l-1)
c
      do i=1,nmx
         G0p(i)=0.d0
         G0m(i)=0.d0
      enddo
c
c  n = 1
      G0p(nlm(1,0))=sqrt(pi/2.)*8.*4.*exp(-2.)
c
c  n = 2
      n=2
      xn=dble(n)
      xnf3=6.
      n_nm1=nlm(n,n-1)
      n_0=nlm(n,0)
      G0p(n_nm1)=sqrt(pi/2.)*(8.*xn/xnf3)*((4.*xn)**xn)*exp(-2.*xn)
      G0m(n_nm1)=(1./(2.*xn))*G0p(n_nm1)
      G0p(n_0)=(2.*xn-1.)*xn*G0p(n_nm1)      
c
      do n=3,nm
         xn=dble(n)
         xnf3=1.d0
         do i=1,2*n-1
            xnf3=xnf3*dble(i)
         enddo
         n_nm1=nlm(n,n-1)
         n_nm2=nlm(n,n-2)
         G0p(n_nm1)=sqrt(pi/2.)*(8.*xn/xnf3)*((4.*xn)**xn)*exp(-2.*xn)
         G0m(n_nm1)=(1./(2.*xn))*G0p(n_nm1)
c
         G0p(n_nm2)=(2.*xn-1.)*xn*G0p(n_nm1)
         G0m(n_nm2)=(2.*xn-1.)*(4.+(xn-1.))*G0m(n_nm1)
c
         do l=n-3,1,-1
            inl=nlm(n,l)
c
            lr=l+2
            n_lrm1=nlm(n,lr-1)
            n_lr=nlm(n,lr)
            xlr=dble(lr)
c
            G0p(inl)=(4.*xn*xn-4.*xlr*xlr+xlr*(2.*xlr-1.))*
     &           G0p(n_lrm1)-
     &           4.*xn*xn*(xn*xn-xlr*xlr)*
     &           G0p(n_lr)
c
            lr=l+1
            n_lrp1=nlm(n,lr+1)
            n_lr=nlm(n,lr)
            xlr=dble(lr)
c
            G0m(inl)=(4.*xn*xn-4.*xlr*xlr+xlr*(2.*xlr+1.))*
     &           G0m(n_lr)-
     &           4.*xn*xn*(xn*xn-(xlr+1.)*(xlr+1.))*
     &           G0m(n_lrp1)
         enddo
c l = 0
         lr=2
         n_lrm1=nlm(n,lr-1)
         n_lr=nlm(n,lr)
         xlr=dble(lr)
         G0p(nlm(n,0))=(4.*xn*xn-4.*xlr*xlr+xlr*(2.*xlr-1.))*
     &           G0p(n_lrm1)-
     &           4.*xn*xn*(xn*xn-xlr*xlr)*
     &           G0p(n_lr)
c
      enddo
c
      return
      end



      double precision function fxh(xk2)
      implicit real*8(a-h,o-z)
      common/cint/y,n,l,lp
c
      xk=sqrt(xk2)
      xn=dble(n)
      xl=dble(l)
      xlp=dble(lp)
      sn2k2=1.+xn*xn*xk2
c
      gfk=gf(n,l,xk,lp)
      theta=sn2k2*gfk*gfk
c
      fxh=sn2k2*sn2k2*theta*exp(-xk2*y)
c
      return
      end



      subroutine rombh(aa,bb,err,fi,fail,k)
      implicit real*8(a-h,o-z)
      dimension tr(100),w(100)
      a=aa
      b=bb
      dx=(b-a)/2.
      fi1=0.5*(fxh(a)+fxh(b))
      fi2=fxh(a+dx)
      tr(1)=dx*(fi1+fi2)
      n=1
      k=1
      w(1)=4.
1     fi=tr(1)
      w(k+1)=4.*w(k)
      n=2*n
      k=k+1
      tdx=dx
      dx=dx/2.
      x=a+dx
      do 2 i=1,n
      fi2=fi2+fxh(x)
2     x=x+tdx
      tr(k)=dx*(fi1+fi2)
      kk=k-1
      do 3 j=1,kk
      m=k-j
3     tr(m)=(w(j)*tr(m+1)-tr(m))/(w(j)-1.)
      fail=abs(tr(1)-fi)/abs(tr(1))
      if(fail-err)6,6,4
4     if(k-100)1,6,6
6     fi=tr(1)
      return
      end



      double precision function exp1(x)
      implicit real*8(a-h,o-z)
      if(x.lt.1.) then
            e1=-dlog(x)-.577216+x-.24991*x*x+.0552*x*x*x-.00976
     &            *x*x*x*x
      else
            e1=dexp(-x)*(x*x+2.334733*x+.250621)/((x*x+3.330657*x
     &            +1.681534)*x)
      endif
      exp1=e1
      return
      end

      double precision function e2(x)
      implicit real*8(a-h,o-z)
      if(x.gt.0.) goto 500
      e2=1.
      goto 400
500   if(x.gt.300.) goto 100
      if(x.gt.1.) goto 200
      e1=-dlog(x)-.577216+x-.24991*x*x+.0552*x*x*x-.00976
     &*x*x*x*x
      goto 300
200   e1=dexp(-x)*(x*x+2.334733*x+.250621)/((x*x+3.330657*x+1.681534
     &)*x)
300   e2=dexp(-x)-x*e1
      goto 400
100   e2=0.
400   continue
      return
      end

      subroutine gamion(x,gion,gamex,gionco)
cn
c  all rates should be divided by den
c  gion = gamma ionization rate ; s**-1
c  gionco = gamma ionization rate (s**-1) of CO ; As a first
c           approximation it is set = gion(ieo,1)
      implicit real*8(a-h,o-z)
c
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      include 'INCLUDE/DATMOL.NOMOL'
      common/cmolcr/simi(mim,ne),simd(mim,ne),ethmi(mim),
     &              ethmd(mim),mi(mim),md(mim)
      common/cmol/xm(mim),xmold(mid,mim),xiem(mid,mie),xemr,
     &            rmd(mie,mion,2),rmc(mie,mion,2),molesin,
     &            moles,iwrmol
      common/cmolgam/iem(mim)
      common/gampar/gaelh,gahe
      common/depthind/id
      common/ciz/iz
      common/cabun/amean,ab(mie),xa(mie,mion),xaold(mid,mie,mion),
     &     nion(mie)
      common/catdat/cq(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),aq(mie,mion,nl,nl)
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
      common/cfedat/cf1(nlf1,nlf1),cf2(nlf2,nlf2),cf3(nlf3,nlf3),
     &     cf4(nlf4,nlf4),cif1(nlf1),cif2(nlf2),cif3(nlf3),cif4(nlf4),
     &     gf1(nlf1),gf2(nlf2),gf3(nlf3),gf4(nlf4),ef1(nlf1),ef2(nlf2),
     &     ef3(nlf3),ef4(nlf4),af1(nlf1,nlf1),af2(nlf2,nlf2),
     &     af3(nlf3,nlf3),af4(nlf4,nlf4),cpf2(3,3)
      common/cionpot/e00(mie,mion)
      common/cchieff/chieff(mie,mion)
      dimension gion(mie,mion),gamex(mie,mion,nlf)
c      dimension ion(mie)
      common/cden/den
      common/enbudg/extot,sumh,he
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/diag/idiag
      common/oldlevpop/xoldie(mid,mie,mion,nl),xoldief1(mid,nlf1),
     &     xoldief2(mid,nlf2),xoldief3(mid,nlf3),xoldief4(mid,nlf4),
     &     xoldieh1(mid,nmax)
      common/cfrac/frionh,frexh3,frexo2
      common/cfrtot/frhe,frion,frexc
      common/csn/i87acl,i87a,i93j,isn
      common/csnmod/absn(mid,mie),ameansn(mid),densn(mid),rsn(midp1),
     &     vrsn(midp1),rsnlab(mid),fillsn(mid),dmsn(mid),iesnm(mid,2),
     &     tdsn,nrsn
      common/cmod/abmod(10,mie),vrmod(mid),rmod(mid),rlab(mid),
     &     dmmod(mid),ameanmod(10),denmod(mid),tdmod,
     &     nrtab(mid),izmod(mid),nrmod,mixm
      character*8 rsnlab
      dimension a(4),exeff(mie,mion,nl),effi(mie,mion)
      dimension exeffh1(nmax)
      data pi/3.1415926e0/,elch/1.60219d-12/,amu/1.660531e-24/
c
      if (i87a.eq.1.or.i93j.eq.1) iz=izmod(id)
c
      do il=1,nmax
         exeffh1(il)=0.
      enddo
      do ie=1,iemax
        do ion=1,mion
          chieff(ie,ion)=0.
          do il=1,nl
             exeff(ie,ion,il)=0.
          enddo
        enddo
      enddo
      xl4=dlog10(x)+4.d0
c
      if (i93j.eq.1) then
c  for 93J ; a rough approx for the gamma deposition
         iz93=iz
         if (iz93.eq.1) iz87=1
         if (iz93.eq.2) iz87=1
         if (iz93.eq.3) iz87=2 
         if (iz93.eq.4) iz87=3 
         if (iz93.eq.5) iz87=4 
         if (iz93.eq.6) iz87=5 
         if (iz93.eq.7) iz87=5 
         if (iz93.eq.8) iz87=6 
         iz=iz87
         if (iz93.eq.5) then
c  gamma ionization for He I from zone iz87=6
c he i
            a(1)=-0.1292d-1
            a(2)= 0.3014d-1
            a(3)=-0.2300d-1
            a(4)= 0.7044d-2
            c=42.76
c            asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
            asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
            chieff(iehe,1)=c*10**asum
            exeff(iehe,1,5)=0.030
         endif
      elseif (isn.eq.1) then
         iz87=0
c  fe - he ; iz87=1
         if (iesnm(id,1).eq.iefe.and.iesnm(id,2).eq.iehe) then
            iz87=1
         elseif (iesnm(id,1).eq.iehe.and.iesnm(id,2).eq.iefe) then
            iz87=1
         elseif (iesnm(id,1).eq.iefe) then
            iz87=1
c  si - s  ; iz87=2
         elseif (iesnm(id,1).eq.iesi.and.iesnm(id,2).eq.ies) then
            iz87=2
         elseif (iesnm(id,1).eq.ies.and.iesnm(id,2).eq.iesi) then
            iz87=2
         elseif (iesnm(id,1).eq.iesi.and.iesnm(id,2).eq.iefe) then
            iz87=2
c  o - si - s ; iz87=3
         elseif (iesnm(id,1).eq.ieo.and.iesnm(id,2).eq.iesi) then
            iz87=3
         elseif (iesnm(id,1).eq.ieo.and.iesnm(id,2).eq.ies) then
            iz87=3
         elseif (iesnm(id,1).eq.iesi.and.iesnm(id,2).eq.ieo) then
            iz87=3
c  o - c  ; iz87=4
         elseif (iesnm(id,1).eq.ieo.and.iesnm(id,2).eq.iec) then
            iz87=4
         elseif (iesnm(id,1).eq.iec.and.iesnm(id,2).eq.ieo) then
            iz87=4
         elseif (iesnm(id,1).eq.iec) then
            iz87=4
c  o      ; iz87=3
         elseif (iesnm(id,1).eq.ieo) then
            iz87=3
c  he   ; iz87=5
         elseif (iesnm(id,1).eq.iehe) then
            iz87=5
c  h - env ; iz87=6
         elseif (iesnm(id,1).eq.ieh) then
            iz87=6
         elseif (iesnm(id,1).eq.iefe) then
c  fe dominates but not together with he
            iz87=2
         endif
cqcq run 99
c         iz87=3
c         iz87=2
cqcq
         if (iz87.eq.0) then
            write(50,*) 'gamion : iz = 0 !',id
            stop
         endif
         iz=iz87
      endif
c
      if (iz.eq.1) then
c
c  fe - he
c
c  fe i ; ie = 13, ion = 1
        ie=iefe
        ion=1
        a(1)=-0.2592d-2
        a(2)= 0.1771d-1
        a(3)= 0.1755d-2
        a(4)= 0.2686d-2
        c=13.06
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ie,ion)=c*10**asum
c  fe ii ; ie = 13, ion = 2
        ie=iefe
        ion=2
        a(1)= 0.1127d-2
        a(2)= 0.8353d-2
        a(3)=-0.9045d-2
        a(4)= 0.5122d-2
        c=17.54
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ie,ion)=c*10**asum
c  fe iii ; ie = 13, ion = 3 ; ad hoc
        ie=iefe
        ion=3
        a(1)= 0.11d-2
        a(2)= 0.84d-2
        a(3)=-0.90d-2
        a(4)= 0.51d-2
        c=35.
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ie,ion)=c*10**asum
c  fe iv ; ie = 13, ion = 4 ; ad hoc
        ie=iefe
        ion=4
        a(1)= 0.11d-2
        a(2)= 0.84d-2
        a(3)=-0.90d-2
        a(4)= 0.51d-2
        c=54.
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ie,ion)=c*10**asum
c  he i ; ie = 2, ion = 1
        a(1)=-0.5616d-2
        a(2)= 0.2013d-1
        a(3)=-0.1804d-1
        a(4)= 0.6516d-2
        c=49.2
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iehe,1)=c*10**asum
        exeff(iehe,1,5)=0.264
c  he ii ; ie = 2, ion = 2
        a(1)=-0.8702d-2
        a(2)=0.3050d-1
        a(3)=-0.2551d-1
        a(4)=0.7624d-2
        c=311.9
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iehe,2)=c*10**asum
c  c i ; ie = 3, ion = 1
        a(1)=-0.1477d-3
        a(2)= 0.9677d-2
        a(3)=-0.8974d-2
        a(4)= 0.5074d-2
        c=9.69
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iec,1)=c*10**asum
c  c ii ; ie = 3, ion = 2
        a(1)= 0.2568d-2
        a(2)= 0.6712d-2
        a(3)=-0.1013d-1
        a(4)= 0.5417d-2
        c=41.40
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iec,2)=c*10**asum
c  o i ; ie = 5, ion = 1
        a(1)=-0.7145d-2
        a(2)= 0.2254d-1
        a(3)=-0.1781d-1
        a(4)= 0.6384d-2
        c=12.25
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieo,1)=c*10**asum
c  o ii ; ie = 5, ion = 2
        a(1)=-0.9599d-2
        a(2)= 0.2710d-1
        a(3)=-0.2235d-1
        a(4)= 0.7152d-2
        c=39.26
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieo,2)=c*10**asum
c  ca i ; ie = 12, ion = 1
        a(1)= 0.4350d-2
        a(2)= 0.2777d-1
        a(3)=-0.7524d-2
        a(4)= 0.4130d-2
        c=3.417
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieca,1)=c*10**asum
c  ca ii ; ie = 12, ion = 2
        a(1)= 0.9099d-3
        a(2)= 0.8544d-2
        a(3)=-0.8160d-2
        a(4)= 0.4978d-2
        c=13.06
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieca,2)=c*10**asum
        exeff(ieca,2,2)=4.43d-5
        exeff(ieca,2,3)=1.15d-3
c  ne i ; ie = 6, ion = 1
        a(1)=-0.8697d-2
        a(2)= 0.2654d-1
        a(3)=-0.2226d-1
        a(4)= 0.7122d-2
        c=19.54
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iene,1)=c*10**asum
c  mg i
        a(1)= 0.9207d-2
        a(2)= 0.1434d-1
        a(3)= 0.9677d-2
        a(4)= 0.1097d-2
        c=6.08
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iemg,1)=c*10**asum
c  mg ii
        a(1)=-0.3504d-2
        a(2)= 0.1509d-1
        a(3)=-0.1229d-1
        a(4)= 0.5402d-2
        c=34.04
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iemg,2)=c*10**asum
c  si i
        a(1)= 0.5863d-2
        a(2)= 0.1177d-1
        a(3)= 0.6210d-2
        a(4)= 0.1977d-2
        c=5.54
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iesi,1)=c*10**asum
c  si ii
        a(1)= 0.7705d-2
        a(2)=-0.3531d-2
        a(3)=-0.7791d-3
        a(4)= 0.3948d-2
        c=21.88
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iesi,2)=c*10**asum
c  s i 
        a(1)=-0.2592d-2
        a(2)= 0.1363d-1
        a(3)=-0.9035d-2
        a(4)= 0.4944d-2
        c=6.03
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ies,1)=c*10**asum
c  s ii
        a(1)= 0.3918d-2
        a(2)= 0.3420d-2
        a(3)=-0.7186d-2
        a(4)= 0.4976d-2
        c=23.44
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ies,2)=c*10**asum
c  ar i
        a(1)=0.2427d-2
        a(2)=0.5861d-2
        a(3)=-0.7965d-2
        a(4)=0.5087d-2
        c=7.55
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iear,1)=c*10**asum
c        
      elseif (iz.eq.2) then
c
c  si - s
c
c  si i ; ie = 9, ion=1
        a(1)= 0.1020d-2
        a(2)= 0.1119d-1
        a(3)=-0.3194d-2
        a(4)= 0.2909d-2
        c=14.93
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iesi,1)=c*10**asum
c  si ii
        a(1)=-0.1220d-1
        a(2)= 0.2988d-1
        a(3)=-0.2188d-1
        a(4)= 0.5958d-2
        c=57.54
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iesi,2)=c*10**asum
c  s i ; ie = 10, ion=1
        a(1)=-0.1370d-1
        a(2)= 0.3118d-1
        a(3)=-0.2138d-1
        a(4)= 0.5638d-2
        c=15.52
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ies,1)=c*10**asum
c  s ii
        a(1)=-0.1614d-1
        a(2)= 0.3661d-1
        a(3)=-0.2588d-1
        a(4)= 0.6428d-2
        c=60.26
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ies,2)=c*10**asum
c  c i
        a(1)=-0.1444d-1
        a(2)= 0.3291d-1
        a(3)=-0.2325d-1
        a(4)= 0.5954d-2
        c=24.83
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iec,1)=c*10**asum
c  c ii
        a(1)=-0.1651d-1
        a(2)= 0.3715d-1
        a(3)=-0.2629d-1
        a(4)= 0.6405d-2
        c=105.0
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iec,2)=c*10**asum
c  mg i
        a(1)= 0.2780d-2
        a(2)= 0.1007d-1
        a(3)= 0.1362d-2
        a(4)= 0.2047d-2
        c=16.37
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iemg,1)=c*10**asum
c  mg ii
        a(1)=-0.1125d-1
        a(2)= 0.2695d-1
        a(3)=-0.1983d-1
        a(4)= 0.5255d-2
        c=84.92
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iemg,2)=c*10**asum
c  o i ; ie = 5, ion = 1
        a(1)=-0.2055d-1
        a(2)= 0.4235d-1
        a(3)=-0.2790d-1
        a(4)=0.6448d-2
        c=30.48
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieo,1)=c*10**asum
c  o ii
        a(1)=-0.2419d-1
        a(2)= 0.4927d-1
        a(3)=-0.3207d-1
        a(4)= 0.7017d-2
        c=95.72
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieo,2)=c*10**asum
c  ca i ; ie = 12, ion = 1
        a(1)=-0.1034d-1
        a(2)= 0.4101d-1
        a(3)=-0.1893d-1
        a(4)= 0.4919d-2
        c=8.855
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieca,1)=c*10**asum
c  ca ii ; ie = 12, ion = 2
        a(1)=-0.1499d-1
        a(2)= 0.3407d-1
        a(3)=-0.2373d-1
        a(4)= 0.6052d-2
        c=33.65
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieca,2)=c*10**asum
        exeff(ieca,2,2)=2.42d-3
        exeff(ieca,2,3)=3.99d-2
c  ne i ; ie = 6, ion = 1
        a(1)=-0.2371d-1
        a(2)= 0.4838d-1
        a(3)=-0.3151d-1
        a(4)= 0.6908d-2
        c=47.53
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iene,1)=c*10**asum
c  fe i ; ie = 13, ion = 1
        a(1)=-0.5525d-2
        a(2)= 0.1611d-1
        a(3)=-0.6897d-2
        a(4)= 0.3489d-2
        c=35.08
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,1)=c*10**asum
c  fe ii ; ie = 13, ion = 2
        a(1)=-0.1493d-1
        a(2)= 0.3394d-1
        a(3)=-0.2386d-1
        a(4)= 0.6024d-2
        c=44.67
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,2)=c*10**asum
c  fe iii  ; ad hoc
        a(1)=-0.15d-1
        a(2)= 0.34d-1
        a(3)=-0.24d-1
        a(4)= 0.60d-2
        c=80.
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,3)=c*10**asum
c  fe iv ; ad hoc
        a(1)=-0.15d-1
        a(2)= 0.34d-1
        a(3)=-0.24d-1
        a(4)= 0.60d-2
        c=130.
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,4)=c*10**asum
c  ar i
        a(1)=-0.1607d-1
        a(2)=0.3625d-1
        a(3)=-0.2546d-1
        a(4)=0.6366d-2
        c=19.50
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iear,1)=c*10**asum
      elseif (iz.eq.3) then
c
c  o - si - s
c
c  c i ; ie = 3, ion = 1
        a(1)=-0.5712d-2
        a(2)= 0.1636d-1
        a(3)=-0.5790d-2
        a(4)= 0.3010d-2
        c=17.95
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iec,1)=c*10**asum
c  c ii
        a(1)=-0.4711d-2
        a(2)= 0.1429d-1
        a(3)=-0.1054d-1
        a(4)= 0.4094d-2
        c=83.75
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iec,2)=c*10**asum
c
c  o i ; ie = 5, ion = 1
        a(1)=-0.1325d-1
        a(2)= 0.2855d-1
        a(3)=-0.1552d-1
        a(4)= 0.4456d-2
        c=24.66
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieo,1)=c*10**asum
        if(x.lt.0.1.and.x.gt.0.01)  then
           exeff(ieo,1,4)=0.0287*(0.01/x)**0.698
           exeff(ieo,1,5)=0.0093*(0.01/x)**0.544
c960616           exeff(ieo,1,2)=0.0287*(0.01/x)**0.698
c960616           exeff(ieo,1,3)=0.0093*(0.01/x)**0.544
        elseif(x.lt.0.01.and.x.gt.0.001)  then
           exeff(ieo,1,4)=0.0766*(0.001/x)**0.426
           exeff(ieo,1,5)=0.0171*(0.001/x)**0.264
c960616           exeff(ieo,1,2)=0.0766*(0.001/x)**0.426
c960616           exeff(ieo,1,3)=0.0171*(0.001/x)**0.264
        elseif(x.lt.0.001.and.x.gt.0.0001)  then
           exeff(ieo,1,4)=0.144*(0.0001/x)**0.274
           exeff(ieo,1,5)=0.0223*(0.0001/x)**0.115
c960616           exeff(ieo,1,2)=0.144*(0.0001/x)**0.274
c960616           exeff(ieo,1,3)=0.0223*(0.0001/x)**0.115
        endif
        exeff(ieo,1,5)=0.0185
        exeff(ieo,1,6)=0.0089
        exeff(ieo,1,7)=0.143
c960616        exeff(ieo,1,3)=0.0185
c960616        exeff(ieo,1,4)=0.0089
c960616        exeff(ieo,1,5)=0.143
c?? guess
        exeff(ieo,1,9)=0.046
        exeff(ieo,1,13)=0.049
c        exeff(ieo,1,9)=0.119
c960616        exeff(ieo,1,7)=0.046
c960616        exeff(ieo,1,8)=0.049
c960616        exeff(ieo,1,9)=0.119
c  o ii
        a(1)=-0.1404d-1
        a(2)= 0.3137d-1
        a(3)=-0.2198d-1
        a(4)= 0.5650d-2
        c=86.10
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieo,2)=c*10**asum
c  ca i ; ie = 12, ion = 1
        a(1)= 0.2240d-1
        a(2)= 0.7385d-1
        a(3)=-0.1687d-1
        a(4)= 0.3121d-2
        c=3.967
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieca,1)=c*10**asum
c  ca ii ; ie = 12, ion = 2
        a(1)=-0.5327d-2
        a(2)= 0.1594d-1
        a(3)=-0.5384d-2
        a(4)= 0.2991d-2
        c=23.93
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieca,2)=c*10**asum
        exeff(ieca,2,2)=3.09d-4
        exeff(ieca,2,3)=5.86d-3
c  si i ; ie = 9 , ion = 1
        a(1)= 0.1159d-1
        a(2)= 0.5411d-1
        a(3)= 0.1721d-2
        a(4)= 0.2654d-3
        c=6.03
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iesi,1)=c*10**asum
c  si ii
        a(1)=-0.2547d-2
        a(2)= 0.7454d-2
        a(3)= 0.7797d-3
        a(4)= 0.2171d-2
        c=38.46
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iesi,2)=c*10**asum
c  s i 
        a(1)=-0.9098d-2
        a(2)= 0.2793d-1
        a(3)=-0.7303d-2
        a(4)= 0.2887d-2
        c=10.40
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ies,1)=c*10**asum
c  s ii
        a(1)=-0.2169d-2
        a(2)= 0.8803d-2
        a(3)=-0.5712d-2
        a(4)= 0.3360d-2
        c=45.29
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ies,2)=c*10**asum
c  mg i
        a(1)= 0.1863d-1
        a(2)= 0.6159d-1
        a(3)= 0.4765d-2
        a(4)=-0.6498d-3
        c=5.88 
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iemg,1)=c*10**asum
c  mg ii
        a(1)=-0.1008d-1
        a(2)= 0.2300d-1
        a(3)=-0.9467d-2
        a(4)= 0.3329d-2
        c=65.61
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iemg,2)=c*10**asum
c  ne i ; ie = 6, ion = 1
        a(1)=-0.1500d-1
        a(2)= 0.3207d-1
        a(3)=-0.2163d-1
        a(4)= 0.5516d-2
        c=42.76
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iene,1)=c*10**asum
c
cfex gamion, wrong gamma-ion for fe for o-si-s
c
c  fe i ; ie = 13, ion = 1
        a(1)= 0.3289d-2
        a(2)= 0.5041d-1
        a(3)= 0.2273d-2
        a(4)= 0.2010d-3
        c=15.03
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,1)=c*10**asum
c  fe ii ; ie = 13, ion = 2
        a(1)=-0.4871d-2
        a(2)= 0.1380d-1
        a(3)=-0.6410d-2
        a(4)= 0.3228d-2
        c=33.50
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,2)=c*10**asum
c  fe iii ; ad hoc
        a(1)=-0.49d-2
        a(2)= 0.14d-1
        a(3)=-0.64d-2
        a(4)= 0.32d-2
        c=70.
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,3)=c*10**asum
c  fe iv ; ad hoc
        a(1)=-0.49d-2
        a(2)= 0.14d-1
        a(3)=-0.64d-2
        a(4)= 0.32d-2
        c=110.
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,4)=c*10**asum
c  ar i
        a(1)=-0.4906d-2
        a(2)=0.1341d-1
        a(3)=-0.6545d-2
        a(4)=0.3388d-2
        c=14.29
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iear,1)=c*10**asum
      elseif (iz.eq.4) then
c
c  o - c
c
c  c i ; ie = 3, ion = 1
        a(1)=-0.6885d-2
        a(2)=0.2153d-1
        a(3)=-0.6086d-2
        a(4)=0.2837d-2
        c=16.90
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iec,1)=c*10**asum
c  c ii
        a(1)=-0.3409d-2
        a(2)= 0.1217d-1
        a(3)=-0.8530d-2
        a(4)= 0.3713d-2
        c=81.28
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iec,2)=c*10**asum
c  o i ; ie = 5, ion = 1
        a(1)=-0.1218d-1
        a(2)= 0.2839d-1
        a(3)=-0.1416d-1
        a(4)= 0.4127d-2
        c=23.77
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieo,1)=c*10**asum
        exeff(ieo,1,4)=0.029
        exeff(ieo,1,5)=0.0096
        exeff(ieo,1,6)=0.0089
        exeff(ieo,1,7)=0.142
        exeff(ieo,1,9)=0.046
        exeff(ieo,1,13)=0.049
c        exeff(ieo,1,9)=0.117
c960616        exeff(ieo,1,2)=0.029
c960616        exeff(ieo,1,3)=0.0096
c960616        exeff(ieo,1,4)=0.0089
c960616        exeff(ieo,1,5)=0.142
c960616        exeff(ieo,1,7)=0.046
c960616        exeff(ieo,1,8)=0.049
c960616        exeff(ieo,1,9)=0.117
c  o ii
        a(1)=-0.1226d-1
        a(2)= 0.2862d-1
        a(3)=-0.2040d-1
        a(4)= 0.5381d-2
        c=85.11
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieo,2)=c*10**asum
c  mg i
        a(1)= 0.4506d-1
        a(2)= 0.7582d-1
        a(3)=-0.6229d-3
        a(4)=-0.2527d-3
        c=4.66 
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iemg,1)=c*10**asum
c  mg ii
        a(1)=-0.1174d-1
        a(2)= 0.2854d-1
        a(3)=-0.9996d-2
        a(4)= 0.3183d-2
        c=62.23
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iemg,2)=c*10**asum
c  si i ; ie = 9 , ion = 1
        a(1)= 0.2593d-1
        a(2)= 0.7092d-1
        a(3)=-0.4206d-2
        a(4)= 0.7223d-3
        c=4.91
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iesi,1)=c*10**asum
c  si ii
        a(1)=-0.4084d-2
        a(2)= 0.1207d-1
        a(3)= 0.1285d-2
        a(4)= 0.1849d-2
        c=35.81
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iesi,2)=c*10**asum
c  ca i ; ie = 12, ion = 1
        a(1)= 0.5320d-1
        a(2)= 0.8277d-1
        a(3)=-0.2158d-1
        a(4)=0.3567d-2
        c=3.240
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieca,1)=c*10**asum
c  ca ii ; ie = 12, ion = 2
        a(1)=-0.5441d-3
        a(2)= 0.1661d-1
        a(3)=-0.4288d-2
        a(4)= 0.2668d-2
        c=22.39
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieca,2)=c*10**asum
        exeff(ieca,2,2)=1.62d-5
        exeff(ieca,2,3)=3.13d-4
c  ne i ; ie = 6, ion = 1
        a(1)=-0.1399d-1
        a(2)= 0.3018d-1
        a(3)=-0.2025d-1
        a(4)= 0.5257d-2
        c=42.17
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iene,1)=c*10**asum
c
cfex gamion, wrong gamma-ion for fe for o-c
c
c  fe i ; ie = 13, ion = 1
        a(1)= 0.3289d-2
        a(2)= 0.5041d-1
        a(3)= 0.2273d-2
        a(4)= 0.2010d-3
        c=15.03
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,1)=c*10**asum
c  fe ii ; ie = 13, ion = 2
        a(1)=-0.4871d-2
        a(2)= 0.1380d-1
        a(3)=-0.6410d-2
        a(4)= 0.3228d-2
        c=33.50
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,2)=c*10**asum
c  fe iii ; ad hoc
        a(1)=-0.49d-2
        a(2)= 0.14d-1
        a(3)=-0.64d-2
        a(4)= 0.32d-2
        c=70.
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,3)=c*10**asum
c  fe iv ; ad hoc
        a(1)=-0.49d-2
        a(2)= 0.14d-1
        a(3)=-0.64d-2
        a(4)= 0.32d-2
        c=110.
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,4)=c*10**asum
c  s i 
        a(1)=-0.8363d-2
        a(2)= 0.3595d-1
        a(3)=-0.8744d-2
        a(4)= 0.2829d-2
        c=9.58
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ies,1)=c*10**asum
c  s ii
        a(1)=-0.1884d-2
        a(2)= 0.8269d-2
        a(3)=-0.3913d-2
        a(4)= 0.2962d-2
        c=43.45
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ies,2)=c*10**asum
c  ar i
        a(1)=-0.4663d-2
        a(2)=0.1368d-1
        a(3)=-0.4912d-2
        a(4)=0.2993d-2
        c=13.61
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iear,1)=c*10**asum
      elseif (iz.eq.5) then
c
c  he (c)
c
c  he i ; ie = 2, ion = 1
        a(1)=-0.1060d-1
        a(2)= 0.2990d-1
        a(3)=-0.1662d-1
        a(4)= 0.5337d-2
        c=47.10
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iehe,1)=c*10**asum
        exeff(iehe,1,2)=0.030
        exeff(iehe,1,3)=0.058
        exeff(iehe,1,5)=0.632
c  he ii ; ie = 2, ion = 2
        a(1)=-0.1706d-1
        a(2)=0.3819d-1
        a(3)=-0.2600d-1
        a(4)=0.6945d-2
        c=330.4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iehe,2)=c*10**asum
c  c i; ie=3, ion=1
        a(1)= 0.7765d-1
        a(2)= 0.3947d-1
        a(3)=-0.2156d-1
        a(4)= 0.5624d-2
        c=5.92
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iec,1)=c*10**asum
c  c ii
        a(1)=-0.7076d-2
        a(2)= 0.2845d-1
        a(3)=-0.1139d-1
        a(4)= 0.4384d-2
        c=36.31
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iec,2)=c*10**asum
c  n i; ie=4, ion=1
        a(1)= 0.3021d-1
        a(2)= 0.4428d-1
        a(3)=-0.2343d-1
        a(4)= 0.6029d-2
        c=9.11
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ien,1)=c*10**asum
c  o i ; ie = 5, ion = 1
        a(1)= 0.2264d-1
        a(2)= 0.4801d-1
        a(3)=-0.2600d-1
        a(4)=0.6425d-2
        c=9.636
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieo,1)=c*10**asum
c  o ii
        a(1)=-0.1288d-1
        a(2)= 0.2807d-1
        a(3)=-0.1879d-1
        a(4)= 0.5859d-2
        c=39.99
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieo,2)=c*10**asum
c  ne i ; ie = 6, ion = 1
        a(1)=-0.1178d-1
        a(2)= 0.3218d-1
        a(3)=-0.2045d-1
        a(4)= 0.5997d-2
        c=19.59
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iene,1)=c*10**asum
c  mg i
        a(1)= 0.2787
        a(2)= 0.1669
        a(3)=-0.6984d-1
        a(4)= 0.1002d-1
        c=1.02
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iemg,1)=c*10**asum
c  mg ii
        a(1)= 0.4814d-1
        a(2)= 0.6605d-1
        a(3)=-0.3362d-1
        a(4)= 0.7096d-2
        c=22.54
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iemg,2)=c*10**asum
c  si i ; ie = 9 , ion = 1
        a(1)= 0.2539
        a(2)= 0.1519
        a(3)=-0.6845d-1
        a(4)= 0.1050d-1
        c=1.11
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iesi,1)=c*10**asum
c  si ii
        a(1)= 0.4088d-1
        a(2)= 0.7718d-1
        a(3)=-0.2891d-1
        a(4)= 0.6055d-2
        c=12.33
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iesi,2)=c*10**asum
c  s i 
        a(1)= 0.1181d0
        a(2)= 0.5546d-1
        a(3)=-0.3107d-1
        a(4)= 0.6792d-2
        c=3.02
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ies,1)=c*10**asum
c  s ii
        a(1)=-0.7619d-2
        a(2)= 0.4196d-1
        a(3)=-0.1341d-1
        a(4)= 0.4319d-2
        c=18.62
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ies,2)=c*10**asum
c  ca i ; ie = 12, ion = 1
        a(1)= 0.2724d0
        a(2)= 0.1209d0
        a(3)=-0.6886d-1
        a(4)= 0.1162d-1
        c=0.8375
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieca,1)=c*10**asum
c  ca ii ; ie = 12, ion = 2
        a(1)= 0.5001d-1
        a(2)= 0.5410d-1
        a(3)=-0.2417d-1
        a(4)= 0.5796d-2
        c=8.24 
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieca,2)=c*10**asum
        exeff(ieca,2,2)=2.87d-6
        exeff(ieca,2,3)=7.87d-5
c
cfex gamion, wrong gamma-ion for fe for he-zone (same as h-zone)
c
c  fe i ; ie = 13, ion=1
        a(1)= 0.1011d0
        a(2)= 0.2084d-1
        a(3)=-0.1181d-1
        a(4)= 0.4609d-2
        c=8.441
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,1)=c*10**asum
c  fe ii ; ie = 13, ion=2
        a(1)=-0.1316d-1
        a(2)= 0.2725d-1
        a(3)=-0.1729d-1
        a(4)= 0.6007d-2
        c=15.52
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,2)=c*10**asum
c  fe iii  ; ad hoc
        a(1)=-0.13d-1
        a(2)= 0.27d-1
        a(3)=-0.17d-1
        a(4)= 0.60d-2
        c=30.
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,3)=c*10**asum
c  fe iv  ; ad hoc
        a(1)=-0.13d-1
        a(2)= 0.27d-1
        a(3)=-0.17d-1
        a(4)= 0.60d-2
        c=50.
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,4)=c*10**asum
c  ar i
        a(1)=0.1163d-1
        a(2)=0.4952d-1
        a(3)=-0.1907d-1
        a(4)=0.5174d-2
        c=5.50
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iear,1)=c*10**asum
      elseif (iz.eq.6) then
c  h
c
c  h i ; ie = 1, ion=1
        a(1)=-0.1383d-1
        a(2)= 0.3011d-1
        a(3)=-0.1607d-1
        a(4)= 0.5735d-2
        c=32.81
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieh,1)=c*10**asum
c        exeff(ieh,1,2)=0.742+0.059
c        exeff(ieh,1,3)=0.108+0.017
c the division between different l states is not correct
        exeff2=0.742+0.059
        exeffh1(2)=exeff2*0.25
        exeffh1(3)=exeff2*0.75
c
c the division between different l states is not correct
        exeff3=0.108+0.017
        exeffh1(4)=exeff3*(1./9.)
        exeffh1(5)=exeff3*(3./9.)
        exeffh1(6)=exeff3*(5./9.)
c
c he i
        a(1)=-0.1292d-1
        a(2)= 0.3014d-1
        a(3)=-0.2300d-1
        a(4)= 0.7044d-2
        c=42.76
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iehe,1)=c*10**asum
        exeff(iehe,1,5)=0.030
c  he ii ; ie = 2, ion = 2
        a(1)=-0.1652d-1
        a(2)=0.3996d-1
        a(3)=-0.2963d-1
        a(4)=0.8020d-2
        c=264.9
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iehe,2)=c*10**asum
c  c i
        a(1)=-0.1294d-1
        a(2)= 0.3135d-1
        a(3)=-0.1878d-1
        a(4)= 0.6176d-2
        c=8.50 
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iec,1)=c*10**asum
c  c ii
        a(1)=-0.3969d-2
        a(2)= 0.1833d-1
        a(3)=-0.1636d-1
        a(4)= 0.6137d-2
        c=36.64
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iec,2)=c*10**asum
c  n i; ie=4, ion=1
        a(1)=-0.1215d-1
        a(2)= 0.2890d-1
        a(3)=-0.2033d-1
        a(4)= 0.6551d-2
        c=10.57
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ien,1)=c*10**asum
c  o i ; ie = 5, ion = 1
        a(1)=-0.1167d-1
        a(2)=0.3196d-1
        a(3)=-0.2269d-1
        a(4)=0.6902d-2
        c=10.59
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieo,1)=c*10**asum
c  o ii
        a(1)=-0.1315d-1
        a(2)= 0.3242d-1
        a(3)=-0.2551d-1
        a(4)= 0.7473d-2
        c=33.88
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieo,2)=c*10**asum
c  mg i
        a(1)= 0.2266d0
        a(2)=-0.3135d-1
        a(3)= 0.4768d-2
        a(4)= 0.2542d-2
        c=3.18 
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iemg,1)=c*10**asum
c  mg ii
        a(1)=-0.1766d-1
        a(2)= 0.3944d-1
        a(3)=-0.2315d-1
        a(4)= 0.6644d-2
        c=29.11
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iemg,2)=c*10**asum
c  si i ; ie = 9 , ion = 1
        a(1)= 0.2028d00
        a(2)=-0.3949d-1
        a(3)= 0.5850d-2
        a(4)= 0.2772d-2
        c=3.17
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iesi,1)=c*10**asum
c  si ii
        a(1)=-0.5790d-2
        a(2)= 0.1466d-1
        a(3)=-0.8207d-2
        a(4)= 0.4630d-2
        c=19.63
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iesi,2)=c*10**asum
c  s i 
        a(1)=-0.1471d-1
        a(2)= 0.4221d-1
        a(3)=-0.2192d-1
        a(4)= 0.6415d-2
        c=5.16
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ies,1)=c*10**asum
c  s ii
        a(1)=-0.3658d-2
        a(2)= 0.1299d-1
        a(3)=-0.1170d-1
        a(4)= 0.5381d-2
        c=20.99
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ies,2)=c*10**asum
c  ca i ; ie = 12, ion = 1
        a(1)= 0.2018d0
        a(2)=-0.3944d-1
        a(3)=-0.1137d-2
        a(4)= 0.4134d-2
        c=2.024
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieca,1)=c*10**asum
c  ca ii ; ie = 12, ion = 2
        a(1)=-0.1092d-1
        a(2)= 0.2889d-1
        a(3)=-0.1743d-1
        a(4)= 0.6007d-2
        c=11.51
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(ieca,2)=c*10**asum
        exeff(ieca,2,2)=4.18d-7
        exeff(ieca,2,3)=1.15d-5
c  ne i ; ie = 6, ion=1
        a(1)=-0.1029d-1
        a(2)= 0.3182d-1
        a(3)=-0.2555d-1
        a(4)= 0.7464d-2
        c=16.71
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iene,1)=c*10**asum
c  fe i ; ie = 13, ion=1
        a(1)= 0.1011d0
        a(2)= 0.2084d-1
        a(3)=-0.1181d-1
        a(4)= 0.4609d-2
        c=8.441
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,1)=c*10**asum
c  fe ii ; ie = 13, ion=2
        a(1)=-0.1316d-1
        a(2)= 0.2725d-1
        a(3)=-0.1729d-1
        a(4)= 0.6007d-2
        c=15.52
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,2)=c*10**asum
c  fe iii  ; ad hoc
        a(1)=-0.13d-1
        a(2)= 0.27d-1
        a(3)=-0.17d-1
        a(4)= 0.60d-2
        c=30.
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,3)=c*10**asum
c  fe iv  ; ad hoc
        a(1)=-0.13d-1
        a(2)= 0.27d-1
        a(3)=-0.17d-1
        a(4)= 0.60d-2
        c=50.
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iefe,4)=c*10**asum
c  ar i
        a(1)=0.1484d-3
        a(2)=0.1705d-1
        a(3)=-0.1432d-1
        a(4)=0.5806d-2
        c=6.69
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        chieff(iear,1)=c*10**asum
      endif
c
      do ie=1,iemax
         do ion=1,2
            if (chieff(ie,ion).gt.0.) then
               gion(ie,ion)=gahe/(chieff(ie,ion)*elch)
            else
               gion(ie,ion)=0.
            endif
         enddo
      enddo
cqcq
c      do ie=1,11
c         gion(ie,2)=0.
c         chieff(ie,2)=0.
c      enddo
c      gion(ieo,2)=0.
c      chieff(ieo,2)=0.
c      gion(iefe,1)=0.
c      chieff(iefe,1)=0.
c      gion(iefe,2)=0.
c      chieff(iefe,2)=0.
c      gion(iefe,3)=0.
c      chieff(iefe,3)=0.
c      gion(iefe,4)=0.
c      chieff(iefe,4)=0.
c      gion(iehe,2)=0.
c      chieff(iehe,2)=0.
c      gion(ieca,2)=0.
c      chieff(ieca,2)=0.
cqcq
c  fe iii, iv
      if (chieff(iefe,3).gt.0.) gion(iefe,3)=gahe/(chieff(iefe,3)*elch)
      if (chieff(iefe,4).gt.0.) gion(iefe,4)=gahe/(chieff(iefe,4)*elch)
c
c preliminary for CO
      gionco=gion(ieo,1)
c
c     heating and excitation efficiency not corrected for iron 
      ex=0.
      if (iz.eq.1) then
c  fe - he
        a(1)= 0.1742d0
        a(2)= 0.2427d-1
        a(3)= 0.9469d-2
        a(4)=-0.3020d-2
        c=0.1050
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        he=c*10**asum
        a(1)=-0.6614d-1
        a(2)= 0.6591d-2
        a(3)=-0.8752d-2
        a(4)=-0.6298d-3
        c=0.348
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        ex=c*10**asum
      elseif (iz.eq.2) then
c  si - s
        a(1)= 0.3469d0
        a(2)=-0.9228d-1
        a(3)= 0.3170d-1
        a(4)=-0.3805d-2
        c=0.08831
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        he=c*10**asum
        a(1)=-0.1161d0
        a(2)= 0.2963d-2
        a(3)=-0.8325d-2
        a(4)= 0.1863d-2
        c=0.322
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        ex=c*10**asum
      elseif (iz.eq.3) then
c  o - si - s
        a(1)= 0.2783d0
        a(2)=-0.4872d-1
        a(3)= 0.1477d-1
        a(4)=-0.1795d-2
        c=0.1255
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        he=c*10**asum
        a(1)=-0.1566d0
        a(2)= 0.3428d-1 
        a(3)=-0.3946d-1
        a(4)= 0.7850d-2
        c=0.237
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        ex=c*10**asum
      elseif (iz.eq.4) then
c  o - c
        a(1)= 0.3199d0
        a(2)=-0.7042d-1
        a(3)= 0.1903d-1
        a(4)=-0.2063d-2
        c=0.1209
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        he=c*10**asum
        a(1)=-0.1618d0
        a(2)= 0.3118d-1 
        a(3)=-0.4824d-1
        a(4)= 0.9615d-2
        c=0.281
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        ex=c*10**asum
      elseif (iz.eq.5) then
c  he
        a(1)= 0.2482d0
        a(2)=-0.7192d-1
        a(3)= 0.3316d-1
        a(4)=-0.4363d-2
        c=0.1399
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        he=c*10**asum
        a(1)=-0.4911d-1
        a(2)=-0.9768d-1
        a(3)= 0.5434d-1
        a(4)=-0.1203d-1
        c=0.303
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        ex=c*10**asum
      elseif (iz.eq.6) then
c  h
        a(1)= 0.1153d0
        a(2)= 0.1710d-1
        a(3)= 0.1927d-1
        a(4)=-0.4129d-2
        c=0.1262
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        he=c*10**asum
        a(1)= 0.3202d-1
        a(2)=-0.1461d0 
        a(3)= 0.8186d-1
        a(4)=-0.1753d-1
        c=0.445 
c        asum=a(1)*xl4+a(2)*xl4**2+a(3)*xl4**3+a(4)*xl4**4
        asum=xl4*(a(1)+xl4*(a(2)+xl4*(a(3)+a(4)*xl4)))
        ex=c*10**asum
      endif
c
c  excitation
c
c  - non-thermal excitation of fe-ions not included
c
c  complications if molecules are included : 
c
c  - in calculating gamex (gamma excitation rate ; s**-1) and extot (total
c    excitation fraction), the atom/ion abundances are not taken into account.
c    This means that the same fraction goes into excitation of the molecules
c    as into the atoms/ions. OK as a first approximation ? OK.
c
c  normalization of exeff
c
      sumexeff=0.
c      do ie=1,iemax
c
c  no non-thermal excitation for iron 
c  ( the maximum dimension for exeff is nl, not nlf)
c
      do ie=1,12
         do ion=1,2
c            if (ie.eq.iefe) then
c                if (ion.eq.1) abxold=ab(ie)*xoldief1(id,1)
c                if (ion.eq.2) abxold=ab(ie)*xoldief2(id,1)
c                if (ion.eq.3) abxold=ab(ie)*xoldief3(id,1)
c                if (ion.eq.4) abxold=ab(ie)*xoldief4(id,1)
c            else
            if (ie.eq.ieh.and.ion.eq.1) then
               abxold=ab(ie)*xoldieh1(id,1)
            else
               abxold=ab(ie)*xoldie(id,ie,ion,1)
            endif
c            endif
            if (abxold.gt.0.) then
               if (ie.eq.1.and.ion.eq.1) then
c  H I
                  do n=1,nlev(ieh,1)
                     sumexeff=sumexeff+exeffh1(n)
                  enddo
               else
                  do n=1,nlev(ie,ion)
                     sumexeff=sumexeff+exeff(ie,ion,n)
                  enddo
               endif
            endif
         enddo
      enddo
c
c  no non-thermal excitation for iron 
c  ( the maximum dimension for exeff is nl, not nlf)
c
c      do ie=1,iemax
      do ie=1,12
         do ion=1,2
c            if (ie.eq.iefe) then
c               if (ion.eq.1) abxold=ab(ie)*xoldief1(id,1)
c               if (ion.eq.2) abxold=ab(ie)*xoldief2(id,1)
c               if (ion.eq.3) abxold=ab(ie)*xoldief3(id,1)
c               if (ion.eq.4) abxold=ab(ie)*xoldief4(id,1)
c            else
            if (ie.eq.ieh.and.ion.eq.1) then
               abxold=ab(ie)*xoldieh1(id,1)
            else
               abxold=ab(ie)*xoldie(id,ie,ion,1)
            endif
c            endif
            if (abxold.gt.0.) then
               if (ie.eq.1.and.ion.eq.1) then
c  H I
                  do n=1,nlev(ieh,1)
                     exeffh1(n)=exeffh1(n)/sumexeff
                  enddo
               else
                  do n=1,nlev(ie,ion)
                     exeff(ie,ion,n)=exeff(ie,ion,n)/sumexeff
                  enddo
               endif
            endif
         enddo
      enddo
c
      extot=0.
c  no non-thermal excitation for iron 
c  ( the maximum dimension for exeff is nl, not nlf)
c
      do ie=1,iemax
         do ion=1,mion
            do il=1,nlf
               gamex(ie,ion,il)=0.
            enddo
         enddo
      enddo

c      do ie=1,iemax
      do ie=1,12
         do ion=1,2
c            if (ie.eq.iefe) then
c               if (ion.eq.1) abxold=ab(ie)*xoldief1(id,1)
c               if (ion.eq.2) abxold=ab(ie)*xoldief2(id,1)
c               if (ion.eq.3) abxold=ab(ie)*xoldief3(id,1)
c               if (ion.eq.4) abxold=ab(ie)*xoldief4(id,1)
c            else
            if (ie.eq.ieh.and.ion.eq.1) then
               abxold=ab(ie)*xoldieh1(id,1)
            else
               abxold=ab(ie)*xoldie(id,ie,ion,1)
            endif
c            endif
            if (abxold.gt.0.) then
               if (ie.eq.1.and.ion.eq.1) then
c  H I
                  do n=1,nlev(ieh,1)
                     if (exeffh1(n).gt.0.) then
                        gamex(ie,ion,n)=ex*exeffh1(n)*gahe/
     &                       (1.602e-12*ehn(nln(n))*abxold)
                        extot=extot+ex*exeffh1(n)
                     else
                        gamex(ie,ion,n)=0.
                     endif
                  enddo
               else
                  do n=1,nlev(ie,ion)
                     if (exeff(ie,ion,n).gt.0.) then
cqcq                     gamex(ie,ion,n)=ex*exeff(ie,ion,n)*gahe/
ccqc     &                    (1.602e-12*e(ie,ion,n))
                        gamex(ie,ion,n)=ex*exeff(ie,ion,n)*gahe/
     &                       (1.602e-12*e(ie,ion,n)*abxold)
                        extot=extot+ex*exeff(ie,ion,n)
                     else
                        gamex(ie,ion,n)=0.
                     endif
                  enddo
               endif
            endif
         enddo
      enddo
      frexo2=ex*exeff(ieo,1,2)
      frexh3=ex*(exeffh1(4)+exeffh1(5)+exeffh1(6))
      frexc=extot
c
c  ionization
c
c  complications if molecules are included : 
c
c  - non-thermal ionization of molecules has to be included.
c    Here, preliminary, the effective ionization potential
c    (chieff) is set to 37 eV, and the ionization potential,
c    unless given, is set to 10 eV. It is assumed that the 
c    effective ionization potential is per molecule.
c
      sumh=0.
      do ie=1,iemax
        do ion=1,mion
          if (chieff(ie,ion).gt.0.) then
            effi(ie,ion)=e00(ie,ion)*ab(ie)*xa(ie,ion)/chieff(ie,ion)
            sumh=sumh+effi(ie,ion)
           else
            effi(ie,ion)=0.
          endif
        enddo
      enddo
      frionh=effi(ieh,1)
      frion=sumh
      sumhm=0.
cnomol      do im=1,imm
cnomol
      do im=1,1
cnomol
         if (xm(im).gt.0.) then
            if (ethmi(im).gt.0.) then
               e00m=ethmi(im)
            else
               e00m=10.
            endif
            chieffm=chieff(iem(im),1)
            effm=e00m*xm(im)/chieffm
            sumhm=sumhm+effm
         endif
      enddo
      sumh=sumh+sumhm
c!!   rest goes into heating
      he=1.-sumh-extot
      frhe=he
c
      gaelh=he*gahe*den
c
      if(idiag.eq.1) then
        ginp=gahe/den
        write(8,991)extot,sumh,he,(1.-extot-sumh),gahe/den
991     format(' gam ',1pe12.4,5e12.4)
        write(8,995)(ginp*effi(ie,1),ie=1,15)
995     format('io',1pe12.4,20e12.4)
        ie=1
        do i=1,nlev(ieh,1)
           if(exeffh1(i).gt.0.) then
              write(8,996)ie,i,exeffh1(i),exeffh1(i)*ginp*ex
           endif
        enddo
c
        do ie=2,15
          do i=1,nl
            if(exeff(ie,1,i).gt.0.) then
              write(8,996)ie,i,exeff(ie,1,i),exeff(ie,1,i)*ginp*ex
996           format('ex',2i3,1pe12.4,5e12.4)
            endif
          enddo
        enddo
      endif
c
      return
      end


      subroutine ebin_long
c
c e1(j) = Midpoint of energy interval j, (eV).
c en(j) = First point of energy interval j, (eV).
c
c => Energy interval j is bounded by en(j) and en(j+1) with midpoint
c    e1(j).
c
c  new emin = 0 
c  emin = 3.3991
c  emax = 30.0 eV
c
      implicit real*8 (a-h,o-z)
      parameter(ne=100)
      common/cebin/e1(ne),en(ne),jmax
c
c  all energies in ev
c
c      jm0=0
      jm0=29
      e2=0.01
      en(1)=0.
      en(2)=0.01
      en(3)=0.02
      en(4)=0.04
      en(5)=0.07
      en(6)=0.10
c
      de0=(3.3991-en(6))/(jm0-5)
      do j=7,jm0
         en(j)=en(j-1)+de0
      enddo
c
      jm1=50
      jm2=20
      jmax=jm1+jm2+jm0
      de1=dlog10(13.618d0/3.3991d0)/dble(jm1-1)
      de2=dlog10(30.d0/13.618d0)/dble(jm2-1)
c      do j=1,jmax+1
      do j=jm0+1,jmax+1
         if (j.le.(jm1+jm0)) then
c            en(j)=3.3991*10.**(de1*(j-1))
            en(j)=3.3991*10.**(de1*(j-jm0-1))
         else
            en(j)=en(jm1+jm0)*10.**(de2*(j-jm1-jm0))   
         endif
      enddo
      e1(1)=0.5*(en(1)+en(2))
      do j1=2,jmax
        e1(j1)=dsqrt(en(j1+1)*en(j1))
      enddo
c
      return
      end



      subroutine ebincf
c
c e1(j) = Midpoint of energy interval j, (eV).
c en(j) = First point of energy interval j, (eV).
c
c => Energy interval j is bounded by en(j) and en(j+1) with midpoint
c    e1(j).
c
c  emin = 3.3991
c  emax = 30.0 eV
c
      implicit real*8 (a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      common/cebin/e1(ne),en(ne),jmax
c
c  all energies in ev
c
      jm1=50
      jm2=20
cq      jm3=21
cq      jmax=jm1+jm2+jm3
      jmax=jm1+jm2
      de1=dlog10(13.618d0/3.3991d0)/dble(jm1-1)
      de2=dlog10(30.d0/13.618d0)/dble(jm2-1)
cq      de3=dlog10(57.d0/30d0)/dble(jm3-1)
      do j=1,jmax+1
         if (j.le.jm1) then
            en(j)=3.3991*10.**(de1*(j-1))
         else
            en(j)=en(jm1)*10.**(de2*(j-jm1))   
cq         elseif (j.le.jm2) then
cq            en(j)=en(jm1)*10.**(de2*(j-jm1))   
cq         else
cq            en(j)=en(jm2)*10.**(de3*(j-jm2))   
         endif
      enddo
      do j1=1,jmax
        e1(j1)=dsqrt(en(j1+1)*en(j1))
      enddo
c
      return
      end



      subroutine ebin
c
c e1(j) = Midpoint of energy interval j, (eV).
c en(j) = First point of energy interval j, (eV).
c
c => Energy interval j is bounded by en(j) and en(j+1) with midpoint
c    e1(j).
c
c  emin = 0.2491 (0.25 eV)
c  emax = 30.0 eV
c  'break points :'  0.2491  (vibrational transitions in CO, dv=1)
c                    0.4991  (vibrational transitions in CO, dv=2)
c                    0.7491  (photo dissociation of H-,C-,O-)
c                    3.3991
c                    13.618
c                    30.0
c  If further photons with energy below 3.3991 eV are included
c  in the flux, more break points are probably needed.
c
      implicit real*8 (a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      common/cebin/e1(ne),en(ne),jmax
c
c  all energies in ev
c
      njm1=5
      njm2=5
      njm3=10
      njm4=50
      njm5=20
      jm1=njm1
      jm2=jm1+njm2
      jm3=jm2+njm3
      jm4=jm3+njm4
      jm5=jm4+njm5
      jmax=njm1+njm2+njm3+njm4+njm5
      de1=dlog10(0.4991d0/0.2491d0)/(dble(njm1)-1.d0)
      de2=dlog10(0.7491d0/0.4991d0)/dble(njm2)
      de3=dlog10(3.3991d0/0.7491d0)/dble(njm3)
      de4=dlog10(13.618d0/3.3991d0)/dble(njm4)
      de5=dlog10(30.d0/13.618d0)/dble(njm5)
c
      do j=1,jmax+1
        if (j.le.jm1) then
         en(j)=0.2491*10.**(de1*(j-1))
        elseif (j.gt.jm1.and.j.le.jm2) then
         en(j)=en(jm1)*10.**(de2*(j-jm1))   
        elseif (j.gt.jm2.and.j.le.jm3) then
         en(j)=en(jm2)*10.**(de3*(j-jm2))   
        elseif (j.gt.jm3.and.j.le.jm4) then
         en(j)=en(jm3)*10.**(de4*(j-jm3))   
c         en(j)=3.3991*10.**(de4*(j-1))   
        elseif (j.gt.jm4.and.j.le.(jm5+1)) then
         en(j)=en(jm4)*10.**(de5*(j-jm4))   
	endif
      enddo
      do j1=1,jmax
        e1(j1)=dsqrt(en(j1+1)*en(j1))
      enddo
c
      return
      end



      subroutine crphigs
c
c  calc. all ground state photo ionization cross sections 
c                                                     ; si(ie,ion,j)
      implicit real*8(a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      common/ctpar/tday,dts,iss,iss1,itd
      common/cebin/em(ne),en(ne),jmax
      common/crosssec/si(mie,mion,ne)
      common/cionpot/e00(mie,mion)
      common/cphionfl/iflphi
c      dimension rmes1(6),rmss1(6)
c      dimension rmes2(4),rmss2(4)
c      dimension rmesi2(5),rmssi2(5)
c      dimension rmear1(6),rmsar1(6)
c      dimension rmena1(10),rmsna1(10)
c
cqcq run 97; run 80.6; run 80.7; run 96; run 77; run 70.8; run 70.9
cqcq run 99; run 80.8  >= run 100
      if (iflphi.eq.0) then
         do ie=1,mie
            do ion=1,mion
               do j1=1,ne
                  si(ie,ion,j1)=0.d0
               enddo
            enddo
         enddo
         return
      endif
c
      do j=1,jmax
         e1=em(j)
c     hi
c  960215 new data from Verner et al 1996
         si(ieh,1,j)=0.d0
         if(e1.gt.e00(ieh,1)) then
c            ps=dsqrt((e1-13.6)/13.6)
c            fe=dexp(-4.d0*datan(ps)/ps)/(1.-dexp(-6.283d0/ps))
c            si(1,1,j)=3.402e-16*(13.6/e1)**4.*fe
            call vernercr(e1,4.298d-1,5.475d4,3.288d1,2.963d0,
     &           0.d0,0.d0,0.d0,sigf)
            si(ieh,1,j)=sigf*1.e-18               
         endif
cqcq
c            si(1,1,j)=0.
cqcq
c     he i
c  960215 new data from Verner et al 1996
         si(iehe,1,j)=0.
c         if(e1.gt.e00(2,1).and.e1.lt.65.4) then
c            si(2,1,j)=si(1,1,j)*(6.53*(e1/24.6)-0.22)
c         elseif(e1.gt.65.4) then
c            si(2,1,j)=si(1,1,j)*(37.-19.1*(65.4/e1)**0.76)
c         endif
c
         if (e1.gt.e00(iehe,1)) then
            call vernercr(e1,1.361d1,9.492d2,1.469d0,3.188d0,
     &           2.039d0,4.434d-1,2.136d0,sigf)
            si(iehe,1,j)=sigf*1.e-18               
         endif
c     c i
c  960215 new data from Verner et al 1996
         si(iec,1,j)=0.
c         if(e1.ge.11.26) then
c            call  cro(1,1,e1,si(3,1,j))
c         endif
c
         if (e1.ge.e00(iec,1)) then
            call vernercr(e1,2.144d0,5.027d2,6.216d1,5.101d0,
     &           9.157d-2,1.133d0,1.607d0,sigf)
            si(iec,1,j)=sigf*1.e-18                           
         endif
c New 990824
c Data from Verner et al 1996
c     c ii
c
         si(iec,2,j)=0.
         if (e1.ge.e00(iec,2)) then
            call vernercr(e1,4.058d-1,8.709d0,1.261d2,8.578d0,
     &           2.093d0,4.929d1,3.234d0,sigf)
            si(iec,2,j)=sigf*1.e-18                           
         endif
c
c     n i reilman and manson 
c  960215 new data from Verner et al 1996
         si(ien,1,j)=0.
c         if(e1.gt.e00(4,1)) then
c            call  cro(2,1,e1,si(4,1,j))
c         endif
c
         if (e1.gt.e00(ien,1)) then
            call vernercr(e1,4.034d0,8.235d2,8.033d1,3.928d0,
     &           9.097d-2,8.598d-1,2.325d0,sigf)
            si(ien,1,j)=sigf*1.e-18               
         endif
c New 990824
c Data from Verner et al 1996
c     n ii
c
         si(ien,2,j)=0.
         if (e1.ge.e00(ien,2)) then
            call vernercr(e1,6.128d-2,1.944d0,8.163d2,8.773d0,
     &           1.043d1,4.280d2,2.030d1,sigf)
            si(ien,2,j)=sigf*1.e-18                           
         endif
c     o i reilman and manson 
c  960215 new data from Verner et al 1996
         si(ieo,1,j)=0.
c         if(e1.ge.13.612) then
c            call  cro(3,1,e1,si(5,1,j))
c         endif
c
c   o i (mcalpine corrected to agree with reilman and manson at
c        high energies).
c
c        if (e1.gt.13.6.and.e1.lt.100.) then
c          si(5,1,j)=9.05e-18*(4.378*(13.6/e1)**1.5-3.378*
c     &(13.6/e1)**2.5)
c        elseif (e1.gt.100.) then
c          si0=1.778
c          al=3.65
c          s=3.
c          e0=100.
c          si(5,1,j)=si0*1.e-18*(al*(e0/e1)**s+(1.-al)*(e0/e1)**(s+1.))
c        endif
         if (e1.gt.e00(ieo,1)) then
            call vernercr(e1,1.240d0,1.745d3,3.784d0,1.764d1, 
     &           7.589d-2,8.698d0,1.271d-1,sigf)
            si(ieo,1,j)=sigf*1.e-18                           
         endif
cqcq
c        si(5,1,j)=0.
cqcq
C-- Ne I from Reilman and Manson, Ap.J. Suppl. 40, 815 (1979)
c  960215 new data from Verner et al 1996
         si(iene,1,j)=0.0d0
c wrong        if(e1.lt.21.564) then
c         if(e1.ge.21.564) then
c            xx=dlog10(e1)
c            silog=-26.2316d0+13.6277d0*xx-6.181d0*xx**2+.7864d0*xx**3
c            si(6,1,j)=1.d1**silog
c         endif
c
         if (e1.gt.e00(iene,1)) then
            call vernercr(e1,4.870d0,4.287d3,5.798d0,
     &           8.355d0,2.434d-1,4.236d-2,5.873d0,sigf)
            si(iene,1,j)=sigf*1.e-18               
         endif
c         
c   Na I
c
c  Data from Reilman and Manson, 1979, ApJ Suppl., 40, 815.
c  960215 new data from Verner et al 1996
c
c         data rmena1/6.,7.,8.,9.,10.,15.,20.,25.,30.,35./
c         data rmsna1/1.254e-3,1.929e-2,5.648e-2,8.685e-2,
c     &    0.1072,0.1252,0.1050,8.507e-2,6.955e-2,5.788e-2/
c         sirm=0.
c         if (e1.gt.e00(iena,1)) then
c            do irm=1,5
c               if (e1.gt.rmena1(irm).and.e1.le.rmena1(irm+1)) then
c                  sirm=rmsna1(irm)+((e1-rmena1(irm))/
c     &                 (rmena1(irm+1)-rmena1(irm)))*
c     &                 (rmsna1(irm+1)-rmsna1(irm))
c               endif
c            enddo
c         endif
c         si(iena,1,j)=sirm*1.e-18
         si(iena,1,j)=0.d0
         if (e1.gt.e00(iena,1)) then
            call vernercr(e1,6.139d0,1.601d0,6.148d3,3.839d0,
     &           0.d0,0.d0,0.d0,sigf)
            si(iena,1,j)=sigf*1.e-18                           
         endif
c
c  Mg I
c!!   old data!!!1
c  960215 new data from Verner et al 1996
c         if(e1.gt.e00(8,1).and.e1.lt.9.178) then
c            si(8,1,j)=cr(e1,7.646,988.3,-701.7,-431.5,53.44,90.06,
c     &                                                0.,0.,18.6836)
c         elseif(e1.ge.9.178.and.e1.lt.9.89) then
c            si(8,1,j)=1.e-20
c         elseif(e1.gt.9.89) then
c            si(8,1,j)=cr(e1,9.89,2.653,-.7339,-6.271,-.0315,9.178,
c     &.654e-4,-6.710,1.0145)
c         endif
c
         si(iemg,1,j)=0.d0
         if (e1.gt.e00(iemg,1)) then
            call vernercr(e1,1.197d1,1.372d8,2.228d-1,1.574d1,
     &           2.805d-1,0.d0,0.d0,sigf)
            si(iemg,1,j)=sigf*1.e-18               
         endif
c  Mg II
c  960215 new data from Verner et al 1996
c         si(8,2,j)=crb(e1,16.2,62.6305,-207.9877,274.1985,
c     &-186.1035,68.2050,-12.6197,.9034,1.7578,8)
c
         si(iemg,2,j)=0.d0
         if (e1.gt.e00(iemg,2)) then
            call vernercr(e1,8.139d0,3.278d0,4.341d7,3.610d0,
     &           0.d0,0.d0,0.d0,sigf)
            si(iemg,2,j)=sigf*1.e-18               
         endif
c  Si I (barfield et al. )
c  960215 new data from Verner et al 1996
c         if(e1.gt.7.90.and.e1.lt.17.8) then
c            call seat(e1,25.7,-.646,1.,7.90,si(9,1,j))
c         elseif(e1.gt.17.8.and.e1.lt.22.4) then
c            call seat(e1,1.047,1.72,-2.,17.8,si(9,1,j))
c         elseif(e1.gt.22.4.and.e1.lt.28.2) then
c            call seat(e1,.409,1.16,1.,22.4,si(9,1,j))
c         elseif(e1.gt.28.2.and.e1.lt.127.) then
c            call seat(e1,.33,5.66,1.68,28.2,si(9,1,j))
c         elseif(e1.gt.127.) then
c            call seat(e1,.122,1.43,2.00,127.,si(9,1,j))
c         endif
c
         si(iesi,1,j)=0.d0
         if (e1.gt.e00(iesi,1)) then
            call vernercr(e1,2.317d1,2.506d1,2.057d1,3.546d0,
     &           2.837d-1,1.672d-5,4.207d-1,sigf)
            si(iesi,1,j)=sigf*1.e-18                           
         endif
c
c  Si II
c  Data from Reilman and Manson, 1979, ApJ Suppl., 40, 815.
c  960215 new data from Verner et al 1996
c
c         data rmesi2/15.,20.,25.,30.,35./
c         data rmssi2/1.206,0.2455,0.6216,0.6751,0.6854/
c         sirm=0.
c         if (e1.gt.e00(iesi,2)) then
c            do irm=1,4
c               if (e1.gt.rmesi2(irm).and.e1.le.rmesi2(irm+1)) then
c                  sirm=rmssi2(irm)+((e1-rmesi2(irm))/
c     &                 (rmesi2(irm+1)-rmesi2(irm)))*
c     &                 (rmssi2(irm+1)-rmssi2(irm))
c               endif
c            enddo
c         endif
c         si(iesi,2,j)=sirm*1.e-18
c
         si(iesi,2,j)=0.d0
         if (e1.gt.e00(iesi,2))then
            call vernercr(e1,2.556d0,4.140d0,1.337d1,1.191d1,
     &           1.570d0,6.634d0,1.272d-1,sigf)
            si(iesi,2,j)=sigf*1.e-18               
         endif
c  S I
c
C   Data from Weisheit and Collins, Ap.J. 210, 299 (1976)
c   (from PL)
C
C-- S I
c         si2p=0.0d0
c         si2s=0.0d0
c         si3p=0.0d0
c         si3s=0.0d0
cc         IF(E1.GT.10.4) SI3P=12.6*(10.4/E1)**.264
c         IF(E1.GT.e00(ies,1)) SI3P=12.6*(e00(ies,1)/E1)**.264
c         IF(E1.GT.20.0) SI3P=10.6*(20.0/E1)**2.33
c         IF(E1.GT.23.9) SI3S=0.14*(E1/23.9)**1.27
c         IF(E1.GT.50.0) SI3S=0.363*(50.0/E1)**.684
c         IF(E1.GT.182.) SI2P=3.66*(182./E1)**2.25
c         IF(E1.GT.245.) SI2S=0.445*(245./E1)**1.54
c         SIWC=(SI2P+SI2S+SI3P+SI3S)*1.E-18
c         si(ies,1,j)=siwc
c
c   S I
c
c  Data from Reilman and Manson, 1979, ApJ Suppl., 40, 815.
c  960215 new data from Verner et al 1996
c
c         data rmes1/10.,15.,20.,25.,30.,35./
c         data rmss1/0.,30.39,6.347,1.684,0.9382,0.9274/
c         sirm=0.
c         if (e1.gt.e00(ies,1)) then
c            do irm=1,5
c               if (e1.gt.rmes1(irm).and.e1.le.rmes1(irm+1)) then
c                  sirm=rmss1(irm)+((e1-rmes1(irm))/
c     &                 (rmes1(irm+1)-rmes1(irm)))*
c     &                 (rmss1(irm+1)-rmss1(irm))
c               endif
c            enddo
c         endif
c         si(ies,1,j)=sirm*1.e-18
         si(ies,1,j)=0.d0
         if (e1.gt.e00(ies,1)) then
            call vernercr(e1,1.808d1,4.564d4,1.0d0,1.361d1,
     &           6.385d-1,9.935d-1,2.486d-1,sigf)
            si(ies,1,j)=sigf*1.e-18               
         endif
c
c  S II
c  Data from Reilman and Manson, 1979, ApJ Suppl., 40, 815.
c  960215 new data from Verner et al 1996
c
c         data rmes2/20.,25.,30.,35./
c         data rmss2/0.,1.192,0.5453,0.9730/
c         sirm=0.
c         if (e1.gt.e00(ies,2)) then
c            do irm=1,3
c               if (e1.gt.rmes2(irm).and.e1.le.rmes2(irm+1)) then
c                  sirm=rmss2(irm)+((e1-rmes2(irm))/
c     &                 (rmes2(irm+1)-rmes2(irm)))*
c     &                 (rmss2(irm+1)-rmss2(irm))
c               endif
c            enddo
c         endif
c         si(ies,2,j)=sirm*1.e-18
         si(ies,2,j)=0.d0
         if (e1.gt.e00(ies,2)) then
            call vernercr(e1,8.787d0,3.136d2,3.442d0,1.281d1,
     &           7.354d-1,2.782d0,1.788d-1,sigf)
            si(ies,2,j)=sigf*1.e-18               
         endif
c         
c   Ar I
c
c  Data from Reilman and Manson, 1979, ApJ Suppl., 40, 815.
c  960215 new data from Verner et al 1996
c
c         data rmear1/10.,15.,20.,25.,30.,35./
c         data rmsar1/0.,56.16,50.22,13.49,3.727,1.517/
c         sirm=0.
c         if (e1.gt.e00(iear,1)) then
c            do irm=1,5
c               if (e1.gt.rmear1(irm).and.e1.le.rmear1(irm+1)) then
c                  sirm=rmsar1(irm)+((e1-rmear1(irm))/
c     &                 (rmear1(irm+1)-rmear1(irm)))*
c     &                 (rmsar1(irm+1)-rmsar1(irm))
c               endif
c            enddo
c         endif
c         si(iear,1,j)=sirm*1.e-18
         si(iear,1,j)=0.d0
         if (e1.gt.e00(iear,1)) then
            call vernercr(e1,1.709d1,2.106d1,2.645d2,4.796d0,
     &           4.185d-1,1.688d0,8.943d-1,sigf)
            si(iear,1,j)=sigf*1.e-18               
         endif

c New 990824
c Data from Verner et al 1996
c     Ar ii
c
         si(iear,2,j)=0.
         if (e1.ge.e00(iear,2)) then
            call vernercr(e1,2.494d1,2.503d1,1.272d2,4.288d0,
     &           5.108d-1,9.299d-1,7.195d-1,sigf)
            si(iear,2,j)=sigf*1.e-18                           
         endif
c
c     ca i  from scott et al j phys 16, 3945, reilman &.. and shine
c  960215 new data from Verner et al 1996
c         if(e1.gt.6.113) then
c            sigb=0.
c            sires=0.
c            ep=(e1-6.63)/4.2e-2
c            if(e1.gt.6.113) sires=7.93e-22*(ep+484.)/(1.+ep**2)
c            if(e1.gt.7.2.and.e1.lt.9.25) sigb=(7.38-0.69*e1)*1.e-20
c            if(e1.gt.9.25.and.e1.lt.15.) sigb=(.61+.481*(e1-9.)**2
c     &            -.0413*(e1-9.)**2)*1.e-19
c            si(12,1,j)=sires+sigb
c         endif
         si(ieca,1,j)=0.d0
         if (e1.gt.e00(ieca,1)) then
            call vernercr(e1,1.278d1,5.370d5,3.162d-1,1.242d1,
     &           4.477d-1,1.012d-3,1.851d-2,sigf)
            si(ieca,1,j)=sigf*1.e-18               
         endif
c     ca ii approx. fit to reilman and manson
c  960215 new data from Verner et al 1996
c         s=0.
c         if(e1.gt.11.87.and.e1.lt.35.) s=0.20*(11.87/e1)**0.0001
c         if(e1.ge.35..and.e1.lt.45.) s=0.10*(35./e1)**1.
c         if(e1.ge.45..and.e1.lt.160.) s=1.5
c         si(12,2,j)=s*1.e-18
         si(ieca,2,j)=0.d0
         if (e1.gt.e00(ieca,2)) then
            call vernercr(e1,1.553d1,1.064d7,7.790d-1,2.130d1,
     &           6.453d-1,2.161d-3,6.706d-2,sigf)
            si(ieca,2,j)=sigf*1.e-18               
         endif
cqcq
c         si(12,2,j)=0.
cqcq

C
C -- Iron from Reilman and Manson (Ap.J. Suppl. 40, 815, 1979) OLD
c
c  NEW data 960211 from Verner, Ferland, Korista, Yakolev
c  1996, in press 'Atomic Data for Astrohysics. II New Analytic Fits 
c  for Photoionization Cross Sections of Atoms and Ions.'
C
C -- Fe I
         si(iefe,1,j)=0.0d0
crm         if(e1.gt.7.87d0.and.e1.le.15.d0) then
crm            si(13,1,j)=0.836d-18*(7.87d0/e1)**9.945
crm            goto 5610
crm         elseif(e1.gt.15.d0.and.e1.lt.35.d0) then
crm            si(13,1,j)=3.288d-18*(e1/15.d0)**1.1
crm            goto 5610
crm         elseif(e1.gt.35.d0.and.e1.le.70.d0) then
crm            si(13,1,j)=8.d-18
crm            goto 5610
crm         elseif(e1.gt.70.d0.and.e1.le.750.d0) then
crm            si(13,1,j)=8.11d-18*(70.d0/e1)**1.6
crm            goto 5610
crm         elseif(e1.gt.750.d0) then
crm            si(13,1,j)=1.46d-18*(750.d0/e1)**2.09
crm         endif
crm 5610    continue
c
            if (e1.ge.e00(iefe,1)) then
               call vernercr(e1,5.461d-2,3.062d-1,2.671d7,7.923d0,
     &              2.069d1,1.382d2,2.481d-1,sigf)
               si(iefe,1,j)=sigf*1.e-18
            endif
cqcq run 16.4
c          si(13,1,j)=0.0d0
cqcq
cqcq
c         if (j.lt.59.or.j.gt.60) then
c         if (j.lt.59) then
c         if (j.ge.59) then
c         if (j.ne.36) then
c            si(13,1,j)=0.0d0
c         endif
c         if (j.eq.32.or.j.eq.36.or.j.eq.43.or.j.eq.50) then
c         if (j.eq.32.or.j.eq.43.or.j.eq.50) then
c            si(13,1,j)=0.0d0
c         endif
cqcq
c
C -- Fe II
         si(iefe,2,j)=0.0d0
crm         if(e1.gt.750.d0) then
crm            si(13,2,j)=1.462d-18*(750.d0/e1)**1.911
crm         elseif(e1.gt.45.d0) then
crm            si(13,2,j)=8.896d-18*(45.d0/e1)**1.378
crm         elseif(e1.gt.25.d0) then
crm            si(13,2,j)=6.095d-18*(e1/25.d0)**.643
crm         elseif(e1.gt.16.16d0) then
crm            si(13,2,j)=0.833d-18*(16.16d0/e1)**9.945
crm         endif
         if (e1.ge.e00(iefe,2)) then
            call vernercr(e1,1.761d-1,4.365d3,6.298d3,5.204d0,
     &           1.141d1,9.272d1,1.075d2,sigf)
            si(iefe,2,j)=sigf*1.e-18
         endif
cqcq run 16.4
c          si(13,2,j)=0.0d0
cqcq
      enddo
c
cqcq run 40.1; run 41.6; run 60.6
cqcq
c      do j=1,jmax
c         si(iena,1,j)=0.
c         si(iemg,1,j)=0.
c         si(iesi,1,j)=0.
c         si(ieca,1,j)=0.
c         si(iefe,1,j)=0.
c      enddo
cqcq
c
      return
      end



      subroutine seat(e1,si0r,alr,sr,e0r,sig)
      implicit real*8(a-h,o-z)
      real*4 si0r,alr,sr,e0r
      si0=dble(si0r)
      al=dble(alr)
      s=dble(sr)
      e0=dble(e0r)
      sig=0.
      if(e1.lt.e0) goto 100
      sig=si0*1.e-18*(al*(e0/e1)**s+(1.-al)*(e0/e1)**(s+1.))
  100 continue
      return
      end

      subroutine cro(i,l,e1,sig)
c
c     reads and interpolates photoionization data from reilman and manson
c
      implicit real*8(a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      character*80 aa
      common/init/initx,initwr,initcr,initdiff,initlin,
     &     initmu(mie,mion),initmuh,initsf
      common/csav6/e(5,15,100),s(5,15,100),jmax(5,15)
      dimension kmax(5)
c     kmax = number of ions of the different elements
      data kmax/5,6,6,4,13/
      if(initcr.eq.0) then
         initcr=1
         open(36,file='ATDAT/photocross.dat2',status= 'old')
         do iz=1,5
            do k=1,kmax(iz)
               read(36,97,err=9,end=9)aa
 97            format(a)
               read(36,*)jjmax
               do j=1,jjmax
                  read(36,*,err=9,end=9)e(iz,k,j),s(iz,k,j)
                  jmax(iz,k)=j
                  e(iz,k,j)=dlog(e(iz,k,j))
                  s(iz,k,j)=dlog(s(iz,k,j))
               enddo
 9             continue
            enddo      
         enddo
         close(36)
      endif
      emin=dexp(e(i,l,1))
      emax=dexp(e(i,l,jmax(i,l)))
      if(e1.gt.emax) then
         j=jmax(i,l)-1
         al=(s(i,l,j+1)-s(i,l,j))/(e(i,l,j+1)-e(i,l,j))
         sig=al*(dlog(e1)-e(i,l,j))+s(i,l,j)
      else
         do j=1,100
            ej=dexp(e(i,l,j+1))
            if(e1.lt.emin.or.e1.lt.ej) then
               al=(s(i,l,j+1)-s(i,l,j))/(e(i,l,j+1)-e(i,l,j))
               sig=al*(dlog(e1)-e(i,l,j))+s(i,l,j)
               goto 1
            endif
         enddo
 1       continue
      endif
      sig=1.e-18*dexp(sig)
c
      return
      end


      double precision function cr(e1,e0r,c1r,c2r,c3r,c4r,c5r,c6r,c7r,
     & s0r)
      implicit real*8(a-h,o-z)
      real*4 e0r,c1r,c2r,c3r,c4r,c5r,c6r,c7r,s0r
c
      e0=dble(e0r)
      c1=dble(c1r)
      c2=dble(c2r)
      c3=dble(c3r)
      c4=dble(c4r)
      c5=dble(c5r)
      c6=dble(c6r)
      c7=dble(c7r)
      s0=dble(s0r)
      cr=0.
      if(e1.gt.e0) then
         el=dlog10(e1)
         s=c1+c2*el+c4*el**3+c6*el**5+c3/el**2+c5/el**4+c7/el**6
         cr=1.e-18*(e0/e1)**s0*10.**s
      endif
c
      return
      end

      double precision function crb(e1,e0r,c1r,c2r,c3r,c4r,c5r,c6r,c7r,
     & s0r,jw)
      implicit real*8(a-h,o-z)
      real*4 e0r,c1r,c2r,c3r,c4r,c5r,c6r,c7r,s0r
      e0=dble(e0r)
      c1=dble(c1r)
      c2=dble(c2r)
      c3=dble(c3r)
      c4=dble(c4r)
      c5=dble(c5r)
      c6=dble(c6r)
      c7=dble(c7r)
      s0=dble(s0r)
      crb=0.
      if(e1.gt.e0) then
        el=dlog10(e1)
        s=c1/el**3+c2/el**2+c3/el+c4+c5*el+c6*el**2+c7*el**3
        crb=1.e-18*(e0/e1)**s0*10.**s
      endif
      return
      end




      double precision function fexp(x)
      implicit real*8 (a-h,o-z)
c
      if (x.ge.0.d0) then
         qe=dexp(x)
      elseif (x.lt.0.d0) then
         if (dabs(x).lt.650.) then
            qe=dexp(x)
         else
            qe=0.d0
         endif
      endif
      fexp=qe
c
      return
      end


      subroutine gamdep(xe,gion,gamex,gionco)
      implicit real*8 (a-h,o-z)
c
c  gion = gamma ionization rate ; s**-1
c  gionco = gamma ionization rate (s**-1) of CO ; As a first
c           approximation it is set = gion(ieo,1)
c
c  - non-thermal ionization of molecules has to be included.
c    see gamion for a preliminary treatment.
c
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      include 'INCLUDE/DATMOL.NOMOL'
c
      logical interp
      common/ctpar/tday,dts,iss,iss1,itd
      common/cabun/amean,ab(mie),xa(mie,mion),xaold(mid,mie,mion),
     &     nion(mie)
      common/gampar/gaelh,gahe
      common/cden/den
      common/cgam/xeg(30),ph(30),pe(30),pi(30),pig(30,mie,mion),
     &     peg(30,mie,mion,nl),im
      common/cionpot/e00(mie,mion)
      common/catdat/c(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),a(mie,mion,nl,nl)
      common/cmlevpop/xhi(nmax),xhei(nl),xoi(nl),xmg1(nl),xsi1(nl),
     &     xcaii(nl),xfei(nlf1),xfeii(nlf2),xfeiii(nlf3),xfeiv(nlf4)
      dimension gion(mie,mion),chieff(mie,mion),gamex(mie,mion,nlf)
      dimension fiei(mie,mion),feei(mie,mion,nl)
      data elch/1.60219d-12/
c
      interp=.false.
      ix=0
      if (xe.lt.xeg(1)) then
         ix=1
         interp=.false.
         write(50,701) ' gamdep : xe too small ',xe,tday
      endif
      do i=2,im
         if (xe.ge.xeg(i-1).and.xe.lt.xeg(i)) then
            ix=i-1
            interp=.true.
         endif
      enddo
      if (xe.gt.xeg(im)) then
         ix=im
         interp=.false.
         write(50,701) ' gamdep : xe too big ',xe,tday
      endif
c
      if (.not.interp) then
         fh=ph(ix)
         fe=pe(ix)
         fi=pi(ix)
         do ie=1,mie
            do ion=1,mion
               fiei(ie,ion)=pig(ix,ie,ion)
               do il=1,nl
                  feei(ie,ion,il)=peg(ix,ie,ion,il)
               enddo
            enddo
         enddo
      elseif (interp) then
         c1=(xe-xeg(ix))/(xeg(ix+1)-xeg(ix))
         fh=ph(ix)+(ph(ix+1)-ph(ix))*c1
         fe=pe(ix)+(pe(ix+1)-pe(ix))*c1
         fi=pi(ix)+(pi(ix+1)-pi(ix))*c1
         do ie=1,mie
            do ion=1,mion
               pig1=pig(ix,ie,ion)
               pig2=pig(ix+1,ie,ion)
               fiei(ie,ion)=pig1+(pig2-pig1)*c1
               do il=1,nl
                  peg1=peg(ix,ie,ion,il)
                  peg2=peg(ix+1,ie,ion,il)
                  feei(ie,ion,il)=peg1+(peg2-peg1)*c1
               enddo
            enddo
         enddo
      endif
c
c  ionization
c
      esum=0.
      do ie=1,mie
         do ion=1,mion
            esum=esum+fiei(ie,ion)
            chieff(ie,ion)=0.
            gion(ie,ion)=0.
            if (fiei(ie,ion).gt.0.) then
c  chieff in eV
               chieff(ie,ion)=e00(ie,ion)*ab(ie)*xa(ie,ion)/
     &             fiei(ie,ion)
            endif
            if (chieff(ie,ion).gt.0.) then
               gion(ie,ion)=gahe/(chieff(ie,ion)*elch)
            endif
         enddo
      enddo
c
c preliminary for CO
      gionco=gion(ieo,1)
c
c  excitation
c
      x1=0.
      if (ie.eq.ieh) x1=xhi(1)
      if (ie.eq.iehe) x1=xhei(1)
      if (ie.eq.ieo) x1=xoi(1)
      if (ie.eq.iemg) x1=xmg1(1)
      if (ie.eq.iesi) x1=xsi1(1)
      if (ie.eq.ieca) x1=xcaii(1)
c
      do ie=1,12
         do ion=1,mion
            do il=1,nl
               esum=esum+feei(ie,ion,il)
               gamex(ie,ion,il)=0.
               if (feei(ie,ion,il).gt.0.) then
c  exeff in eV
                  exeff=e(ie,ion,il)*ab(ie)*x1/feei(ie,ion,il)
               endif
               if (exeff.gt.0.) then
                  gamex(ie,ion,il)=gahe/(exeff*elch)
               endif
            enddo
         enddo
      enddo
c  
c  heating
c  gaelh = the energy s**-1 that goes into heating
c
      he=1.-esum
      if (abs(fh-he).gt.0.02) write(50,701) ' gamdep : cons. problems '
      gaelh=he*gahe*den
c
c  see gamion
c      if(idiag.eq.1) then
c        ginp=gahe/den
c        write(8,991)extot,sumh,he,(1.-extot-sumh),gahe/den
c991     format(' gam ',1pe12.4,5e12.4)
c        write(8,995)(ginp*effi(ie,1),ie=1,15)
c995     format('io',1pe12.4,20e12.4)
c        do ie=1,15
c          do i=1,nl
c            if(exeff(ie,1,i).gt.0.) then
c              write(8,996)ie,i,exeff(ie,1,i),exeff(ie,1,i)*ginp*ex
c996           format('ex',2i3,1pe12.4,5e12.4)
c            endif
c          enddo
c        enddo
c      endif
c
 701  format(a,1pe12.4,e12.4)
      return
      end



      subroutine gexc(pem,nls,feeihi,feeihei,feeioi,feeinai,
     &     feeimgi,feeimgii,feeisi1,feeicaii,feeifei,feeifeii)
      implicit real*8 (a-h,o-z)
c      parameter(mies=55,mil=15)
      parameter(mies=69,mil=15)
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      real*4 pem(mies,mil)
      integer nls(mies)
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
      common/cieion/ieion(mie,mion)
      common/depthind/id
      dimension feeihi(mid,nmax),feeihei(mid,nl),feeioi(mid,nl)
      dimension feeinai(mid,nl),feeimgi(mid,nl),feeimgii(mid,nl)
      dimension feeicaii(mid,nl),feeifei(mid,nlf1),feeifeii(mid,nlf2)
      dimension feeisi1(mid,nl)
c
c  feeihi(id,il)  = the fraction of the deposited energy going into
c                     H I, level il (IOR-number) at depth id.
c  feeihei(id,il)  = the fraction of the deposited energy going into
c                     He I, level il (IOR-number) at depth id.
c  feeioi(id,il)  = the fraction of the deposited energy going into
c                     O I, level il (IOR-number) at depth id.
c  feeinai(id,il)  = the fraction of the deposited energy going into
c                     Na I, level il (IOR-number) at depth id.
c  feeimgi(id,il)  = the fraction of the deposited energy going into
c                     Mg I, level il (IOR-number) at depth id.
c  feeimgii(id,il)  = the fraction of the deposited energy going into
c                     Mg II, level il (IOR-number) at depth id.
c  feeisi1(id,il)  = the fraction of the deposited energy going into
c                     Si I, level il (IOR-number) at depth id.
c  feeicaii(id,il)  = the fraction of the deposited energy going into
c                     Ca II, level il (IOR-number) at depth id.
c  feeifei(id,il)  = the fraction of the deposited energy going into
c                     Fe I, level il (IOR-number) at depth id.
c  feeifeii(id,il)  = the fraction of the deposited energy going into
c                     Fe II, level il (IOR-number) at depth id.
c  pem(ies,ils)     = the fraction of the deposited energy going into
c                     element ies, level ils (SF-number).
c
c ( - The energy going into levels in SF which are not included in IOR
c    are distributed among the levels in IOR in proportion to the energy
c    going into that level. This makes the energy going into excitation 
c    the same even of the levels are not quite the same. 
c    Ex : He I, O I, Ca II.)
c
c  - No energy is assumed to go into those levels in IOR which are not
c    included in SF. 
c
      do il=1,nmax
         feeihi(id,il)=0.
      enddo
      do il=1,nl
         feeihei(id,il)=0.
         feeioi(id,il)=0.
         feeicaii(id,il)=0.
      enddo
      do il=1,nlf1
         feeifei(id,il)=0.
      enddo
      do il=1,nlf2
         feeifeii(id,il)=0.
      enddo
c ***********
      ie=ieh
      ion=1
      ies=ieion(ie,ion)
c  H I
      feeihi(id,1)=0.
ch
      if (nn.ge.2) then
c  2s
         feeihi(id,2)=pem(ies,1)
c  2p
         feeihi(id,3)=pem(ies,2)
      else
         feeihi(id,2)=pem(ies,1)+pem(ies,2)
      endif
c
      if (nn.ge.3) then
c  3s
         feeihi(id,4)=pem(ies,3)
c  3p
         feeihi(id,5)=pem(ies,4)
c  3d
         feeihi(id,6)=pem(ies,5)
      else
         feeihi(id,3)=pem(ies,3)+pem(ies,4)+pem(ies,5)
      endif
c
      if (nn.ge.4) then
c  4s
         feeihi(id,7)=pem(ies,6)
c  4p
         feeihi(id,8)=pem(ies,7)
c  4d
         feeihi(id,9)=pem(ies,8)
c  4f
         feeihi(id,10)=pem(ies,9)
      else
         feeihi(id,4)=pem(ies,6)+pem(ies,7)+pem(ies,8)+pem(ies,9)
      endif
ch
c
c  feeihi(2) : n=2              (IOR)
c  pem(1)  : 2s  ,pem(2) : 2p (SF)
ch      feeihi(id,2)=pem(ies,1)+pem(ies,2)
c  feeihi(3) : n=3
c  pem(3,4,5) : 3s,3p,3d
ch      feeihi(id,3)=pem(ies,3)+pem(ies,4)+pem(ies,5)
c  feeihi(4) : n=4
c  pem(6,7,8,9) : 4s,4p,4d,4f
ch      feeihi(id,4)=pem(ies,6)+pem(ies,7)+pem(ies,8)+pem(ies,9)
c
ch      feeihi(id,5)=0.
c
c ***********
      ie=iehe
      ion=1
      ies=ieion(ie,ion)
c  He I
      feeihei(id,1)=0.
c
c  2s 3S
      feeihei(id,2)=pem(ies,1)
c  2s 1S
      feeihei(id,3)=pem(ies,2)
c  2p 3P
      feeihei(id,4)=pem(ies,3)
c  2p 1P
      feeihei(id,5)=pem(ies,4)
c  3s 3S
c      feeihei(id,6)=pem(ies,8)
      feeihei(id,6)=pem(ies,5)
c  3p 3P
c      feeihei(id,7)=pem(ies,9)
      feeihei(id,7)=pem(ies,6)
c  3d 3D
c      feeihei(id,8)=pem(ies,10)
      feeihei(id,8)=pem(ies,7)
c
      feeihei(id,9)=0.
      feeihei(id,10)=0.
      feeihei(id,11)=0.
      feeihei(id,12)=0.
      feeihei(id,13)=0.
      feeihei(id,14)=0.
      feeihei(id,15)=0.
      feeihei(id,16)=0.
c
c  SF : 5 = 3 1S ; 6 = 3 1P ; 7 = 3 1D
c  these levels are not included in IOR =>
c  the non-thermal exc. to these levels is assumed
c  to end up in levels 2 1S and 2 1P instead.     
c
c      fex=pem(ies,5)+pem(ies,6)+pem(ies,7)
c      fe35=feeihei(id,3)+feeihei(id,5)
c      fet=fex+fe35
c  2s 1S
c      feeihei(id,3)=feeihei(id,3)*fet/fe35
c  2p 1P
c      feeihei(id,5)=feeihei(id,5)*fet/fe35
c
      sumsf=0.
      do j=1,nls(ies)
         sumsf=sumsf+pem(ies,j)
      enddo
      n=nlev(ie,ion)
      sum=0.
      do j=1,n
         sum=sum+feeihei(id,j)
      enddo
      if (sum.gt.0.) then
         do j=1,n
            feeihei(id,j)=feeihei(id,j)*sumsf/sum
         enddo
      endif
c
c ***********
      ie=ieo
      ion=1
      ies=ieion(ie,ion)
c  O I
c 3P
      feeioi(id,1)=0.
      feeioi(id,2)=0.
      feeioi(id,3)=0.
c
c  1D
      feeioi(id,4)=pem(ies,1)
c  1S
      feeioi(id,5)=pem(ies,2)
c  3s 5S
      feeioi(id,6)=pem(ies,3)
c  3s 3S
      feeioi(id,7)=pem(ies,4)
c  3p 5P
      feeioi(id,8)=pem(ies,5)
c  3p 3P
      feeioi(id,9)=pem(ies,6)
c  4s 5S
      feeioi(id,10)=pem(ies,10)
c  4s 3S
      feeioi(id,11)=pem(ies,7)
c  3d 5D
      feeioi(id,12)=pem(ies,9)
c  3d 3D
      feeioi(id,13)=pem(ies,8)
c  3s' 3D
cior      feeioi(id,9)=pem(ies,15)
c      feeioi(id,9)=pem(ies,8)
c
c ( SF :  7 = 4s 3S 
c        9 = 4p 3P ; 10 = 3d 5D ; 11 = 4s 5S 
c       12 = 4p 5P ; 13 = 4d 5D ; 14 = 4d 3D
c  these levels are not included in IOR =>
c  the non-thermal exc. to these levels is assumed
c  in some lower lying levels instead (pretty much ad hoc)
c        (SF)                 (IOR) 
c  4s 3S  (7) => 3p 3P (7) 
c  4p 3P  (9) => 3s 3S (5)
c  3d 5D (10) => 3p 5P (6), 3s 5S (4)
c  4s 5S (11) => 3p 5P (6), 3s 5S (4)
c  4p 5P (12) => 3s 5S (4)
c  4d 5D (13) => 3p 5P (6), 3s 5S (4)
c  4d 3D (14) => 3s 3S (5), 3p 3P (7)  )
c
c      feeioi(id,7)=feeioi(id,7)+pem(ies,7)
c
c      feeioi(id,5)=feeioi(id,5)+pem(ies,9)
c
c      fex=pem(ies,10)+pem(ies,11)+pem(ies,13)
c      fe64=feeioi(id,6)+feeioi(id,4)
c      fet=fex+fe64
c      feeioi(id,6)=feeioi(id,6)*fet/fe64
c      feeioi(id,4)=feeioi(id,4)*fet/fe64
c
c      feeioi(id,4)=feeioi(id,4)+pem(ies,12)
c
c      fe57=feeioi(id,5)+feeioi(id,7)
c      fet=fe57+pem(ies,14)
c      feeioi(id,5)=feeioi(id,5)*fet/fe57
c      feeioi(id,7)=feeioi(id,7)*fet/fe57
c
      sumsf=0.
      do j=1,nls(ies)
         sumsf=sumsf+pem(ies,j)
      enddo
      sum=0.
      n=nlev(ie,ion)
      do j=1,n
         sum=sum+feeioi(id,j)
      enddo
      if (sum.gt.0.) then
         do j=1,n
            feeioi(id,j)=feeioi(id,j)*sumsf/sum
         enddo
      endif
c
c ***********
      ie=iena
      ion=1
      ies=ieion(ie,ion)
c  Na I
      feeinai(id,1)=0.
c
c  3s 2S -> 5p 2P
c
      feeinai(id,2)=pem(ies,1)
c
c ***********
      ie=iemg
      ion=1
      ies=ieion(ie,ion)
c  Mg I
      feeimgi(id,1)=0.
c
c  3s 1S -> 3p 3P
c
      feeimgi(id,2)=pem(ies,1)
c
c  3s 1S -> 3p 1P
c
      feeimgi(id,3)=pem(ies,2)
c
      sumsf=0.
      do j=1,nls(ies)
         sumsf=sumsf+pem(ies,j)
      enddo
      sum=0.
      n=3
      do j=1,n
         sum=sum+feeimgi(id,j)
      enddo
      if (sum.gt.0.) then
         do j=1,n
            feeimgi(id,j)=feeimgi(id,j)*sumsf/sum
         enddo
      endif
c
c ***********
      ie=iemg
      ion=2
      ies=ieion(ie,ion)
c  Mg II
      feeimgii(id,1)=0.
c
c  3s 2S -> 3p 2P
c
      feeimgii(id,2)=pem(ies,1)
c
      sumsf=0.
      do j=1,nls(ies)
         sumsf=sumsf+pem(ies,j)
      enddo
      sum=0.
      n=1
      do j=1,n
         sum=sum+feeimgii(id,j)
      enddo
      if (sum.gt.0.) then
         do j=1,n
            feeimgii(id,j)=feeimgii(id,j)*sumsf/sum
         enddo
      endif
c
c ***********
c 970218
      ie=iesi
      ion=1
      ies=ieion(ie,ion)
c  Si I
      feeisi1(id,1)=0.
      feeisi1(id,2)=0.
      feeisi1(id,3)=0.
c
c  3P -> 1D
c
cnew9804      feeisi1(id,2)=pem(ies,1)
      feeisi1(id,4)=pem(ies,1)
c
c  3P -> 1S 
c
cnew9804      feeisi1(id,3)=pem(ies,2)
      feeisi1(id,5)=pem(ies,2)
c
      sumsf=0.
      do j=1,nls(ies)
         sumsf=sumsf+pem(ies,j)
      enddo
      sum=0.
      n=5
      do j=1,n
         sum=sum+feeisi1(id,j)
      enddo
      if (sum.gt.0.) then
         do j=1,n
            feeisi1(id,j)=feeisi1(id,j)*sumsf/sum
         enddo
      endif
c
c ***********
      ie=ieca
      ion=2
      ies=ieion(ie,ion)
c  Ca II
      feeicaii(id,1)=0.
c
c  3d ; the energy is divided between the two sub-levels 
c       according to statistical weights.
c
      feeicaii(id,2)=pem(ies,1)*(4./10.)
      feeicaii(id,3)=pem(ies,1)*(6./10.)
c      feeicaii(id,2)=pem(ies,1)
c
c  4p ; the energy is divided between the two sub-levels 
c       according to statistical weights.
c
      feeicaii(id,4)=pem(ies,2)*(2./6.)
      feeicaii(id,5)=pem(ies,2)*(4./6.)
c      feeicaii(id,3)=pem(ies,2)
c
      sumsf=0.
      do j=1,nls(ies)
         sumsf=sumsf+pem(ies,j)
      enddo
      sum=0.
      n=nlev(ie,ion)
      do j=1,n
         sum=sum+feeicaii(id,j)
      enddo
      if (sum.gt.0.) then
         do j=1,n
            feeicaii(id,j)=feeicaii(id,j)*sumsf/sum
         enddo
      endif
c
c ***********
      ie=iefe
      ion=1
      ies=ieion(ie,ion)
c  Fe I
c
c  z5D ; the energy is divided between the sub-levels 
c       depending on the fraction of the mean 
c       osc.strength going to that particular level 
c       (see calculation in /cf1/cecilia/DATA/ATDAT/FE/feoscfe.f).
c
      feeifei(id,54)=pem(ies,1)*(0.378)
      feeifei(id,56)=pem(ies,1)*(0.277)
      feeifei(id,58)=pem(ies,1)*(0.193)
      feeifei(id,61)=pem(ies,1)*(0.114)
      feeifei(id,62)=pem(ies,1)*(0.038)
c
c  z5F ; the energy is divided between the sub-levels 
c       depending on the fraction of the mean 
c       osc.strength going to that particular level 
c       (see calculation in /cf1/cecilia/DATA/ATDAT/FE/feoscfe.f).
c
      feeifei(id,65)=pem(ies,2)*(0.334)
      feeifei(id,66)=pem(ies,2)*(0.259)
      feeifei(id,67)=pem(ies,2)*(0.194)
      feeifei(id,69)=pem(ies,2)*(0.134)
      feeifei(id,70)=pem(ies,2)*(0.079)
c
c  z5P ; the energy is divided between the sub-levels 
c       depending on the fraction of the mean 
c       osc.strength going to that particular level 
c       (see calculation in /cf1/cecilia/DATA/ATDAT/FE/feoscfe.f).
c
      feeifei(id,73)=pem(ies,3)*(0.456)
      feeifei(id,78)=pem(ies,3)*(0.337)
      feeifei(id,79)=pem(ies,3)*(0.207)
c
c  y5D ; the energy is divided between the sub-levels 
c       depending on the fraction of the mean 
c       osc.strength going to that particular level 
c       (see calculation in /cf1/cecilia/DATA/ATDAT/FE/feoscfe.f).
c
      feeifei(id,89)=pem(ies,4)*(0.353)
      feeifei(id,91)=pem(ies,4)*(0.284)
      feeifei(id,94)=pem(ies,4)*(0.194)
      feeifei(id,96)=pem(ies,4)*(0.126)
      feeifei(id,98)=pem(ies,4)*(0.043)
c
c  y5F ; the energy is divided between the sub-levels 
c       depending on the fraction of the mean 
c       osc.strength going to that particular level 
c       (see calculation in /cf1/cecilia/DATA/ATDAT/FE/feoscfe.f).
c
      feeifei(id,92)=pem(ies,5)*(0.276)
      feeifei(id,97)=pem(ies,5)*(0.260)
      feeifei(id,99)=pem(ies,5)*(0.216)
      feeifei(id,101)=pem(ies,5)*(0.156)
      feeifei(id,104)=pem(ies,5)*(0.092)
c
c  z5G ; the energy is divided between the sub-levels 
c       depending on the fraction of the mean 
c       osc.strength going to that particular level 
c       (see calculation in /cf1/cecilia/DATA/ATDAT/FE/feoscfe.f).
c
      feeifei(id,105)=pem(ies,6)*(0.335)
      feeifei(id,106)=pem(ies,6)*(0.)
      feeifei(id,107)=pem(ies,6)*(0.368)
      feeifei(id,109)=pem(ies,6)*(0.221)
      feeifei(id,111)=pem(ies,6)*(0.076)
c
c  y5P ; the energy is divided between the sub-levels 
c       depending on the fraction of the mean 
c       osc.strength going to that particular level 
c       (see calculation in /cf1/cecilia/DATA/ATDAT/FE/feoscfe.f).
c
      feeifei(id,114)=pem(ies,7)*(0.480)
      feeifei(id,115)=pem(ies,7)*(0.328)
      feeifei(id,117)=pem(ies,7)*(0.192)
c  
      n=nlev(ie,ion)
      sumsf=0.
      do j=1,nls(ies)
         sumsf=sumsf+pem(ies,j)
      enddo
      sum=0.
      do j=1,n
         sum=sum+feeifei(id,j)
      enddo
      if (sum.gt.0.) then
         do j=1,n
            feeifei(id,j)=feeifei(id,j)*sumsf/sum
         enddo
      elseif (sum.eq.0.) then
cnew 000110
c When using a small Fe I modelatom, place the excitations
c to levels above the highest level in the modelatom used,
c in the highest level included in the modelatom.
c
         feeifei(id,n)=sumsf
      endif
c ***********
      ie=iefe
      ion=2
      ies=ieion(ie,ion)
c  Fe II
c
c  a4F ; the energy is divided between the sub-levels 
c       according to statistical weights.
c
      feeifeii(id,6)=pem(ies,1)*(10./28.)
      feeifeii(id,7)=pem(ies,1)*(8./28.)
      feeifeii(id,8)=pem(ies,1)*(6./28.)
      feeifeii(id,9)=pem(ies,1)*(4./28.)
c
c  a4D ; the energy is divided between the sub-levels 
c       according to statistical weights.
c
      feeifeii(id,10)=pem(ies,2)*(8./20.)
      feeifeii(id,11)=pem(ies,2)*(6./20.)
      feeifeii(id,12)=pem(ies,2)*(4./20.)
      feeifeii(id,13)=pem(ies,2)*(2./20.)
c
c  a4P ; the energy is divided between the sub-levels 
c       according to statistical weights.
c
      feeifeii(id,14)=pem(ies,3)*(6./12.)
      feeifeii(id,15)=pem(ies,3)*(4./12.)
      feeifeii(id,16)=pem(ies,3)*(2./12.)
c
c  b4P ; the energy is divided between the sub-levels 
c       according to statistical weights.
c
      feeifeii(id,24)=pem(ies,4)*(6./12.)
      feeifeii(id,30)=pem(ies,4)*(4./12.)
      feeifeii(id,31)=pem(ies,4)*(2./12.)
c
c  b4F ; the energy is divided between the sub-levels 
c       according to statistical weights.
c
      feeifeii(id,32)=pem(ies,5)*(10./28.)
      feeifeii(id,33)=pem(ies,5)*(8./28.)
      feeifeii(id,34)=pem(ies,5)*(6./28.)
      feeifeii(id,35)=pem(ies,5)*(4./28.)
c
c  a6S 
c
      feeifeii(id,36)=pem(ies,6)
c
c  z6Do ; the energy is divided between the sub-levels 
c       according to statistical weights.
c
      feeifeii(id,64)=pem(ies,7)*(10./30.)
      feeifeii(id,65)=pem(ies,7)*(8./30.)
      feeifeii(id,66)=pem(ies,7)*(6./30.)
      feeifeii(id,67)=pem(ies,7)*(4./30.)
      feeifeii(id,68)=pem(ies,7)*(2./30.)
c
c  z6Fo ; the energy is divided between the sub-levels 
c       according to statistical weights.
c
      feeifeii(id,69)=pem(ies,8)*(12./42.)
      feeifeii(id,70)=pem(ies,8)*(10./42.)
      feeifeii(id,71)=pem(ies,8)*(8./42.)
      feeifeii(id,72)=pem(ies,8)*(6./42.)
      feeifeii(id,73)=pem(ies,8)*(4./42.)
      feeifeii(id,74)=pem(ies,8)*(2./42.)
c
c  z6Po ; the energy is divided between the sub-levels 
c       according to statistical weights.
c
      feeifeii(id,75)=pem(ies,9)*(8./18.)
      feeifeii(id,76)=pem(ies,9)*(6./18.)
      feeifeii(id,77)=pem(ies,9)*(4./18.)
c
c  z4Po ; the energy is divided between the sub-levels 
c       according to statistical weights.
c
      feeifeii(id,88)=pem(ies,10)*(6./12.)
      feeifeii(id,89)=pem(ies,10)*(4./12.)
      feeifeii(id,90)=pem(ies,10)*(2./12.)
c
      n=nlev(ie,ion)
      sumsf=0.
      do j=1,nls(ies)
         sumsf=sumsf+pem(ies,j)
      enddo
      sum=0.
      do j=1,n
         sum=sum+feeifeii(id,j)
      enddo
      if (sum.gt.0.) then
         do j=1,n
            feeifeii(id,j)=feeifeii(id,j)*sumsf/sum
         enddo
      elseif (sum.eq.0.) then
cnew 000110
c When using a small Fe II modelatom, place the excitations
c to levels above the highest level in the modelatom used,
c in the highest level included in the modelatom.
c
         feeifeii(id,n)=sumsf
      endif
c ***********
c  No gamma-exitations for the other elements
c
      return
      end


      subroutine gamsf(xe)
      implicit real*8(a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
c      parameter(mies=55,mil=15)
      parameter(mies=69,mil=15)
      real*4 xesf,abin(mies),ph,peie(mies),pem(mies,mil)
      real*4 pim(mies),pio(4),xieff(mies),abinold
      real*4 xesfx
      integer nls(mies)
      common/ctpar/tday,dts,iss,iss1,itd
      common/cabun/amean,ab(mie),xa(mie,mion),xaold(mid,mie,mion),
     &     nion(mie)
      common/init/initx,initwr,initcr,initdiff,initlin,
     &     initmu(mie,mion),initmuh,initsf
      common/depthind/id
      common/cieion/ieion(mie,mion)
      common/cfrsf/frhe(mid),fiei(mid,mie,mion),feeihi(mid,nmax),
     &     feeihei(mid,nl),feeioi(mid,nl),feeicaii(mid,nl),
     &     feeinai(mid,nl),feeimgi(mid,nl),feeimgii(mid,nl),
     &     feeisi1(mid,nl),
     &     feeifei(mid,nlf1),feeifeii(mid,nlf2),fio(mid,4)
c     &     fieoii2d,fieoii2p
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/cebin/e1(ne),en(ne),jmax
      common/csn/i87acl,i87a,i93j,isn
      common/cmod/abmod(10,mie),vrmod(mid),rmod(mid),rlab(mid),
     &     dmmod(mid),ameanmod(10),denmod(mid),tdmod,
     &     nrtab(mid),izmod(mid),nrmod,mixm
      common/cfrac/frionh,frexh3,frexo2
      common/cfrtot/frheat,frion,frexc
c
      common/csav7/abinold(mid,mies),nik0
c
      xesf=xe
c
c  excitations included (iexfl=1)
      iexfl=1
c  excitations not included (iexfl=0); run 22.2; run 22.3
cqcq run 99
c      iexfl=0
c
      if (initsf.eq.0) then
         nik0=0
         call abindex(ieion)
         initsf=1
         write(100,704) i93j,i87a,isn,iss1,mixm,nrmod
c
         do i=1,mid
            do ies=1,mies
               abinold(i,ies)=0.
            enddo
         enddo
c
      else
c  atan for the energy bins are not calc.
         iatafl=0
      endif
c  the input abundances for sf
      do i=1,mies
         abin(i)=0.
      enddo
c
      xesfx=0.
      do ie=1,mie
         do ion=1,mion
            ies=ieion(ie,ion)
            if (ies.gt.0) then
               abxa=ab(ie)*xa(ie,ion)
c               abin(ies)=ab(ie)*xa(ie,ion)
               abin(ies)=abxa
               if (abxa.lt.1d-37.and.abxa.gt.0.) then
                  abin(ies)=1e-37
               endif
c               if (id.eq.6) then
c                  write(*,*) ie,ion,abin(ies)
c               endif
cqcq
c  do not let the abundances change by more than 10% between
c  two deposition calculations
c               if (abin(ies).gt.0.) then
c               if (ab(ie).gt.0.1) then
c                  if (abinold(id,ies).ne.0.) then
c                     dabin=(abinold(id,ies)-abin(ies))/abin(ies)
c                     if (abs(dabin).gt.0.50) then
c                        if (dabin.gt.0.) then
c                           abin(ies)=abinold(id,ies)/1.10
c                        elseif(dabin.lt.0.) then
c                           abin(ies)=abinold(id,ies)*1.10
c                        endif
c                        write(50,705) ie,ion,ab(ie)*xa(ie,ion),
c     &                       abin(ies)
c                     endif
c                  endif
c               endif
c               xesfx=xesfx+abin(ies)*(ion-1)
c            else
c               absum=0.
c               do i=ion-1,1,-1
c                  absum=absum+abin(ieion(ie,i))
c               enddo
c               dxesf=(ab(ie)-absum)*(ion-1)
c               xesfx=xesfx+dxesf
c               goto 9
            endif
         enddo
c 9       continue
      enddo
c      if (xesfx.gt.0.) xesf=xesfx
c
c  save the abundances for which the deposition is calculatedc
c      do ies=1,mies
c         abinold(id,ies)=abin(ies)
c      enddo
cqcq
c
c these threshold values for xe are chosen just ad hoc     
c
      if (xesf.gt.0.1) then
         nik=1000
      elseif (xesf.gt.0.01) then
         nik=1500
      elseif(xesf.gt.0.001) then
         nik=2000
      else
         nik=2000
      endif
      if (nik.ne.nik0) then
c  atan for every,given energy bin are calc. every time
c  the energy binning is changed
         iatafl=1
         nik0=nik
      endif
c
      call sf(xesf,abin,iexfl,iatafl,ifail,ph,peie,pem,pim,
     &     pio,xieff,nls,nik)
c
      pi=0.
      pe=0.
      do ies=1,mies
         pi=pi+pim(ies)
         do il=1,mil
            pe=pe+pem(ies,il)
         enddo
      enddo
      sumfr=ph+pe+pi
c
c  ionization
c
      esum=0.
      do ie=1,mie
         do ion=1,mion
            ies=ieion(ie,ion)
            fiei(id,ie,ion)=0.
            if (ies.gt.0) fiei(id,ie,ion)=pim(ies)
            esum=esum+fiei(id,ie,ion)
         enddo
      enddo
      frion=esum
cqcq
c  special treatment of ionization of o i followed by excitation
c  in o ii. explicitly treat the ionization/excitation of 
c  the levels 2Do and 2Po in o ii.
c
      do i=1,4
         fio(id,i)=0.
c         fio(id,i)=pio(i)
      enddo
cqcq
      esum=esum-fiei(id,ieo,1)
      fio(id,1)=0.
      fio(id,2)=pio(2)*3.3/16.9
      fio(id,3)=pio(3)*5.0/18.6
      fio(id,4)=0.
      fiei(id,ieo,1)=pio(1)+pio(2)+pio(3)+pio(4)-
     &     fio(id,2)-fio(id,3)
      esum=esum+fiei(id,ieo,1)
      frion=esum
cqcq
c      fieoii2d=pio(2)
c      fieoii2p=pio(3)
c
c  ionization of the highest ionization state of an ion is
c  assumed to finally end up as heating
c  no good !! does not work
      hisum=0.
c      do ie=1,mie
c         if (ie.eq.ieca.or.ie.eq.iefe) then
c            ion=3
c         else
c            ion=2
c         endif
c         hisum=hisum+fiei(ie,ion)
c         fiei(ie,ion)=0.
c      enddo
cqcq
c
c  excitation
c
c  exc of levels in o ii when ionizing of o i
c
      esum=esum+fio(id,2)+fio(id,3)
      if (iexfl.eq.1) then

         call gexc(pem,nls,feeihi,feeihei,feeioi,feeinai,
     &        feeimgi,feeimgii,feeisi1,feeicaii,feeifei,feeifeii)
cqcq
         do il=nlev(ieh,1)+1,nmax
            feeihi(id,il)=0.
         enddo
         do il=nlev(iehe,1)+1,nl
            feeihei(id,il)=0.
         enddo
         do il=3,nl
            feeinai(id,il)=0.
         enddo
         do il=4,nl
            feeimgi(id,il)=0.
         enddo
         do il=3,nl
            feeimgii(id,il)=0.
         enddo
         do il=6,nl
            feeisi1(id,il)=0.
         enddo
         do il=nlev(ieo,1)+1,nl
            feeioi(id,il)=0.
         enddo
         do il=nlev(ieca,2)+1,nl
            feeicaii(id,il)=0.
         enddo
         do il=nlev(iefe,1)+1,nlf1
            feeifei(id,il)=0.
         enddo
         do il=nlev(iefe,2)+1,nlf2
            feeifeii(id,il)=0.
         enddo
cqcq         
      elseif (iexfl.eq.0) then
         do il=1,nmax
            feeihi(id,il)=0.
         enddo
         do il=1,nl
            feeihei(id,il)=0.
            feeinai(id,il)=0.
            feeimgi(id,il)=0.
            feeimgii(id,il)=0.
            feeisi1(id,il)=0.
            feeioi(id,il)=0.
            feeicaii(id,il)=0.
         enddo
         do il=1,nlf1
            feeifei(id,il)=0.
         enddo
         do il=1,nlf2
            feeifeii(id,il)=0.
         enddo
      endif
      do il=1,nmax
         esum=esum+feeihi(id,il)
      enddo
      do il=1,nl
         esum=esum+feeihei(id,il)
         esum=esum+feeinai(id,il)
         esum=esum+feeimgi(id,il)
         esum=esum+feeimgii(id,il)
         esum=esum+feeisi1(id,il)
         esum=esum+feeioi(id,il)
         esum=esum+feeicaii(id,il)
      enddo
      do il=1,nlf1
         esum=esum+feeifei(id,il)
      enddo
      do il=1,nlf2
         esum=esum+feeifeii(id,il)
      enddo
      frexo2=feeioi(id,2)
      frexc=esum-frion
c
c  heating
c  gaelh = the energy s**-1 that goes into heating
c
      he=1.-esum
      herr=abs(ph-he)
      esum=esum+ph
      if (herr.gt.0.02) then
         write(50,702) ' gamdep-problems ',id,ifail,tday,xesf,xe,
     &        herr,sumfr,esum
      else
         write(50,702) ' gamsf',id,ifail,tday,xesf,xe,herr,sumfr,
     &        esum
      endif
      he=he+hisum
      frhe(id)=dble(he)
      frheat=frhe(id)
c
      ifil=100
      write(ifil,704) id
      write(ifil,703) xe,frhe(id)
      do ie=1,iemax
         write(ifil,703) (fiei(id,ie,ion),ion=1,mion)
      enddo
      write(ifil,703) (fio(id,i),i=1,4)
      write(ifil,703) (feeihi(id,il),il=1,nlev(ieh,1))
      write(ifil,703) (feeihei(id,il),il=1,nlev(iehe,1))
      write(ifil,703) (feeioi(id,il),il=1,nlev(ieo,1))
      write(ifil,703) (feeinai(id,il),il=1,2)
      write(ifil,703) (feeimgi(id,il),il=1,3)
      write(ifil,703) (feeimgii(id,il),il=1,2)
      write(ifil,703) (feeisi1(id,il),il=1,3)
      write(ifil,703) (feeicaii(id,il),il=1,nlev(ieca,2))
      ies=ieion(iefe,1)
      write(ifil,703) (pem(ies,il),il=1,7)
      ies=ieion(iefe,2)
      write(ifil,703) (pem(ies,il),il=1,10)
c
c      write(33,701) tday,xe,ph,he,pi,pe,sumfr,nik,ifail 
c      
 701  format(1pe12.4,6e12.4,i6,i4)
 702  format(a,2i4,1pe12.4,10e12.4)
 703  format(1pe12.4,20e12.4)
 704  format(10i5)
 705  format('gamsf-ab',2i5,1pe12.4,1e12.4)
c
      return
      end


      subroutine gdep(gion,gamex,gionco)
      implicit real*8 (a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      common/cabun/amean,ab(mie),xa(mie,mion),xaold(mid,mie,mion),
     &     nion(mie)
      common/ctpar/tday,dts,iss,iss1,itd
      common/gampar/gaelh,gahe
      common/cden/den
      common/depthind/id
      common/cionpot/e00(mie,mion)
      common/cmlevpop/xhi(nmax),xhei(nl),xoi(nl),xmg1(nl),xsi1(nl),
     &     xcaii(nl),xfei(nlf1),xfeii(nlf2),xfeiii(nlf3),xfeiv(nlf4)
      common/catdat/cq(mie,mion,nl,nl),ci(mie,mion,nl),g(mie,mion,nl),
     &e(mie,mion,nl),aq(mie,mion,nl,nl)
      common/catdath/ghn(nm),ehn(nm),ah(nmax,nmax),
     &     nmn,nlm(nm,0:nm),nnm(nm),nln(nmax),nn
      common/cfrsf/frhe(mid),fiei(mid,mie,mion),feeihi(mid,nmax),
     &     feeihei(mid,nl),feeioi(mid,nl),feeicaii(mid,nl),
     &     feeinai(mid,nl),feeimgi(mid,nl),feeimgii(mid,nl),
     &     feeisi1(mid,nl),
     &     feeifei(mid,nlf1),feeifeii(mid,nlf2),fio(mid,4)
      common/cchieff/chieff(mie,mion)
      common/cfedat/cf1(nlf1,nlf1),cf2(nlf2,nlf2),cf3(nlf3,nlf3),
     &     cf4(nlf4,nlf4),cif1(nlf1),cif2(nlf2),cif3(nlf3),cif4(nlf4),
     &     gf1(nlf1),gf2(nlf2),gf3(nlf3),gf4(nlf4),ef1(nlf1),ef2(nlf2),
     &     ef3(nlf3),ef4(nlf4),af1(nlf1,nlf1),af2(nlf2,nlf2),
     &     af3(nlf3,nlf3),af4(nlf4,nlf4),cpf2(3,3)
      dimension gion(mie,mion),gamex(mie,mion,nlf)
      data elch/1.60219d-12/
c
c  ionization
c
      frion=0.
      do ie=1,mie
         do ion=1,mion
            chieff(ie,ion)=0.
            gion(ie,ion)=0.
            frion=frion+fiei(id,ie,ion)
c            if (fiei(id,ie,ion).gt.1.d-40) then
            if (fiei(id,ie,ion).gt.0.) then
c  chieff in eV
               chieff(ie,ion)=e00(ie,ion)*ab(ie)*xa(ie,ion)/
     &             fiei(id,ie,ion)
            endif
c            if (chieff(ie,ion).gt.1.d-40) then
            if (chieff(ie,ion).gt.0.) then
               gion(ie,ion)=gahe/(chieff(ie,ion)*elch)
            endif
         enddo
      enddo
c preliminary for CO
      gionco=gion(ieo,1)
c
c  excitation
c
c  - non-thermal excitation of fe ii included separately
c
      frexc=0.
c
      do ie=1,iemax
         do ion=1,mion
            do il=1,nlf
               gamex(ie,ion,il)=0.
            enddo
         enddo
      enddo
c
c  h i
      ie=ieh
      ion=1
      x1=xhi(1)
      do il=1,nmax
         gamex(ie,ion,il)=0.
         exeff=0.
         feei=feeihi(id,il)
         frexc=frexc+feei
         if (feei.gt.0.) then
c  exeff in eV
            nil=nln(il)
            exeff=ehn(nil)*ab(ie)*x1/feei
         endif
         if (exeff.gt.0.) then
            gamex(ie,ion,il)=gahe/(exeff*elch)
         endif
      enddo
c      do ie=1,iemax
      do ie=2,12
         do ion=1,mion
c
            x1=0.
ch            if (ie.eq.ieh.and.ion.eq.1) x1=xhi(1)
            if (ie.eq.iehe.and.ion.eq.1) x1=xhei(1)
            if (ie.eq.ieo.and.ion.eq.1) x1=xoi(1)
            if (ie.eq.iena.and.ion.eq.1) x1=xa(iena,1)
c            if (ie.eq.iemg.and.ion.eq.1) x1=xa(iemg,1)
            if (ie.eq.iemg.and.ion.eq.1)then
               x1=xmg1(1)
            endif
            if (ie.eq.iemg.and.ion.eq.2) x1=xa(iemg,2)
c            if (ie.eq.iesi.and.ion.eq.1) x1=xa(iesi,1)
            if (ie.eq.iesi.and.ion.eq.1) x1=xsi1(1)+xsi1(2)+xsi1(3)
            if (ie.eq.ieca.and.ion.eq.2) x1=xcaii(1)
c
            do il=1,nl
               gamex(ie,ion,il)=0.
               exeff=0.
ch               if (ie.eq.ieh.and.ion.eq.1) then
ch                  feei=feeihi(id,il)
               if (ie.eq.iehe.and.ion.eq.1) then
                  feei=feeihei(id,il)
               elseif (ie.eq.ieo.and.ion.eq.1) then
                  feei=feeioi(id,il)
               elseif (ie.eq.iena.and.ion.eq.1) then
                  feei=feeinai(id,il)
               elseif (ie.eq.iemg.and.ion.eq.1) then
                  feei=feeimgi(id,il)
               elseif (ie.eq.iemg.and.ion.eq.2) then
                  feei=feeimgii(id,il)
               elseif (ie.eq.iesi.and.ion.eq.1) then
                  feei=feeisi1(id,il)
               elseif (ie.eq.ieca.and.ion.eq.2) then
                  feei=feeicaii(id,il)
               else
                  feei=0.
               endif
               frexc=frexc+feei
               if (feei.gt.0.) then
c  exeff in eV
                  exeff=e(ie,ion,il)*ab(ie)*x1/feei
               endif
               if (exeff.gt.0.) then
                  gamex(ie,ion,il)=gahe/(exeff*elch)
               endif
            enddo
         enddo
      enddo
c
c  non-thermal exc of fe i
      ie=iefe
      ion=1
      x1=0.
      gsum=0.
      do il=1,5
         x1=x1+xfei(il)
         gsum=gsum+gf1(il)
      enddo
      do il=1,nlf1
         gamex(ie,ion,il)=0.
         exeff=0.
         feei=feeifei(id,il)
         frexc=frexc+feei
         if (feei.gt.0.) then
c  exeff in eV
            efmean=0.
            do i=1,5
               ei=ef1(il)-ef1(i)
               efmean=efmean+ei*gf1(i)/gsum
            enddo
            exeff=efmean*ab(ie)*x1/feei
         endif
         if (exeff.gt.0.) then
            gamex(ie,ion,il)=gahe/(exeff*elch)
         endif
      enddo
c
c  non-thermal exc of fe ii
      ie=iefe
      ion=2
      x1=0.
      gsum=0.
      do il=1,5
         x1=x1+xfeii(il)
         gsum=gsum+gf2(il)
      enddo
      do il=1,nlf2
         gamex(ie,ion,il)=0.
         exeff=0.
         feei=feeifeii(id,il)
         frexc=frexc+feei
         if (feei.gt.0.) then
c  exeff in eV
            efmean=0.
            do i=1,5
               ei=ef2(il)-ef2(i)
               efmean=efmean+ei*gf2(i)/gsum
            enddo
            exeff=efmean*ab(ie)*x1/feei
         endif
         if (exeff.gt.0.) then
            gamex(ie,ion,il)=gahe/(exeff*elch)
         endif
      enddo
c  heating
c  gaelh = the energy s**-1 that goes into heating
c
      gaelh=frhe(id)*gahe*den
c
      return
      end


      subroutine abindex(ieion)
      implicit real*8 (a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      dimension ieion(mie,mion)
c
c   ieion(ie,ion)=ies
c   ie,ion = the element and ion numbering in ior (DATIE)
c   ies    = the element numbering in sf (/SFIOR/DATIE)
c
c   if ieion = 0 is that element not included in sf
c
c New numbering 990825 (including higher ionization stages, up
c                       to V for all elements)
c                       
      do ie=1,mie
         do ion=1,mion
            ieion(ie,ion)=0
         enddo
      enddo
c
c                                 H 
      ieion(ieh,1)=1
c                                 He
      ieion(iehe,1)=2
      ieion(iehe,2)=3
c                                 C 
      ieion(iec,1)=4
      ieion(iec,2)=5
c      ieion(iec,3)=6
c      ieion(iec,4)=7
c      ieion(iec,5)=8
c                                 N 
      ieion(ien,1)=9
c      ieion(ien,2)=10
c      ieion(ien,3)=11
c      ieion(ien,4)=12
c      ieion(ien,5)=13
c                                 O 
      ieion(ieo,1)=14
      ieion(ieo,2)=15
c      ieion(ieo,3)=16
c      ieion(ieo,4)=17
c      ieion(ieo,5)=18
c                                 Ne 
      ieion(iene,1)=19
      ieion(iene,2)=20
      ieion(iene,3)=21
      ieion(iene,4)=22
c      ieion(iene,5)=23
c                                 Na 
      ieion(iena,1)=24
c      ieion(iena,2)=25
c      ieion(iena,3)=26
c      ieion(iena,4)=27
c      ieion(iena,5)=28
c                                 Mg 
      ieion(iemg,1)=29
      ieion(iemg,2)=30
c      ieion(iemg,3)=31
c      ieion(iemg,4)=32
c      ieion(iemg,5)=33
c                                 Si
      ieion(iesi,1)=34
      ieion(iesi,2)=35
c      ieion(iesi,3)=36
c      ieion(iesi,4)=37
c      ieion(iesi,5)=38
c                                 S
      ieion(ies,1)=39
      ieion(ies,2)=40
      ieion(ies,3)=41
      ieion(ies,4)=42
c      ieion(ies,5)=43
c                                 Ar
      ieion(iear,1)=44
      ieion(iear,2)=45
      ieion(iear,3)=46
      ieion(iear,4)=47
c      ieion(iear,5)=48
c                                 Ca
      ieion(ieca,1)=49
      ieion(ieca,2)=50
c      ieion(ieca,3)=51
c      ieion(ieca,4)=52
c      ieion(ieca,5)=53
c                                 Fe
      ieion(iefe,1)=54
      ieion(iefe,2)=55
      ieion(iefe,3)=56
      ieion(iefe,4)=57
c      ieion(iefe,5)=58
c      ieion(iefe,6)=59
c                                 Co
c      ieion(ieco,1)=60
c      ieion(ieco,2)=61
c      ieion(ieco,3)=62
c      ieion(ieco,4)=63
c      ieion(ieco,5)=64
c                                 Ni
c      ieion(ieni,1)=65
c      ieion(ieni,2)=66
c      ieion(ieni,3)=67
c      ieion(ieni,4)=68
c      ieion(ieni,5)=69
c
      return
      end



      subroutine twolat(wl,om,g1,g2,a21,ge2,alf2,xel,z,xa0,xa1,te,
     & rl,cint,fp21,t00,s00,fjscat,phxl,j21)
c      subroutine twolat(wl,om,g1,g2,a21,ge2,xel,z,te,rl,cint,fp21,
c     & t00,s00,fjscat,j21)
      implicit real*8(a-h,o-z)
c     **************************************************************
c     *****
c     this routine calculates the heat loss and observed flux from
c     a line with the escape probability method.
c     wl = wavelegth (a)
c     om = collision strength
c     g1,g2 = stat.weights of the lower and upper levels
c     a21 = transition probability
c     ge2 = non-thermal excitation of level 2
c     xel = electron fraction
c     z = total abundance of the ion
c     te = temperature
c     rl = loss rate in erg cm**3 /s
c     cint = observed flux
c
c     950628
c
c     980323 : include recombination (effective or direct) to the 
c              upper of the two levels 
c     alf2= recombination coeff. (direct or effective) to upper level
c     *****
c     ****************************************************************
      include 'INCLUDE/PARAM.IOR'
      parameter(itmax=30)
      common/ctpar/tday,dts,iss,iss1,itd
      common/cfj/fj(ne),fjidm1(ne),fjc(0:ne),fjcidm1(ne)
      common/cden/den
      common/cphed/phed(0:ne),phextot,phextc,phextl,phexlm(mid,lmax),
     &     eq1(1200),klq1(1200),kl1m
      common/depthind/id
cqcq run 18
      common/ctwolat/x1,be21,pe12,pe21,c12,c21
cqcq  
c      
      data pi/3.1415926e0/,elch/1.60219e-12/,amu/1.660531e-24/
      data tol/0.00001/
c hc=h*c=6.626e-27 erg s *2.9979e10 cm/s = 1.9864e-16 erg cm
      data hc/1.9864e-16/
c
      if (om.le.0.and.ge2.le.0.) then
         cool=0.
         rl=cool
         cint=0.
         fp21=0.
         t00=0.
         s00=0.
      else
c
c  encgs = transition energy in erg

         encgs=1.989e-8/wl
c
         en=1.4413e8/wl
         ente=en/te
         if(en/te.gt.250.) ente=250.
c
         ge20=ge2
         dene=xel*den
         time=8.64e4*tday
         wl21u3=(wl*1e-8)**3
c
         c21=8.629e-6*om/(g2*dsqrt(te))
         c12=(g2/g1)*c21*fexp(-ente)
c
c
c  Photoexcitation/deexcitation (stimulated)
c
c  change the unit from ../eV to ../Hz
c  excitation/deexcitation due to continuumflux
         flc=fjc(j21)/2.41838e14
cnew9802 include line scattering
         flc=flc+fjscat
         pe21=0.5*(wl*wl)*1.e-16*a21*flc/encgs
         pe12=(g2/g1)*pe21
         if (pe12.gt.0.) then
            frl=fjscat/flc
            frc=1.-frl
            pe12c=pe12*frc
            pe21c=pe21*frc
            pe12l=pe12*frl
            pe21l=pe21*frl
         else
            pe12c=0.d0
            pe21c=0.d0
            pe12l=0.d0
            pe21l=0.d0
         endif
c
c  start values
c   
         x1=0.9
         x2=0.1
c
         do it=1,itmax
            ge2=ge20/x1
c
c  the escape probability be21
c
            t00=wl21u3*time*a21*g2*den*z*x1*(1.-g1*x2
     &           /(g2*x1))/(8.*pi*g1)
            if (t00.lt.0.01) then
               be21=1.-0.5*t00+(t00**2)/6.
            elseif(t00.ge.0.01) then
               be21=(1.-fexp(-t00))/t00
            endif
c
            clow=(dene*c12+ge2+pe12*be21)
            q=(dene*c21+be21*(a21+pe21))/clow
            if (alf2.gt.0.) then
               q2=xa1*dene*alf2/(clow*xa0)
            else
               q2=0.d0
            endif
            x2n=(1.+q2)/(1.+q)
c            x2n=1./(1.+q)
            x1n=1.-x2n
c
            dx=abs((x2-x2n)/x2n)
            x1=x1n
            x2=x2n
            if (dx.lt.tol) goto 9
c
         enddo
         if (it.ge.itmax) write(50,*) ' no conv in twolat'
 9       continue
c
         cool=encgs*z*(x1*c12-x2*c21)
         rl=cool
c
         const=z*be21*encgs/(den*xel)
         cint=const*x2*a21
         fp21=const*x2*pe21
         phextot=phextot+const*(x1*pe12-x2*pe21)
         phextc=phextc+const*(x1*pe12c-x2*pe21c)
         phxl=const*(x1*pe12l-x2*pe21l)
         phextl=phextl+phxl
c
         phed(j21)=phed(j21)+const*(x1*pe12-x2*pe21)
         if (x2.le.1.e-40) then
            s00=0.d0
         else
            s00=(2.*hc/wl21u3)/((x1*g2)/(x2*g1)-1)
         endif
      endif
c
      return
      end

c*********************************************************************   
      double precision function spline(p1,p2,p3,p4,p5,x)
C
C     Devised by Burgess & Tully (A & A 254, 436, '92).
C************************************************************           
      implicit real*8(a-h,o-z)
C
      s=1.d0/30.d0
      s2=32.d0*s*(19.d0*p1-43.d0*p2+30.d0*p3-7.d0*p4+p5)
      s3=160.d0*s*(-p1+7.d0*p2-12.d0*p3+7.d0*p4-p5)
      s4=32.d0*s*(p1-7.d0*p2+30.d0*p3-43.d0*p4+19.d0*p5)
      if(x.gt.0.25d0) goto 1
      x0=x-0.125d0
      t3=0.0d0
      t2=0.5d0*s2
      t1=4.d0*(p2-p1)
      t0=0.5d0*(p1+p2)-0.015625d0*t2
      goto 4
    1 if(x.gt.0.5d0) goto 2
      x0=x-0.375d0
      t3=20.d0*s*(s3-s2)
      t2=0.25d0*(s2+s3)
      t1=4.0d0*(p3-p2)-0.015625d0*t3
      t0=0.5d0*(p2+p3)-0.015625d0*t2
      goto 4
    2 if(x.gt.0.75d0) goto 3
      x0=x-0.625d0
      t3=20.d0*s*(s4-s3)
      t2=0.25d0*(s4+s3)
      t1=4.0d0*(p4-p3)-0.015625d0*t3
      t0=0.5d0*(p4+p3)-0.015625d0*t2
      goto 4
    3 x0=x-0.875d0
      t3=0.0d0
      t2=0.5d0*s4
      t1=4.d0*(p5-p4)
      t0=0.5d0*(p5+p4)-0.015625d0*t2
    4 spline=t0+x0*(t1+x0*(t2+x0*t3))
C  
      return
      end
c*********************************************************************   
c*********************************************************************   
      double precision function upsil(k,eij,c,p1,p2,p3,p4,p5,t)
C
C     Devised by Burgess & Tully (A & A 254, 436, '92).
C************************************************************           
      implicit real*8(a-h,o-z)
C
      teij=1.57888d5*eij
      e=dabs(t/teij)
      if(k.eq.1.or.k.eq.4) x=dlog((e+c)/c)/dlog(e+c)
      if(k.eq.2.or.k.eq.3) x=e/(e+c)
      y=spline(p1,p2,p3,p4,p5,x)
      if(k.eq.1) y=y*dlog(e+2.71828d0)
      if(k.eq.3) y=y/(e+1)
      if(k.eq.4) y=y*dlog(e+c)
      upsil=y
C
      return
      end
c*********************************************************************   


      subroutine readferec
      implicit real*8 (a-h,o-z)
      common/cferec_f1/tef1(81),rectf1(81)
      common/cferecl_f1/rcmpf1(13,81),rcsumf1(10,81)
      common/cferec_f2/tef2(81),rectf2(81),rcmpf2(42,81),rcsumf2(17,81)
      dimension tv(81)
c
c Read total Fe I recombination coefficients
      open(22,file='ATDAT/FEDAT/FEREC/rec.tot.fe1',status='old')
      do i=1,4
         read(22,*) 
      enddo
      do i=1,81
         read(22,*) tef1(i),rectf1(i)
      enddo
      close(22)
c Read total Fe II recombination coefficients
      open(22,file='ATDAT/FEDAT/FEREC/rec.tot.fe2',status='old')
      do i=1,4
         read(22,*) 
      enddo
      do i=1,81
         read(22,*) tef2(i),rectf2(i)
      enddo
      close(22)
c Read Fe I recombination coefficients to specific multiplets
      open(22,file='ATDAT/FEDAT/FEREC/rec.lev.fe1',status='old')
      read(22,*)
      read(22,701) (tv(i),i=1,81)
c
      do imp=1,13
         read(22,*)
         read(22,701) (rcmpf1(imp,it),it=1,81)
      enddo
c Read Fe I sum of recombination coefficients to higher multiplets
      read(22,*)
      do imps=1,10
         read(22,702) (rcsumf1(imps,it),it=1,81)
      enddo
 701  format(19x,85e10.2)
 702  format(11x,85e10.2)
      close(22)
c Read Fe II recombination coefficients to specific multiplets
      open(22,file='ATDAT/FEDAT/FEREC/rec.lev.fe2',status='old')
      read(22,*)
      read(22,701) (tv(i),i=1,81)
      do imp=1,42
         read(22,*)
         read(22,701) (rcmpf2(imp,it),it=1,81)
      enddo
c Read Fe II sum of recombination coefficients to higher multiplets
      read(22,*)
      do imps=1,17
         read(22,702) (rcsumf2(imps,it),it=1,81)
      enddo
      close(22)
c
      return
      end



      subroutine readferec_big
      implicit real*8 (a-h,o-z)
      common/cferec_f1/tef1(81),rectf1(81)
      common/cferecl_f1_big/rect5f1(81),rect7f1(81),rrcf1(81,538),
     &     isymf1(538)
      common/cferec_f2/tef2(81),rectf2(81),rcmpf2(42,81),rcsumf2(17,81)
      dimension tv(81)
c
c  Big Fe I atom
c
      do i=1,538
         isymf1(i)=0
      enddo
c Read Fe I recombination coefficients
      open(22,file='ATDAT/FEDAT/FEREC/rec.big.fe1',status='old')
      do i=1,15
         read(22,*) 
      enddo
      read(22,703) (tef1(it),it=1,81)
 703  format(1pe11.4,6e11.4)
      do it=1,81
         do il=1,538
            rrcf1(it,il)=0.d0
         enddo
      enddo
c
      read(22,*)
      read(22,*)
c
      do il=1,538
         read(22,704) ii,incl
         if (ii.ne.il) write(*,*) ii,il
         if (incl.eq.1) then
            backspace(22)
            read(22,704) ii,incl,isymf1(il)
            read(22,703) (rrcf1(it,il),it=1,81)
         endif
      enddo
 704  format(2i4,4x,i1)
c
      read(22,*)
      read(22,*)
c read total recomb. coeff. for quintets
      read(22,703) (rect5f1(it),it=1,81)
c
      read(22,*)
      read(22,*)
c read total recomb. coeff. for septets
      read(22,703) (rect7f1(it),it=1,81)
c
      read(22,*)
      read(22,*)
c read total recomb. coeff.
      read(22,703) (rectf1(it),it=1,81)
c
      close(22)
c Read total Fe II recombination coefficients
      open(22,file='ATDAT/FEDAT/FEREC/rec.tot.fe2',status='old')
      do i=1,4
         read(22,*) 
      enddo
      do i=1,81
         read(22,*) tef2(i),rectf2(i)
      enddo
      close(22)
c Read Fe II recombination coefficients to specific multiplets
      open(22,file='ATDAT/FEDAT/FEREC/rec.lev.fe2',status='old')
      read(22,*)
      read(22,701) (tv(i),i=1,81)
      do imp=1,42
         read(22,*)
         read(22,701) (rcmpf2(imp,it),it=1,81)
      enddo
c Read Fe II sum of recombination coefficients to higher multiplets
      read(22,*)
      do imps=1,17
         read(22,702) (rcsumf2(imps,it),it=1,81)
      enddo
      close(22)
 701  format(19x,85e10.2)
 702  format(11x,85e10.2)
c
      return
      end


      subroutine ferecomb(te,recof1,recof2,recof3)
      implicit real*8 (a-h,o-z)
cnew000225 New Nahar-recombination data for Fe III (from CF)
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/cferec_f1/tef1(81),rectf1(81)
      common/cferecl_f1/rcmpf1(13,81),rcsumf1(10,81)
      common/cferec_f2/tef2(81),rectf2(81),rcmpf2(42,81),rcsumf2(17,81)
      common/cfedat/cf1(nlf1,nlf1),cf2(nlf2,nlf2),cf3(nlf3,nlf3),
     &     cf4(nlf4,nlf4),cif1(nlf1),cif2(nlf2),cif3(nlf3),cif4(nlf4),
     &     gf1(nlf1),gf2(nlf2),gf3(nlf3),gf4(nlf4),ef1(nlf1),ef2(nlf2),
     &     ef3(nlf3),ef4(nlf4),af1(nlf1,nlf1),af2(nlf2,nlf2),
     &     af3(nlf3,nlf3),af4(nlf4,nlf4),cpf2(3,3)
      dimension recof1(nlf1),recof2(nlf2),recof3(nlf3)
c
c Fe I
      do il=1,nlf1
         recof1(il)=0.d0
      enddo
      if (te.lt.tef1(1)) then
         tconst=0.d0
         it1=1
         it2=1
      elseif (te.ge.tef1(81)) then
         tconst=0.d0
         it1=81
         it2=81
      else
         do i=1,80
            if (te.ge.tef1(i).and.te.lt.tef1(i+1)) then
               it1=i
               it2=i+1
               goto 9
            endif
         enddo
 9       continue
         tconst=(te-tef1(it1))/(tef1(it2)-tef1(it1))
      endif
c
      rctfe1=rectf1(it1)+tconst*(rectf1(it2)-rectf1(it1))
c
c a 5De ; imp=1, il=1(g=9),2(7),3(5),4(3),5(1); gsum=25
c Add 5De-sum; imps=2
      rcmp=rcmpf1(1,it1)+tconst*(rcmpf1(1,it2)-rcmpf1(1,it1))
      rcsum=rcsumf1(2,it1)+tconst*(rcsumf1(2,it2)-rcsumf1(2,it1))
      rcmp=rcmp+rcsum
      recof1(1)=rcmp*0.36
      recof1(2)=rcmp*0.28
      recof1(3)=rcmp*0.20
      recof1(4)=rcmp*0.12
      recof1(5)=rcmp*0.04
c a 5Fe ; imp=2, il=6(11),7(9),8(7),9(5),10(3); gsum=35
c Add 5Fe-sum; imps=3
      rcmp=rcmpf1(2,it1)+tconst*(rcmpf1(2,it2)-rcmpf1(2,it1))
      rcsum=rcsumf1(3,it1)+tconst*(rcsumf1(3,it2)-rcsumf1(3,it1))
      rcmp=rcmp+rcsum
      recof1(6)=rcmp*0.3143
      recof1(7)=rcmp*0.2571
      recof1(8)=rcmp*0.2
      recof1(9)=rcmp*0.1429
      recof1(10)=rcmp*0.0857
c a 5Pe ; imp=3, il=14(7),15(5),16(3); gsum=15
c Add 5Pe-sum; imps=1
      rcmp=rcmpf1(3,it1)+tconst*(rcmpf1(3,it2)-rcmpf1(3,it1))
      rcsum=rcsumf1(1,it1)+tconst*(rcsumf1(1,it2)-rcsumf1(1,it1))
      rcmp=rcmp+rcsum
      recof1(14)=rcmp*0.4667
      recof1(15)=rcmp*0.3333
      recof1(16)=rcmp*0.2
c z 7Do ; imp=4, il=18(11),21(9),23(7),25(5),26(3); gsum=35
c Add 7Do-sum; imps=9
      rcmp=rcmpf1(4,it1)+tconst*(rcmpf1(4,it2)-rcmpf1(4,it1))
      rcsum=rcsumf1(9,it1)+tconst*(rcsumf1(9,it2)-rcsumf1(9,it1))
      rcmp=rcmp+rcsum
      recof1(18)=rcmp*0.3143
      recof1(21)=rcmp*0.2571
      recof1(23)=rcmp*0.2
      recof1(25)=rcmp*0.1429
      recof1(26)=rcmp*0.0857
c z 7Fo ; imp=5, il=34(13),36(11),38(9),40(7),41(5),42(3),43(1); gsum=49
c Add 7Fo-sum; imps=10
      rcmp=rcmpf1(5,it1)+tconst*(rcmpf1(5,it2)-rcmpf1(5,it1))
      rcsum=rcsumf1(10,it1)+tconst*(rcsumf1(10,it2)-rcsumf1(10,it1))
      rcmp=rcmp+rcsum
      recof1(34)=rcmp*0.2653
      recof1(36)=rcmp*0.2245
      recof1(38)=rcmp*0.1837
      recof1(40)=rcmp*0.1429
      recof1(41)=rcmp*0.1020
      recof1(42)=rcmp*0.0612
      recof1(43)=rcmp*0.0204
c z 7Po ; imp=6, il=44(9),47(7),50(5); gsum=21
c Add 7Po-sum; imps=8
      rcmp=rcmpf1(6,it1)+tconst*(rcmpf1(6,it2)-rcmpf1(6,it1))
      rcsum=rcsumf1(8,it1)+tconst*(rcsumf1(8,it2)-rcsumf1(8,it1))
      rcmp=rcmp+rcsum
      recof1(44)=rcmp*0.4286
      recof1(47)=rcmp*0.3333
      recof1(50)=rcmp*0.2381
c z 5Do ; imp=7, il=54(9),56(7),58(5),61(3),62(1); gsum=25
      rcmp=rcmpf1(7,it1)+tconst*(rcmpf1(7,it2)-rcmpf1(7,it1))
      recof1(54)=rcmp*0.36
      recof1(56)=rcmp*0.28
      recof1(58)=rcmp*0.20
      recof1(61)=rcmp*0.12
      recof1(62)=rcmp*0.04
c z 5Fo ; imp=8, il=65(11),66(9),67(7),69(5),70(3); gsum=35
      rcmp=rcmpf1(8,it1)+tconst*(rcmpf1(8,it2)-rcmpf1(8,it1))
      recof1(65)=rcmp*0.3143
      recof1(66)=rcmp*0.2571
      recof1(67)=rcmp*0.2
      recof1(69)=rcmp*0.1429
      recof1(70)=rcmp*0.0857
c z 5Po ; imp=9, il=73(7),78(5),79(3); gsum=15
      rcmp=rcmpf1(9,it1)+tconst*(rcmpf1(9,it2)-rcmpf1(9,it1))
      recof1(73)=rcmp*0.4667
      recof1(78)=rcmp*0.3333
      recof1(79)=rcmp*0.2
c y 5Do ; imp=10, il=89(9),91(7),94(5),96(3),98(1); gsum=25
c Add 5Do-sum; imps=5
      rcmp=rcmpf1(10,it1)+tconst*(rcmpf1(10,it2)-rcmpf1(10,it1))
      rcsum=rcsumf1(5,it1)+tconst*(rcsumf1(5,it2)-rcsumf1(5,it1))
      rcmp=rcmp+rcsum
      recof1(89)=rcmp*0.36
      recof1(91)=rcmp*0.28
      recof1(94)=rcmp*0.20
      recof1(96)=rcmp*0.12
      recof1(98)=rcmp*0.04
c y 5Fo ; imp=11, il=92(11),97(9),99(7),101(5),104(3); gsum=35
c Add 5Fo-sum; imps=6
      rcmp=rcmpf1(11,it1)+tconst*(rcmpf1(11,it2)-rcmpf1(11,it1))
      rcsum=rcsumf1(6,it1)+tconst*(rcsumf1(6,it2)-rcsumf1(6,it1))
      rcmp=rcmp+rcsum
      recof1(92)=rcmp*0.3143
      recof1(97)=rcmp*0.2571
      recof1(99)=rcmp*0.2
      recof1(101)=rcmp*0.1429
      recof1(104)=rcmp*0.0857
c z 5Go ; imp=12, il=105(11),106(13),107(9),109(7),111(5); gsum=45
c Add 5Go-sum; imps=7
      rcmp=rcmpf1(12,it1)+tconst*(rcmpf1(12,it2)-rcmpf1(12,it1))
      rcsum=rcsumf1(7,it1)+tconst*(rcsumf1(7,it2)-rcsumf1(7,it1))
      rcmp=rcmp+rcsum
      recof1(105)=rcmp*0.2444
      recof1(106)=rcmp*0.2889
      recof1(107)=rcmp*0.2
      recof1(109)=rcmp*0.1556
      recof1(111)=rcmp*0.1111
c y 5Po ; imp=13, il=114(7),115(5),117(3); gsum=15
c Add 5Po-sum; imps=4
      rcmp=rcmpf1(13,it1)+tconst*(rcmpf1(13,it2)-rcmpf1(13,it1))
      rcsum=rcsumf1(4,it1)+tconst*(rcsumf1(4,it2)-rcsumf1(4,it1))
      rcmp=rcmp+rcsum
      recof1(114)=rcmp*0.4667
      recof1(115)=rcmp*0.3333
      recof1(117)=rcmp*0.2
c
c Add the remaining recombinations to the 8 highest levels 
c (114 - 121; y5P, y3F, y3D) weighted
c with their statistical weights so that the total recombination 
c coefficient is correct ; gsum=42 (7,5,7,3,5,7,5,3)
c
      recsum=0.d0
cqcq run 70.7; run 80.1; run 80.2; runs >= 70.7
c      do il=1,nlf1
      do il=1,nlev(iefe,1)
         recsum=recsum+recof1(il)
      enddo
      recadd=rctfe1-recsum
      if (recadd.lt.0.) then
         write(50,*) 'Error in calc. rec. coeff. for Fe I !!'
      endif
cqcq run 70.4 ; run 70.5; run 70.7; run 80.1; run 80.2; runs >=70.4
c      recof1(114)=recof1(114)+recadd*0.1667
c      recof1(115)=recof1(115)+recadd*0.1190
c      recof1(116)=recof1(116)+recadd*0.1667
c      recof1(117)=recof1(117)+recadd*0.0714
c      recof1(118)=recof1(118)+recadd*0.1190
c      recof1(119)=recof1(119)+recadd*0.1667
c      recof1(120)=recof1(120)+recadd*0.1190
c      recof1(121)=recof1(121)+recadd*0.0715
cqcq run 70.4; run 70.5; run 70.7; run 80.1; run 80.2; runs >=70.4
c add the remaining recomb. contribution to
c all the levels, not only the highest
      gsum=0.d0
cqcq run 70.7; run 80.1; run 80.2; runs >=70.7
c      do il=1,121
      do il=1,nlev(iefe,1)
         gsum=gsum+gf1(il)
      enddo
cqcq run 70.7; run 80.1; run 80.2; runs >=70.7
c      do il=1,121
      do il=1,nlev(iefe,1)
         recof1(il)=recof1(il)+recadd*gf1(il)/gsum
      enddo
cqcq   
c
c Fe II
c
      do il=1,nlf2
         recof2(il)=0.d0
      enddo
      if (te.lt.tef2(1)) then
         tconst=0.d0
         it1=1
         it2=1
      elseif (te.ge.tef2(81)) then
         tconst=0.d0
         it1=81
         it2=81
      else
         do i=1,80
            if (te.ge.tef2(i).and.te.lt.tef2(i+1)) then
               it1=i
               it2=i+1
               goto 10
            endif
         enddo
 10      continue
         tconst=(te-tef2(it1))/(tef2(it2)-tef2(it1))
      endif
c
      rctfe2=rectf2(it1)+tconst*(rectf2(it2)-rectf2(it1))
c
c a 6De ; imp=1, il=1(10),2(8),3(6),4(4),5(2); gsum=30
c Add 6De-sum; imps=3
      rcmp=rcmpf2(1,it1)+tconst*(rcmpf2(1,it2)-rcmpf2(1,it1))
      rcsum=rcsumf2(3,it1)+tconst*(rcsumf2(3,it2)-rcsumf2(3,it1))
      rcmp=rcmp+rcsum
      recof2(1)=rcmp*0.3333
      recof2(2)=rcmp*0.2667
      recof2(3)=rcmp*0.2
      recof2(4)=rcmp*0.1333
      recof2(5)=rcmp*0.0667
c a 4Fe ; imp=2, il=6(10),7(8),8(6),9(4); gsum=28
      rcmp=rcmpf2(2,it1)+tconst*(rcmpf2(2,it2)-rcmpf2(2,it1))
      recof2(6)=rcmp*0.3571
      recof2(7)=rcmp*0.2857
      recof2(8)=rcmp*0.2143
      recof2(9)=rcmp*0.1429
c a 4De ; imp=3, il=10(8),11(6),12(4),13(2); gsum=20
      rcmp=rcmpf2(3,it1)+tconst*(rcmpf2(3,it2)-rcmpf2(3,it1))
      recof2(10)=rcmp*0.4
      recof2(11)=rcmp*0.3
      recof2(12)=rcmp*0.2
      recof2(13)=rcmp*0.1
c a 4Pe ; imp=4, il=14(6),15(4),16(2); gsum=12
      rcmp=rcmpf2(4,it1)+tconst*(rcmpf2(4,it2)-rcmpf2(4,it1))
      recof2(14)=rcmp*0.5
      recof2(15)=rcmp*0.3333
      recof2(16)=rcmp*0.1667
c b 4Pe ; imp=5, il=24(6),30(4),31(2); gsum=12
      rcmp=rcmpf2(5,it1)+tconst*(rcmpf2(5,it2)-rcmpf2(5,it1))
      recof2(24)=rcmp*0.5
      recof2(30)=rcmp*0.3333
      recof2(31)=rcmp*0.1667
c a 4He ; imp=6, il=25(14),27(12),28(10),29(8); gsum=44
c Add 4He-sum; imps=15
      rcmp=rcmpf2(6,it1)+tconst*(rcmpf2(6,it2)-rcmpf2(6,it1))
      rcsum=rcsumf2(15,it1)+tconst*(rcsumf2(15,it2)-rcsumf2(15,it1))
      rcmp=rcmp+rcsum
      recof2(25)=rcmp*0.3182
      recof2(27)=rcmp*0.2727
      recof2(28)=rcmp*0.2273
      recof2(29)=rcmp*0.1818
c b 4Fe ; imp=7, il=32(10),33(8),34(6),35(4); gsum=28
      rcmp=rcmpf2(7,it1)+tconst*(rcmpf2(7,it2)-rcmpf2(7,it1))
      recof2(32)=rcmp*0.3571
      recof2(33)=rcmp*0.2857
      recof2(34)=rcmp*0.2143
      recof2(35)=rcmp*0.1429
c a 6Se ; imp=8, il=36(6) 
c Add 6Se-sum; imps=1
      rcmp=rcmpf2(8,it1)+tconst*(rcmpf2(8,it2)-rcmpf2(8,it1))
      rcsum=rcsumf2(1,it1)+tconst*(rcsumf2(1,it2)-rcsumf2(1,it1))
      rcmp=rcmp+rcsum
      recof2(36)=rcmp
c a 4Ge ; imp=9, il=37(12),39(10),40(8),41(6); gsum=36
      rcmp=rcmpf2(9,it1)+tconst*(rcmpf2(9,it2)-rcmpf2(9,it1))
      recof2(37)=rcmp*0.3333
      recof2(39)=rcmp*0.2778
      recof2(40)=rcmp*0.2222
      recof2(41)=rcmp*0.1667
c b 4De ; imp=10, il=49(4),50(2),51(6),52(8); gsum=20
      rcmp=rcmpf2(10,it1)+tconst*(rcmpf2(10,it2)-rcmpf2(10,it1))
      recof2(49)=rcmp*0.2
      recof2(50)=rcmp*0.1
      recof2(51)=rcmp*0.3
      recof2(52)=rcmp*0.4
c z 6Do ; imp=11, il=64(10),65(8),66(6),67(4),68(2); gsum=30
c Add 6Do-sum; imps=4
      rcmp=rcmpf2(11,it1)+tconst*(rcmpf2(11,it2)-rcmpf2(11,it1))
      rcsum=rcsumf2(4,it1)+tconst*(rcsumf2(4,it2)-rcsumf2(4,it1))
      rcmp=rcmp+rcsum
      recof2(64)=rcmp*0.3333
      recof2(65)=rcmp*0.2667
      recof2(66)=rcmp*0.2
      recof2(67)=rcmp*0.1333
      recof2(68)=rcmp*0.0667
c z 6Fo ; imp=12, il=69(12),70(10),71(8),72(6),73(4),74(2); gsum=42
c Add 6Fo-sum; imps=5
      rcmp=rcmpf2(12,it1)+tconst*(rcmpf2(12,it2)-rcmpf2(12,it1))
      rcsum=rcsumf2(5,it1)+tconst*(rcsumf2(5,it2)-rcsumf2(5,it1))
      rcmp=rcmp+rcsum
      recof2(69)=rcmp*0.2857
      recof2(70)=rcmp*0.2381
      recof2(71)=rcmp*0.1905
      recof2(72)=rcmp*0.1429
      recof2(73)=rcmp*0.0952
      recof2(74)=rcmp*0.0476
c z 6Po ; imp=13, il=75(8),76(6),77(4); gsum=18
c Add 6Po-sum; imps=2
      rcmp=rcmpf2(13,it1)+tconst*(rcmpf2(13,it2)-rcmpf2(13,it1))
      rcsum=rcsumf2(2,it1)+tconst*(rcsumf2(2,it2)-rcsumf2(2,it1))
      rcmp=rcmp+rcsum
      recof2(75)=rcmp*0.4445
      recof2(76)=rcmp*0.3333
      recof2(77)=rcmp*0.2222
c z 4Fo ; imp=14, il=78(10),80(8),85(6),87(4); gsum=28
      rcmp=rcmpf2(14,it1)+tconst*(rcmpf2(14,it2)-rcmpf2(14,it1))
      recof2(78)=rcmp*0.3571
      recof2(80)=rcmp*0.2857
      recof2(85)=rcmp*0.2143
      recof2(87)=rcmp*0.1429
c z 4Do ; imp=15, il=79(8),81(6),84(4),86(2); gsum=20
      rcmp=rcmpf2(15,it1)+tconst*(rcmpf2(15,it2)-rcmpf2(15,it1))
      recof2(79)=rcmp*0.4
      recof2(81)=rcmp*0.3
      recof2(84)=rcmp*0.2
      recof2(86)=rcmp*0.1
c z 4Po ; imp=16, il=88(6),89(4),90(2); gsum=12
      rcmp=rcmpf2(16,it1)+tconst*(rcmpf2(16,it2)-rcmpf2(16,it1))
      recof2(88)=rcmp*0.5
      recof2(89)=rcmp*0.3333
      recof2(90)=rcmp*0.1667
c c 4Pe ; imp=17, il=93(2),94(4),99(6); gsum=12
      rcmp=rcmpf2(17,it1)+tconst*(rcmpf2(17,it2)-rcmpf2(17,it1))
      recof2(93)=rcmp*0.1667
      recof2(94)=rcmp*0.3333
      recof2(99)=rcmp*0.5
c c 4Fe ; imp=18, il=95(4),96(6),97(10),98(8); gsum=28
c Add 4Fe-sum; imps=11
      rcmp=rcmpf2(18,it1)+tconst*(rcmpf2(18,it2)-rcmpf2(18,it1))
      rcsum=rcsumf2(11,it1)+tconst*(rcsumf2(11,it2)-rcsumf2(11,it1))
      rcmp=rcmp+rcsum
      recof2(95)=rcmp*0.1429
      recof2(96)=rcmp*0.2143
      recof2(97)=rcmp*0.3571
      recof2(98)=rcmp*0.2857
c b 4Ge ; imp=19, il=101(12),102(10),103(6),104(8); gsum=36
c Add 4Ge-sum; imps=13
      rcmp=rcmpf2(19,it1)+tconst*(rcmpf2(19,it2)-rcmpf2(19,it1))
      rcsum=rcsumf2(13,it1)+tconst*(rcsumf2(13,it2)-rcsumf2(13,it1))
      rcmp=rcmp+rcsum
      recof2(101)=rcmp*0.3333
      recof2(102)=rcmp*0.2778
      recof2(103)=rcmp*0.1667
      recof2(104)=rcmp*0.2222
c d 4Pe ; imp=20, il=108(6),109(4),110(2); gsum=12
c Add 4Pe-sum; imps=7
      rcmp=rcmpf2(20,it1)+tconst*(rcmpf2(20,it2)-rcmpf2(20,it1))
      rcsum=rcsumf2(7,it1)+tconst*(rcsumf2(7,it2)-rcsumf2(7,it1))
      rcmp=rcmp+rcsum
      recof2(108)=rcmp*0.5
      recof2(109)=rcmp*0.3333
      recof2(110)=rcmp*0.1667
c z 4So ; imp=21, il=113(4) 
      rcmp=rcmpf2(21,it1)+tconst*(rcmpf2(21,it2)-rcmpf2(21,it1))
      recof2(113)=rcmp
c c 4De ; imp=22, il=114(8),115(2),117(4),118(6); gsum=20
c Add 4De-sum; imps=9
      rcmp=rcmpf2(22,it1)+tconst*(rcmpf2(22,it2)-rcmpf2(22,it1))
      rcsum=rcsumf2(9,it1)+tconst*(rcsumf2(9,it2)-rcsumf2(9,it1))
      rcmp=rcmp+rcsum
      recof2(114)=rcmp*0.4
      recof2(115)=rcmp*0.1
      recof2(117)=rcmp*0.2
      recof2(118)=rcmp*0.3
c y 4Po ; imp=23, il=116(6),125(2),128(4); gsum=12
      rcmp=rcmpf2(23,it1)+tconst*(rcmpf2(23,it2)-rcmpf2(23,it1))
      recof2(116)=rcmp*0.5
      recof2(125)=rcmp*0.1667
      recof2(128)=rcmp*0.3333
c z 4Go ; imp=24, il=119(12),120(10),123(8),126(6); gsum=36
      rcmp=rcmpf2(24,it1)+tconst*(rcmpf2(24,it2)-rcmpf2(24,it1))
      recof2(119)=rcmp*0.3333
      recof2(120)=rcmp*0.2778
      recof2(123)=rcmp*0.2222
      recof2(126)=rcmp*0.1667
c z 4Ho ; imp=25, il=121(14),122(12),124(10),127(8); gsum=44
      rcmp=rcmpf2(25,it1)+tconst*(rcmpf2(25,it2)-rcmpf2(25,it1))
      recof2(121)=rcmp*0.3182
      recof2(122)=rcmp*0.2727
      recof2(124)=rcmp*0.2273
      recof2(127)=rcmp*0.1818
c z 4Io ; imp=26, il=129(16),130(10),131(14),132(12); gsum=52
c Add 4Io-sum; imps=17
      rcmp=rcmpf2(26,it1)+tconst*(rcmpf2(26,it2)-rcmpf2(26,it1))
      rcsum=rcsumf2(17,it1)+tconst*(rcsumf2(17,it2)-rcsumf2(17,it1))
      rcmp=rcmp+rcsum
      recof2(129)=rcmp*0.3077
      recof2(130)=rcmp*0.1923
      recof2(131)=rcmp*0.2692
      recof2(132)=rcmp*0.2308
c y 4Do ; imp=27, il=133(8),138(6),139(2),140(4); gsum=20
      rcmp=rcmpf2(27,it1)+tconst*(rcmpf2(27,it2)-rcmpf2(27,it1))
      recof2(133)=rcmp*0.4
      recof2(138)=rcmp*0.3
      recof2(139)=rcmp*0.1
      recof2(140)=rcmp*0.2
c y 4Fo ; imp=28, il=134(8),135(6),136(10),137(4); gsum=28
      rcmp=rcmpf2(28,it1)+tconst*(rcmpf2(28,it2)-rcmpf2(28,it1))
      recof2(134)=rcmp*0.2857
      recof2(135)=rcmp*0.2143
      recof2(136)=rcmp*0.3571
      recof2(137)=rcmp*0.1429
c x 4Do ; imp=29, il=141(8),142(6),143(4),144(2); gsum=20
      rcmp=rcmpf2(29,it1)+tconst*(rcmpf2(29,it2)-rcmpf2(29,it1))
      recof2(141)=rcmp*0.4
      recof2(142)=rcmp*0.3
      recof2(143)=rcmp*0.2
      recof2(144)=rcmp*0.1
c y 4Go ; imp=30, il=145(12),146(10),147(8),148(6); gsum=36
      rcmp=rcmpf2(30,it1)+tconst*(rcmpf2(30,it2)-rcmpf2(30,it1))
      recof2(145)=rcmp*0.3333
      recof2(146)=rcmp*0.2778
      recof2(147)=rcmp*0.2222
      recof2(148)=rcmp*0.1667
c x 4Go ; imp=31, il=149(12),150(10),151(8),153(6); gsum=36
      rcmp=rcmpf2(31,it1)+tconst*(rcmpf2(31,it2)-rcmpf2(31,it1))
      recof2(149)=rcmp*0.3333
      recof2(150)=rcmp*0.2778
      recof2(151)=rcmp*0.2222
      recof2(153)=rcmp*0.1667
c x 4Fo ; imp=32, il=152(10),154(8),157(6),159(4); gsum=28
      rcmp=rcmpf2(32,it1)+tconst*(rcmpf2(32,it2)-rcmpf2(32,it1))
      recof2(152)=rcmp*0.3571
      recof2(154)=rcmp*0.2857
      recof2(157)=rcmp*0.2143
      recof2(159)=rcmp*0.1429
c y 4Ho ; imp=33, il=155(14),156(12),158(10),160(8); gsum=44
c Add 4Ho-sum; imps=16
      rcmp=rcmpf2(33,it1)+tconst*(rcmpf2(33,it2)-rcmpf2(33,it1))
      rcsum=rcsumf2(16,it1)+tconst*(rcsumf2(16,it2)-rcsumf2(16,it1))
      rcmp=rcmp+rcsum
      recof2(155)=rcmp*0.3182
      recof2(156)=rcmp*0.2727
      recof2(158)=rcmp*0.2273
      recof2(160)=rcmp*0.1818
c w 4Po ; imp=34, il=161(6),162(4),164(2); gsum=12
      rcmp=rcmpf2(34,it1)+tconst*(rcmpf2(34,it2)-rcmpf2(34,it1))
      recof2(161)=rcmp*0.5
      recof2(162)=rcmp*0.3333
      recof2(164)=rcmp*0.1667
c w 4Fo ; imp=35, il=163(4),165(6),166(8),169(10); gsum=28
      rcmp=rcmpf2(35,it1)+tconst*(rcmpf2(35,it2)-rcmpf2(35,it1))
      recof2(163)=rcmp*0.1429
      recof2(165)=rcmp*0.2143
      recof2(166)=rcmp*0.2857
      recof2(169)=rcmp*0.3571
c w 4Do ; imp=36, il=167(2),168(6),170(8),171(4); gsum=20
      rcmp=rcmpf2(36,it1)+tconst*(rcmpf2(36,it2)-rcmpf2(36,it1))
      recof2(167)=rcmp*0.1
      recof2(168)=rcmp*0.3
      recof2(170)=rcmp*0.4
      recof2(171)=rcmp*0.2
c v 4Do ; imp=37, il=172(2),173(4),174(6),175(8); gsum=20
      rcmp=rcmpf2(37,it1)+tconst*(rcmpf2(37,it2)-rcmpf2(37,it1))
      recof2(172)=rcmp*0.1
      recof2(173)=rcmp*0.2
      recof2(174)=rcmp*0.3
      recof2(175)=rcmp*0.4
c w 4Go ; imp=38, il=176(6),177(8),178(10),179(12); gsum=36
c Add 4Go-sum; imps=14
      rcmp=rcmpf2(38,it1)+tconst*(rcmpf2(38,it2)-rcmpf2(38,it1))
      rcsum=rcsumf2(14,it1)+tconst*(rcsumf2(14,it2)-rcsumf2(14,it1))
      rcmp=rcmp+rcsum
      recof2(176)=rcmp*0.1667
      recof2(177)=rcmp*0.2222
      recof2(178)=rcmp*0.2778
      recof2(179)=rcmp*0.3333
c y 4So ; imp=39, il=180(4); gsum=4
c Add 4So-sum; imps=6
      rcmp=rcmpf2(39,it1)+tconst*(rcmpf2(39,it2)-rcmpf2(39,it1))
      rcsum=rcsumf2(6,it1)+tconst*(rcsumf2(6,it2)-rcsumf2(6,it1))
      rcmp=rcmp+rcsum
      recof2(180)=rcmp
c u 4Po ; imp=40, il=181(2),182(4),183(6); gsum=12
c Add 4Po-sum; imps=8
      rcmp=rcmpf2(40,it1)+tconst*(rcmpf2(40,it2)-rcmpf2(40,it1))
      rcsum=rcsumf2(8,it1)+tconst*(rcsumf2(8,it2)-rcsumf2(8,it1))
      rcmp=rcmp+rcsum
      recof2(181)=rcmp*0.1667
      recof2(182)=rcmp*0.3333
      recof2(183)=rcmp*0.5
c t 4Do ; imp=41, il=184(2),185(4),186(6),187(8); gsum=20
c Add 4Do-sum; imps=10
      rcmp=rcmpf2(41,it1)+tconst*(rcmpf2(41,it2)-rcmpf2(41,it1))
      rcsum=rcsumf2(10,it1)+tconst*(rcsumf2(10,it2)-rcsumf2(10,it1))
      rcmp=rcmp+rcsum
      recof2(184)=rcmp*0.1
      recof2(185)=rcmp*0.2
      recof2(186)=rcmp*0.3
      recof2(187)=rcmp*0.4
c t 4Fo ; imp=42, il=188(4),189(6),190(10),191(8); gsum=28
c Add 4Fo-sum; imps=12
      rcmp=rcmpf2(42,it1)+tconst*(rcmpf2(42,it2)-rcmpf2(42,it1))
      rcsum=rcsumf2(12,it1)+tconst*(rcsumf2(12,it2)-rcsumf2(12,it1))
      rcmp=rcmp+rcsum
      recof2(188)=rcmp*0.1429
      recof2(189)=rcmp*0.2143
      recof2(190)=rcmp*0.3571
      recof2(191)=rcmp*0.2857
c
c Add the remaining recombinations to the 4 highest levels 
c (188 - 191; t4F) weighted
c with their statistical weights so that the total recombination 
c coefficient is correct ; gsum=28 (4,6,10,8)
c
      recsum=0.d0
cqcq run 70.7; run 80.1; run 80.2; runs >= 70.7
c      do il=1,nlf2
      do il=1,nlev(iefe,2)
         recsum=recsum+recof2(il)
      enddo
      recadd=rctfe2-recsum
      if (recadd.lt.0.) then
         write(50,*) 'Error in calc. rec. coeff. for Fe II !!'
      endif
cqcq run 70.7; run 80.1; run 80.2; runs >= 70.7
c      recof2(188)=recof2(188)+recadd*0.1429
c      recof2(189)=recof2(189)+recadd*0.2143
c      recof2(190)=recof2(190)+recadd*0.3571
c      recof2(191)=recof2(191)+recadd*0.2857
c add the remaining recomb. contribution to
c all the levels, not only the highest
      gsum=0.d0
cqcq run 70.7; run 80.1; run 80.2; runs >= 70.7
c      do il=1,nlf2
      do il=1,nlev(iefe,2)
         gsum=gsum+gf2(il)
      enddo
cqcq run 70.7; run 80.1; run 80.2; runs >= 70.7
c      do il=1,nlf2
      do il=1,nlev(iefe,2)
         recof2(il)=recof2(il)+recadd*gf2(il)/gsum
      enddo
cqcq   
c
c Fe III
c
cnew000225
c New recombination data (from CF) from Nahar
c for Fe IV - Fe III
c
      do il=1,nlf3
         recof3(il)=0.d0
      enddo
c
c add 4.7e-12 to obtain the total recombination coeff
      recod65D=1.06e-12+4.7e-12
      recof3(1)=recod65D*9./25.
      recof3(2)=recod65D*7./25.
      recof3(3)=recod65D*5./25.
      recof3(4)=recod65D*3./25.
      recof3(5)=recod65D*1./25.
      reco4p5P=1.11e-13
      recof3(20)=reco4p5P*7./15.            
      recof3(21)=reco4p5P*5./15.            
      recof3(22)=reco4p5P*3./15.
      reco4p5F=6.62e-14
      recof3(23)=reco4p5F*11./35. 
      recof3(27)=reco4p5F*9./35. 
      recof3(28)=reco4p5F*7./35. 
      recof3(29)=reco4p5F*3./35. 
      recof3(30)=reco4p5F*5./35. 
      reco4p5P=8.72e-14
      recof3(36)=reco4p5P*7./15. 
      recof3(37)=reco4p5P*5./15. 
      recof3(38)=reco4p5P*3./15. 
      reco4p5F=9.23e-14
      recof3(42)=reco4p5F*3./35. 
      recof3(43)=reco4p5F*5./35. 
      recof3(44)=reco4p5F*7./35. 
      recof3(45)=reco4p5F*9./35. 
      recof3(46)=reco4p5F*11./35. 
      reco4D4p5D=1.14e-13
      recof3(50)=reco4D4p5D*9./25. 
      recof3(51)=reco4D4p5D*1./25. 
      reco4D4p5P=6.89e-14
      recof3(52)=reco4D4p5P*3./15. 
      recof3(53)=reco4D4p5P*5./15. 
      recof3(54)=reco4D4p5P*7./15. 
      reco4F4p5F=4.90e-14
      recof3(59)=reco4F4p5F*9./35. 
      recof3(60)=reco4F4p5F*7./35. 
      recof3(61)=reco4F4p5F*5./35. 
      recof3(62)=reco4F4p5F*11./35. 
      recof3(63)=reco4F4p5F*3./35. 
      reco4F4p5D=1.10e-13
      recof3(66)=reco4F4p5D*9./25.                        
      recof3(67)=reco4F4p5D*7./25.
      recof3(68)=reco4F4p5D*5./25.
      recof3(69)=reco4F4p5D*3./25.
      recof3(70)=reco4F4p5D*1./25.
      reco6S5p5P=4.34e-14
      recof3(71)=reco6S5p5P*7./15.
      recof3(72)=reco6S5p5P*5./15.
      recof3(73)=reco6S5p5P*3./15.
      reco6S4f5F=7.01e-14
      recof3(76)=reco6S4f5F*3./35.
      recof3(77)=reco6S4f5F*5./35.
      recof3(78)=reco6S4f5F*7./35.
      recof3(79)=reco6S4f5F*9./35.
      recof3(80)=reco6S4f5F*11./35.
      reco4P5p5P=4.98e-14
      recof3(94)=reco4P5p5P*7./15.
      recof3(95)=reco4P5p5P*5./15.
      recof3(96)=reco4P5p5P*3./15.
      reco4D5p5D=6.76e-14
      recof3(97)=reco4D5p5D*9./25.
      recof3(98)=reco4D5p5D*3./25.
      recof3(99)=reco4D5p5D*7./25.
      recof3(100)=reco4D5p5D*5./25.
      reco6S5f5F=6.18e-14
      recof3(101)=reco6S5f5F*3./25.
      recof3(102)=reco6S5f5F*5./25.
      recof3(103)=reco6S5f5F*7./25.
      recof3(104)=reco6S5f5F*9./25.
      recof3(105)=reco6S5f5F*11./25.
      reco4F5p5F=4.17e-14
      recof3(106)=reco4F5p5F*11./35.
      recof3(107)=reco4F5p5F*9./35.
      recof3(108)=reco4F5p5F*7./35.
      recof3(109)=reco4F5p5F*5./35.
      recof3(110)=reco4F5p5F*3./35.
      do j=1,112
         recof3(j)=recof3(j)*(5000./te)**0.43
      enddo
c
      return
      end



      subroutine ferecomb_big(te,recof1,recof2)
      implicit real*8 (a-h,o-z)
      include 'INCLUDE/PARAM.IOR'
      include 'INCLUDE/DATIE'
c
c run 70.6 Next time a big Fe I atom is needed fix 
c the recombination to the highest levels !!!!
c
      common/cnlev/nlev(mie,mion),nlevm(mie,mion),nlevfe(mion,mid),
     &     nlevfe1m,nlevfe2m,nlevfe3m,nlevfe4m
      common/cferec_f1/tef1(81),rectf1(81)
      common/cferecl_f1_big/rect5f1(81),rect7f1(81),rrcf1(81,538),
     &     isymf1(538)
      common/cferec_f2/tef2(81),rectf2(81),rcmpf2(42,81),rcsumf2(17,81)
      common/cfedat/cf1(nlf1,nlf1),cf2(nlf2,nlf2),cf3(nlf3,nlf3),
     &     cf4(nlf4,nlf4),cif1(nlf1),cif2(nlf2),cif3(nlf3),cif4(nlf4),
     &     gf1(nlf1),gf2(nlf2),gf3(nlf3),gf4(nlf4),ef1(nlf1),ef2(nlf2),
     &     ef3(nlf3),ef4(nlf4),af1(nlf1,nlf1),af2(nlf2,nlf2),
     &     af3(nlf3,nlf3),af4(nlf4,nlf4),cpf2(3,3)
c
      dimension recof1(nlf1),recof2(nlf2)
c
c Fe I
      do il=1,nlf1
         recof1(il)=0.d0
      enddo
      if (te.lt.tef1(1)) then
         tconst=0.d0
         it1=1
         it2=1
      elseif (te.ge.tef1(81)) then
         tconst=0.d0
         it1=81
         it2=81
      else
         do i=1,80
            if (te.ge.tef1(i).and.te.lt.tef1(i+1)) then
               it1=i
               it2=i+1
               goto 9
            endif
         enddo
 9       continue
         tconst=(te-tef1(it1))/(tef1(it2)-tef1(it1))
      endif
c
      rctfe1=rectf1(it1)+tconst*(rectf1(it2)-rectf1(it1))
      rct5fe1=rect5f1(it1)+tconst*(rect5f1(it2)-rect5f1(it1))
      rct7fe1=rect7f1(it1)+tconst*(rect7f1(it2)-rect7f1(it1))
c
      do il=1,538
         recof1(il)=rrcf1(it1,il)+
     &        tconst*(rrcf1(it2,il)-rrcf1(it1,il))
      enddo
c
c Fe II
c
      do il=1,nlf2
         recof2(il)=0.d0
      enddo
      if (te.lt.tef2(1)) then
         tconst=0.d0
         it1=1
         it2=1
      elseif (te.ge.tef2(81)) then
         tconst=0.d0
         it1=81
         it2=81
      else
         do i=1,80
            if (te.ge.tef2(i).and.te.lt.tef2(i+1)) then
               it1=i
               it2=i+1
               goto 10
            endif
         enddo
 10      continue
         tconst=(te-tef2(it1))/(tef2(it2)-tef2(it1))
      endif
c
      rctfe2=rectf2(it1)+tconst*(rectf2(it2)-rectf2(it1))
c
c a 6De ; imp=1, il=1(10),2(8),3(6),4(4),5(2); gsum=30
c Add 6De-sum; imps=3
      rcmp=rcmpf2(1,it1)+tconst*(rcmpf2(1,it2)-rcmpf2(1,it1))
      rcsum=rcsumf2(3,it1)+tconst*(rcsumf2(3,it2)-rcsumf2(3,it1))
      rcmp=rcmp+rcsum
      recof2(1)=rcmp*0.3333
      recof2(2)=rcmp*0.2667
      recof2(3)=rcmp*0.2
      recof2(4)=rcmp*0.1333
      recof2(5)=rcmp*0.0667
c a 4Fe ; imp=2, il=6(10),7(8),8(6),9(4); gsum=28
      rcmp=rcmpf2(2,it1)+tconst*(rcmpf2(2,it2)-rcmpf2(2,it1))
      recof2(6)=rcmp*0.3571
      recof2(7)=rcmp*0.2857
      recof2(8)=rcmp*0.2143
      recof2(9)=rcmp*0.1429
c a 4De ; imp=3, il=10(8),11(6),12(4),13(2); gsum=20
      rcmp=rcmpf2(3,it1)+tconst*(rcmpf2(3,it2)-rcmpf2(3,it1))
      recof2(10)=rcmp*0.4
      recof2(11)=rcmp*0.3
      recof2(12)=rcmp*0.2
      recof2(13)=rcmp*0.1
c a 4Pe ; imp=4, il=14(6),15(4),16(2); gsum=12
      rcmp=rcmpf2(4,it1)+tconst*(rcmpf2(4,it2)-rcmpf2(4,it1))
      recof2(14)=rcmp*0.5
      recof2(15)=rcmp*0.3333
      recof2(16)=rcmp*0.1667
c b 4Pe ; imp=5, il=24(6),30(4),31(2); gsum=12
      rcmp=rcmpf2(5,it1)+tconst*(rcmpf2(5,it2)-rcmpf2(5,it1))
      recof2(24)=rcmp*0.5
      recof2(30)=rcmp*0.3333
      recof2(31)=rcmp*0.1667
c a 4He ; imp=6, il=25(14),27(12),28(10),29(8); gsum=44
c Add 4He-sum; imps=15
      rcmp=rcmpf2(6,it1)+tconst*(rcmpf2(6,it2)-rcmpf2(6,it1))
      rcsum=rcsumf2(15,it1)+tconst*(rcsumf2(15,it2)-rcsumf2(15,it1))
      rcmp=rcmp+rcsum
      recof2(25)=rcmp*0.3182
      recof2(27)=rcmp*0.2727
      recof2(28)=rcmp*0.2273
      recof2(29)=rcmp*0.1818
c b 4Fe ; imp=7, il=32(10),33(8),34(6),35(4); gsum=28
      rcmp=rcmpf2(7,it1)+tconst*(rcmpf2(7,it2)-rcmpf2(7,it1))
      recof2(32)=rcmp*0.3571
      recof2(33)=rcmp*0.2857
      recof2(34)=rcmp*0.2143
      recof2(35)=rcmp*0.1429
c a 6Se ; imp=8, il=36(6) 
c Add 6Se-sum; imps=1
      rcmp=rcmpf2(8,it1)+tconst*(rcmpf2(8,it2)-rcmpf2(8,it1))
      rcsum=rcsumf2(1,it1)+tconst*(rcsumf2(1,it2)-rcsumf2(1,it1))
      rcmp=rcmp+rcsum
      recof2(36)=rcmp
c a 4Ge ; imp=9, il=37(12),39(10),40(8),41(6); gsum=36
      rcmp=rcmpf2(9,it1)+tconst*(rcmpf2(9,it2)-rcmpf2(9,it1))
      recof2(37)=rcmp*0.3333
      recof2(39)=rcmp*0.2778
      recof2(40)=rcmp*0.2222
      recof2(41)=rcmp*0.1667
c b 4De ; imp=10, il=49(4),50(2),51(6),52(8); gsum=20
      rcmp=rcmpf2(10,it1)+tconst*(rcmpf2(10,it2)-rcmpf2(10,it1))
      recof2(49)=rcmp*0.2
      recof2(50)=rcmp*0.1
      recof2(51)=rcmp*0.3
      recof2(52)=rcmp*0.4
c z 6Do ; imp=11, il=64(10),65(8),66(6),67(4),68(2); gsum=30
c Add 6Do-sum; imps=4
      rcmp=rcmpf2(11,it1)+tconst*(rcmpf2(11,it2)-rcmpf2(11,it1))
      rcsum=rcsumf2(4,it1)+tconst*(rcsumf2(4,it2)-rcsumf2(4,it1))
      rcmp=rcmp+rcsum
      recof2(64)=rcmp*0.3333
      recof2(65)=rcmp*0.2667
      recof2(66)=rcmp*0.2
      recof2(67)=rcmp*0.1333
      recof2(68)=rcmp*0.0667
c z 6Fo ; imp=12, il=69(12),70(10),71(8),72(6),73(4),74(2); gsum=42
c Add 6Fo-sum; imps=5
      rcmp=rcmpf2(12,it1)+tconst*(rcmpf2(12,it2)-rcmpf2(12,it1))
      rcsum=rcsumf2(5,it1)+tconst*(rcsumf2(5,it2)-rcsumf2(5,it1))
      rcmp=rcmp+rcsum
      recof2(69)=rcmp*0.2857
      recof2(70)=rcmp*0.2381
      recof2(71)=rcmp*0.1905
      recof2(72)=rcmp*0.1429
      recof2(73)=rcmp*0.0952
      recof2(74)=rcmp*0.0476
c z 6Po ; imp=13, il=75(8),76(6),77(4); gsum=18
c Add 6Po-sum; imps=2
      rcmp=rcmpf2(13,it1)+tconst*(rcmpf2(13,it2)-rcmpf2(13,it1))
      rcsum=rcsumf2(2,it1)+tconst*(rcsumf2(2,it2)-rcsumf2(2,it1))
      rcmp=rcmp+rcsum
      recof2(75)=rcmp*0.4445
      recof2(76)=rcmp*0.3333
      recof2(77)=rcmp*0.2222
c z 4Fo ; imp=14, il=78(10),80(8),85(6),87(4); gsum=28
      rcmp=rcmpf2(14,it1)+tconst*(rcmpf2(14,it2)-rcmpf2(14,it1))
      recof2(78)=rcmp*0.3571
      recof2(80)=rcmp*0.2857
      recof2(85)=rcmp*0.2143
      recof2(87)=rcmp*0.1429
c z 4Do ; imp=15, il=79(8),81(6),84(4),86(2); gsum=20
      rcmp=rcmpf2(15,it1)+tconst*(rcmpf2(15,it2)-rcmpf2(15,it1))
      recof2(79)=rcmp*0.4
      recof2(81)=rcmp*0.3
      recof2(84)=rcmp*0.2
      recof2(86)=rcmp*0.1
c z 4Po ; imp=16, il=88(6),89(4),90(2); gsum=12
      rcmp=rcmpf2(16,it1)+tconst*(rcmpf2(16,it2)-rcmpf2(16,it1))
      recof2(88)=rcmp*0.5
      recof2(89)=rcmp*0.3333
      recof2(90)=rcmp*0.1667
c c 4Pe ; imp=17, il=93(2),94(4),99(6); gsum=12
      rcmp=rcmpf2(17,it1)+tconst*(rcmpf2(17,it2)-rcmpf2(17,it1))
      recof2(93)=rcmp*0.1667
      recof2(94)=rcmp*0.3333
      recof2(99)=rcmp*0.5
c c 4Fe ; imp=18, il=95(4),96(6),97(10),98(8); gsum=28
c Add 4Fe-sum; imps=11
      rcmp=rcmpf2(18,it1)+tconst*(rcmpf2(18,it2)-rcmpf2(18,it1))
      rcsum=rcsumf2(11,it1)+tconst*(rcsumf2(11,it2)-rcsumf2(11,it1))
      rcmp=rcmp+rcsum
      recof2(95)=rcmp*0.1429
      recof2(96)=rcmp*0.2143
      recof2(97)=rcmp*0.3571
      recof2(98)=rcmp*0.2857
c b 4Ge ; imp=19, il=101(12),102(10),103(6),104(8); gsum=36
c Add 4Ge-sum; imps=13
      rcmp=rcmpf2(19,it1)+tconst*(rcmpf2(19,it2)-rcmpf2(19,it1))
      rcsum=rcsumf2(13,it1)+tconst*(rcsumf2(13,it2)-rcsumf2(13,it1))
      rcmp=rcmp+rcsum
      recof2(101)=rcmp*0.3333
      recof2(102)=rcmp*0.2778
      recof2(103)=rcmp*0.1667
      recof2(104)=rcmp*0.2222
c d 4Pe ; imp=20, il=108(6),109(4),110(2); gsum=12
c Add 4Pe-sum; imps=7
      rcmp=rcmpf2(20,it1)+tconst*(rcmpf2(20,it2)-rcmpf2(20,it1))
      rcsum=rcsumf2(7,it1)+tconst*(rcsumf2(7,it2)-rcsumf2(7,it1))
      rcmp=rcmp+rcsum
      recof2(108)=rcmp*0.5
      recof2(109)=rcmp*0.3333
      recof2(110)=rcmp*0.1667
c z 4So ; imp=21, il=113(4) 
      rcmp=rcmpf2(21,it1)+tconst*(rcmpf2(21,it2)-rcmpf2(21,it1))
      recof2(113)=rcmp
c c 4De ; imp=22, il=114(8),115(2),117(4),118(6); gsum=20
c Add 4De-sum; imps=9
      rcmp=rcmpf2(22,it1)+tconst*(rcmpf2(22,it2)-rcmpf2(22,it1))
      rcsum=rcsumf2(9,it1)+tconst*(rcsumf2(9,it2)-rcsumf2(9,it1))
      rcmp=rcmp+rcsum
      recof2(114)=rcmp*0.4
      recof2(115)=rcmp*0.1
      recof2(117)=rcmp*0.2
      recof2(118)=rcmp*0.3
c y 4Po ; imp=23, il=116(6),125(2),128(4); gsum=12
      rcmp=rcmpf2(23,it1)+tconst*(rcmpf2(23,it2)-rcmpf2(23,it1))
      recof2(116)=rcmp*0.5
      recof2(125)=rcmp*0.1667
      recof2(128)=rcmp*0.3333
c z 4Go ; imp=24, il=119(12),120(10),123(8),126(6); gsum=36
      rcmp=rcmpf2(24,it1)+tconst*(rcmpf2(24,it2)-rcmpf2(24,it1))
      recof2(119)=rcmp*0.3333
      recof2(120)=rcmp*0.2778
      recof2(123)=rcmp*0.2222
      recof2(126)=rcmp*0.1667
c z 4Ho ; imp=25, il=121(14),122(12),124(10),127(8); gsum=44
      rcmp=rcmpf2(25,it1)+tconst*(rcmpf2(25,it2)-rcmpf2(25,it1))
      recof2(121)=rcmp*0.3182
      recof2(122)=rcmp*0.2727
      recof2(124)=rcmp*0.2273
      recof2(127)=rcmp*0.1818
c z 4Io ; imp=26, il=129(16),130(10),131(14),132(12); gsum=52
c Add 4Io-sum; imps=17
      rcmp=rcmpf2(26,it1)+tconst*(rcmpf2(26,it2)-rcmpf2(26,it1))
      rcsum=rcsumf2(17,it1)+tconst*(rcsumf2(17,it2)-rcsumf2(17,it1))
      rcmp=rcmp+rcsum
      recof2(129)=rcmp*0.3077
      recof2(130)=rcmp*0.1923
      recof2(131)=rcmp*0.2692
      recof2(132)=rcmp*0.2308
c y 4Do ; imp=27, il=133(8),138(6),139(2),140(4); gsum=20
      rcmp=rcmpf2(27,it1)+tconst*(rcmpf2(27,it2)-rcmpf2(27,it1))
      recof2(133)=rcmp*0.4
      recof2(138)=rcmp*0.3
      recof2(139)=rcmp*0.1
      recof2(140)=rcmp*0.2
c y 4Fo ; imp=28, il=134(8),135(6),136(10),137(4); gsum=28
      rcmp=rcmpf2(28,it1)+tconst*(rcmpf2(28,it2)-rcmpf2(28,it1))
      recof2(134)=rcmp*0.2857
      recof2(135)=rcmp*0.2143
      recof2(136)=rcmp*0.3571
      recof2(137)=rcmp*0.1429
c x 4Do ; imp=29, il=141(8),142(6),143(4),144(2); gsum=20
      rcmp=rcmpf2(29,it1)+tconst*(rcmpf2(29,it2)-rcmpf2(29,it1))
      recof2(141)=rcmp*0.4
      recof2(142)=rcmp*0.3
      recof2(143)=rcmp*0.2
      recof2(144)=rcmp*0.1
c y 4Go ; imp=30, il=145(12),146(10),147(8),148(6); gsum=36
      rcmp=rcmpf2(30,it1)+tconst*(rcmpf2(30,it2)-rcmpf2(30,it1))
      recof2(145)=rcmp*0.3333
      recof2(146)=rcmp*0.2778
      recof2(147)=rcmp*0.2222
      recof2(148)=rcmp*0.1667
c x 4Go ; imp=31, il=149(12),150(10),151(8),153(6); gsum=36
      rcmp=rcmpf2(31,it1)+tconst*(rcmpf2(31,it2)-rcmpf2(31,it1))
      recof2(149)=rcmp*0.3333
      recof2(150)=rcmp*0.2778
      recof2(151)=rcmp*0.2222
      recof2(153)=rcmp*0.1667
c x 4Fo ; imp=32, il=152(10),154(8),157(6),159(4); gsum=28
      rcmp=rcmpf2(32,it1)+tconst*(rcmpf2(32,it2)-rcmpf2(32,it1))
      recof2(152)=rcmp*0.3571
      recof2(154)=rcmp*0.2857
      recof2(157)=rcmp*0.2143
      recof2(159)=rcmp*0.1429
c y 4Ho ; imp=33, il=155(14),156(12),158(10),160(8); gsum=44
c Add 4Ho-sum; imps=16
      rcmp=rcmpf2(33,it1)+tconst*(rcmpf2(33,it2)-rcmpf2(33,it1))
      rcsum=rcsumf2(16,it1)+tconst*(rcsumf2(16,it2)-rcsumf2(16,it1))
      rcmp=rcmp+rcsum
      recof2(155)=rcmp*0.3182
      recof2(156)=rcmp*0.2727
      recof2(158)=rcmp*0.2273
      recof2(160)=rcmp*0.1818
c w 4Po ; imp=34, il=161(6),162(4),164(2); gsum=12
      rcmp=rcmpf2(34,it1)+tconst*(rcmpf2(34,it2)-rcmpf2(34,it1))
      recof2(161)=rcmp*0.5
      recof2(162)=rcmp*0.3333
      recof2(164)=rcmp*0.1667
c w 4Fo ; imp=35, il=163(4),165(6),166(8),169(10); gsum=28
      rcmp=rcmpf2(35,it1)+tconst*(rcmpf2(35,it2)-rcmpf2(35,it1))
      recof2(163)=rcmp*0.1429
      recof2(165)=rcmp*0.2143
      recof2(166)=rcmp*0.2857
      recof2(169)=rcmp*0.3571
c w 4Do ; imp=36, il=167(2),168(6),170(8),171(4); gsum=20
      rcmp=rcmpf2(36,it1)+tconst*(rcmpf2(36,it2)-rcmpf2(36,it1))
      recof2(167)=rcmp*0.1
      recof2(168)=rcmp*0.3
      recof2(170)=rcmp*0.4
      recof2(171)=rcmp*0.2
c v 4Do ; imp=37, il=172(2),173(4),174(6),175(8); gsum=20
      rcmp=rcmpf2(37,it1)+tconst*(rcmpf2(37,it2)-rcmpf2(37,it1))
      recof2(172)=rcmp*0.1
      recof2(173)=rcmp*0.2
      recof2(174)=rcmp*0.3
      recof2(175)=rcmp*0.4
c w 4Go ; imp=38, il=176(6),177(8),178(10),179(12); gsum=36
c Add 4Go-sum; imps=14
      rcmp=rcmpf2(38,it1)+tconst*(rcmpf2(38,it2)-rcmpf2(38,it1))
      rcsum=rcsumf2(14,it1)+tconst*(rcsumf2(14,it2)-rcsumf2(14,it1))
      rcmp=rcmp+rcsum
      recof2(176)=rcmp*0.1667
      recof2(177)=rcmp*0.2222
      recof2(178)=rcmp*0.2778
      recof2(179)=rcmp*0.3333
c y 4So ; imp=39, il=180(4); gsum=4
c Add 4So-sum; imps=6
      rcmp=rcmpf2(39,it1)+tconst*(rcmpf2(39,it2)-rcmpf2(39,it1))
      rcsum=rcsumf2(6,it1)+tconst*(rcsumf2(6,it2)-rcsumf2(6,it1))
      rcmp=rcmp+rcsum
      recof2(180)=rcmp
c u 4Po ; imp=40, il=181(2),182(4),183(6); gsum=12
c Add 4Po-sum; imps=8
      rcmp=rcmpf2(40,it1)+tconst*(rcmpf2(40,it2)-rcmpf2(40,it1))
      rcsum=rcsumf2(8,it1)+tconst*(rcsumf2(8,it2)-rcsumf2(8,it1))
      rcmp=rcmp+rcsum
      recof2(181)=rcmp*0.1667
      recof2(182)=rcmp*0.3333
      recof2(183)=rcmp*0.5
c t 4Do ; imp=41, il=184(2),185(4),186(6),187(8); gsum=20
c Add 4Do-sum; imps=10
      rcmp=rcmpf2(41,it1)+tconst*(rcmpf2(41,it2)-rcmpf2(41,it1))
      rcsum=rcsumf2(10,it1)+tconst*(rcsumf2(10,it2)-rcsumf2(10,it1))
      rcmp=rcmp+rcsum
      recof2(184)=rcmp*0.1
      recof2(185)=rcmp*0.2
      recof2(186)=rcmp*0.3
      recof2(187)=rcmp*0.4
c t 4Fo ; imp=42, il=188(4),189(6),190(10),191(8); gsum=28
c Add 4Fo-sum; imps=12
      rcmp=rcmpf2(42,it1)+tconst*(rcmpf2(42,it2)-rcmpf2(42,it1))
      rcsum=rcsumf2(12,it1)+tconst*(rcsumf2(12,it2)-rcsumf2(12,it1))
      rcmp=rcmp+rcsum
      recof2(188)=rcmp*0.1429
      recof2(189)=rcmp*0.2143
      recof2(190)=rcmp*0.3571
      recof2(191)=rcmp*0.2857
c
c Add the remaining recombinations to the 4 highest levels 
c (188 - 191; t4F) weighted
c with their statistical weights so that the total recombination 
c coefficient is correct ; gsum=28 (4,6,10,8)
c
      recsum=0.d0
cqcq >run 70.7
c      do il=1,nlf2
      do il=1,nlev(iefe,2)
         recsum=recsum+recof2(il)
      enddo
      recadd=rctfe2-recsum
      if (recadd.lt.0.) then
         write(50,*) 'Error in calc. rec. coeff. for Fe II !!'
      endif
cqcq > run 70.7
c      recof2(188)=recof2(188)+recadd*0.1429
c      recof2(189)=recof2(189)+recadd*0.2143
c      recof2(190)=recof2(190)+recadd*0.3571
c      recof2(191)=recof2(191)+recadd*0.2857
c add the remaining recomb. contribution to
c all the levels, not only the highest
      gsum=0.d0
cqcq >run 70.7
c      do il=1,nlf2
      do il=1,nlev(iefe,2)
         gsum=gsum+gf2(il)
      enddo
cqcq >run 70.7
c      do il=1,nlf2
      do il=1,nlev(iefe,2)
         recof2(il)=recof2(il)+recadd*gf2(il)/gsum
      enddo
cqcq   
c
      return
      end
